@@ -176,7 +176,10 @@ int c128ui_init(void)
             cartridge_enable,
             cartridge_disable,
             cartridge_can_save_image,
-            cartridge_can_flush_image);
+            cartridge_can_flush_image,
+            cartridge_set_default,
+            cartridge_unset_default,
+            cartridge_get_info_list);
 
     /* ui_cart_set_detect_func(cartridge_detect); only cbm2/plus4 */
     ui_cart_set_list_func(cartridge_get_info_list);
@@ -194,13 +197,15 @@ int c128ui_init(void)
 
     /* push VDC display to front depending on 40/80 key */
     if (resources_get_int("C128ColumnKey", &forty) >= 0) {
-        debug_gtk3("col mode: %d.", forty ? 40 : 80);
+        GtkWidget *window;
 
-        if (!forty) {
-            GtkWidget *window = ui_get_window_by_index(1); /* VDC */
-            if (window != NULL) {
-                gtk_window_present(GTK_WINDOW(window));
-            }
+        if (forty) {
+            window = ui_get_window_by_index(0); /* VICIIe */
+        } else {
+            window = ui_get_window_by_index(1); /* VDC */
+        }
+        if (window != NULL) {
+            gtk_window_present(GTK_WINDOW(window));
         }
     }
 
@@ -209,7 +214,6 @@ int c128ui_init(void)
         GtkWidget *window;
 
         if (hide_vdc) {
-            debug_gtk3("Attempting to hide the VDC window according to C128HideVDC");
             window = ui_get_window_by_index(1); /* VDC */
             if (window != NULL) {
                 gtk_widget_hide(window);
