@@ -136,7 +136,10 @@ int cbm2ui_init(void)
             NULL, /* cartridge_enable */
             NULL, /* cartridge_disable */
             NULL, /* cartridge_can_save_image */
-            NULL  /* cartridge_can_flush_image */);
+            NULL, /* cartridge_can_flush_image */
+            NULL,
+            NULL,
+            NULL);
 
     /* ui_cart_set_detect_func(cartridge_detect); only cbm2/plus4 */
     /*ui_cart_set_list_func(cartridge_get_info_list);*/
