@@ -59,7 +59,7 @@ typedef struct vice_opengl_renderer_context_s {
 
     /** \brief A queue of backbuffers ready for painting to the widget */
     void *render_queue;
-    
+
 #ifdef MACOSX_SUPPORT
     /** \brief native child window for OpenGL to draw on */
     void *native_view;
@@ -81,7 +81,7 @@ typedef struct vice_opengl_renderer_context_s {
 
     /** \brief size of the backing layer in pixels. Can be higher than native_view_width (high dpi) */
     unsigned int gl_backing_layer_width;
-    
+
     /** \brief size of the backing layer in pixels. Can be higher than native_view_height (high dpi) */
     unsigned int gl_backing_layer_height;
 
@@ -93,7 +93,7 @@ typedef struct vice_opengl_renderer_context_s {
 
     /** \brief size of the native view in gtk coordinates (which are not pixels) */
     unsigned int native_view_width;
-    
+
     /** \brief size of the native view in gtk coordinates (which are not pixels) */
     unsigned int native_view_height;
 
@@ -102,24 +102,27 @@ typedef struct vice_opengl_renderer_context_s {
 
     /** \brief minimum size for the drawing area, based on emu and aspect ratio settings */
     unsigned int native_view_min_height;
-    
+
     /** \brief background colour for the native view */
     float native_view_bg_r;
-    
+
     /** \brief background colour for the native view */
     float native_view_bg_g;
-    
+
     /** \brief background colour for the native view */
     float native_view_bg_b;
 
-    /** \brief The OpenGL program that comprises our vertex and fragment shaders. */
-    GLuint program;
+    /** \brief GLSL shader */
+    GLuint shader_builtin;
 
-    /** \brief The index of the "position" parameter in the shader program. */
-    GLuint position_index;
+    /** \brief GLSL shader */
+    GLuint shader_builtin_interlaced;
 
-    /** \brief The index of the "texCoord" parameter in the shader program. */
-    GLuint tex_coord_index;
+    /** \brief GLSL shader */
+    GLuint shader_bicubic;
+
+    /** \brief GLSL shader */
+    GLuint shader_bicubic_interlaced;
 
     /** \brief The vertex buffer object that holds our vertex data. */
     GLuint vbo;
@@ -128,29 +131,30 @@ typedef struct vice_opengl_renderer_context_s {
     GLuint vao;
 
     /** \brief The texture identifier for the GPU's copy of our  machine display. */
-    GLuint texture;
+    GLuint current_frame_texture;
+    unsigned int current_frame_width;
+    unsigned int current_frame_height;
+    bool interlaced;
+    int current_interlace_field;
+    float pixel_aspect_ratio;
+
+    /** \brief The texture identifier for the GPU's copy of our  machine display. */
+    GLuint previous_frame_texture;
+    unsigned int previous_frame_width;
+    unsigned int previous_frame_height;
 
     /** \brief size of the next frame to be emulated */
     unsigned int emulated_width_next;
-    
+
     /** \brief size of the next frame to be emulated */
     unsigned int emulated_height_next;
 
     /** \brief pixel aspect ratio of the next frame to be emulated */
     float pixel_aspect_ratio_next;
 
-    /** \brief size of the last frame to be rendered on host */
-    unsigned int emulated_width_last_rendered;
-    
-    /** \brief size of the last frame to be rendered on host */
-    unsigned int emulated_height_last_rendered;
-
-    /** \brief pixel aspect ratio of the last frame to be rendered on host */
-    float pixel_aspect_ratio_last_rendered;
-
     /** \brief when the last frame was rendered */
     unsigned long last_render_time;
-    
+
     /** \brief cached value of the vsync resource to avoid setting it each frame */
     unsigned long cached_vsync_resource;
 
@@ -159,7 +163,7 @@ typedef struct vice_opengl_renderer_context_s {
 void vice_opengl_renderer_create_child_view(GtkWidget *widget, vice_opengl_renderer_context_t *context);
 void vice_opengl_renderer_resize_child_view(vice_opengl_renderer_context_t *context);
 void vice_opengl_renderer_destroy_child_view(vice_opengl_renderer_context_t *context);
-     
+
 void vice_opengl_renderer_make_current(vice_opengl_renderer_context_t *context);
 void vice_opengl_renderer_set_viewport(vice_opengl_renderer_context_t *context);
 void vice_opengl_renderer_set_vsync(vice_opengl_renderer_context_t *context, bool enable_vsync);
