@@ -140,15 +140,18 @@ int plus4ui_init(void)
             NULL, /* cartridge_enable */
             NULL, /* cartridge_disable */
             NULL, /* cartridge_can_save_image */
-            NULL  /* cartridge_can_flush_image */);
+            NULL, /* cartridge_can_flush_image */
+            cartridge_set_default,
+            cartridge_unset_default,
+            cartridge_get_info_list);
 
     /* ui_cart_set_detect_func(cartridge_detect); only cbm2/plus4 */
-/*    ui_cart_set_list_func(cartridge_get_info_list); */
+    ui_cart_set_list_func(cartridge_get_info_list);
     ui_cart_set_attach_func(cartridge_attach_image);
-/*    ui_cart_set_freeze_func(cartridge_trigger_freeze); */
+    ui_cart_set_freeze_func(cartridge_trigger_freeze);
     ui_cart_set_detach_func(cartridge_detach_image);
-    /*ui_cart_set_set_default_func(cartridge_set_default);*/
-    /*ui_cart_set_unset_default_func(cartridge_unset_default);*/
+    ui_cart_set_set_default_func(cartridge_set_default);
+    ui_cart_set_unset_default_func(cartridge_unset_default);
 
     settings_model_widget_set_memhack_func(plus4_memory_hacks_desc);
 
