@@ -84,12 +84,10 @@ void ui_destroy_main_window(int index);
 void ui_dispatch_events(void);
 void ui_exit(void);
 
-void ui_update_statusbars(void);
-
 int  ui_is_fullscreen(void);
 void ui_trigger_resize(void);
-gboolean ui_fullscreen_callback(GtkWidget *widget, gpointer user_data);
-gboolean ui_fullscreen_decorations_callback(GtkWidget *widget, gpointer user_data);
+
+
 
 GtkWindow *ui_get_active_window(void);
 video_canvas_t *ui_get_active_canvas(void);
@@ -99,15 +97,28 @@ video_canvas_t *ui_get_active_canvas(void);
  */
 int  ui_pause_active(void);
 void ui_pause_enable(void);
+bool ui_pause_loop_iteration(void);
 void ui_pause_enter_monitor(void);
 void ui_pause_disable(void);
 void ui_pause_toggle(void);
 
-gboolean ui_toggle_pause(void);
-gboolean ui_advance_frame(void);
+/*
+ * UI 'actions' for the custom hotkeys, perhaps move to uicommands.c/uiactions.c?
+ */
+gboolean ui_action_toggle_pause(void);
+gboolean ui_action_toggle_warp(void);
+gboolean ui_action_advance_frame(void);
+gboolean ui_action_toggle_fullscreen(void);
+gboolean ui_action_toggle_fullscreen_decorations(void);
+void     ui_action_set_speed(int speed);
+void     ui_action_set_fps(int fps);
+
+gboolean ui_speed_custom_toggled(GtkWidget *widget, gpointer data);
+gboolean ui_fps_custom_toggled(GtkWidget *widget, gpointer data);
+gboolean ui_cpu_speed_callback(GtkWidget *widget, gpointer data);
+gboolean ui_fps_callback(GtkWidget *widget, gpointer data);
 
 void ui_update_lightpen(void);
-
 void ui_enable_crt_controls(int enabled);
 void ui_enable_mixer_controls(int enabled);
 
