@@ -38,6 +38,7 @@
 #include "attach.h"
 #include "autostart.h"
 #include "drive.h"
+#include "log.h"
 #include "tape.h"
 #include "debug_gtk3.h"
 #include "basedialogs.h"
@@ -50,30 +51,26 @@
 #include "mainlock.h"
 #include "resources.h"
 #include "ui.h"
+#include "uiapi.h"
 #include "uistatusbar.h"
 #include "uimachinewindow.h"
 #include "lastdir.h"
 
 #include "uidiskattach.h"
 
-#ifndef SANDBOX_MODE
 /** \brief  File type filters for the dialog
  */
 static ui_file_filter_t filters[] = {
-    { "Disk images", file_chooser_pattern_disk },
-    { "Compressed files", file_chooser_pattern_compressed },
-    { "All files", file_chooser_pattern_all },
+    { "Disk images",        file_chooser_pattern_disk },
+    { "Compressed files",   file_chooser_pattern_compressed },
+    { "All files",          file_chooser_pattern_all },
     { NULL, NULL }
 };
-#endif
 
-#ifndef SANDBOX_MODE
 /** \brief  Array to keep track of drive numbers for each unit
  */
 static int unit_drive_nums[NUM_DISK_UNITS] = { 0, 0, 0, 0};
-#endif
 
-#ifndef SANDBOX_MODE
 /** \brief  Preview widget reference
  */
 static GtkWidget *preview_widget = NULL;
@@ -81,7 +78,6 @@ static GtkWidget *preview_widget = NULL;
 /** \brief  Drive number widget reference
  */
 static GtkWidget *driveno_widget = NULL;
-#endif
 
 
 /** \brief  Last directory used
@@ -91,8 +87,14 @@ static GtkWidget *driveno_widget = NULL;
  * ui_disk_attach_shutdown() on emulator shutdown.
  */
 static gchar *last_dir = NULL;
+
+/** \brief  Last filename used */
 static gchar *last_file = NULL;
 
+/** \brief  Reference to the custom 'Autostart' button
+ */
+static GtkWidget *autostart_button;
+
 
 /** \brief  Unit number to attach disk to
  */
@@ -102,29 +104,8 @@ static int unit_number = DRIVE_UNIT_DEFAULT;
  */
 static int drive_number = 0;
 
-#if 0
-/** \brief  Update the last directory reference
- *
- * \param[in]   widget  dialog
- */
-static void update_last_dir(GtkWidget *widget)
-{
-    gchar *new_dir;
-
-    new_dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(widget));
-    debug_gtk3("new dir = '%s'.", new_dir);
-    if (new_dir != NULL) {
-        /* clean up previous value */
-        if (last_dir != NULL) {
-            g_free(last_dir);
-        }
-        last_dir = new_dir;
-    }
-}
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Handler for the 'toggled' event of the 'show hidden files' checkbox
  *
  * \param[in]   widget      checkbox triggering the event
@@ -135,14 +116,10 @@ static void on_hidden_toggled(GtkWidget *widget, gpointer user_data)
     int state;
 
     state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
-    debug_gtk3("show hidden files: %s.", state ? "enabled" : "disabled");
-
     gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER(user_data), state);
 }
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Handler for the 'toggled' event of the 'attach read-only' checkbox
  *
  * \param[in]   widget      checkbox triggering the event
@@ -153,14 +130,10 @@ static void on_readonly_toggled(GtkWidget *widget, gpointer user_data)
     int state;
 
     state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
-    debug_gtk3("read-only: %s.", state ? "enabled" : "disabled");
-
     resources_set_int_sprintf("AttachDevice%dReadonly", state, unit_number);
 }
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Handler for the "update-preview" event
  *
  * \param[in]   chooser file chooser dialog
@@ -177,22 +150,15 @@ static void on_update_preview(GtkFileChooser *chooser, gpointer data)
         if (path != NULL) {
             gchar *filename_locale = file_chooser_convert_to_locale(path);
 
-            debug_gtk3("called with '%s'.", path);
-
             content_preview_widget_set_image(preview_widget, filename_locale);
             g_free(path);
             g_free(filename_locale);
         }
         g_object_unref(file);
     }
-#if 0
-    content_preview_widget_set_index(preview_widget, 2);
-#endif
 }
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Callback for the drive unit widget
  *
  * Updates the drive number widget, depending on the drive unit's type:
@@ -205,20 +171,12 @@ static void on_unit_changed(int unit)
 {
     int dual = drive_is_dualdrive_by_devnr(unit);
 
-    debug_gtk3(
-        "Unit #%d: type %d, dual-drive: %s.",
-        unit,
-        drive_get_type_by_devnr(unit),
-        dual ? "TRUE" : "FALSE");
-
     gtk_widget_set_sensitive(driveno_widget, dual);
     drive_no_widget_update(driveno_widget,
                            unit_drive_nums[unit - DRIVE_UNIT_MIN]);
 }
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Callback for the drive number widget
  *
  * Update drive number for the current unit, to later be used to restore the
@@ -228,34 +186,27 @@ static void on_unit_changed(int unit)
  */
 static void on_drive_num_changed(int drive)
 {
-    debug_gtk3("Got drive number %d for unit #%d.", drive, unit_number);
-
     unit_drive_nums[unit_number - DRIVE_UNIT_MIN] = drive;
 
 }
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Trigger autostarting a disk image
  *
  * \param[in,out]   widget      dialog
- * \param[in]       user_data   file index in the image
+ * \param[in]       index       file index in the image's directory listing
+ * \param[in]       autostart   issue "RUN:" after loading
  */
-static void do_autostart(GtkWidget *widget, gpointer user_data)
+static void do_autostart(GtkWidget *widget, int index, int autostart)
 {
     gchar *filename;
     gchar *filename_locale;
-    int index = GPOINTER_TO_INT(user_data);
-
-    mainlock_assert_lock_obtained();
 
     lastdir_update(widget, &last_dir, &last_file);
     filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
     /* convert filename to current locale */
     filename_locale = file_chooser_convert_to_locale(filename);
 
-    debug_gtk3("Autostarting file '%s'.", filename);
     /* if this function exists, why is there no attach_autodetect()
      * or something similar? -- compyx */
     if (autostart_disk(
@@ -266,90 +217,155 @@ static void do_autostart(GtkWidget *widget, gpointer user_data)
                 index,  /* Program number? Probably used when clicking
                            in the preview widget to load the proper
                            file in an image */
-                AUTOSTART_MODE_RUN) < 0) {
+                autostart ? AUTOSTART_MODE_RUN : AUTOSTART_MODE_LOAD) < 0) {
         /* oeps */
-        debug_gtk3("autostart disk attach failed.");
+        log_error(LOG_ERR, "autostart disk attach failed.");
+        ui_error("Autostart disk attach failed.");
     }
     g_free(filename);
     g_free(filename_locale);
-    gtk_widget_destroy(widget);
 }
-#endif
 
-#if 0
-static void on_file_activated(GtkWidget *chooser, gpointer data)
+
+/** \brief  Attach a disk image
+ *
+ * \param[in,out]   widget      dialog
+ * \param[in]       user_data   file index in the image
+ */
+static void do_attach(GtkWidget *widget, gpointer user_data)
 {
-    do_autostart(chooser, data);
+    gchar *filename;
+    gchar *filename_locale;
+    gchar buffer[1024];
+
+    lastdir_update(widget, &last_dir, &last_file);
+
+    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
+    /* convert filename to current locale */
+    filename_locale = file_chooser_convert_to_locale(filename);
+
+    /* copied from Gtk2: I fail to see how brute-forcing your way
+        * through file types is 'smart', but hell, it works */
+    if (file_system_attach_disk(unit_number, drive_number, filename_locale) < 0) {
+        /* failed */
+        g_snprintf(buffer, 1024, "Unit #%d: failed to attach '%s'",
+                unit_number, filename);
+    } else {
+        g_snprintf(buffer, 1024, "Unit #%d: attached '%s'",
+                unit_number, filename);
+    }
+    ui_display_statustext(buffer, 1);
+    g_free(filename_locale);
 }
-#endif
 
 
-#ifndef SANDBOX_MODE
+/** \brief  Handler for 'selection-changed' event of the preview widget
+ *
+ * Checks if a proper selection was made and activates the 'Autostart' button
+ * if so, disabled it otherwise
+ *
+ * \param[in]   chooser Parent dialog
+ * \param[in]   data    Extra event data (unused)
+ */
+static void on_selection_changed(GtkFileChooser *chooser, gpointer data)
+{
+    gchar *filename;
+
+    filename = gtk_file_chooser_get_filename(chooser);
+    if (filename != NULL) {
+        gtk_widget_set_sensitive(autostart_button, TRUE);
+        g_free(filename);
+    } else {
+        gtk_widget_set_sensitive(autostart_button, FALSE);
+    }
+}
+
+
+
 /** \brief  Handler for 'response' event of the dialog
  *
  * This handler is called when the user clicks a button in the dialog.
  *
  * \param[in]   widget      the dialog
  * \param[in]   response_id response ID
- * \param[in]   user_data   extra data (unused)
- *
- * TODO:    proper (error) messages, which requires implementing ui_error() and
- *          ui_message() and moving them into gtk3/widgets to avoid circular
- *          references
+ * \param[in]   user_data   unit number
  */
-static void on_response(GtkWidget *widget, gint response_id,
-                        gpointer user_data)
+static void on_response(GtkWidget *widget, gint response_id, gpointer user_data)
 {
-    gchar *filename;
-    gchar *filename_locale;
-    gchar buffer[1024];
+    gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
+    int index = content_preview_widget_get_index(preview_widget);
+    int autostart = 0;
 
-#ifdef HAVE_DEBUG_GTK3UI
-    int index = GPOINTER_TO_INT(user_data);
-    debug_gtk3("got response ID %d, index %d.", response_id, index);
-#endif
+    resources_get_int("AutostartOnDoubleclick", &autostart);
 
-    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
+    /* first, to make the following logic less funky, map some events to others,
+       depending on whether autostart-on-doubleclick is enabled or not, and
+       depending on the event coming from the preview window or not. */
+    switch (response_id) {
+        /* double-click on file in the preview widget when autostart-on-doubleclick is NOT enabled */
+        case VICE_RESPONSE_AUTOLOAD_INDEX:
+        /* double-click on file in the preview widget when autostart-on-doubleclick is enabled */
+        case VICE_RESPONSE_AUTOSTART_INDEX:
+            if ((index < 0) || (filename == NULL)) {
+                response_id = VICE_RESPONSE_INVALID;   /* drop this event */
+            }
+            break;
+        /* 'Open' button clicked when autostart-on-doubleclick is enabled */
+        case VICE_RESPONSE_CUSTOM_OPEN:
+            if (filename == NULL) {
+                response_id = VICE_RESPONSE_INVALID;   /* drop this event */
+            } else if (index >= 0) {
+                response_id = VICE_RESPONSE_AUTOLOAD_INDEX;
+            }
+            break;
+        /* double-click on file in the main file chooser,
+           'Autostart' button when autostart-on-doubleclick is enabled
+           'Open' button when autostart-on-doubleclick is NOT enabled */
+        case GTK_RESPONSE_ACCEPT:
+            if (filename == NULL) {
+                response_id = VICE_RESPONSE_INVALID;   /* drop this event */
+            } else if ((index >= 0) && (autostart == 0)) {
+                response_id = VICE_RESPONSE_AUTOLOAD_INDEX;
+            } else if ((index >= 0) && (autostart == 1)) {
+                response_id = VICE_RESPONSE_AUTOSTART_INDEX;
+            } else if (autostart == 0) {
+                response_id = VICE_RESPONSE_CUSTOM_OPEN;
+            } else {
+                response_id = VICE_RESPONSE_AUTOSTART;
+            }
+            break;
+        default:
+            break;
+    }
 
     switch (response_id) {
+        /* 'Open' button clicked when autostart-on-doubleclick is enabled */
+        case VICE_RESPONSE_CUSTOM_OPEN:
+            do_attach(widget, user_data);
 
-        /* 'Open' button, double-click on file */
-        case GTK_RESPONSE_ACCEPT:
-
-            lastdir_update(widget, &last_dir, &last_file);
-            filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
-            /* convert filename to current locale */
-            filename_locale = file_chooser_convert_to_locale(filename);
-
-            /* ui_message("Opening file '%s' ...", filename); */
-            debug_gtk3("Attaching file '%s' to unit #%d.",
-                    filename, unit_number);
-
-            /* copied from Gtk2: I fail to see how brute-forcing your way
-             * through file types is 'smart', but hell, it works */
-            if (file_system_attach_disk(unit_number, drive_number, filename_locale) < 0) {
-                /* failed */
-                debug_gtk3("disk attach failed.");
-                g_snprintf(buffer, 1024, "Unit #%d: failed to attach '%s'",
-                        unit_number, filename);
-            } else {
-                g_snprintf(buffer, 1024, "Unit #%d: attached '%s'",
-                        unit_number, filename);
-            }
-            ui_display_statustext(buffer, 1);
-            g_free(filename_locale);
+            mainlock_release();
             gtk_widget_destroy(widget);
+            mainlock_obtain();
             break;
 
-        /* 'Autostart' button clicked */
+        /* 'Autostart' button clicked when autostart-on-doubleclick is NOT enabled */
         case VICE_RESPONSE_AUTOSTART:
-            /* did we actually get a filename? */
-            if (filename != NULL) {
-                do_autostart(widget, user_data);
-                mainlock_release();
-                gtk_widget_destroy(widget);
-                mainlock_obtain();
-            }
+        /* double-click on file in the preview widget when autostart-on-doubleclick is enabled */
+        case VICE_RESPONSE_AUTOSTART_INDEX:
+            do_autostart(widget, index + 1, 1);
+
+            mainlock_release();
+            gtk_widget_destroy(widget);
+            mainlock_obtain();
+            break;
+
+        /* double-click on file in the preview widget when autostart-on-doubleclick is NOT enabled */
+        case VICE_RESPONSE_AUTOLOAD_INDEX:
+            do_autostart(widget, index + 1, 0);
+
+            mainlock_release();
+            gtk_widget_destroy(widget);
+            mainlock_obtain();
             break;
 
         /* 'Close'/'X' button */
@@ -367,62 +383,8 @@ static void on_response(GtkWidget *widget, gint response_id,
     }
 }
 
-#else
-
-static void on_response(GtkWidget *widget, gint response_id,
-                        gpointer user_data)
-{
-    gchar *filename;
-    gchar *filename_locale;
-    gchar buffer[1024];
-    GtkFileChooser *chooser = GTK_FILE_CHOOSER(widget);
-
-    debug_gtk3("Reponse-ID: %d", response_id);
-
-    filename = gtk_file_chooser_get_filename(chooser);
-    debug_gtk3("FIlename: '%s'", filename);
-
-    switch (response_id) {
-        case GTK_RESPONSE_ACCEPT:
-
-            lastdir_update(widget, &last_dir, &last_file);
-            filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
-            /* convert filename to current locale */
-            filename_locale = file_chooser_convert_to_locale(filename);
-
-            /* ui_message("Opening file '%s' ...", filename); */
-            debug_gtk3("Attaching file '%s' to unit #%d.",
-                    filename, unit_number);
-
-            /* copied from Gtk2: I fail to see how brute-forcing your way
-             * through file types is 'smart', but hell, it works */
-            if (file_system_attach_disk(unit_number, filename_locale) < 0) {
-                /* failed */
-                debug_gtk3("disk attach failed.");
-                g_snprintf(buffer, 1024, "Unit #%d: failed to attach '%s'",
-                        unit_number, filename);
-            } else {
-                g_snprintf(buffer, 1024, "Unit #%d: attached '%s'",
-                        unit_number, filename);
-            }
-            ui_display_statustext(buffer, 1);
-            g_free(filename_locale);
-            break;
-        default:
-            break;
-    }
-
-    if (filename != NULL) {
-        g_free(filename);
-    }
-    mainlock_release();
-    gtk_native_dialog_destroy(GTK_NATIVE_DIALOG(chooser));
-    mainlock_obtain();
-}
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Create the 'extra' widget
  *
  * \param[in]   parent  parent widget
@@ -461,10 +423,6 @@ static GtkWidget *create_extra_widget(GtkWidget *parent, int unit)
 
     /* add drive number widget */
     driveno_widget = drive_no_widget_create(0, &drive_number, on_drive_num_changed);
-#if 0
-    drive_type = drive_get_type_by_devnr(unit);
-    debug_gtk3("Drive type for unit #%d: %d.", unit, drive_type);
-#endif
     gtk_widget_set_sensitive(driveno_widget, drive_is_dualdrive_by_devnr(unit));
 
     gtk_grid_attach(GTK_GRID(grid),
@@ -474,10 +432,8 @@ static GtkWidget *create_extra_widget(GtkWidget *parent, int unit)
     gtk_widget_show_all(grid);
     return grid;
 }
-#endif
 
 
-#ifndef SANDBOX_MODE
 
 /** \brief  Create the disk attach dialog
  *
@@ -490,6 +446,9 @@ static GtkWidget *create_disk_attach_dialog(GtkWidget *parent, int unit)
 {
     GtkWidget *dialog;
     size_t i;
+    int autostart = 0;
+
+    resources_get_int("AutostartOnDoubleclick", &autostart);
 
     /* create new dialog */
     dialog = gtk_file_chooser_dialog_new(
@@ -497,11 +456,32 @@ static GtkWidget *create_disk_attach_dialog(GtkWidget *parent, int unit)
             ui_get_active_window(),
             GTK_FILE_CHOOSER_ACTION_OPEN,
             /* buttons */
-            "Open", GTK_RESPONSE_ACCEPT,
-            "Autostart", VICE_RESPONSE_AUTOSTART,
-            "Close", GTK_RESPONSE_REJECT,
+            /* buttons */
             NULL, NULL);
 
+    /* We need to manually add the buttons here, not using the constructor
+     * above, to keep the order of the buttons the same as in the other
+     * 'attach' dialogs. Gtk doesn't allow getting references to buttons when
+     * added via the constructor, meaning we cannot get a reference to the
+     * "Autostart" button in order to "grey it out".
+     */
+    /* to handle the "double click means autostart" option, we need to always
+       connect a button to GTK_RESPONSE_ACCEPT, else double clicks will stop
+       working */
+    if (autostart) {
+        gtk_dialog_add_button(GTK_DIALOG(dialog), "Attach / Load", VICE_RESPONSE_CUSTOM_OPEN);
+        autostart_button = gtk_dialog_add_button(GTK_DIALOG(dialog),
+                                                "Autostart",
+                                                GTK_RESPONSE_ACCEPT);
+    } else {
+        gtk_dialog_add_button(GTK_DIALOG(dialog), "Attach / Load", GTK_RESPONSE_ACCEPT);
+        autostart_button = gtk_dialog_add_button(GTK_DIALOG(dialog),
+                                                "Autostart",
+                                                VICE_RESPONSE_AUTOSTART);
+    }
+
+    gtk_widget_set_sensitive(autostart_button, FALSE);
+    gtk_dialog_add_button(GTK_DIALOG(dialog), "Close", GTK_RESPONSE_REJECT);
     /* set modal so mouse-grab doesn't get triggered */
     gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
 
@@ -516,7 +496,7 @@ static GtkWidget *create_disk_attach_dialog(GtkWidget *parent, int unit)
                                       create_extra_widget(dialog, unit));
 
     preview_widget = content_preview_widget_create(
-            dialog, diskcontents_filesystem_read, on_response);
+            dialog, diskcontents_filesystem_read, on_response, unit);
     gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(dialog),
             preview_widget);
 
@@ -533,38 +513,12 @@ static GtkWidget *create_disk_attach_dialog(GtkWidget *parent, int unit)
             G_CALLBACK(on_response), GINT_TO_POINTER(0));
     g_signal_connect(dialog, "update-preview",
             G_CALLBACK(on_update_preview), NULL);
-#if 0
-    /* Double click: autostart */
-    g_signal_connect(dialog, "file-activated",
-            G_CALLBACK(on_file_activated), NULL);
-#endif
+    g_signal_connect_unlocked(dialog, "selection-changed",
+            G_CALLBACK(on_selection_changed), NULL);
     return dialog;
 
 }
 
-#else
-
-static GtkFileChooserNative *create_disk_attach_dialog_native(GtkWidget *parent,
-                                                              int unit)
-{
-    GtkFileChooserNative *dialog;
-
-    /* create new dialog */
-    dialog = gtk_file_chooser_native_new(
-            "Attach a disk image (NATIVE)",
-            ui_get_active_window(),
-            GTK_FILE_CHOOSER_ACTION_OPEN,
-            NULL,
-            NULL);
-
-    g_signal_connect(dialog, "response",
-            G_CALLBACK(on_response),
-            GINT_TO_POINTER(0));    /* index of file to start? */
-
-    return dialog;
-}
-#endif
-
 
 /** \brief  Callback for the "smart-attach" and "attach to #%d" menu items
  *
@@ -578,16 +532,10 @@ static GtkFileChooserNative *create_disk_attach_dialog_native(GtkWidget *parent,
  */
 gboolean ui_disk_attach_dialog_show(GtkWidget *widget, gpointer user_data)
 {
-#ifndef SANDBOX_MODE
     GtkWidget *dialog;
 
     dialog = create_disk_attach_dialog(widget, GPOINTER_TO_INT(user_data));
     gtk_widget_show(dialog);
-#else
-    GtkFileChooserNative *dialog;
-    dialog = create_disk_attach_dialog_native(widget, GPOINTER_TO_INT(user_data));
-    gtk_native_dialog_show(GTK_NATIVE_DIALOG(dialog));
-#endif
     return TRUE;
 }
 
@@ -613,7 +561,6 @@ gboolean ui_disk_detach_callback(GtkWidget *widget, gpointer user_data)
     unit = GPOINTER_TO_INT(user_data) >> 8;
     drive = GPOINTER_TO_INT(user_data) & 0xff;
 
-    debug_gtk3("Detaching unit #%d, drive %d.", unit, drive);
     file_system_detach_disk(unit, drive);
     return TRUE;
 }
