@@ -42,15 +42,8 @@
 #endif
 
 #ifdef MACOSX_SUPPORT
-#import <objc/message.h>
 #import <CoreGraphics/CGEvent.h>
 
-/* The proper way to use objc_msgSend is to cast it into the right shape each time */
-#define OBJC_MSGSEND(return_type, ...) ((return_type (*)(__VA_ARGS__))objc_msgSend)
-#define OBJC_MSGSEND_STRET(...) ((void (*)(__VA_ARGS__))objc_msgSend_stret)
-
-/* For some reason this isn't in the GDK quartz headers */
-id gdk_quartz_window_get_nswindow (GdkWindow *window);
 #elif defined(WIN32_COMPILE)
 #include <windows.h>
 #endif
@@ -72,6 +65,10 @@ id gdk_quartz_window_get_nswindow (GdkWindow *window);
 #include "uimachinemenu.h"
 #include "uimachinewindow.h"
 
+#ifdef MACOSX_SUPPORT
+#include "macOS-util.h"
+#endif
+
 /* FIXME:   someone please add Doxygen docs for this, I can guess what it means
  *          but I'll probably get it wrong. --compyx
  */
@@ -122,29 +119,29 @@ static int host_delta_origin_y = 0.0;
 static void warp(int x, int y)
 {
 #ifdef MACOSX_SUPPORT
-    
+
     CGWarpMouseCursorPosition(CGPointMake(x, y));
     CGAssociateMouseAndMouseCursorPosition(true);
-    
+
 #elif defined(WIN32_COMPILE)
-    
+
     SetCursorPos(x, y);
-    
+
 #else /* xlib */
-    
+
     GtkWidget *gtk_widget = ui_get_window_by_index(PRIMARY_WINDOW);
     GdkWindow *gdk_window = gtk_widget_get_window(gtk_widget);
     Display *display = GDK_DISPLAY_XDISPLAY(gdk_window_get_display(gdk_window));
-    
+
     XWarpPointer(display, None, GDK_WINDOW_XID(gdk_window), 0, 0, 0, 0, x, y);
-    
+
 #endif
 }
 
 static void mouse_host_capture(int warp_x, int warp_y)
 {
     enable_capture = true;
-    
+
     warp(warp_x, warp_y);
 
     /* future mouse moments will be captured relative from here */
@@ -162,14 +159,14 @@ static void mouse_host_uncapture(void)
 static void mouse_host_moved(float x, float y)
 {
     float delta_x, delta_y;
-    
+
     if (!enable_capture) {
         return;
     }
-    
+
     delta_x = x - host_delta_origin_x;
     delta_y = y - host_delta_origin_y;
-    
+
     if (delta_x || delta_y) {
         mouse_move(delta_x, delta_y);
         warp(host_delta_origin_x, host_delta_origin_y);
@@ -235,11 +232,13 @@ static gboolean event_box_motion_cb(GtkWidget *widget,
     video_canvas_t *canvas = (video_canvas_t *)user_data;
 
     _mouse_still_frames = 0;
-    
+
     if (event->type != GDK_MOTION_NOTIFY) {
         return FALSE;
     }
 
+    pthread_mutex_lock(&canvas->lock);
+
     /* GDK_ENTER_NOTIFY isn't reliable on fullscreen transitions, so we reenable this here too */
     if (canvas->still_frame_callback_id == 0) {
         canvas->still_frame_callback_id =
@@ -251,26 +250,22 @@ static gboolean event_box_motion_cb(GtkWidget *widget,
 
     GdkEventMotion *motion = (GdkEventMotion *)event;
 
-    // printf("mouse move %f, %f  (%f, %f)\n", motion->x, motion->y, motion->x_root, motion->y_root); fflush(stdout);
+    /* printf("mouse move %f, %f  (%f, %f)\n", motion->x, motion->y, motion->x_root, motion->y_root); fflush(stdout); */
 
     if (enable_capture) {
 
         /* mouse movement, translate motion into window coordinates */
         int widget_x, widget_y;
         gtk_widget_translate_coordinates(widget, gtk_widget_get_toplevel(widget), 0, 0, &widget_x, &widget_y);
-        
+
 #ifdef MACOSX_SUPPORT
-        
-        void    *native_window  = gdk_quartz_window_get_nswindow(gtk_widget_get_window(widget));
-        id      content_view    = OBJC_MSGSEND(id, id, SEL)(native_window, sel_getUid("contentView"));
-        
         CGRect native_frame, content_rect;
-        OBJC_MSGSEND_STRET(CGRect *, id, SEL)(&native_frame, native_window, sel_getUid("frame"));
-        OBJC_MSGSEND_STRET(CGRect *, id, SEL)(&content_rect, content_view,  sel_getUid("frame"));
-        
+
+        vice_macos_get_widget_frame_and_content_rect(widget, &native_frame, &content_rect);
+
         /* macOS CoreGraphics coordinates origin is bottom-left of primary display */
         size_t main_display_height = CGDisplayPixelsHigh(CGMainDisplayID());
-        
+
         mouse_host_moved(
                                   native_frame.origin.x + widget_x + motion->x,
             main_display_height - native_frame.origin.y - content_rect.size.height + widget_y + motion->y
@@ -289,21 +284,22 @@ static gboolean event_box_motion_cb(GtkWidget *widget,
             (widget_y + motion->y) * scale);
 
 #endif
-        
+
+        pthread_mutex_unlock(&canvas->lock);
         return FALSE;
     }
-    
+
     /*
      * Mouse isn't captured, so we update the pen position.
      */
 
     double render_w = canvas->geometry->screen_size.width;
     double render_h = canvas->geometry->last_displayed_line - canvas->geometry->first_displayed_line + 1;
-    
+
     /* There might be some sweet off-by-0.5 bugs here */
     int pen_x = (motion->x - canvas->screen_origin_x) * render_w / canvas->screen_display_w;
     int pen_y = (motion->y - canvas->screen_origin_y) * render_h / canvas->screen_display_h;
-    
+
     if (pen_x < 0 || pen_y < 0 || pen_x >= render_w || pen_y >= render_h) {
         /* Mouse pointer is offscreen, so the light pen is disabled. */
         canvas->pen_x = -1;
@@ -314,6 +310,7 @@ static gboolean event_box_motion_cb(GtkWidget *widget,
         canvas->pen_y = pen_y;
     }
 
+    pthread_mutex_unlock(&canvas->lock);
     return FALSE;
 }
 
@@ -339,6 +336,8 @@ static gboolean event_box_mouse_button_cb(GtkWidget *widget, GdkEvent *event, gp
 
     if (event->type == GDK_BUTTON_PRESS) {
         int button = ((GdkEventButton *)event)->button;
+
+        pthread_mutex_lock(&canvas->lock);
         if (button == 1) {
             /* Left mouse button */
             canvas->pen_buttons |= LP_HOST_BUTTON_1;
@@ -346,6 +345,7 @@ static gboolean event_box_mouse_button_cb(GtkWidget *widget, GdkEvent *event, gp
             /* Right mouse button */
             canvas->pen_buttons |= LP_HOST_BUTTON_2;
         }
+        pthread_mutex_unlock(&canvas->lock);
 
         /* Don't send button events if the mouse isn't captured */
         if (_mouse_enabled) {
@@ -353,6 +353,8 @@ static gboolean event_box_mouse_button_cb(GtkWidget *widget, GdkEvent *event, gp
         }
     } else if (event->type == GDK_BUTTON_RELEASE) {
         int button = ((GdkEventButton *)event)->button;
+
+        pthread_mutex_lock(&canvas->lock);
         if (button == 1) {
             /* Left mouse button */
             canvas->pen_buttons &= ~LP_HOST_BUTTON_1;
@@ -360,12 +362,14 @@ static gboolean event_box_mouse_button_cb(GtkWidget *widget, GdkEvent *event, gp
             /* Right mouse button */
             canvas->pen_buttons &= ~LP_HOST_BUTTON_2;
         }
+        pthread_mutex_unlock(&canvas->lock);
 
         /* Don't send button events if the mouse isn't captured */
         if (_mouse_enabled) {
             mouse_button(button - 1, 0);
         }
     }
+    
     /* Ignore all other mouse button events, though we'll be sent
      * things like double- and triple-click. */
     return FALSE;
@@ -547,7 +551,7 @@ static gboolean event_box_cross_cb(GtkWidget *widget, GdkEvent *event, gpointer
          * clicking the canvas altered grab status. */
         return FALSE;
     }
-    
+
     if (crossing->type == GDK_ENTER_NOTIFY) {
         _mouse_still_frames = 0;
         if (canvas->still_frame_callback_id == 0) {
@@ -572,9 +576,11 @@ static gboolean event_box_cross_cb(GtkWidget *widget, GdkEvent *event, gpointer
                 gtk_widget_remove_tick_callback(canvas->event_box, canvas->still_frame_callback_id);
                 canvas->still_frame_callback_id = 0;
             }
+            pthread_mutex_lock(&canvas->lock);
             canvas->pen_x = -1;
             canvas->pen_y = -1;
             canvas->pen_buttons = 0;
+            pthread_mutex_unlock(&canvas->lock);
         }
     }
 
@@ -631,7 +637,7 @@ static void machine_window_create(video_canvas_t *canvas)
 
     g_signal_connect_unlocked(new_event_box, "enter-notify-event", G_CALLBACK(event_box_cross_cb), canvas);
     g_signal_connect_unlocked(new_event_box, "leave-notify-event", G_CALLBACK(event_box_cross_cb), canvas);
-    
+
     /* Important mouse event handling to bypass the lock and be immediately visible to the emulator */
     g_signal_connect_unlocked(new_event_box, "motion-notify-event", G_CALLBACK(event_box_motion_cb), canvas);
     g_signal_connect_unlocked(new_event_box, "button-press-event", G_CALLBACK(event_box_mouse_button_cb), canvas);
@@ -662,46 +668,42 @@ void ui_mouse_grab_pointer(void)
 {
     GtkWidget *window;
     float warp_x, warp_y;
-    
+
     if (!_mouse_enabled) {
         return;
     }
 
     window = ui_get_window_by_index(PRIMARY_WINDOW);
-    
+
     if (!window) {
         /* Probably mouse grab via config or command line, we'll grab it later via on_focus_in_event(). */
         return;
     }
-    
+
     /*
      * We warp the mouse to the center of the primary window and move it back there
      * each time we detect mouse movement.
      */
-        
+
 #ifdef MACOSX_SUPPORT
-        
-    void *native_window = gdk_quartz_window_get_nswindow(gtk_widget_get_window(window));
-    id   content_view   = OBJC_MSGSEND(id, id, SEL)(native_window, sel_getUid("contentView"));
-    
+
     CGRect native_frame, content_rect;
-    OBJC_MSGSEND_STRET(CGRect *, id, SEL)(&native_frame, native_window, sel_getUid("frame"));
-    OBJC_MSGSEND_STRET(CGRect *, id, SEL)(&content_rect, content_view,  sel_getUid("frame"));
+    vice_macos_get_widget_frame_and_content_rect(window, &native_frame, &content_rect);
 
     /* macOS CoreGraphics coordinates origin is bottom-left of primary display */
     size_t main_display_height = CGDisplayPixelsHigh(CGMainDisplayID());
 
     warp_x =                       native_frame.origin.x + (int)(content_rect.size.width  / 2.0f);
     warp_y = main_display_height - native_frame.origin.y - (int)(content_rect.size.height / 2.0f);
-        
+
 #else
-        
+
     /* First calculate destination relateive to window */
     int window_width, window_height;
     gtk_window_get_size(GTK_WINDOW(window), &window_width, &window_height);
 
     int scale = gtk_widget_get_scale_factor(window);
-    
+
     warp_x = (float)window_width  / 2.0f * scale;
     warp_y = (float)window_height / 2.0f * scale;
 
@@ -713,7 +715,7 @@ void ui_mouse_grab_pointer(void)
 
     warp_x += window_x * scale;
     warp_y += window_y * scale;
-            
+
 #endif
 #endif
 
