@@ -33,20 +33,21 @@
 
 #include <gtk/gtk.h>
 
+#include "vice_gtk3.h"
 #include "attach.h"
 #include "autostart.h"
 #include "cartridge.h"
 #include "drive.h"
 #include "tape.h"
 #include "debug_gtk3.h"
-#include "basedialogs.h"
 #include "contentpreviewwidget.h"
 #include "diskcontents.h"
+#include "diskimage.h"
+#include "driveimage.h"
 #include "tapecontents.h"
 #include "machine.h"
 #include "mainlock.h"
 #include "resources.h"
-#include "filechooserhelpers.h"
 #include "ui.h"
 #include "uiapi.h"
 #include "uimachinewindow.h"
@@ -56,35 +57,33 @@
 
 #include "uismartattach.h"
 
+/* #define HAVE_DEBUG_GTK3UI */
 
-#ifndef SANDBOX_MODE
 /** \brief  File type filters for the dialog
  */
 static ui_file_filter_t filters[] = {
-    { "All files", file_chooser_pattern_all },
-    { "Disk images", file_chooser_pattern_disk },
-    { "Tape images", file_chooser_pattern_tape },
-    { "Cartridge images", file_chooser_pattern_cart },
-    { "Program files", file_chooser_pattern_program },
-    { "Snapshot files", file_chooser_pattern_snapshot },
-    { "Archives files", file_chooser_pattern_archive },
-    { "Compressed files", file_chooser_pattern_compressed },
+    { "All files",          file_chooser_pattern_all },
+    { "Disk images",        file_chooser_pattern_disk },
+    { "Tape images",        file_chooser_pattern_tape },
+    { "Cartridge images",   file_chooser_pattern_cart },
+    { "Program files",      file_chooser_pattern_program },
+    { "Snapshot files",     file_chooser_pattern_snapshot },
+    { "Archives files",     file_chooser_pattern_archive },
+    { "Compressed files",   file_chooser_pattern_compressed },
     { NULL, NULL }
 };
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Preview widget reference
  */
 static GtkWidget *preview_widget = NULL;
-#endif
 
 
 /** \brief  Last directory used
  *
- * When an image is attached, this is set to the directory of that file. Since
- * it's heap-allocated by Gtk3, it must be freed with a call to
+ * When an image is attached, this is set to the directory of that file.
+ *
+ * Since it's heap-allocated by Gtk3, it must be freed with a call to
  * ui_smart_attach_shutdown() on emulator shutdown.
  */
 static gchar *last_dir = NULL;
@@ -101,16 +100,15 @@ static GtkWidget *autostart_button;
 
 /** \brief  Trigger autostart
  *
- * \param[in]   widget  dialog
- * \param[in]   data    file index in the directory preview
+ * \param[in]   widget      dialog
+ * \param[in]   index       file index in the directory preview
+ * \param[in]   autostart   flag: 0: just load, 1: autostart
  */
-static void do_autostart(GtkWidget *widget, gpointer data)
+static void do_autostart(GtkWidget *widget, int index, int autostart)
 {
     gchar *filename;
     gchar *filename_locale;
 
-    int index = GPOINTER_TO_INT(data);
-
     lastdir_update(widget, &last_dir, &last_file);
     filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
     filename_locale = file_chooser_convert_to_locale(filename);
@@ -123,7 +121,7 @@ static void do_autostart(GtkWidget *widget, gpointer data)
                 index,  /* Program number? Probably used when clicking
                            in the preview widget to load the proper
                            file in an image */
-                AUTOSTART_MODE_RUN) < 0) {
+                autostart ? AUTOSTART_MODE_RUN : AUTOSTART_MODE_LOAD) < 0) {
         /* oeps
          *
          * I currently can't find a way to use a non-blocking Gtk Error dialog
@@ -134,7 +132,105 @@ static void do_autostart(GtkWidget *widget, gpointer data)
     }
     g_free(filename);
     g_free(filename_locale);
-    gtk_widget_destroy(widget);
+}
+
+/** \brief  try attach a disk image, change drive type if needed
+ *
+ * \param[in]   unit_number
+ * \param[in]   drive_number
+ * \param[in]   filename_locale
+ */
+static int try_attach_disk(int unit_number, int drive_number, char *filename_locale)
+{
+
+    if (file_system_attach_disk(unit_number, drive_number, filename_locale) < 0) {
+        /* failed */
+        return -1;
+    } else {
+        /* shitty code, we really need to extend the drive API to
+        * get at these sorts for things without breaking into core code
+        */
+        struct disk_image_s *diskimg = file_system_get_image(unit_number, drive_number);
+
+        if (diskimg == NULL) {
+            log_error(LOG_ERR, "Failed to get disk image for unit %d.", unit_number);
+            return -1;
+        } else {
+            int chk = drive_check_image_format(diskimg->type, 0);
+            log_message(LOG_DEFAULT, "mounted image is type: %u, %schanging drive.",
+                        diskimg->type, (chk < 0) ? "" : "not ");
+            /* change drive type only when image does not work in current drive */
+            if (chk < 0) {
+                if (resources_set_int_sprintf("Drive%dType", drive_image_type_to_drive_type(diskimg->type), unit_number) < 0) {
+                    log_error(LOG_ERR, "Failed to set drive type.");
+                }
+            }
+
+            /* detach disk before reattaching */
+            file_system_detach_disk(unit_number, drive_number);
+
+            if (file_system_attach_disk(unit_number, drive_number, filename_locale) < 0) {
+                return -1;
+            }
+        }
+    }
+    return 0;
+}
+
+/** \brief  Do smart attach
+ *
+ * \param[in]   widget  dialog
+ * \param[in]   data    file index in the directory preview
+ */
+static void do_smart_attach(GtkWidget *widget, gpointer data)
+{
+    gchar *filename_locale;
+    gchar *filename;
+
+    lastdir_update(widget, &last_dir, &last_file);
+    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
+    filename_locale = file_chooser_convert_to_locale(filename);
+
+    /* Smart attach for C64/C128/Plus4
+        *
+        * This tries to attach a file as a cartridge image, which is only
+        * valid for C64/C128/Plus4
+        */
+    if ((machine_class == VICE_MACHINE_C64)
+            || (machine_class == VICE_MACHINE_C64SC)
+            || (machine_class == VICE_MACHINE_SCPU64)
+            || (machine_class == VICE_MACHINE_C128)
+            || (machine_class == VICE_MACHINE_PLUS4)) {
+        if (try_attach_disk(DRIVE_UNIT_DEFAULT, 0, filename_locale) < 0
+                && tape_image_attach(1, filename_locale) < 0
+                && autostart_snapshot(filename_locale, NULL) < 0
+                && cartridge_attach_image(CARTRIDGE_CRT, filename_locale) < 0
+                && autostart_prg(filename_locale, AUTOSTART_MODE_LOAD) < 0) {
+            /* failed */
+            log_error(LOG_ERR, "smart attach failed for '%s' failed", filename);
+        }
+    } else if (machine_class == VICE_MACHINE_VIC20) {
+        if (try_attach_disk(DRIVE_UNIT_DEFAULT, 0, filename_locale) < 0
+                && tape_image_attach(1, filename_locale) < 0
+                && autostart_snapshot(filename_locale, NULL) < 0
+                /* && autostart_prg(filename_locale, AUTOSTART_MODE_LOAD) < 0 */
+                && cartridge_attach_image(CARTRIDGE_CRT, filename_locale) < 0) {
+            /* failed */
+            log_error(LOG_ERR, "smart attach failed for '%s' failed", filename);
+        }
+    } else {
+        /* Smart attach for other emulators: don't try to attach a file
+            * as a cartidge, it'll result in false positives
+            */
+        if (try_attach_disk(DRIVE_UNIT_DEFAULT, 0, filename_locale) < 0
+                && tape_image_attach(1, filename_locale) < 0
+                && autostart_snapshot(filename_locale, NULL) < 0)
+        {
+            log_error(LOG_ERR, "Failed to smart attach '%s'",
+                    filename_locale);
+        }
+    }
+    g_free(filename_locale);
 }
 
 
@@ -146,7 +242,7 @@ static void do_autostart(GtkWidget *widget, gpointer data)
  * \param[in]   chooser Parent dialog
  * \param[in]   data    Extra event data (unused)
  */
-void on_selection_changed(GtkFileChooser *chooser, gpointer data)
+static void on_selection_changed(GtkFileChooser *chooser, gpointer data)
 {
     gchar *filename;
 
@@ -160,7 +256,6 @@ void on_selection_changed(GtkFileChooser *chooser, gpointer data)
 }
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Handler for the "update-preview" event
  *
  * \param[in]   chooser file chooser dialog
@@ -184,10 +279,8 @@ static void on_update_preview(GtkFileChooser *chooser, gpointer data)
         g_object_unref(file);
     }
 }
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Handler for the 'toggled' event of the 'show hidden files' checkbox
  *
  * \param[in]   widget      checkbox triggering the event
@@ -200,10 +293,8 @@ static void on_hidden_toggled(GtkWidget *widget, gpointer user_data)
     state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
     gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER(user_data), state);
 }
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Handler for the 'toggled' event of the 'attach read-only' checkbox
  *
  * \param[in]   widget      checkbox triggering the event
@@ -216,7 +307,6 @@ static void on_readonly_toggled(GtkWidget *widget, gpointer user_data)
     state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
     resources_set_int_sprintf("AttachDevice%dReadonly", state, DRIVE_UNIT_DEFAULT);
 }
-#endif
 
 
 /** \brief  Handler for 'response' event of the dialog
@@ -225,7 +315,7 @@ static void on_readonly_toggled(GtkWidget *widget, gpointer user_data)
  *
  * \param[in]   widget      the dialog
  * \param[in]   response_id response ID
- * \param[in]   user_data   index in the preview widget
+ * \param[in]   user_data   unit number
  *
  * TODO:    proper (error) messages, which requires implementing ui_error() and
  *          ui_message() and moving them into gtk3/widgets to avoid circular
@@ -233,67 +323,80 @@ static void on_readonly_toggled(GtkWidget *widget, gpointer user_data)
  */
 static void on_response(GtkWidget *widget, gint response_id, gpointer user_data)
 {
-    gchar *filename;
-    gchar *filename_locale;
+    gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
+    int index = content_preview_widget_get_index(preview_widget);
+    int autostart = 0;
 
-#ifdef HAVE_DEBUG_GTK3UI
-    int index = GPOINTER_TO_INT(user_data);
-    debug_gtk3("got response ID %d, index %d.", response_id, index);
-#endif
+    resources_get_int("AutostartOnDoubleclick", &autostart);
 
-    /* gonna needs this in multiple checks */
-    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
+    /* first, to make the following logic less funky, map some events to others,
+       depending on whether autostart-on-doubleclick is enabled or not, and
+       depending on the event coming from the preview window or not. */
+    switch (response_id) {
+        /* double-click on file in the preview widget when autostart-on-doubleclick is NOT enabled */
+        case VICE_RESPONSE_AUTOLOAD_INDEX:
+        /* double-click on file in the preview widget when autostart-on-doubleclick is enabled */
+        case VICE_RESPONSE_AUTOSTART_INDEX:
+            if ((index < 0) || (filename == NULL)) {
+                response_id = VICE_RESPONSE_INVALID;   /* drop this event */
+            }
+            break;
+        /* 'Open' button clicked when autostart-on-doubleclick is enabled */
+        case VICE_RESPONSE_CUSTOM_OPEN:
+            if (filename == NULL) {
+                response_id = VICE_RESPONSE_INVALID;   /* drop this event */
+            } else if (index >= 0) {
+                response_id = VICE_RESPONSE_AUTOLOAD_INDEX;
+            }
+            break;
+        /* double-click on file in the main file chooser,
+           'Autostart' button when autostart-on-doubleclick is enabled
+           'Open' button when autostart-on-doubleclick is NOT enabled */
+        case GTK_RESPONSE_ACCEPT:
+            if (filename == NULL) {
+                response_id = VICE_RESPONSE_INVALID;   /* drop this event */
+            } else if ((index >= 0) && (autostart == 0)) {
+                response_id = VICE_RESPONSE_AUTOLOAD_INDEX;
+            } else if ((index >= 0) && (autostart == 1)) {
+                response_id = VICE_RESPONSE_AUTOSTART_INDEX;
+            } else if (autostart == 0) {
+                response_id = VICE_RESPONSE_CUSTOM_OPEN;
+            } else {
+                response_id = VICE_RESPONSE_AUTOSTART;
+            }
+            break;
+        default:
+            break;
+    }
 
     switch (response_id) {
+        /* 'Open' button clicked when autostart-on-doubleclick is enabled */
+        case VICE_RESPONSE_CUSTOM_OPEN:
+            do_smart_attach(widget, user_data);
 
-        /* 'Open' button, double-click on file */
-        case GTK_RESPONSE_ACCEPT:
-            lastdir_update(widget, &last_dir, &last_file);
-            filename_locale = file_chooser_convert_to_locale(filename);
-
-            /* Smart attach for C64/C128
-             *
-             * This tries to attach a file as a cartridge image, which is only
-             * valid for C64/C128
-             */
-            if ((machine_class == VICE_MACHINE_C64)
-                    || (machine_class == VICE_MACHINE_C64SC)
-                    || (machine_class == VICE_MACHINE_SCPU64)
-                    || (machine_class == VICE_MACHINE_C128)) {
-                if (file_system_attach_disk(DRIVE_UNIT_DEFAULT, 0, filename_locale) < 0
-                        && tape_image_attach(1, filename_locale) < 0
-                        && autostart_snapshot(filename_locale, NULL) < 0
-                        && cartridge_attach_image(CARTRIDGE_CRT, filename_locale) < 0
-                        && autostart_prg(filename_locale, AUTOSTART_MODE_LOAD) < 0) {
-                    /* failed (TODO: perhaps a proper error message?) */
-                    debug_gtk3("smart attach failed.");
-                }
-            } else {
-                /* Smart attach for other emulators: don't try to attach a file
-                 * as a cartidge, it'll result in false positives
-                 */
-                if (file_system_attach_disk(DRIVE_UNIT_DEFAULT, 0, filename_locale) < 0
-                        && tape_image_attach(1, filename_locale) < 0
-                        && autostart_snapshot(filename_locale, NULL) < 0)
-                {
-                    /* failed (TODO: perhaps a proper error message?) */
-                    log_error(LOG_ERR, "Failed to smart attach '%s'",
-                            filename_locale);
-                }
-            }
-            g_free(filename_locale);
+            mainlock_release();
             gtk_widget_destroy(widget);
+            mainlock_obtain();
             break;
 
-        /* 'Autostart' button clicked */
+        /* 'Autostart' button clicked when autostart-on-doubleclick is NOT enabled */
         case VICE_RESPONSE_AUTOSTART:
-            /* do we actually have a file to autostart? */
-            if (filename != NULL) {
-                do_autostart(widget, user_data);
-                mainlock_release();
-                gtk_widget_destroy(widget);
-                mainlock_obtain();
-            }
+        /* double-click on file in the preview widget when autostart-on-doubleclick is enabled */
+        case VICE_RESPONSE_AUTOSTART_INDEX:
+            do_autostart(widget, index + 1, 1);
+
+            mainlock_release();
+            gtk_widget_destroy(widget);
+            mainlock_obtain();
+            break;
+
+        /* double-click on file in the preview widget when autostart-on-doubleclick is NOT enabled */
+        case VICE_RESPONSE_AUTOLOAD_INDEX:
+            do_autostart(widget, index + 1, 0);
+
+            mainlock_release();
+            gtk_widget_destroy(widget);
+            mainlock_obtain();
             break;
 
         /* 'Close'/'X' button */
@@ -312,7 +415,6 @@ static void on_response(GtkWidget *widget, gint response_id, gpointer user_data)
 }
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Create the 'extra' widget
  *
  * \param[in]   parent  parent widget (unused)
@@ -326,8 +428,7 @@ static GtkWidget *create_extra_widget(GtkWidget *parent)
     GtkWidget *readonly_check;
     int readonly_state;
 
-    grid = gtk_grid_new();
-    gtk_grid_set_column_spacing(GTK_GRID(grid), 8);
+    grid = vice_gtk3_grid_new_spaced(0, 8);
 
     hidden_check = gtk_check_button_new_with_label("Show hidden files");
     g_signal_connect(hidden_check, "toggled", G_CALLBACK(on_hidden_toggled),
@@ -344,10 +445,8 @@ static GtkWidget *create_extra_widget(GtkWidget *parent)
     gtk_widget_show_all(grid);
     return grid;
 }
-#endif
 
 
-#ifndef SANDBOX_MODE
 /** \brief  Wrapper around disk/tape contents readers
  *
  * First treats \a path as disk image file and when that fails it falls back
@@ -369,11 +468,8 @@ static image_contents_t *read_contents_wrapper(const char *path)
     }
     return content;
 }
-#endif
 
 
-
-#ifndef SANDBOX_MODE
 /** \brief  Create the smart-attach dialog
  *
  * \param[in]   parent  parent widget, used to get the top level window
@@ -388,6 +484,9 @@ static GtkWidget *create_smart_attach_dialog(GtkWidget *parent)
 {
     GtkWidget *dialog;
     size_t i;
+    int autostart = 0;
+
+    resources_get_int("AutostartOnDoubleclick", &autostart);
 
     /* create new dialog */
     dialog = gtk_file_chooser_dialog_new(
@@ -403,10 +502,21 @@ static GtkWidget *create_smart_attach_dialog(GtkWidget *parent)
      * added via the constructor, meaning we cannot get a reference to the
      * "Autostart" button in order to "grey it out".
      */
-    gtk_dialog_add_button(GTK_DIALOG(dialog), "Open", GTK_RESPONSE_ACCEPT);
-    autostart_button = gtk_dialog_add_button(GTK_DIALOG(dialog),
-                                             "Autostart",
-                                             VICE_RESPONSE_AUTOSTART);
+    /* to handle the "double click means autostart" option, we need to always
+       connect a button to GTK_RESPONSE_ACCEPT, else double clicks will stop
+       working */
+    if (autostart) {
+        gtk_dialog_add_button(GTK_DIALOG(dialog), "Attach / Load", VICE_RESPONSE_CUSTOM_OPEN);
+        autostart_button = gtk_dialog_add_button(GTK_DIALOG(dialog),
+                                                "Autostart",
+                                                GTK_RESPONSE_ACCEPT);
+    } else {
+        gtk_dialog_add_button(GTK_DIALOG(dialog), "Attach / Load", GTK_RESPONSE_ACCEPT);
+        autostart_button = gtk_dialog_add_button(GTK_DIALOG(dialog),
+                                                "Autostart",
+                                                VICE_RESPONSE_AUTOSTART);
+    }
+
     gtk_widget_set_sensitive(autostart_button, FALSE);
     gtk_dialog_add_button(GTK_DIALOG(dialog), "Close", GTK_RESPONSE_REJECT);
 
@@ -421,7 +531,7 @@ static GtkWidget *create_smart_attach_dialog(GtkWidget *parent)
             create_extra_widget(dialog));
 
     preview_widget = content_preview_widget_create(dialog,
-            read_contents_wrapper, on_response);
+            read_contents_wrapper, on_response, 0);
     gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(dialog),
             preview_widget);
 
@@ -443,38 +553,6 @@ static GtkWidget *create_smart_attach_dialog(GtkWidget *parent)
 
 }
 
-#else
-
-/** \brief  Create the smart-attach dialog
- *
- * \param[in]   parent  parent widget, used to get the top level window
- *
- * \return  GtkFileChooserNative
- *
- * \todo    Figure out how to only enable the 'Autostart' button when an actual
- *          file/image has been selected. And when I do, make sure it's somehow
- *          reusable for other 'open file' dialogs'.
- */
-static GtkFileChooserNative *create_smart_attach_dialog(void *parent)
-{
-    GtkFileChooserNative *dialog;
-
-    /* create new dialog */
-    dialog = gtk_file_chooser_native_new(
-            "Smart-attach a file",
-            ui_get_active_window(),
-            GTK_FILE_CHOOSER_ACTION_OPEN,
-            /* buttons */
-            NULL, NULL);
-
-    /* connect "reponse" handler: the `user_data` argument gets filled in when
-     * the "response" signal is emitted: a response ID */
-    g_signal_connect(dialog, "response", G_CALLBACK(on_response), NULL);
-
-    return dialog;
-}
-#endif
-
 
 /** \brief  Callback for the File menu's "smart-attach" item
  *
@@ -487,17 +565,10 @@ static GtkFileChooserNative *create_smart_attach_dialog(void *parent)
  */
 gboolean ui_smart_attach_dialog_show(GtkWidget *widget, gpointer user_data)
 {
-#ifndef SANDBOX_MODE
-
     GtkWidget *dialog;
 
     dialog = create_smart_attach_dialog(widget);
     gtk_widget_show(dialog);
-#else
-    GtkFileChooserNative *dialog;
-    dialog = create_smart_attach_dialog((gpointer)widget);
-    gtk_native_dialog_show(GTK_NATIVE_DIALOG(dialog));
-#endif
     return TRUE;
 
 }
