@@ -32,18 +32,20 @@
 #include <gtk/gtk.h>
 
 
-gboolean uisnapshot_open_file(GtkWidget *parent, gpointer user_data);
-gboolean uisnapshot_save_file(GtkWidget *parent, gpointer user_data);
+gboolean ui_snapshot_open_file(GtkWidget *parent, gpointer user_data);
+gboolean ui_snapshot_save_file(GtkWidget *parent, gpointer user_data);
 
-gboolean uisnapshot_quickload_snapshot(GtkWidget *parent, gpointer user_data);
-gboolean uisnapshot_quicksave_snapshot(GtkWidget *parent, gpointer user_data);
+gboolean ui_snapshot_quickload_snapshot(GtkWidget *parent, gpointer user_data);
+gboolean ui_snapshot_quicksave_snapshot(GtkWidget *parent, gpointer user_data);
 
-gboolean uisnapshot_history_select_dir(GtkWidget *parent, gpointer user_data);
-gboolean uisnapshot_history_record_start(GtkWidget *parent, gpointer user_data);
-gboolean uisnapshot_history_record_stop(GtkWidget *parent, gpointer user_data);
-gboolean uisnapshot_history_playback_start(GtkWidget *parent, gpointer user_data);
-gboolean uisnapshot_history_playback_stop(GtkWidget *parent, gpointer user_data);
-gboolean uisnapshot_history_milestone_set(GtkWidget *parent, gpointer user_data);
-gboolean uisnapshot_history_milestone_reset(GtkWidget *parent, gpointer user_data);
+#if 0
+gboolean ui_snapshot_history_select_dir(GtkWidget *parent, gpointer user_data);
+#endif
+gboolean ui_snapshot_history_record_start(GtkWidget *parent, gpointer user_data);
+gboolean ui_snapshot_history_record_stop(GtkWidget *parent, gpointer user_data);
+gboolean ui_snapshot_history_playback_start(GtkWidget *parent, gpointer user_data);
+gboolean ui_snapshot_history_playback_stop(GtkWidget *parent, gpointer user_data);
+gboolean ui_snapshot_history_milestone_set(GtkWidget *parent, gpointer user_data);
+gboolean ui_snapshot_history_milestone_reset(GtkWidget *parent, gpointer user_data);
 
 #endif
