@@ -31,8 +31,8 @@
 #include "vice.h"
 #include <gtk/gtk.h>
 
-void ui_tape_attach_callback(GtkWidget *widget, gpointer user_data);
-void ui_tape_detach_callback(GtkWidget *widget, gpointer user_data);
+gboolean ui_tape_attach_callback(GtkWidget *widget, gpointer user_data);
+gboolean ui_tape_detach_callback(GtkWidget *widget, gpointer user_data);
 void ui_tape_attach_shutdown(void);
 
 #endif
