@@ -38,12 +38,14 @@
 #include <stdlib.h>
 #include <gtk/gtk.h>
 
+#include "debug_gtk3.h"
 #include "debug.h"
 #include "lib.h"
 #include "machine.h"
 #include "psid.h"
 #include "ui.h"
 #include "uiabout.h"
+#include "uiactions.h"
 #include "uicmdline.h"
 #include "uicommands.h"
 #include "uicompiletimefeatures.h"
@@ -109,11 +111,13 @@ static GSList *tune_submenu_group = NULL;
  */
 static ui_menu_item_t reset_submenu[] = {
     { "Soft reset", UI_MENU_TYPE_ITEM_ACTION,
-        "reset-soft", ui_machine_reset_callback, GINT_TO_POINTER(MACHINE_RESET_MODE_SOFT),
-        GDK_KEY_F9, VICE_MOD_MASK, true },
+      ACTION_RESET_SOFT,
+      ui_machine_reset_callback, NULL, GINT_TO_POINTER(MACHINE_RESET_MODE_SOFT),
+      GDK_KEY_F9, VICE_MOD_MASK, true },
     { "Hard reset", UI_MENU_TYPE_ITEM_ACTION,
-        "reset-hard", ui_machine_reset_callback, GINT_TO_POINTER(MACHINE_RESET_MODE_HARD),
-        GDK_KEY_F12, VICE_MOD_MASK, true },
+      ACTION_RESET_HARD,
+      ui_machine_reset_callback, NULL, GINT_TO_POINTER(MACHINE_RESET_MODE_HARD),
+      GDK_KEY_F12, VICE_MOD_MASK, true },
 
     UI_MENU_TERMINATOR
 };
@@ -123,8 +127,9 @@ static ui_menu_item_t reset_submenu[] = {
  */
 static ui_menu_item_t file_menu[] = {
     { "Load PSID file ...", UI_MENU_TYPE_ITEM_ACTION,
-        "load-psid", uisidattach_show_dialog, NULL,
-        GDK_KEY_L, VICE_MOD_MASK, true },
+      "load-psid",
+      uisidattach_show_dialog, NULL, NULL,
+      GDK_KEY_L, VICE_MOD_MASK, true },
 
     UI_MENU_SEPARATOR,
 
@@ -132,38 +137,42 @@ static ui_menu_item_t file_menu[] = {
      *      contains sound recording options
      */
     { "Record sound file ...", UI_MENU_TYPE_ITEM_ACTION,
-        "sound-save", ui_media_dialog_show, NULL,
-        GDK_KEY_R, VICE_MOD_MASK | GDK_SHIFT_MASK, false },
+      "sound-save",
+      ui_media_dialog_show, NULL, NULL,
+      GDK_KEY_R, VICE_MOD_MASK|GDK_SHIFT_MASK, false },
 
     { "Stop sound recording", UI_MENU_TYPE_ITEM_ACTION,
-        "sound-stop", ui_media_stop_recording, NULL,
-        GDK_KEY_S, VICE_MOD_MASK | GDK_SHIFT_MASK, false },
+      "sound-stop",
+      ui_media_stop_recording, NULL, NULL,
+      GDK_KEY_S, VICE_MOD_MASK|GDK_SHIFT_MASK, false },
 
     UI_MENU_SEPARATOR,
 
     /* monitor */
     { "Activate monitor", UI_MENU_TYPE_ITEM_ACTION,
-        "monitor", ui_monitor_activate_callback, NULL,
+      "monitor",
+      ui_monitor_activate_callback, NULL, NULL,
 #ifdef MACOSX_SUPPORT
-        /* use Command-Option-M on Mac */
-        GDK_KEY_M, VICE_MOD_MASK | GDK_MOD1_MASK,
+      /* use Command-Option-M on Mac */
+      GDK_KEY_M, VICE_MOD_MASK|GDK_MOD1_MASK,
 #else
-        GDK_KEY_H, VICE_MOD_MASK,
+      GDK_KEY_H, VICE_MOD_MASK,
 #endif
-        false
-    },
+      false },
 
     UI_MENU_SEPARATOR,
 
     { "Reset", UI_MENU_TYPE_SUBMENU,
-        NULL, NULL, reset_submenu,
-        0, 0, false },
+      "reset-submenu",
+      NULL, NULL, reset_submenu,
+      0, 0, false },
 
     UI_MENU_SEPARATOR,
 
     { "Exit player", UI_MENU_TYPE_ITEM_ACTION,
-        "exit", ui_close_callback, NULL,
-        GDK_KEY_Q, VICE_MOD_MASK, true },
+      ACTION_QUIT,
+      ui_close_callback, NULL, NULL,
+      GDK_KEY_Q, VICE_MOD_MASK, true },
 
     UI_MENU_TERMINATOR
 };
@@ -186,8 +195,9 @@ static ui_menu_item_t settings_menu[] = {
      *      added to the settings dialog
      */
     { "Override PSID settings", UI_MENU_TYPE_ITEM_CHECK,
-        "psid-keep-env", ui_toggle_resource, (void *)"PSIDKeepEnv",
-        0, 0, false },
+      "psid-keep-env",
+      ui_toggle_resource, (void*)"PSIDKeepEnv", NULL,
+      0, 0, false },
 
     UI_MENU_TERMINATOR
 };
@@ -197,25 +207,28 @@ static ui_menu_item_t settings_menu[] = {
  */
 #ifdef DEBUG
 static ui_menu_item_t debug_menu[] = {
-    { "Trace mode ...", UI_MENU_TYPE_ITEM_ACTION,
-        "tracemode", ui_debug_trace_mode_dialog_show, NULL,
-        0, 0, false },
+    { "Trace mode...", UI_MENU_TYPE_ITEM_ACTION,
+      "tracemode",
+      ui_debug_trace_mode_dialog_show, NULL, NULL,
+      0, 0, false },
 
     UI_MENU_SEPARATOR,
 
     { "Main CPU trace", UI_MENU_TYPE_ITEM_CHECK,
-        "trace-maincpu", (void *)(ui_toggle_resource), (void *)"MainCPU_TRACE",
-        0, 0, false },
+      "trace-maincpu",
+      ui_toggle_resource, (void*)"MainCPU_TRACE", NULL,
+      0, 0, false },
 
     UI_MENU_SEPARATOR,
 
-
     { "Autoplay playback frames ...", UI_MENU_TYPE_ITEM_ACTION,
-        "playframes", ui_debug_playback_frames_dialog_show, NULL,
-        0, 0, false },
+      ACTION_DEBUG_AUTOPLAYBACK_FRAMES,
+      ui_debug_playback_frames_dialog_show, NULL, NULL,
+      0, 0, false },
     { "Save core dump", UI_MENU_TYPE_ITEM_CHECK,
-        "coredump", (void *)(ui_toggle_resource), (void *)"DoCoreDump",
-        0, 0, false },
+      "coredump",
+      ui_toggle_resource, (void*)"DoCoreDump", NULL,
+      0, 0, false },
 
     UI_MENU_TERMINATOR
 };
@@ -226,21 +239,26 @@ static ui_menu_item_t debug_menu[] = {
  */
 static ui_menu_item_t help_menu[] = {
     { "Browse manual", UI_MENU_TYPE_ITEM_ACTION,
-        "manual", ui_open_manual_callback, NULL,
-        0, 0, true },
+      "manual",
+      ui_open_manual_callback, NULL, NULL,
+      0, 0, true },
     { "Command line options ...", UI_MENU_TYPE_ITEM_ACTION,
-        "cmdline", uicmdline_dialog_show, NULL,
-        0, 0, true },
+      "cmdline",
+      uicmdline_dialog_show, NULL, NULL,
+      0, 0, true },
     { "Compile time features ...", UI_MENU_TYPE_ITEM_ACTION,
-        "features", uicompiletimefeatures_dialog_show, NULL,
-        0, 0, true },
+      "features",
+      uicompiletimefeatures_dialog_show, NULL, NULL,
+      0, 0, true },
     { "About VICE", UI_MENU_TYPE_ITEM_ACTION,
-        "about", ui_about_dialog_callback, NULL,
-        0, 0, true },
+      "about",
+      ui_about_dialog_callback, NULL, NULL,
+      0, 0, true },
 
     UI_MENU_TERMINATOR
 };
 
+
 /** \brief  Play a tune when selected with the Tune menu
  *
  * \param[in,out]   menuitem    menu item
@@ -279,6 +297,12 @@ void ui_vsid_tune_menu_set_tune_count(int count) {
     long i;
     char *buf;
 
+
+    if (tune_submenu == NULL || !GTK_IS_CONTAINER(tune_submenu)) {
+        debug_gtk3("tune_submenu invalid.");
+        return;
+    }
+
     gtk_container_foreach(GTK_CONTAINER(tune_submenu), remove_item_from_menu, NULL);
     for (i = count; i >0; i--) {
         buf = lib_msprintf("Tune %s%d", i < 10 ? "_" :"", i);
