@@ -32,7 +32,10 @@
 #include <stdlib.h>
 #include <string.h>
 
+/* for GULONG_TO_POINTER() and GPOINTER_TO_ULONG() */
+#include "archdep_defs.h"
 #include "basewidget_types.h"
+#include "debug_gtk3.h"
 #include "lib.h"
 #include "log.h"
 #include "resourcehelpers.h"
@@ -132,7 +135,7 @@ static gboolean set_combo_int_id(GtkComboBox *combo, int id)
  *
  * Frees the heap-allocated copy of the resource name
  *
- * \param[im]   combo       combo box
+ * \param[in]   combo       combo box
  * \param[in]   user_data   extra event data (unused)
  */
 static void on_combo_int_destroy(GtkWidget *combo, gpointer user_data)
@@ -145,7 +148,7 @@ static void on_combo_int_destroy(GtkWidget *combo, gpointer user_data)
  *
  * Updates the resource connected to the combo box
  *
- * \param[im]   combo       combo box
+ * \param[in]   combo       combo box
  * \param[in]   user_data   extra event data (unused)
  */
 static void on_combo_int_changed(GtkComboBox *combo, gpointer user_data)
@@ -156,8 +159,23 @@ static void on_combo_int_changed(GtkComboBox *combo, gpointer user_data)
     resource = resource_widget_get_resource_name(GTK_WIDGET(combo));
     if (get_combo_int_id(combo, &id)) {
         if (resources_set_int(resource, id) < 0) {
+            gulong handler_id;
+            int prev;
+
             log_error(LOG_ERR, "failed to set resource '%s' to %d\n",
                     resource, id);
+
+            /* set combo back to its previous (valid) id */
+            prev = resource_widget_get_int(GTK_WIDGET(combo), "PreviousID");
+            handler_id = GPOINTER_TO_ULONG(g_object_get_data(G_OBJECT(combo),
+                                                             "ChangedHandlerID"));
+            g_signal_handler_block(G_OBJECT(combo), handler_id);
+            set_combo_int_id(combo, prev);
+            g_signal_handler_unblock(G_OBJECT(combo), handler_id);
+
+        } else {
+            /* valid, so update the previous ID */
+            resource_widget_set_int(GTK_WIDGET(combo), "PreviousID", id);
         }
     } else {
         log_error(LOG_ERR, "failed to get ID for resource '%s'\n", resource);
@@ -167,8 +185,8 @@ static void on_combo_int_changed(GtkComboBox *combo, gpointer user_data)
 
 /** \brief  Create a combo box to control an integer resource
  *
- * \param[in]   combo   combo box
- * \param[in]   entries list of entries for the combo box
+ * \param[in,out]   combo   combo box
+ * \param[in]       entries list of entries for the combo box
  *
  * \return  GtkComboBox
  */
@@ -180,6 +198,7 @@ static GtkWidget *resource_combo_box_int_new_helper(
     GtkCellRenderer *renderer;
     const char *resource;
     int current;
+    gulong handler_id;
 
     /* setup combo box with model and renderers */
     model = create_combo_int_model(entries);
@@ -212,7 +231,9 @@ static GtkWidget *resource_combo_box_int_new_helper(
 
     /* remember original value for reset() */
     resource_widget_set_int(combo, "ResourceOrig", current);
-
+    /* used to reset combo box to its previous state if setting the resource
+     * fails */
+    resource_widget_set_int(combo, "PreviousID", current);
 
     /* register methods to be used by the resource widget manager */
     resource_widget_register_methods(
@@ -221,9 +242,10 @@ static GtkWidget *resource_combo_box_int_new_helper(
             vice_gtk3_resource_combo_box_int_factory,
             vice_gtk3_resource_combo_box_int_sync);
 
-
     /* connect signal handlers */
-    g_signal_connect(combo, "changed", G_CALLBACK(on_combo_int_changed), NULL);
+    handler_id = g_signal_connect(combo, "changed", G_CALLBACK(on_combo_int_changed), NULL);
+    /* used to temporarily block the signal handler to avoid extra resource_set_int() calls */
+    g_object_set_data(G_OBJECT(combo), "ChangedHandlerID", GULONG_TO_POINTER(handler_id));
     g_signal_connect_unlocked(combo, "destroy", G_CALLBACK(on_combo_int_destroy), NULL);
 
     gtk_widget_show(combo);
@@ -318,6 +340,8 @@ GtkWidget *vice_gtk3_resource_combo_box_int_new_with_label(
  *
  * \param[in,out]   widget  integer resource combo box
  * \param[in]       id      new ID for the combo box resource
+ *
+ * \return  TRUE if the new \a id was set
  */
 gboolean vice_gtk3_resource_combo_box_int_set(GtkWidget *widget, int id)
 {
@@ -340,6 +364,8 @@ gboolean vice_gtk3_resource_combo_box_int_set(GtkWidget *widget, int id)
  *
  * \param[in]   widget  integer resource combo box
  * \param[out]  dest    object to store ID
+ *
+ * \return  TRUE if the ID was properly read
  */
 gboolean vice_gtk3_resource_combo_box_int_get(GtkWidget *widget, int *dest)
 {
@@ -361,6 +387,8 @@ gboolean vice_gtk3_resource_combo_box_int_get(GtkWidget *widget, int *dest)
 /** \brief  Reset \a widget to its factory default
  *
  * \param[in,out]   widget  integer resource combo box
+ *
+ * \return  TRUE if the widget was reset to its factory value
  */
 gboolean vice_gtk3_resource_combo_box_int_factory(GtkWidget *widget)
 {
@@ -400,13 +428,21 @@ gboolean vice_gtk3_resource_combo_box_int_sync(GtkWidget *widget)
 {
     const char *resource_name;
     int resource_value;
+    gulong handler_id;
+    gboolean result;
 
     resource_name = resource_widget_get_resource_name(widget);
     if (resources_get_int(resource_name, &resource_value) < 0) {
         return FALSE;
     }
 
-    return set_combo_int_id(GTK_COMBO_BOX(widget), resource_value);
+    handler_id = GPOINTER_TO_ULONG(g_object_get_data(G_OBJECT(widget),
+                                                     "ChangedHandlerID"));
+    debug_gtk3("changed handler id = %lu.", handler_id);
+    g_signal_handler_block(G_OBJECT(widget), handler_id);
+    result = set_combo_int_id(GTK_COMBO_BOX(widget), resource_value);
+    g_signal_handler_unblock(G_OBJECT(widget), handler_id);
+    return result;
 }
 
 
@@ -417,12 +453,12 @@ gboolean vice_gtk3_resource_combo_box_int_sync(GtkWidget *widget)
  * Presents a combo box with strings, and uses strings for keys
  */
 
-/** \brief  Handler for the "destroy" event of the combo box
+/** \brief  Handler for the 'destroy' event of the combo box
  *
  * Frees the heap-allocated copy of the resource name
  *
- * \param[im]   combo       combo box
- * \param[in]   user_data   extra event data (unused)
+ * \param[in,out]   combo       combo box
+ * \param[in]       user_data   extra event data (unused)
  */
 static void on_combo_str_destroy(GtkWidget *combo, gpointer user_data)
 {
@@ -431,11 +467,11 @@ static void on_combo_str_destroy(GtkWidget *combo, gpointer user_data)
 }
 
 
-/** \brief  Handler for the "changed" event of the string combo box
+/** \brief  Handler for the 'changed' event of the string combo box
  *
  * Updates the resource connected to the combo box
  *
- * \param[im]   combo       combo box
+ * \param[in]   combo       combo box
  * \param[in]   user_data   extra event data (unused)
  */
 static void on_combo_str_changed(GtkWidget *combo, gpointer user_data)
@@ -531,7 +567,7 @@ GtkWidget *vice_gtk3_resource_combo_box_str_new(
 
 /** \brief  Create a combo box to control a string resource
  *
- * \param[in]   resource    resource name
+ * \param[in]   fmt         resource name, printf-style format string
  * \param[in]   entries     list of entries for the combo box
  *
  * \return  GtkComboBoxText
@@ -592,8 +628,10 @@ GtkWidget *vice_gtk3_resource_combo_box_str_new_with_label(
  *
  * Set new ID of the combo box
  *
- * \param[in,out]   combo   string resource combo box
+ * \param[in,out]   widget  string resource combo box
  * \param[in]       id      new ID of the combo box
+ *
+ * \return  TRUE if the new \a id was set
  */
 gboolean vice_gtk3_resource_combo_box_str_set(GtkWidget *widget, const char *id)
 {
@@ -617,8 +655,10 @@ gboolean vice_gtk3_resource_combo_box_str_set(GtkWidget *widget, const char *id)
  * value will be returned. On failure `FALSE` will be returned and `NULL`stored
  * in \a *dest.
  *
- * \param[in]   combo   string resource combo box
+ * \param[in]   widget  string resource combo box
  * \param[out]  dest    object to store ID
+ *
+ * \return  TRUE if the \a id was set
  */
 gboolean vice_gtk3_resource_combo_box_str_get(GtkWidget *widget, const char **dest)
 {
@@ -633,10 +673,11 @@ gboolean vice_gtk3_resource_combo_box_str_get(GtkWidget *widget, const char **de
 }
 
 
-
 /** \brief  Reset string combo box to its factory default
  *
  * \param[in,out]   widget  string resource combo box
+ *
+ * \return  TRUE if the factory default was set
  */
 gboolean vice_gtk3_resource_combo_box_str_factory(GtkWidget *widget)
 {
@@ -656,6 +697,8 @@ gboolean vice_gtk3_resource_combo_box_str_factory(GtkWidget *widget)
  * Restores the state of the widget as it was when instanciated.
  *
  * \param[in,out]   widget  string resource combo box
+ *
+ * \return  TRUE if the combo box was reset
  */
 gboolean vice_gtk3_resource_combo_box_str_reset(GtkWidget *widget)
 {
@@ -671,6 +714,8 @@ gboolean vice_gtk3_resource_combo_box_str_reset(GtkWidget *widget)
  * Updates the widget state to what its resource currently is.
  *
  * \param[in,out]   widget  string resource combo box
+ *
+ * \return  TRUE if the widget was synchronized with its resource
  */
 gboolean vice_gtk3_resource_combo_box_str_sync(GtkWidget *widget)
 {
