@@ -44,7 +44,6 @@ gboolean vice_gtk3_resource_entry_get(GtkWidget *entry, const char **dest);
 gboolean vice_gtk3_resource_entry_factory(GtkWidget *entry);
 gboolean vice_gtk3_resource_entry_reset(GtkWidget *widget);
 gboolean vice_gtk3_resource_entry_sync(GtkWidget *widget);
-gboolean vice_gtk3_resource_entry_apply(GtkWidget *widget);
 
 
 /*
@@ -53,6 +52,8 @@ gboolean vice_gtk3_resource_entry_apply(GtkWidget *widget);
  */
 
 GtkWidget *vice_gtk3_resource_entry_full_new(const char *resource);
+GtkWidget *vice_gtk3_resource_entry_full_new_sprintf(const char *fmt, ...);
+
 void vice_gtk3_resource_entry_full_disable_auto_update(GtkWidget *widget);
 
 gboolean vice_gtk3_resource_entry_full_set(GtkWidget *entry, const char *new);
@@ -60,7 +61,6 @@ gboolean vice_gtk3_resource_entry_full_get(GtkWidget *widget, const char **dest)
 gboolean vice_gtk3_resource_entry_full_reset(GtkWidget *entry);
 gboolean vice_gtk3_resource_entry_full_sync(GtkWidget *entry);
 gboolean vice_gtk3_resource_entry_full_factory(GtkWidget *entry);
-gboolean vice_gtk3_resource_entry_full_apply(GtkWidget *entry);
 
 
 #endif
