@@ -33,10 +33,12 @@
 #include "vice.h"
 #include <gtk/gtk.h>
 
-#include "debug_gtk3.h"
-#include "basewidgets.h"
 #include "basedialogs.h"
+#include "basewidgets.h"
+#include "basewidget_types.h"
+#include "debug_gtk3.h"
 #include "filechooserhelpers.h"
+#include "resourcehelpers.h"
 #include "widgethelpers.h"
 
 #endif
