@@ -38,10 +38,8 @@
 
 #include <gtk/gtk.h>
 
-#include "basewidgets.h"
-#include "debug_gtk3.h"
+#include "vice_gtk3.h"
 #include "resources.h"
-#include "widgethelpers.h"
 
 #include "cbm2rammappingwidget.h"
 
@@ -76,7 +74,14 @@ GtkWidget *cbm2_ram_mapping_widget_create(void)
     GtkWidget *grid;
     int i;
 
-    grid = uihelpers_create_grid_with_label("Map RAM into bank 15", 1);
+    grid = vice_gtk3_grid_new_spaced_with_label(
+            -1, 0, "Map RAM into bank 15", 1);
+    g_object_set(grid, "margin-left", 8, NULL);
+
+    /*update margins of the label */
+    GtkWidget *label = gtk_grid_get_child_at(GTK_GRID(grid), 0, 0);
+    g_object_set(label, "margin-bottom", 8, NULL);
+
     for (i = 0; mappings[i].text != NULL; i++) {
         GtkWidget *check;
 
