@@ -44,12 +44,10 @@
 
 #include <gtk/gtk.h>
 
-#include "basewidgets.h"
-#include "debug_gtk3.h"
+#include "vice_gtk3.h"
 #include "drivewidgethelpers.h"
 #include "machine.h"
 #include "resources.h"
-#include "widgethelpers.h"
 
 #include "driverpmwidget.h"
 
@@ -85,7 +83,7 @@ GtkWidget *drive_rpm_widget_create(int unit)
     GtkWidget *wobble_amp;
     GtkWidget *label;
 
-    grid = uihelpers_create_grid_with_label("RPM settings", 2);
+    grid = vice_gtk3_grid_new_spaced_with_label(-1, -1, "RPM settings", 2);
     g_object_set_data(G_OBJECT(grid), "UnitNumber", GINT_TO_POINTER(unit));
 
     /* RPM */
