@@ -69,9 +69,9 @@ static void save_filename_callback(GtkDialog *dialog,
 }
 
 
-/** \brief  Handler for the "clicked" event of the "Save As" button
+/** \brief  Handler for the 'clicked' event of the "Save As" button
  *
- * \param[in]   widget      button
+ * \param[in]   widget      button (unused)
  * \param[in]   user_data   extra event data (unused)
  */
 static void on_save_clicked(GtkWidget *widget, gpointer user_data)
@@ -83,14 +83,13 @@ static void on_save_clicked(GtkWidget *widget, gpointer user_data)
 }
 
 
-/** \brief  Handler for the "clicked" event of the "Flush now" button
+/** \brief  Handler for the 'clicked' event of the "Flush now" button
  *
- * \param[in]   widget      button
+ * \param[in]   widget      button (unused)
  * \param[in]   user_data   extra event data (unused)
  */
 static void on_flush_clicked(GtkWidget *widget, gpointer user_data)
 {
-    debug_gtk3("flushing EF image.");
     if (carthelpers_flush_func(CARTRIDGE_EASYFLASH) < 0) {
         vice_gtk3_message_error("VICE core",
                 "Failed to flush the EasyFlash image");
@@ -113,9 +112,7 @@ GtkWidget *easyflash_widget_create(GtkWidget *parent)
     GtkWidget *save_button;
     GtkWidget *flush_button;
 
-    grid = gtk_grid_new();
-    gtk_grid_set_column_spacing(GTK_GRID(grid), 16);
-    gtk_grid_set_row_spacing(GTK_GRID(grid), 8);
+    grid = vice_gtk3_grid_new_spaced(VICE_GTK3_DEFAULT, VICE_GTK3_DEFAULT);
 
     jumper = vice_gtk3_resource_check_button_new(
             "EasyFlashJumper", "Set Easy Flash jumper");
@@ -138,7 +135,7 @@ GtkWidget *easyflash_widget_create(GtkWidget *parent)
             (gboolean)(carthelpers_can_save_func(CARTRIDGE_EASYFLASH)));
 
     /* Flush image now */
-    flush_button = gtk_button_new_with_label("Flush image now");
+    flush_button = gtk_button_new_with_label("Save image");
     gtk_grid_attach(GTK_GRID(grid), flush_button, 1, 1, 1, 1);
     g_signal_connect(flush_button, "clicked", G_CALLBACK(on_flush_clicked),
             NULL);
