@@ -38,14 +38,13 @@
 #include <gtk/gtk.h>
 #include <stdlib.h>
 
-
-#include "debug_gtk3.h"
-#include "joy.h"
+#include "vice_gtk3.h"
 #include "joystick.h"
 #include "lib.h"
 #include "machine.h"
 #include "resources.h"
-#include "widgethelpers.h"
+#include "filechooserhelpers.h"
+
 #include "joystickdevicewidget.h"
 
 
@@ -115,12 +114,12 @@ GtkWidget *joystick_device_widget_create(int device, const char *title)
 {
     GtkWidget *grid;
     GtkWidget *combo;
-    int id, i1, i2;
+    int id, i1, i2;     /* really? */
     int current;
 
     resources_get_int_sprintf("JoyDevice%d", &current, device + 1);
 
-    grid = uihelpers_create_grid_with_label(title, 1);
+    grid = vice_gtk3_grid_new_spaced_with_label(-1, -1, title, 1);
 
     combo = gtk_combo_box_text_new();
     g_object_set(combo, "margin-left", 16, NULL);
@@ -140,17 +139,22 @@ GtkWidget *joystick_device_widget_create(int device, const char *title)
     joystick_ui_reset_device_list();
     for (i2 = 0; (device_list[i2].name = joystick_ui_get_next_device_name(&id)) != NULL; i2++) {
         char idstr[32];
+        gchar *utf8;
 
         if (i2 >= MAX_EXTRA_DEVICES) {
             break;
         }
 
+        /* convert name from locale to UTF-8 to be used in the list */
+        utf8 = file_chooser_convert_from_locale(device_list[i2].name);
+
         device_list[i2].id = id;
         g_snprintf(idstr, 32, "%d", device_list[i2].id);
 
         gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(combo),
                                   idstr,
-                                  device_list[i2].name);
+                                  utf8);
+        g_free(utf8);
         if (device_list[i2].id == current) {
             gtk_combo_box_set_active(GTK_COMBO_BOX(combo), i1+i2);
         }
@@ -167,7 +171,7 @@ GtkWidget *joystick_device_widget_create(int device, const char *title)
 
 /** \brief  Set joystick device \a widget to \a id
  *
- * \param]in,out]   widget  joystick device widget
+ * \param[in,out]   widget  joystick device widget
  * \param[in]       id      new value for the \a widget
  */
 void joystick_device_widget_update(GtkWidget *widget, int id)
@@ -181,7 +185,6 @@ void joystick_device_widget_update(GtkWidget *widget, int id)
     /* get combo box widget */
     combo = gtk_grid_get_child_at(GTK_GRID(widget), 0, 1);
     if (combo != NULL && GTK_IS_COMBO_BOX_TEXT(combo)) {
-        debug_gtk3("updating widget to %d.", id);
         gtk_combo_box_set_active_id(GTK_COMBO_BOX(combo), id_str);
     }
 }
