@@ -6,7 +6,7 @@
 
 /*
  * $VICERES MIDIEnable      x64 x64sc xscpu64 x128 xvic
- * $VICERES MIDIMode        x64 x64sc xscpu64 x128 xvic
+ * $VICERES MIDIMode        x64 x64sc xscpu64 x128
  * $VICERE$S MIDIDriver     x64 x64sc xscpu64 x128 xvic
  *  (Unix only)
  * $VICERES MIDIInDev       x64 x64sc xscpu64 x128 xvic
@@ -60,14 +60,20 @@
 static GtkWidget *midi_enable;
 /** \brief  MIDI mode combobox */
 static GtkWidget *midi_mode;
+/** \brief  MIDI in device  */
 static GtkWidget *midi_in_entry;
+/** \brief  MIDI out device */
 static GtkWidget *midi_out_entry;
 #ifdef ARCHDEP_OS_UNIX
 # ifdef ARCHDEP_OS_MACOS
+/** \brief  MIDI name entry */
 static GtkWidget *midi_name_entry;
 # else
+/** \brief  MIDI driver widget */
 static GtkWidget *midi_driver;
+/** \brief  MIDI in file browser */
 static GtkWidget *midi_in_browse;
+/** \brief  MIDI out file browser */
 static GtkWidget *midi_out_browse;
 # endif
 #endif
@@ -107,7 +113,9 @@ static void on_midi_enable_toggled(GtkWidget *widget, gpointer user_data)
 {
     int state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-    gtk_widget_set_sensitive(midi_mode, state);
+    if (machine_class != VICE_MACHINE_VIC20) {
+        gtk_widget_set_sensitive(midi_mode, state);
+    }
 #ifdef ARCHDEP_OS_UNIX
 # ifdef ARCHDEP_OS_MACOS
     gtk_widget_set_sensitive(midi_name_entry, state);
@@ -132,7 +140,6 @@ static void midi_in_filename_callback(GtkDialog *dialog,
                                       gpointer data)
 {
     if (filename != NULL) {
-        debug_gtk3("Setting MIDIInDev to '%s'", filename);
         vice_gtk3_resource_entry_full_set(midi_in_browse, filename);
         g_free(filename);
     }
@@ -163,7 +170,6 @@ static void midi_out_filename_callback(GtkDialog *dialog,
                                        gpointer data)
 {
     if (filename != NULL) {
-        debug_gtk3("Setting MIDIOutDev to '%s'", filename);
         vice_gtk3_resource_entry_full_set(midi_out_browse, filename);
         g_free(filename);
     }
@@ -171,7 +177,7 @@ static void midi_out_filename_callback(GtkDialog *dialog,
 }
 
 
-/** \brief  Handler for the "clicked" event of the MIDI-Out "Browse" button
+/** \brief  Handler for the 'clicked' event of the MIDI-Out "Browse" button
  *
  * \param[in]   widget      button
  * \param[in]   user_data   text entry to store new filename
@@ -229,7 +235,7 @@ static GtkWidget *create_midi_driver_widget(void)
 
 /** \brief  Create MIDI settings widget
  *
- * \param[in]   parent  parent widget
+ * \param[in]   parent  parent widget (unused)
  *
  * \return  GtkGrid
  */
@@ -239,19 +245,19 @@ GtkWidget *midi_widget_create(GtkWidget *parent)
     GtkWidget *label;
     int row;
 
-    grid = gtk_grid_new();
-    gtk_grid_set_column_spacing(GTK_GRID(grid), 16);
-    gtk_grid_set_row_spacing(GTK_GRID(grid), 8);
+    grid = vice_gtk3_grid_new_spaced(VICE_GTK3_DEFAULT, VICE_GTK3_DEFAULT);
 
     midi_enable = create_midi_enable_widget();
     gtk_grid_attach(GTK_GRID(grid), midi_enable, 0, 0, 3, 1);
 
-    label = gtk_label_new("MIDI mode");
-    gtk_widget_set_halign(label, GTK_ALIGN_START);
-    g_object_set(label, "margin-left", 16, NULL);
-    gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
-    midi_mode = create_midi_mode_widget();
-    gtk_grid_attach(GTK_GRID(grid), midi_mode, 1, 1, 1, 1);
+    if (machine_class != VICE_MACHINE_VIC20) {
+        label = gtk_label_new("MIDI mode");
+        gtk_widget_set_halign(label, GTK_ALIGN_START);
+        g_object_set(label, "margin-left", 16, NULL);
+        gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
+        midi_mode = create_midi_mode_widget();
+        gtk_grid_attach(GTK_GRID(grid), midi_mode, 1, 1, 1, 1);
+    }
 
     row = 2;
 
