@@ -34,16 +34,21 @@
 
 #include <gtk/gtk.h>
 
-#include "widgethelpers.h"
-#include "debug_gtk3.h"
+#include "vice_gtk3.h"
 #include "resources.h"
 
 #include "petmiscwidget.h"
 
 
+/** \brief  Crtc checkbox */
 static GtkWidget *crtc_widget = NULL;
+
+/** \brief  Blank-on-EOI checkbox */
 static GtkWidget *blank_widget = NULL;
 
+/** \brief  Screen mirrors like the 2001 checkbox */
+static GtkWidget *screen2001_widget = NULL;
+
 /** \brief  User-defined callback for changes in the Crtc resource
  */
 static void (*user_callback_crtc)(int) = NULL;
@@ -52,6 +57,10 @@ static void (*user_callback_crtc)(int) = NULL;
  */
 static void (*user_callback_blank)(int) = NULL;
 
+/** \brief  User-defined callback for changes in the Screen2001 resource
+ */
+static void (*user_callback_screen2001)(int) = NULL;
+
 
 /** \brief  Handler for the "toggled" event of the CRTC check button
  *
@@ -69,10 +78,8 @@ static void on_crtc_toggled(GtkWidget *widget, gpointer user_data)
     new_val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
     if (new_val != old_val) {
-        debug_gtk3("setting Crtc to %s.", new_val ? "ON" : "OFF");
         resources_set_int("Crtc", new_val);
         if (user_callback_crtc != NULL) {
-            debug_gtk3("calling user_callback_crtc(%d).", new_val);
             user_callback_crtc(new_val);
         }
     }
@@ -95,13 +102,34 @@ static void on_blank_toggled(GtkWidget *widget, gpointer user_data)
     new_val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
     if (new_val != old_val) {
-        debug_gtk3("setting EoiBlank to %s.", new_val ? "ON" : "OFF");
         resources_set_int("EoiBlank", new_val);
         if (user_callback_blank != NULL) {
-            debug_gtk3("calling user_callback_blank(%d).", new_val);
             user_callback_blank(new_val);
         }
+    }
+}
 
+
+/** \brief  Handler for the "toggled" event of the Screen2001 check button
+ *
+ * Sets the Screen2001 resource when it has been changed.
+ *
+ * \param[in]   widget      radio button triggering the event
+ * \param[in]   user_data   extra data (unused)
+ */
+static void on_screen2001_toggled(GtkWidget *widget, gpointer user_data)
+{
+    int old_val;
+    int new_val;
+
+    resources_get_int("Screen2001", &old_val);
+    new_val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+    if (new_val != old_val) {
+        resources_set_int("Screen2001", new_val);
+        if (user_callback_screen2001 != NULL) {
+            user_callback_screen2001(new_val);
+        }
     }
 }
 
@@ -117,57 +145,89 @@ GtkWidget *pet_misc_widget_create(void)
     GtkWidget *grid;
     int crtc;
     int blank;
+    int screen2001;
 
     user_callback_crtc = NULL;
     user_callback_blank = NULL;
+    user_callback_screen2001 = NULL;
 
     resources_get_int("Crtc", &crtc);
     resources_get_int("EoiBlank", &blank);
+    resources_get_int("Screen2001", &screen2001);
 
-    grid = uihelpers_create_grid_with_label("Miscellaneous", 1);
+    grid = vice_gtk3_grid_new_spaced_with_label(-1, -1, "Miscellaneous", 1);
     gtk_grid_set_column_spacing(GTK_GRID(grid), 8);
 
     crtc_widget = gtk_check_button_new_with_label("CRTC chip enable");
     g_object_set(crtc_widget, "margin-left", 16, NULL);
-    blank_widget = gtk_check_button_new_with_label("EOI blanks screen");
+    blank_widget = gtk_check_button_new_with_label("2001 quirk: EOI blanks screen");
+    g_object_set(blank_widget, "margin-left", 16, NULL);
+    screen2001_widget = gtk_check_button_new_with_label("2001 quirk: extra screen mirrors");
+    g_object_set(screen2001_widget, "margin-left", 16, NULL);
+
     gtk_grid_attach(GTK_GRID(grid), crtc_widget, 0, 1, 1, 1);
-    gtk_grid_attach(GTK_GRID(grid), blank_widget, 1, 1, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), blank_widget, 0, 2, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), screen2001_widget, 0, 3, 1, 1);
 
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(crtc_widget), crtc);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(blank_widget), blank);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(screen2001_widget), blank);
 
     g_signal_connect(crtc_widget, "toggled", G_CALLBACK(on_crtc_toggled), NULL);
     g_signal_connect(blank_widget, "toggled", G_CALLBACK(on_blank_toggled), NULL);
+    g_signal_connect(screen2001_widget, "toggled", G_CALLBACK(on_screen2001_toggled), NULL);
 
     gtk_widget_show_all(grid);
     return grid;
 }
 
 
-void pet_misc_widget_set_crtc_callback(GtkWidget *widget,
-                                       void (*func)(int))
+/** \brief  Set function to trigger on Crtc checkbox toggle
+ *
+ * \param[in]   func    callback function
+ */
+void pet_misc_widget_set_crtc_callback(void (*func)(int))
 {
     user_callback_crtc = func;
 }
 
 
-void pet_misc_widget_set_blank_callback(GtkWidget *widget,
-                                       void (*func)(int))
+/** \brief  Set function to trigger on EOI-blank checkbox toggle
+ *
+ * \param[in]   func    callback function
+ */
+void pet_misc_widget_set_blank_callback(void (*func)(int))
 {
     user_callback_blank = func;
 }
 
 
+/** \brief  Set function to trigger on Screen2001 checkbox toggle
+ *
+ * \param[in]   func    callback function
+ */
+void pet_misc_widget_set_screen2001_callback(void (*func)(int))
+{
+    user_callback_screen2001 = func;
+}
+
+
+/** \brief  Synchronize \a widget with its resources
+ *
+ * Synchronize the PET misc widget's child widgets with their resources.
+ *
+ * \param[in,out]   widget  PET misc widget
+ */
 void pet_misc_widget_sync(GtkWidget *widget)
 {
     int state;
-    GtkWidget *check;
 
     resources_get_int("Crtc", &state);
-    check = gtk_grid_get_child_at(GTK_GRID(widget), 0, 1);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), state);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(crtc_widget), state);
 
     resources_get_int("EoiBlank", &state);
-    check = gtk_grid_get_child_at(GTK_GRID(widget), 1, 1);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), state);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(blank_widget), state);
+
+    resources_get_int("Screen2001", &state);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(screen2001_widget), state);
 }
