@@ -37,27 +37,11 @@
 #include "lib.h"
 #include "machine.h"
 #include "crtcontrolwidget.h"
-#include "resourcewidgetmanager.h"
 #include "uisettings.h"
 
 #include "settings_crt.h"
 
 
-/** \brief  Resource widget manager object
- */
-static resource_widget_manager_t manager;
-
-
-/** \brief  Clean up memory used by the widget manager
- *
- * \param[in]   widget  widget triggering the event (unused)
- * \param[in]   data    extra event data (unused)
- */
-static void on_destroy(GtkWidget *widget, gpointer data)
-{
-    vice_resource_widget_manager_exit(&manager);
-}
-
 
 /** \brief  Create a CRT settings widget for the settings UI
  *
@@ -74,9 +58,6 @@ GtkWidget *settings_crt_widget_create(GtkWidget *parent)
     GtkWidget *chip1 = NULL;
     GtkWidget *chip2 = NULL;
 
-    vice_resource_widget_manager_init(&manager);
-    ui_settings_set_resource_widget_manager(&manager);
-
     grid = vice_gtk3_grid_new_spaced(VICE_GTK3_DEFAULT, VICE_GTK3_DEFAULT);
 
     switch (machine_class) {
@@ -109,18 +90,12 @@ GtkWidget *settings_crt_widget_create(GtkWidget *parent)
     /* add VIC/VICII/TED/CRTC */
     if (chip1 != NULL) {
         gtk_grid_attach(GTK_GRID(grid), chip1, 0, 0, 1, 1);
-        vice_resource_widget_manager_add_widget(&manager, chip1, NULL,
-                crt_control_widget_reset, NULL, NULL);
     }
     /* add VDC in case of c128 */
     if (chip2 != NULL) {
         gtk_grid_attach(GTK_GRID(grid), chip2, 0, 1, 1, 1);
-        vice_resource_widget_manager_add_widget(&manager, chip2, NULL,
-                crt_control_widget_reset, NULL, NULL);
     }
 
-    g_signal_connect(grid, "destroy", G_CALLBACK(on_destroy), NULL);
-
     gtk_widget_show_all(grid);
     return grid;
 }
