@@ -54,6 +54,7 @@
 
 #ifdef HAVE_RAWNET
 static void clean_iface_list(void);
+static void clean_driver_list(void);
 #endif
 
 
@@ -67,6 +68,7 @@ static void on_settings_ethernet_destroy(GtkWidget *widget, gpointer data)
 {
 #ifdef HAVE_RAWNET
     clean_iface_list();
+    clean_driver_list();
 #endif
 }
 
@@ -81,6 +83,13 @@ static void on_settings_ethernet_destroy(GtkWidget *widget, gpointer data)
  */
 static vice_gtk3_combo_entry_str_t *iface_list;
 
+/** \brief  List of available drivers
+ *
+ * This list is dynamically generated and destroyed when the main widget
+ * is destroyed.
+ */
+static vice_gtk3_combo_entry_str_t *driver_list;
+
 
 /** \brief  Build interface list for the combo box
  *
@@ -141,6 +150,64 @@ static gboolean build_iface_list(void)
 }
 
 
+/** \brief  Build driver list for the combo box
+ *
+ * \return  TRUE if the list was generated
+ */
+static gboolean build_driver_list(void)
+{
+    int num = 0;
+    char *driver_name;
+    char *driver_desc;
+
+    /* get number of adapters */
+    if (!rawnet_enumdriver_open()) {
+        return FALSE;
+    }
+    while (rawnet_enumdriver(&driver_name, &driver_desc)) {
+        lib_free(driver_name);
+        if (driver_desc != NULL) {
+            lib_free(driver_desc);
+        }
+        num++;
+    }
+    rawnet_enumdriver_close();
+
+    /* allocate memory for list */
+    driver_list = lib_malloc((size_t)(num + 1) * sizeof *driver_list);
+
+    /* now add the list items */
+    if (!rawnet_enumdriver_open()) {
+        lib_free(driver_list);
+        driver_list = NULL;
+        return FALSE;
+    }
+
+    num = 0;
+    while (rawnet_enumdriver(&driver_name, &driver_desc)) {
+        driver_list[num].id = lib_strdup(driver_name);
+        /*
+         * On Windows, the description string seems to be always present, on
+         * Unix this isn't the case and NULL can be returned.
+         */
+        if (driver_desc == NULL) {
+            driver_list[num].name = lib_strdup(driver_name);
+        } else {
+            driver_list[num].name = lib_msprintf("%s (%s)", driver_name, driver_desc);
+        }
+        lib_free(driver_name);
+        if (driver_desc != NULL) {
+            lib_free(driver_desc);
+        }
+
+        num++;
+    }
+    driver_list[num].id = NULL;
+    driver_list[num].name = NULL;
+    rawnet_enumdriver_close();
+    return TRUE;
+}
+
 /** \brief  Free memory used by the interface list
  */
 static void clean_iface_list(void)
@@ -157,6 +224,36 @@ static void clean_iface_list(void)
     }
 }
 
+/** \brief  Free memory used by the driver list
+ */
+static void clean_driver_list(void)
+{
+    if (driver_list != NULL) {
+        int num = 0;
+        while (driver_list[num].id != NULL) {
+            lib_free(driver_list[num].id);
+            lib_free(driver_list[num].name);
+            num++;
+        }
+        lib_free(driver_list);
+        driver_list = NULL;
+    }
+}
+
+
+static GtkWidget *create_driver_combo(void)
+{
+    GtkWidget *combo;
+
+    if (build_driver_list()) {
+        combo = vice_gtk3_resource_combo_box_str_new(
+                "ETHERNET_DRIVER",
+                driver_list);
+    } else {
+        combo = gtk_combo_box_text_new();
+    }
+    return combo;
+}
 
 
 /** \brief  Create combo box to select the ethernet interface
@@ -191,9 +288,9 @@ GtkWidget *settings_ethernet_widget_create(GtkWidget *parent)
     GtkWidget *label;
     char *text;
 #ifdef HAVE_RAWNET
-    GtkWidget *combo;
+    GtkWidget *iface_combo;
+    GtkWidget *driver_combo;
     bool available = archdep_ethernet_available();
-    debug_gtk3("Ethernet available = %s\n", available ? "True" : "False");
 #endif
 
     grid = vice_gtk3_grid_new_spaced(VICE_GTK3_DEFAULT, VICE_GTK3_DEFAULT);
@@ -222,19 +319,25 @@ GtkWidget *settings_ethernet_widget_create(GtkWidget *parent)
     }
 
 #ifdef HAVE_RAWNET
-    label = gtk_label_new("Ethernet device");
+    label = gtk_label_new("Ethernet driver:");
+    gtk_widget_set_halign(label, GTK_ALIGN_START);
+    driver_combo = create_driver_combo();
+    gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), driver_combo, 1, 0, 1, 1);
+
+    label = gtk_label_new("Ethernet interface:");
     gtk_widget_set_halign(label, GTK_ALIGN_START);
 
-    combo = create_device_combo();
-    gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
-    gtk_grid_attach(GTK_GRID(grid), combo, 1, 0, 1, 1);
+    iface_combo = create_device_combo();
+    gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), iface_combo, 1, 1, 1, 1);
 
     if (!available) {
-        gtk_widget_set_sensitive(combo, FALSE);
+        gtk_widget_set_sensitive(iface_combo, FALSE);
         label = gtk_label_new(NULL);
 # ifdef ARCHDEP_OS_UNIX
         gtk_label_set_markup(GTK_LABEL(label),
-                "<i>VICE needs to be run as <tt>root</tt> to be able to use ethernet emulation.</i>");
+                "<i>VICE needs TUN/TAP support or the proper permissions (with libpcap) to be able to use ethernet emulation.</i>");
 # elif defined(ARCHDEP_OS_WINDOWS)
         gtk_label_set_markup(GTK_LABEL(label),
                 "<i><tt>wpcap.dll</tt> not found, please install WinPCAP to use ethernet emulation.</i>");
