@@ -97,61 +97,112 @@ static const vice_gtk3_radiogroup_entry_t c64dtv_revisions[] = {
 };
 
 
+/*
+ * Function pointers
+ */
+
+/** \brief  Function to call to get the model for the current machine
+ */
+static int (*get_model_func)(void);
+
+/** \brief  Function to call to get the memhack name */
+static const char *(*get_memhack_func)(int hack);
+
+
 /** \brief  Machine model widget
  *
- * Used by all machines
+ * This widget controls which machine model is used.
+ *
+ * Changing the machine model means a lot of other resources being changed, and
+ * changing a resource changes the model (or invalidates it).
  */
 static GtkWidget *machine_widget = NULL;
 
-
 /** \brief  CIA model widget
  *
  * Used by C64, C64SC, SCPU64, C64DTV, C128, CBM5x0, CBM-II, VSID
  */
 static GtkWidget *cia_widget = NULL;
 
-
-static int (*get_model_func)(void);
-
-static const char *(*get_memhack_func)(int hack);
-
-
 /** \brief  Video model widget
  */
 static GtkWidget *video_widget = NULL;
 
+/** \brief  RAM widget */
 static GtkWidget *ram_widget = NULL;
+
+/** \brief  Memhacks widget
+ *
+ * Hardware hacks related to installed memory, x64/x64sc only.
+ */
 static GtkWidget *memhack_widget = NULL;
+
+/** \brief  ACIA widget
+ *
+ * Plus4 only.
+ */
 static GtkWidget *acia_widget = NULL;
+
+/** \brief  V364 speech widget
+ *
+ * Used only in xplus4.
+ */
 static GtkWidget *speech_widget = NULL;
+
+/** \brief  VDC display widget
+ *
+ * Used in x128.
+ */
 static GtkWidget *vdc_widget = NULL;
+
+/** \brief  SID widget */
 static GtkWidget *sid_widget = NULL;
+
+/** \brief  KERNAL widget */
 static GtkWidget *kernal_widget = NULL;
+
+/** \brief  PET video widget */
 static GtkWidget *pet_video_size_widget = NULL;
+
+/** \brief  PET keyboard widget */
 static GtkWidget *pet_keyboard_widget = NULL;
+
+/** \brief  PET miscellaneous settings widget */
 static GtkWidget *pet_misc_widget = NULL;
+
+/** \brief  PET I/O size widget */
 static GtkWidget *pet_io_widget = NULL;
+
+/** \brief  PET RAM9 ($9000-$9fff) widget */
 static GtkWidget *pet_ram9_widget = NULL;
+
+/** \brief  PET RAMA ($a000-$afff) widget */
 static GtkWidget *pet_rama_widget = NULL;
 
+/** \brief  C64 DTV revision widget */
 static GtkWidget *c64dtv_rev_widget = NULL;
+
+/** \brief  C64 DTV Hummer ADC widget */
 static GtkWidget *c64dtv_hummer_adc_widget = NULL;
+
+/** \brief  Reset with IEC checkbox */
 static GtkWidget *reset_with_iec_widget = NULL;
 
+/** \brief  C64 "discrete glue logic" radio button */
+static GtkWidget *c64_discrete_radio = NULL;
 
+/** \brief  C64 "custom glue logic" radio button */
+static GtkWidget *c64_custom_radio = NULL;
+
+
+
+/** \brief  Function called video model changes
+ *
+ * \param[in]   model   new videochip model
+ */
 static void video_model_callback(int model)
 {
-#ifdef HAVE_DEBUG_GTK3UI
-    int true_model = -1;
-#endif
-
-    debug_gtk3("got model %d", model);
-
     if (get_model_func != NULL) {
-#ifdef HAVE_DEBUG_GTK3UI
-        true_model = get_model_func();
-#endif
-        debug_gtk3("got true model %d", true_model);
         machine_model_widget_update(machine_widget);
         if (machine_class == VICE_MACHINE_PLUS4) {
             plus4_debug_dump_resources();
@@ -165,17 +216,25 @@ static void video_model_callback(int model)
  * x128-specific callbacks
  */
 
+
+/** \brief  Function called on VDC revision changes
+ *
+ * \param[in]   revision    new VDC revision (unused)
+ */
 static void vdc_revision_callback(int revision)
 {
-    debug_gtk3("got VDC revision %d.", revision);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
 }
 
 
+/** \brief  Function called on VDC RAM changes
+ *
+ * \param[in]   state   new VDC RAM state (unused)
+ */
 static void vdc_ram_callback(int state)
-{    debug_gtk3("Got VDC 64KiB RAM state %d.", state);
+{
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
@@ -183,19 +242,14 @@ static void vdc_ram_callback(int state)
 
 /* }}} */
 
+
+/** \brief  Function called on SID model changes
+ *
+ * \param[in]   model   new SID model
+ */
 static void sid_model_callback(int model)
 {
-#ifdef HAVE_DEBUG_GTK3UI
-    int true_model = -1;
-#endif
-
-    debug_gtk3("got model %d", model);
-
     if (get_model_func != NULL) {
-#ifdef HAVE_DEBUG_GTK3UI
-        true_model = get_model_func();
-#endif
-        debug_gtk3("got true model %d", true_model);
         machine_model_widget_update(machine_widget);
     }
 }
@@ -210,27 +264,21 @@ static void sid_model_callback(int model)
  */
 static void kernal_revision_callback(int rev)
 {
-#ifdef HAVE_DEBUG_GTK3UI
-    int true_model = -1;
-
-    if (get_model_func != NULL) {
-        true_model = get_model_func();
-        debug_gtk3("Got model ID: %d.", true_model);
-    }
-#endif
-    debug_gtk3("Got revision %d.", rev);
     machine_model_widget_update(machine_widget);
 }
 
 
+/** \brief  Function called on IEC checkbox toggles
+ *
+ * \param[in]   widget  IEC widget (unused)
+ * \param[in]   data    extra event data (unused)
+ */
 static void iec_callback(GtkWidget *widget, gpointer data)
 {
-    debug_gtk3("Called.");
     machine_model_widget_update(machine_widget);
 }
 
 
-
 /** \brief  Callback for CIA model changes
  *
  * \param[in]   cia_num     CIA number (1 or 2)
@@ -238,8 +286,6 @@ static void iec_callback(GtkWidget *widget, gpointer data)
  */
 static void cia_model_callback(int cia_num, int cia_model)
 {
-    debug_gtk3("got CIA %d, model %d", cia_num, cia_model);
-
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
@@ -250,70 +296,106 @@ static void cia_model_callback(int cia_num, int cia_model)
 
 /** \brief  Callback for PET RAM size changes
  *
- * \param[in]   size    RAM size in KiB
+ * \param[in]   size    RAM size in KiB (unused)
  */
 static void pet_ram_size_callback(int size)
 {
-    debug_gtk3("Called with %d RAM.", size);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
 }
 
 
+/** \brief  Function called on PET I/O size changes
+ *
+ * \param[in]   size    new I/O size (unused)
+ */
 static void pet_video_size_callback(int size)
 {
-    debug_gtk3("Called with %d video size.", size);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
 }
 
 
+/** \brief  Function called on PET keyboard type changes
+ *
+ * \param[in]   type    new keyboard type (unused)
+ */
 static void pet_keyboard_type_callback(int type)
 {
-    debug_gtk3("called with keyboard type %d.", type);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
 }
 
+
+/** \brief  Function called on PET CRTC changes
+ *
+ * \param[in]   state   new CRTC state (unused)
+ */
 static void pet_crtc_callback(int state)
 {
-    debug_gtk3("called with CRTC %d.", state);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
 }
 
+
+/** \brief  Function called on PET blank-on-eoi changes
+ *
+ * \param[in]   state   new blank-on-eoi state (unused)
+ */
 static void pet_blank_callback(int state)
 {
-    debug_gtk3("called with EOI-blank %d.", state);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
 }
 
 
+/** \brief  Function called on PET screen-mirrors-like-2001 changes
+ *
+ * \param[in]   state   new blank-on-eoi state (unused)
+ */
+static void pet_screen2001_callback(int state)
+{
+    if (get_model_func != NULL) {
+        machine_model_widget_update(machine_widget);
+    }
+}
+
+
+/** \brief  Function called on PET I/O changes
+ *
+ * \param[in]   state   new I/O state (unused)
+ */
 static void pet_io_callback(int state)
 {
-    debug_gtk3("called with IO size %d.", state);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
 }
 
+
+/** \brief  Function called on PET RAM9 changes
+ *
+ * \param[in]   state   new RAM9 state (unused)
+ */
 static void pet_ram9_callback(int state)
 {
-    debug_gtk3("called with RAM9 type %d.", state);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
 }
 
+
+/** \brief  Function called on PET RAMA changes
+ *
+ * \param[in]   state   new RAMA state (unused)
+ */
 static void pet_rama_callback(int state)
 {
-    debug_gtk3("called with RAMA type %d.", state);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
@@ -323,7 +405,8 @@ static void pet_rama_callback(int state)
 
 /* {{{ Plus4 glue logic and helpers */
 
-
+/** \brief  Debug hook: dump Plus4-related resources on stdoud
+ */
 static void plus4_debug_dump_resources(void)
 {
 #ifdef HAVE_DEBUG_GTK3UI
@@ -393,13 +476,11 @@ static void plus4_mem_size_callback(GtkWidget *widget, int value)
     int size = 0;
 
     resources_get_int("RamSize", &size);
+#if 0
     debug_gtk3("Got new value: %dKiB, RamSize = %d", value, size);
     debug_gtk3("Calling plus4_memory_expansion_widget_sync(): ");
-    if (plus4_memory_expansion_widget_sync()) {
-        debug_gtk3("OK.");
-    } else {
-        debug_gtk3("failed.");
-    }
+#endif
+    plus4_memory_expansion_widget_sync();
     machine_model_widget_update(machine_widget);
     plus4_debug_dump_resources();
 }
@@ -417,18 +498,10 @@ static void plus4_mem_hack_callback(GtkWidget *widget, int value)
     int size = 0;
 
     resources_get_int("RamSize", &size);
-    debug_gtk3("Got new MemoryHack value: %d, RamSize = %d", value, size);
 
     plus4_memory_size_widget_sync();
-    debug_gtk3("Setting RamSize widget sensitivity to %s",
-            value == MEMORY_HACK_NONE ? "TRUE" : "FALSE");
     gtk_widget_set_sensitive(ram_widget, value == MEMORY_HACK_NONE);
-    debug_gtk3("Calling plus4_memory_size_widget_sync(): ");
-    if (plus4_memory_size_widget_sync()) {
-        debug_gtk3("OK.");
-    } else {
-        debug_gtk3("failed.");
-    }
+    plus4_memory_size_widget_sync();
 
     machine_model_widget_update(machine_widget);
     plus4_debug_dump_resources();
@@ -442,7 +515,6 @@ static void plus4_mem_hack_callback(GtkWidget *widget, int value)
  */
 static void plus4_acia_widget_callback(GtkWidget *widget, int value)
 {
-    debug_gtk3("Called with value %d\n", value);
     machine_model_widget_update(machine_widget);
 }
 
@@ -454,7 +526,6 @@ static void plus4_acia_widget_callback(GtkWidget *widget, int value)
  */
 static void v364_speech_widget_callback(GtkWidget *widget, int value)
 {
-    debug_gtk3("Called with value %d\n", value);
     machine_model_widget_update(machine_widget);
 }
 
@@ -467,12 +538,15 @@ static void v364_speech_widget_callback(GtkWidget *widget, int value)
  */
 static void c64_misc_widget_sync(void);
 
+
+/** \brief  Callback triggered on changing machine model
+ *
+ * \param[in]   model   machine model
+ */
 static void machine_model_handler_c64(int model)
 {
     GtkWidget *sid_group;
 
-    debug_gtk3("Got model change for C64: %d", model);
-
     /* synchronize video chip widget */
     video_model_widget_update(video_widget);
 
@@ -557,11 +631,10 @@ static void machine_model_handler_c128(int model)
  * Calls model widget update
  *
  * \param[in]   widget      radio button triggering the callback (unused)
- * \param[in]   revision    new DTV revision
+ * \param[in]   revision    new DTV revision (unused)
  */
 static void dtv_revision_callback(GtkWidget *widget, int revision)
 {
-    debug_gtk3("got revision %d.", revision);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
@@ -572,11 +645,10 @@ static void dtv_revision_callback(GtkWidget *widget, int revision)
  *
  * Calls model widget update
  *
- * \param[in]   model   new VIC-II model
+ * \param[in]   model   new VIC-II model (unused)
  */
 static void dtv_video_callback(int model)
 {
-    debug_gtk3("got video model %d.", model);
     machine_model_widget_update(machine_widget);
 }
 
@@ -604,7 +676,7 @@ static void c64dtv_hummer_adc_callback(GtkWidget *widget, int value)
 
 /** \brief  Callback for DTV machine model changes
  *
- * Updates the DTV revision and VIC-II model widgets
+ * Updates the DTV revision and VIC-II model widgets.
  *
  * \param[in]   model   DTV model
  */
@@ -628,7 +700,6 @@ static void machine_model_handler_c64dtv(int model)
             /* 3: V3 PAL */
             break;
     }
-    debug_gtk3("setting revision to %d", rev);
 
     /* update revision widget */
     group = gtk_grid_get_child_at(GTK_GRID(c64dtv_rev_widget), 0, 1);
@@ -656,7 +727,6 @@ static void machine_model_handler_c64dtv(int model)
  */
 static void vic20_video_callback(int model)
 {
-    debug_gtk3("got video model %d.", model);
     machine_model_widget_update(machine_widget);
 }
 
@@ -695,7 +765,6 @@ static void machine_model_handler_vic20(int model)
  */
 static void plus4_video_callback(int model)
 {
-    debug_gtk3("got video model %d.", model);
     machine_model_widget_update(machine_widget);
     plus4_debug_dump_resources();
 }
@@ -721,7 +790,6 @@ static void plus4_memory_callback(int ram, int hack)
  */
 static void machine_model_handler_plus4(int model)
 {
-    debug_gtk3("called with model %d.", model);
     video_model_widget_update(video_widget);
     plus4_memory_size_widget_sync();
     plus4_acia_widget_sync();
@@ -738,44 +806,58 @@ static void machine_model_handler_plus4(int model)
 
 /** \brief  Callback for the CBM-II 5x0 VIC-II model (sync factor)
  *
- * Calls model widget update
+ * Calls model widget update.
  *
  * \param[in]   model   new VIC-II model
  */
 static void cbm5x0_video_callback(int model)
 {
-    debug_gtk3("got video model %d.", model);
     machine_model_widget_update(machine_widget);
 }
 
 
 /** \brief  Callback for the CBM-II 6x0/7x0 CRTC model (sync factor)
  *
- * Calls model widget update
+ * Calls model widget update.
  *
  * \param[in]   model   new VIC-II model
  */
 static void cbm2_video_callback(int model)
 {
-    debug_gtk3("got video model %d.", model);
     machine_model_widget_update(machine_widget);
 }
 
 
+/** \brief  Callback for the CBM-II 6x0/7x0 ModelLine switches
+ *
+ * Calls model widget update.
+ *
+ * \param[in]   widget      switches widget (unused)
+ * \param[in]   model_line  new mode line value (unused)
+ */
 static void cbm2_switches_callback(GtkWidget *widget, int model_line)
 {
-    debug_gtk3("called with model_line %d.", model_line);
     machine_model_widget_update(machine_widget);
 }
 
 
+/** \brief  Callback for CBM-II 6x0/7x0 memory size changes
+ *
+ * Calls model widget update.
+ *
+ * \param[in]   widget  memory size widget (unused)
+ * \param[in]   size    new memory size (unused)
+ */
 static void cbm2_memory_size_callback(GtkWidget *widget, int size)
 {
-    debug_gtk3("called with memory size %d.", size);
     machine_model_widget_update(machine_widget);
 }
 
 
+/** \brief  Callback for CBM 5x0 model changes
+ *
+ * \param[in]   model   new model
+ */
 static void machine_model_handler_cbm5x0(int model)
 {
     video_model_widget_update(video_widget);
@@ -783,6 +865,10 @@ static void machine_model_handler_cbm5x0(int model)
 }
 
 
+/** \brief  Callback for CBM 6x0/7x0 model changes
+ *
+ * \param[in]   model   new model
+ */
 static void machine_model_handler_cbm6x0(int model)
 {
     video_model_widget_update(video_widget);
@@ -808,9 +894,12 @@ static void pet_set_ram9a_sensitivity(void)
 }
 
 
+/** \brief  Callback for PET model changes
+ *
+ * \param[in]   model   new model
+ */
 static void machine_model_handler_pet(int model)
 {
-    debug_gtk3("Called.");
     pet_ram_size_widget_sync(ram_widget);
     pet_video_size_widget_sync(pet_video_size_widget);
     pet_keyboard_type_widget_sync(pet_keyboard_widget);
@@ -822,15 +911,12 @@ static void machine_model_handler_pet(int model)
 }
 
 
-
 /** \brief  Generic callback for machine model changes
  *
  * \param[in]   model
  */
 static void machine_model_callback(int model)
 {
-    debug_gtk3("got model %d.", model);
-
     switch (machine_class) {
         case VICE_MACHINE_C64:      /* fall through */
         case VICE_MACHINE_C64SC:    /* fall through */
@@ -865,7 +951,7 @@ static void machine_model_callback(int model)
 }
 
 
-/** \brief  Handler for the "toggled" event of the C64 Glue Logic radio buttons
+/** \brief  Handler for the 'toggled' event of the C64 "Glue Logic" radio buttons
  *
  * \param[in]   widget      radio button triggering the event
  * \param[in]   user_data   glue value (int)
@@ -875,8 +961,6 @@ static void on_c64_glue_toggled(GtkWidget *widget, gpointer user_data)
     int glue = GPOINTER_TO_INT(user_data);
 
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
-        debug_gtk3("setting GlueLogic to %s.",
-                glue == 0 ? "discrete" : "252535-01");
         resources_set_int("GlueLogic", glue);
         machine_model_widget_update(machine_widget);
     }
@@ -893,6 +977,7 @@ static void c64_reset_with_iec_sync(void)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(reset_with_iec_widget), iecreset);
 }
 
+
 /** \brief  Create widget to toggle "Reset-to-IEC"
  *
  * \return  GtkGrid
@@ -907,6 +992,7 @@ static GtkWidget *create_reset_with_iec_widget(void)
     return reset_with_iec_widget;
 }
 
+
 /** \brief  Create widget to toggle "Go64Mode"
  *
  * \return  GtkGrid
@@ -918,8 +1004,6 @@ static GtkWidget *create_go64_widget(void)
 }
 
 
-GtkWidget *c64_discrete_radio = NULL;
-GtkWidget *c64_custom_radio = NULL;
 
 /** \brief  Sync "Glue Logic" widget with the associated resource
  *
@@ -990,11 +1074,7 @@ static GtkWidget *create_c64_misc_widget(void)
     GtkWidget *iec_widget;
     GtkWidget *glue_widget = NULL;
 
-#if 0
-    grid = uihelpers_create_grid_with_label("Miscellaneous", 1);
-#else
     grid = vice_gtk3_grid_new_spaced_with_label(-1, -1, "Miscellaneous", 1);
-#endif
     iec_widget = create_reset_with_iec_widget();
     g_object_set(iec_widget, "margin-left", 16, NULL);
     gtk_grid_attach(GTK_GRID(grid), iec_widget, 0, 1, 1, 1);
@@ -1099,7 +1179,7 @@ static GtkWidget *create_c64_layout(GtkWidget *grid)
     gtk_grid_attach(GTK_GRID(grid), sid_widget, 1, 1, 1, 1);
 
     /* CIA1 & CIA2 widget */
-    cia_widget = cia_model_widget_create(machine_widget, 2);
+    cia_widget = cia_model_widget_create(2);
     cia_model_widget_set_callback(cia_widget, cia_model_callback);
     gtk_grid_attach(GTK_GRID(grid), cia_widget, 0, 2, 2, 1);
 
@@ -1133,7 +1213,7 @@ static GtkWidget *create_c128_layout(GtkWidget *grid)
     GtkWidget *misc_widget;
 
     /* wrap machine model and machine type widgets in a single widget */
-    machine_wrapper = gtk_grid_new();
+    machine_wrapper = vice_gtk3_grid_new_spaced(0, 16);
 
     /* add machine model widget */
     gtk_grid_attach(GTK_GRID(machine_wrapper), machine_widget, 0, 0, 1, 1);
@@ -1145,7 +1225,7 @@ static GtkWidget *create_c128_layout(GtkWidget *grid)
     gtk_grid_attach(GTK_GRID(grid), machine_wrapper, 0, 0, 1, 1);
 
     /* wrap VIC-II, VDC and CIA1/2 in a single widget */
-    col2_wrapper = gtk_grid_new();
+    col2_wrapper = vice_gtk3_grid_new_spaced(0, 16);
 
     /* VIC-II model widget */
     video_widget = video_model_widget_create(machine_widget);
@@ -1154,16 +1234,14 @@ static GtkWidget *create_c128_layout(GtkWidget *grid)
 
     /* VDC model widget */
     vdc_widget = vdc_model_widget_create();
-    /* XXX: I'm sure I had/have a reason for this: */
-    vdc_model_widget_connect_signals(vdc_widget);
-    vdc_model_widget_set_revision_callback(vdc_widget, vdc_revision_callback);
-    vdc_model_widget_set_ram_callback(vdc_widget, vdc_ram_callback);
+    vdc_model_widget_set_revision_callback(vdc_revision_callback);
+    vdc_model_widget_set_ram_callback(vdc_ram_callback);
     /* align with other widgets */
     g_object_set(vdc_widget, "margin-left", 8, NULL);
     gtk_grid_attach(GTK_GRID(col2_wrapper), vdc_widget, 0, 1, 1, 1);
 
     /* CIA1 & CIA2 widget */
-    cia_widget = cia_model_widget_create(machine_widget, 2);
+    cia_widget = cia_model_widget_create(2);
     cia_model_widget_set_callback(cia_widget, cia_model_callback);
     gtk_grid_attach(GTK_GRID(col2_wrapper), cia_widget, 0, 2, 1, 1);
     gtk_grid_attach(GTK_GRID(grid), col2_wrapper, 1, 0, 1, 1);
@@ -1286,7 +1364,7 @@ static GtkWidget *create_plus4_layout(GtkWidget *grid)
     gtk_grid_attach(GTK_GRID(grid), ram_widget, 1, 1, 1, 1);
 
     resources_get_int("MemoryHack", &hack);
-    gtk_widget_set_sensitive(memhack_widget, hack == MEMORY_HACK_NONE);
+    gtk_widget_set_sensitive(ram_widget, hack == MEMORY_HACK_NONE);
 
     /* ACIA widget */
     acia_widget = plus4_acia_widget_create();
@@ -1305,7 +1383,9 @@ static GtkWidget *create_plus4_layout(GtkWidget *grid)
 
 /** \brief  Creat PET layout
  *
- * Uses a GtkStack and GtkStackSwitcher to reduce vertical space
+ * Create PET layout using a GtkStack and GtkStackSwitcher to reduce space.
+ *
+ * \param[in,out]   grid    main grid to add widgets to
  *
  * \return  \a grid
  */
@@ -1329,7 +1409,7 @@ static GtkWidget *create_pet_layout(GtkWidget *grid)
     gtk_grid_attach(GTK_GRID(pet_grid), pet_keyboard_widget, 1, 0, 1, 1);
 
     pet_video_size_widget = pet_video_size_widget_create();
-    pet_video_size_widget_set_callback(pet_video_size_widget, pet_video_size_callback);
+    pet_video_size_widget_set_callback(pet_video_size_callback);
     gtk_grid_attach(GTK_GRID(pet_grid), pet_video_size_widget, 1, 1, 1, 1);
 
     ram_widget = pet_ram_size_widget_create();
@@ -1337,21 +1417,22 @@ static GtkWidget *create_pet_layout(GtkWidget *grid)
     gtk_grid_attach(GTK_GRID(pet_grid), ram_widget, 2, 0, 1, 1);
 
     pet_io_widget = pet_io_size_widget_create();
-    pet_io_size_widget_set_callback(pet_io_widget, pet_io_callback);
+    pet_io_size_widget_set_callback(pet_io_callback);
     gtk_grid_attach(GTK_GRID(pet_grid), pet_io_widget, 2, 1, 1, 1);
 
     pet_ram9_widget = pet_ram9_widget_create();
-    pet_ram9_widget_set_callback(pet_ram9_widget, pet_ram9_callback);
+    pet_ram9_widget_set_callback(pet_ram9_callback);
     gtk_grid_attach(GTK_GRID(pet_grid), pet_ram9_widget, 3, 0, 1, 1);
 
     pet_rama_widget = pet_rama_widget_create();
-    pet_rama_widget_set_callback(pet_rama_widget, pet_rama_callback);
+    pet_rama_widget_set_callback(pet_rama_callback);
     pet_set_ram9a_sensitivity();
     gtk_grid_attach(GTK_GRID(pet_grid), pet_rama_widget, 3, 1, 1, 1);
 
     pet_misc_widget = pet_misc_widget_create();
-    pet_misc_widget_set_crtc_callback(pet_misc_widget, pet_crtc_callback);
-    pet_misc_widget_set_blank_callback(pet_misc_widget, pet_blank_callback);
+    pet_misc_widget_set_crtc_callback(pet_crtc_callback);
+    pet_misc_widget_set_blank_callback(pet_blank_callback);
+    pet_misc_widget_set_screen2001_callback(pet_screen2001_callback);
     gtk_grid_attach(GTK_GRID(pet_grid), pet_misc_widget, 1, 2, 2, 1);
 
     /* SuperPET widgets */
@@ -1380,7 +1461,6 @@ static GtkWidget *create_pet_layout(GtkWidget *grid)
     gtk_grid_attach(GTK_GRID(grid), stack, 0, 1, 1, 1);
 
     gtk_widget_show_all(grid);
-    debug_gtk3("Done creating PET model dialog");
     return grid;
 }
 
@@ -1410,7 +1490,7 @@ static GtkWidget *create_cbm5x0_layout(GtkWidget *grid)
     gtk_grid_attach(GTK_GRID(grid), sid_widget, 1, 1, 1, 1);
 
     /* CIA1 widget */
-    cia_widget = cia_model_widget_create(machine_widget, 1);
+    cia_widget = cia_model_widget_create(1);
     cia_model_widget_set_callback(cia_widget, cia_model_callback);
     gtk_grid_attach(GTK_GRID(grid), cia_widget, 2, 0, 1, 1);
 
@@ -1465,7 +1545,7 @@ static GtkWidget *create_cbm6x0_layout(GtkWidget *grid)
     gtk_grid_attach(GTK_GRID(grid), switches_widget, 2, 0, 1, 1);
 
     /* CIA1 widget */
-    cia_widget = cia_model_widget_create(machine_widget, 1);
+    cia_widget = cia_model_widget_create(1);
     cia_model_widget_set_callback(cia_widget, cia_model_callback);
     gtk_grid_attach(GTK_GRID(grid), cia_widget, 1, 1, 2, 1);
 
@@ -1512,6 +1592,9 @@ static GtkWidget *create_layout(void)
 {
     GtkWidget *grid = gtk_grid_new();
 
+    gtk_grid_set_column_spacing(GTK_GRID(grid), 16);
+    gtk_grid_set_row_spacing(GTK_GRID(grid), 16);
+
     /* Can't add the machine model widget here, since it differs a lot in size
      * depending on the machine, which may result in odd layouts. So we need
      * to determine per machine how many grid rows the machine model widget
@@ -1659,7 +1742,7 @@ void settings_model_widget_set_model_func(int (*func)(void))
  *
  * \param[in]   func    function to get a string for a memhack ID (int)
  */
-void settings_model_widget_set_memhack_func(const char *(func)(int))
+void settings_model_widget_set_memhack_func(const char *(*func)(int))
 {
     get_memhack_func = func;
 }
