@@ -4,15 +4,20 @@
  * \author  Bas Wassink <b.wassink@ziggo.nl>
  */
 
-/**
+/*
  * $VICERES Acia1Enable     x64 x64sc xscpu64 x128 xvic
  * $VICERES Acia1Dev        x64 x64sc xscpu64 x128 xvic xplus4 xcbm5x0 xcbm2
  * $VICERES Acia1Base       x64 x64sc xscpu64 x128 xvic
  * $VICERES Acia1Irq        x64 x64sc xscpu64 x128 xvic
  * $VICERES Acia1Mode       x64 x64sc xscpu64 x128 xvic
- * $VICERES RsUserEnable    x64 x64sc xscpu64 x128 xvic
+ * $VICERES UserportDevice  x64 x64sc xscpu64 x128 xvic
  * $VICERES RsUserBaud      x64 x64sc xscpu64 x128 xvic
  * $VICERES RsUserDev       x64 x64sc xscpu64 x128 xvic
+ * $VICERES RsUserUP9600     x64 x64sc xscpu64 x128 xvic
+ * $VICERES RsUserRTSInv    x64 x64sc xscpu64 x128 xvic
+ * $VICERES RsUserCTSInv    x64 x64sc xscpu64 x128 xvic
+ * $VICERES RsUserDSRInv    x64 x64sc xscpu64 x128 xvic
+ * $VICERES RsUserDCDInv    x64 x64sc xscpu64 x128 xvic
  * $VICERES RsDevice1       x64 x64sc xscpu64 x128 xvic xplus4 xcbm5x0 xcbm2
  * $VICERES RsDevice2       x64 x64sc xscpu64 x128 xvic xplus4 xcbm5x0 xcbm2
  * $VICERES RsDevice3       x64 x64sc xscpu64 x128 xvic xplus4 xcbm5x0 xcbm2
@@ -54,9 +59,12 @@
 #include "lib.h"
 #include "machine.h"
 #include "resources.h"
-
+#include "userport.h"
+#include "userportrsinterfacewidget.h"
 #include "settings_rs232.h"
 
+#include "rsuser.h"
+
 
 /** \brief  List of ACIA devices
  *
@@ -128,6 +136,8 @@ static const vice_gtk3_combo_entry_int_t rsuser_baud_rates[] = {
     { "2400",   2400 },
     { "4800",   4800 },
     { "9600",   9600 },
+    { "38400",  38400},
+    { "57600",  57600},
     { NULL, -1 }
 };
 
@@ -158,6 +168,66 @@ static const vice_gtk3_combo_entry_int_t serial_baud_rates_plus4[] = {
     { NULL, -1 }
 };
 
+/** \brief  Userport baud rate widget */
+static GtkWidget *rsuser_baud_widget;
+
+/** \brief  Extra event handler for the userport RS-232 interface type changes
+ *
+ * \param[in]   widget      Userport RS-232 interface type combobox
+ */
+static void on_rsinterface_changed(GtkWidget *widget)
+{
+    int up9600;
+
+    GtkWidget *baud;
+    GtkWidget *rts;
+    GtkWidget *cts;
+    GtkWidget *dcd;
+    GtkWidget *dsr;
+    GtkWidget *dtr;
+    GtkWidget *grid;
+    int index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+
+
+    resources_get_int("RsUserUP9600", &up9600);
+    baud = rsuser_baud_widget;
+    grid = gtk_widget_get_parent(widget);
+    rts = gtk_grid_get_child_at((GtkGrid*)grid, 1, 2);
+    cts = gtk_grid_get_child_at((GtkGrid*)grid, 2, 2);
+    dsr = gtk_grid_get_child_at((GtkGrid*)grid, 1, 3);
+    dcd = gtk_grid_get_child_at((GtkGrid*)grid, 2, 3);
+    dtr = gtk_grid_get_child_at((GtkGrid*)grid, 1, 4);
+
+    if (rts != NULL && cts != NULL && dcd != NULL && dsr != NULL && dtr != NULL) {
+        switch (index) {
+            case USERPORT_RS_NONINVERTED:
+                vice_gtk3_resource_check_button_set(rts, false);
+                vice_gtk3_resource_check_button_set(cts, false);
+                /*vice_gtk3_resource_check_button_set(dcd, false);*/
+                vice_gtk3_resource_check_button_set(dsr, false);
+                vice_gtk3_resource_check_button_set(dtr, false);
+                resources_set_int("RsUserUP9600", 0);
+                break;
+            case USERPORT_RS_INVERTED:
+                vice_gtk3_resource_check_button_set(rts, true);
+                vice_gtk3_resource_check_button_set(cts, true);
+                /*vice_gtk3_resource_check_button_set(dcd, true);*/
+                vice_gtk3_resource_check_button_set(dsr, true);
+                vice_gtk3_resource_check_button_set(dtr, true);
+                resources_set_int("RsUserUP9600", 0);
+                break;
+            case USERPORT_RS_UP9600:
+                if (baud != NULL) {
+                    vice_gtk3_resource_combo_box_int_set(baud, 9600);
+                }
+                resources_set_int("RsUserUP9600", 1);
+                break;
+            default:
+                resources_set_int("RsUserUP9600", 0);
+                break;
+        }
+    }
+}
 
 
 /** \brief  Helper: create left-aligned, 16 px indented label
@@ -397,13 +467,62 @@ static GtkWidget *create_acia_widget(void)
         row++;
     }
 
-
     gtk_widget_show_all(grid);
     return grid;
 }
 
 
-/** \brief  Create user-port RS233 emulation settings widget
+
+/** \brief  Handler for the 'toggled' event of the userport enable checkbox
+ *
+ * Set userport device to USERPORT_DEVICE_RS232 or USERPORT_DEVICE_NONE.
+ *
+ * \param[in]   self    userport enable check button
+ * \param[in]   data    extra event data (unused)
+ */
+static void on_userport_enable_toggled(GtkWidget *self, gpointer data)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self))) {
+        /* set global userport device to rs232 modem */
+        resources_set_int("UserportDevice", USERPORT_DEVICE_RS232_MODEM);
+    } else {
+        /* set global userport device to none */
+        resources_set_int("UserportDevice", USERPORT_DEVICE_NONE);
+    }
+}
+
+
+/** \brief  Create toggle button to enable the userport rs232 device
+ *
+ * Set userport device to USERPORT_DEVICE_RS232 or USERPORT_DEVICE_NONE.
+ *
+ * The old resource "RsUserEnable" has been removed in the new userport system
+ * and we now set/unset the global "UserportDevice" resource.
+ *
+ *
+ * \return  GtkCheckButton
+ */
+static GtkWidget *create_userport_enable_widget(void)
+{
+    GtkWidget *check;
+    int device;
+
+    if (resources_get_int("UserportDevice", &device) < 0) {
+        device = USERPORT_DEVICE_NONE;
+    }
+
+    check = gtk_check_button_new_with_label("Enable userport RS232 emulation");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
+                                 device == USERPORT_DEVICE_RS232_MODEM);
+    g_signal_connect(check,
+                     "toggled",
+                     G_CALLBACK(on_userport_enable_toggled),
+                     NULL);
+    return check;
+}
+
+
+/** \brief  Create user-port RS232 emulation settings widget
  *
  * \note    C64,C128 and VIC-20 only
  *
@@ -414,8 +533,9 @@ static GtkWidget *create_userport_widget(void)
     GtkWidget *grid;
     GtkWidget *label;
     GtkWidget *rsuser_enable_widget;
-    GtkWidget *rsuser_baud_widget;
     GtkWidget *rsuser_device_widget;
+    GtkWidget *rsuser_rsinterface_widget;
+    int up9600;
 
     grid = vice_gtk3_grid_new_spaced(VICE_GTK3_DEFAULT, VICE_GTK3_DEFAULT);
     label = gtk_label_new(NULL);
@@ -423,32 +543,38 @@ static GtkWidget *create_userport_widget(void)
     gtk_widget_set_halign(label, GTK_ALIGN_START);
     gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 4, 1);
 
-    rsuser_enable_widget = vice_gtk3_resource_check_button_new(
-            "RsUserEnable", "Enable Userport RS232 emulation");
+    rsuser_enable_widget = create_userport_enable_widget();
     gtk_widget_set_halign(rsuser_enable_widget, GTK_ALIGN_START);
     g_object_set(rsuser_enable_widget, "margin-left", 16, NULL);
     gtk_grid_attach(GTK_GRID(grid), rsuser_enable_widget, 0, 1, 4, 1);
 
+    /* RS-232 Interface Widget */
+    rsuser_rsinterface_widget = userport_rsinterface_widget_create();
+    gtk_grid_attach(GTK_GRID(grid), rsuser_rsinterface_widget, 0, 2, 4, 1);
+
     label = create_indented_label("Device");
     rsuser_device_widget = vice_gtk3_resource_combo_box_int_new(
             "RsUserDev", acia_devices);
-    gtk_grid_attach(GTK_GRID(grid), label, 0, 2, 1, 1);
-    gtk_grid_attach(GTK_GRID(grid), rsuser_device_widget, 1, 2, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), label, 0, 3, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), rsuser_device_widget, 1, 3, 1, 1);
     label = create_indented_label("Baud");
     rsuser_baud_widget = vice_gtk3_resource_combo_box_int_new(
             "RsUserBaud", rsuser_baud_rates);
-    gtk_grid_attach(GTK_GRID(grid), label, 2, 2, 1, 1);
-    gtk_grid_attach(GTK_GRID(grid), rsuser_baud_widget, 3, 2, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), label, 2, 3, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), rsuser_baud_widget, 3, 3, 1, 1);
+    resources_get_int("RsUserUP9600", &up9600);
 
     gtk_widget_show_all(grid);
+    userport_rsinterface_widget_add_callback((GtkGrid*)rsuser_rsinterface_widget, on_rsinterface_changed);
     return grid;
 }
 
 
 /** \brief  Create RS232 devices widget
  *
+ * XXX: only supports Unix, Windows appears do things differently.
  *
- * XXX: only supports Unix, Windows appears do things differently
+ * \return  GtkGrid
  */
 static GtkWidget *create_rs232_devices_widget(void)
 {
@@ -486,7 +612,7 @@ static GtkWidget *create_rs232_devices_widget(void)
     gtk_grid_attach(GTK_GRID(grid), label, 2, 1, 1, 1);
     gtk_grid_attach(GTK_GRID(grid), ser1_baud_widget, 3, 1, 1, 1);
     ser1_ip232_widget = vice_gtk3_resource_check_button_new(
-            "RsDevice1ip232", "IP232");    
+            "RsDevice1ip232", "IP232");
     gtk_grid_attach(GTK_GRID(grid), ser1_ip232_widget, 4, 1, 1, 1);
 
     label = create_indented_label("Serial 2");
@@ -500,7 +626,7 @@ static GtkWidget *create_rs232_devices_widget(void)
     gtk_grid_attach(GTK_GRID(grid), label, 2, 2, 1, 1);
     gtk_grid_attach(GTK_GRID(grid), ser2_baud_widget, 3, 2, 1, 1);
     ser2_ip232_widget = vice_gtk3_resource_check_button_new(
-            "RsDevice2ip232", "IP232");    
+            "RsDevice2ip232", "IP232");
     gtk_grid_attach(GTK_GRID(grid), ser2_ip232_widget, 4, 2, 1, 1);
 
     label = create_indented_label("Serial 3");
@@ -514,7 +640,7 @@ static GtkWidget *create_rs232_devices_widget(void)
     gtk_grid_attach(GTK_GRID(grid), label, 2, 3, 1, 1);
     gtk_grid_attach(GTK_GRID(grid), ser3_baud_widget, 3, 3, 1, 1);
     ser3_ip232_widget = vice_gtk3_resource_check_button_new(
-            "RsDevice3ip232", "IP232");    
+            "RsDevice3ip232", "IP232");
     gtk_grid_attach(GTK_GRID(grid), ser3_ip232_widget, 4, 3, 1, 1);
 
     label = create_indented_label("Serial 4");
@@ -528,9 +654,9 @@ static GtkWidget *create_rs232_devices_widget(void)
     gtk_grid_attach(GTK_GRID(grid), label, 2, 4, 1, 1);
     gtk_grid_attach(GTK_GRID(grid), ser4_baud_widget, 3, 4, 1, 1);
     ser4_ip232_widget = vice_gtk3_resource_check_button_new(
-            "RsDevice4ip232", "IP232");    
+            "RsDevice4ip232", "IP232");
     gtk_grid_attach(GTK_GRID(grid), ser4_ip232_widget, 4, 4, 1, 1);
-    
+
     gtk_widget_show_all(grid);
     return grid;
 }
@@ -540,7 +666,7 @@ static GtkWidget *create_rs232_devices_widget(void)
  *
  * Invalid for PET, C64DTV and VSID
  *
- * \param[in]   parent  parent widget
+ * \param[in]   parent  parent widget (unused)
  *
  * \return  GtkGrid
  */
@@ -593,4 +719,3 @@ GtkWidget *settings_rs232_widget_create(GtkWidget *parent)
 
     return grid;
 }
-
