@@ -57,28 +57,48 @@ static const vice_gtk3_radiogroup_entry_t recstart_modes[] = {
 static GtkWidget *histdir_entry;
 
 
-/** \brief  Handler for the "clicked" event of the "browse" button
+/** \brief  Callback for the directory-select dialog
  *
- * \param[in]   widget      widget triggering the event
- * \param[in]   user_data   extra event data (unused)
+ * \param[in,out]   dialog      directory-select dialog
+ * \param[in,out]   filename    filename (NULL if canceled)
+ * \param[in]       param       extra data (unused)
+ *
+ * \todo    Replace with resourcebrowser
  */
-static void on_histdir_browse_clicked(GtkWidget *widget, gpointer user_data)
+static void histdir_browse_callback(GtkDialog *dialog,
+                                    gchar *filename,
+                                    gpointer param)
 {
-    char *filename;
-    const char *current;
-
-    if (resources_get_string("EventSnapshotDir", &current) < 0) {
-        debug_gtk3("failed to get current history directory, using NULL.");
-        current = NULL;
-    }
-
-    filename = vice_gtk3_select_directory_dialog(
-            "Select history directory", NULL, TRUE, current);
     if (filename != NULL) {
-        debug_gtk3("Setting EventSnapshotDir to '%s'.", filename);
         vice_gtk3_resource_entry_full_set(histdir_entry, filename);
         g_free(filename);
     }
+    gtk_widget_destroy(GTK_WIDGET(dialog));
+}
+
+
+/** \brief  Handler for the 'clicked' event of the "browse" button
+ *
+ * \param[in]   widget      widget triggering the event
+ * \param[in]   user_data   extra event data (unused)
+ *
+ * \todo    Replace with resourcebrowser
+ */
+static void on_histdir_browse_clicked(GtkWidget *widget, gpointer user_data)
+{
+    GtkWidget *dialog;
+    const char *current;
+
+    resources_get_string("EventSnapshotDir", &current);
+
+    dialog = vice_gtk3_select_directory_dialog(
+            "Select history directory",
+            NULL,
+            TRUE,
+            current,
+            histdir_browse_callback,
+            NULL);
+    gtk_widget_show(dialog);
 }
 
 
@@ -87,6 +107,8 @@ static void on_histdir_browse_clicked(GtkWidget *widget, gpointer user_data)
  * \param[in]   parent  parent widget
  *
  * \return  GtkGrid
+ *
+ * \todo    Use resourcebrowser to control "EventSnapshotDir" resource
  */
 GtkWidget *settings_snapshot_widget_create(GtkWidget *parent)
 {
@@ -96,9 +118,7 @@ GtkWidget *settings_snapshot_widget_create(GtkWidget *parent)
     GtkWidget *histdir_browse;
     GtkWidget *recmode_widget;
 
-    grid = gtk_grid_new();
-    gtk_grid_set_column_spacing(GTK_GRID(grid), 16);
-    gtk_grid_set_row_spacing(GTK_GRID(grid), 8);
+    grid = vice_gtk3_grid_new_spaced(VICE_GTK3_DEFAULT, VICE_GTK3_DEFAULT);
 
     label = gtk_label_new("History directory");
     gtk_widget_set_halign(label, GTK_ALIGN_START);
