@@ -36,8 +36,7 @@
 
 #include <gtk/gtk.h>
 
-#include "widgethelpers.h"
-#include "debug_gtk3.h"
+#include "vice_gtk3.h"
 #include "resources.h"
 #include "machine.h"
 #include "log.h"
@@ -45,13 +44,29 @@
 #include "videomodelwidget.h"
 
 
+/** \brief  Title for the widget
+ *
+ * Set once via ${emu}ui.c during UI initialization.
+ */
 static const char *widget_title = NULL;
+
+/** \brief  Name of the video model resource
+ *
+ * Set once via ${emu}ui.c during UI initialization.
+ */
 static const char *resource_name = NULL;
+
+/** \brief  List of video models
+ *
+ * Set once via ${emu}ui.c during UI initialization.
+ */
 static const vice_gtk3_radiogroup_entry_t *model_list = NULL;
-static GtkWidget *machine_widget = NULL;
+
 
 
 /** \brief  Get index in model list for model-ID \a model
+ *
+ * \param[in]   model   model index
  *
  * \return  index in list or -1 when not found
  */
@@ -69,13 +84,8 @@ static int get_model_index(int model)
 static void on_model_toggled(GtkWidget *widget, gpointer user_data)
 {
     int model_id = GPOINTER_TO_INT(user_data);
-#if 0
-    debug_gtk3("got model ID %d.", model_id);
-#endif
+
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
-#if 0
-        debug_gtk3("setting '%s' to %d.", resource_name, model_id);
-#endif
         if (resources_set_int(resource_name, model_id) < 0) {
             log_error(LOG_ERR, "failed to set %s to %d\n",
                     resource_name, model_id);
@@ -87,9 +97,6 @@ static void on_model_toggled(GtkWidget *widget, gpointer user_data)
             if (parent != NULL) {
                 callback = g_object_get_data(G_OBJECT(parent), "ExtraCallback");
                 if (callback != NULL) {
-#if 0
-                    debug_gtk3("triggering extra callback with %d.", model_id);
-#endif
                     callback(model_id);
                 }
             }
@@ -99,6 +106,8 @@ static void on_model_toggled(GtkWidget *widget, gpointer user_data)
 
 
 /** \brief  Set title of the widget
+ *
+ * Called from the ${emu}ui.c file.
  *
  * \param[in]   title   title for the widget
  */
@@ -143,9 +152,13 @@ GtkWidget *video_model_widget_create(GtkWidget *machine)
     GtkRadioButton *last = NULL;
     GSList *group = NULL;
     int i;
+    GtkWidget *title;
 
-    machine_widget = machine;
-    grid = uihelpers_create_grid_with_label(widget_title, 1);
+    grid = vice_gtk3_grid_new_spaced_with_label(
+            VICE_GTK3_DEFAULT, 0,
+            widget_title, 1);
+    title = gtk_grid_get_child_at(GTK_GRID(grid), 0, 0);
+    g_object_set(title, "margin-bottom", 8, NULL);
 
     if (model_list != NULL) {
         for (i = 0; model_list[i].name != NULL; i++) {
@@ -181,9 +194,7 @@ void video_model_widget_update(GtkWidget *widget)
 
     resources_get_int(resource_name, &model_id);
     index = get_model_index(model_id);
-#if 0
-    debug_gtk3("got resource %d, index %d.", model_id, index);
-#endif
+
     while ((radio = gtk_grid_get_child_at(GTK_GRID(widget), 0, i + 1)) != NULL) {
         if (GTK_IS_RADIO_BUTTON(radio) && (index == i)) {
             /* don't update radio button when it's already active, inhibiting
@@ -209,9 +220,6 @@ void video_model_widget_connect_signals(GtkWidget *widget)
 
     while ((radio = gtk_grid_get_child_at(GTK_GRID(widget), 0, i + 1)) != NULL) {
         if (GTK_IS_RADIO_BUTTON(radio)) {
-#if 0
-            debug_gtk3("adding model ID %d.", model_list[i].id);
-#endif
             g_signal_connect(radio, "toggled", G_CALLBACK(on_model_toggled),
                     GINT_TO_POINTER(model_list[i].id));
         } else {
