@@ -31,6 +31,9 @@
 
 #include <gtk/gtk.h>
 
-GtkWidget *vsid_playlist_widget_create(void);
+GtkWidget * vsid_playlist_widget_create(void);
+
+gboolean    vsid_playlist_widget_append_file(const gchar *path);
+void        vsid_playlist_widget_remove_file(int row);
 #endif
 
