@@ -78,8 +78,8 @@ static const cmdline_option_t cmdline_options_common[] =
 };
 
 
-// /** \brief  Flag indicating pause mode
-//  */
+/** \brief  Flag indicating pause mode
+ */
 static int is_paused = 0;
 
 
@@ -90,19 +90,19 @@ static int is_paused = 0;
 void fullscreen_capability(struct cap_fullscreen_s *cap_fullscreen)
 {
     /* printf("%s\n", __func__); */
-    
+
     return;
 }
 
 
-// /** \brief  Initialize command line options (generic)
-//  *
-//  * \return  0 on success, -1 on failure
-//  */
+/** \brief  Initialize command line options (generic)
+ *
+ * \return  0 on success, -1 on failure
+ */
 int ui_cmdline_options_init(void)
 {
     /* printf("%s\n", __func__); */
-    
+
     return cmdline_register_options(cmdline_options_common);
 }
 
@@ -121,7 +121,7 @@ int ui_cmdline_options_init(void)
 char *ui_get_file(const char *format, ...)
 {
     /* printf("%s\n", __func__); */
-    
+
     /*
      * Also not called when trying to play back events, at least, I've never
      * seen this called.
@@ -135,10 +135,19 @@ char *ui_get_file(const char *format, ...)
  *
  * \param[in]   argc    pointer to main()'s argc
  * \param[in]   argv    main()'s argv
+ *
+ */
+void ui_init_with_args(int *argc, char **argv)
+{
+    printf("Initialising headless ui with args\n");
+}
+
+
+/** \brief  Initialize Gtk3/GLib
  *
  * \return  0 on success, -1 on failure
  */
-int ui_init(int *argc, char **argv)
+int ui_init(void)
 {
     printf("Initialising headless ui\n");
 
@@ -157,7 +166,7 @@ int ui_init(int *argc, char **argv)
 int ui_init_finish(void)
 {
     /* printf("%s\n", __func__); */
-    
+
     return 0;
 }
 
@@ -175,7 +184,7 @@ int ui_init_finish(void)
 int ui_init_finalize(void)
 {
     /* printf("%s\n", __func__); */
-    
+
     return 0;
 }
 
@@ -215,7 +224,7 @@ ui_jam_action_t ui_jam_dialog(const char *format, ...)
 int ui_resources_init(void)
 {
     /* printf("%s\n", __func__); */
-    
+
     return 0;
 }
 
@@ -253,7 +262,7 @@ void ui_dispatch_events(void)
 int ui_extend_image_dialog(void)
 {
     /* printf("%s\n", __func__); */
-    
+
     /* FIXME: this dialog needs to be implemented. */
     NOT_IMPLEMENTED();
     return 0;
@@ -267,7 +276,7 @@ int ui_extend_image_dialog(void)
 void ui_error(const char *format, ...)
 {
     /* printf("%s\n", __func__); */
-    
+
     char *buffer;
     va_list ap;
 
@@ -287,7 +296,7 @@ void ui_error(const char *format, ...)
 void ui_message(const char *format, ...)
 {
     /* printf("%s\n", __func__); */
-    
+
     char *buffer;
     va_list ap;
 
@@ -300,6 +309,17 @@ void ui_message(const char *format, ...)
 }
 
 
+bool ui_pause_loop_iteration(void)
+{
+    /* printf("%s\n", __func__); */
+    /*
+    ui_dispatch_next_event();
+    g_usleep(10000);
+    */
+    return is_paused;
+}   
+
+
 /** \brief  Keeps the ui events going while the emulation is paused
  *
  * \param[in]   addr    unused
@@ -308,13 +328,12 @@ void ui_message(const char *format, ...)
 static void pause_trap(uint16_t addr, void *data)
 {
     /* printf("%s\n", __func__); */
-    
-    // vsync_suspend_speed_eval();
-    // sound_suspend();
-    // while (is_paused) {
-    //     ui_dispatch_next_event();
-    //     g_usleep(10000);
-    // }
+/*
+    vsync_suspend_speed_eval();
+    sound_suspend();
+ 
+    while (ui_pause_loop_iteration());
+*/
 }
 
 
@@ -325,7 +344,7 @@ static void pause_trap(uint16_t addr, void *data)
 int ui_pause_active(void)
 {
     /* printf("%s\n", __func__); */
-    
+
     return is_paused;
 }
 
@@ -335,7 +354,7 @@ int ui_pause_active(void)
 void ui_pause_enable(void)
 {
     /* printf("%s\n", __func__); */
-    
+
     if (!ui_pause_active()) {
         is_paused = 1;
         interrupt_maincpu_trigger_trap(pause_trap, 0);
@@ -348,7 +367,7 @@ void ui_pause_enable(void)
 void ui_pause_disable(void)
 {
     /* printf("%s\n", __func__); */
-    
+
     if (ui_pause_active()) {
         is_paused = 0;
     }
@@ -361,3 +380,10 @@ void ui_update_lightpen(void)
 {
     /* printf("%s\n", __func__); */
 }
+
+
+/* FIXME: temporary hack to allow using ui_hotkeys_init() from src/main.c */
+void ui_hotkeys_init(void)
+{
+    /* NOP */
+}
