@@ -57,12 +57,19 @@ extern void sdljoy_set_hotkey(SDL_Event e, ui_menu_entry_t *value);
 extern void sdljoy_set_keypress(SDL_Event e, int row, int col);
 extern void sdljoy_set_extra(SDL_Event e, int type);
 extern void sdljoy_unset(SDL_Event e);
+extern void sdljoy_set_joystick_axis(SDL_Event e, int port, int pot);
+extern char *get_joy_pin_mapping_string(int joy, int pin);
+extern char *get_joy_pot_mapping_string(int joynr, int pot);
+extern void sdljoy_delete_pin_mapping(int port, int pin);
+extern void sdljoy_delete_pot_mapping(int port, int pot);
+extern int sdljoy_get_joynum_for_event(Uint8 event_device_id);
+extern int sdljoy_rescan(void);
+extern void sdljoy_clear_presses(void);
+
 #endif
 
 extern void sdljoy_swap_ports(void);
-extern void sdljoy_swap_userport_ports(void);
 extern int sdljoy_get_swap_ports(void) ;
-extern int sdljoy_get_swap_userport_ports(void) ;
 
 #define JOYDEV_NONE     0
 #define JOYDEV_NUMPAD   1
