@@ -267,7 +267,7 @@ SDLKey SDL1x_to_SDL2x_Keys(SDLKey key)
 static inline int sdlkbd_key_mod_to_index(SDLKey key, SDLMod mod)
 {
     int i = 0;
-    
+
     if (mod & KMOD_SHIFT) {
         i |= (1 << 0);
     }
@@ -362,7 +362,7 @@ int sdlkbd_hotkeys_load(const char *filename)
         return -1;
     }
 
-    fp = sysfile_open(filename, &complete_path, MODE_READ_TEXT);
+    fp = sysfile_open(filename, machine_name, &complete_path, MODE_READ_TEXT);
 
     if (fp == NULL) {
         log_warning(sdlkbd_log, "Failed to open `%s'.", filename);
@@ -570,7 +570,7 @@ ui_menu_action_t sdlkbd_release(SDLKey key, SDLMod mod)
 void kbd_arch_init(void)
 {
 #ifdef SDL_DEBUG
-    fprintf(stderr, "%s: hotkey table size %u (%lu bytes)\n", __func__, SDLKBD_UI_HOTKEYS_MAX, SDLKBD_UI_HOTKEYS_MAX * sizeof(ui_menu_entry_t *));
+    fprintf(stderr, "%s: hotkey table size %d (%lu bytes)\n", __func__, SDLKBD_UI_HOTKEYS_MAX, SDLKBD_UI_HOTKEYS_MAX * sizeof(ui_menu_entry_t *));
 #endif
 
     sdlkbd_log = log_open("SDLKeyboard");
@@ -578,9 +578,9 @@ void kbd_arch_init(void)
     sdlkbd_keyword_clear();
     /* first load the defaults, then patch them with the user defined hotkeys */
     if (machine_class == VICE_MACHINE_VSID) {
-        sdlkbd_hotkeys_load("sdl_hotkeys_vsid.vkm");
+        sdlkbd_hotkeys_load("sdl-hotkeys-vsid.vhk");
     } else {
-        sdlkbd_hotkeys_load("sdl_hotkeys.vkm");
+        sdlkbd_hotkeys_load("sdl-hotkeys.vhk");
     }
     sdlkbd_hotkeys_load(hotkey_file);
 }
@@ -620,7 +620,96 @@ void kbd_initialize_numpad_joykeys(int* joykeys)
     joykeys[10] = SDL2x_to_SDL1x_Keys(SDLK_KP_ENTER);
 }
 
-const char *kbd_get_menu_keyname(void)
+static char *kbd_get_full_keyname(int mod_key, int key)
 {
-    return SDL_GetKeyName(SDL1x_to_SDL2x_Keys(sdl_ui_menukeys[0]));
+    char *mod_key_string = NULL;
+    char *retval = NULL;
+
+    switch (mod_key) {
+        case 1:
+            mod_key_string = "Shift+";
+            break;
+        case 2:
+            mod_key_string = "Alt+";
+            break;
+        case 3:
+            mod_key_string = "Alt+Shift+";
+            break;
+        case 4:
+            mod_key_string = "Ctrl+";
+            break;
+        case 5:
+            mod_key_string = "Ctrl+Shift+";
+            break;
+        case 6:
+            mod_key_string = "Ctrl+Alt+";
+            break;
+        case 7:
+            mod_key_string = "Ctrl+Alt+Shift+";
+            break;
+        case 8:
+            mod_key_string = "Meta+";
+            break;
+        case 9:
+            mod_key_string = "Meta+Shift+";
+            break;
+        case 10:
+            mod_key_string = "Meta+Alt+";
+            break;
+        case 11:
+            mod_key_string = "Meta+Alt+Shift+";
+            break;
+        case 12:
+            mod_key_string = "Meta+Ctrl+";
+            break;
+        case 13:
+            mod_key_string = "Meta+Ctrl+Shift+";
+            break;
+        case 14:
+            mod_key_string = "Meta+Ctrl+Alt+";
+            break;
+        case 15:
+            mod_key_string = "Meta+Ctrl+Alt+Shift+";
+            break;
+    }
+    if (mod_key_string != NULL) {
+        retval = util_concat(mod_key_string, SDL_GetKeyName(SDL1x_to_SDL2x_Keys(key)), NULL);
+    } else {
+        retval = lib_strdup(SDL_GetKeyName(SDL1x_to_SDL2x_Keys(key)));
+    }
+    return retval;
+}
+
+char *kbd_get_menu_keyname(void)
+{
+    int mod_key = (sdl_ui_menukeys[0] / SDL_NUM_SCANCODES);
+    int key = sdl_ui_menukeys[0] - (mod_key * SDL_NUM_SCANCODES);
+
+    return kbd_get_full_keyname(mod_key, key);
+}
+
+char *kbd_get_path_keyname(char *path)
+{
+    char *hotkey_path;
+    int i;
+    int mod_key = 0;
+    int key = 0;
+    int found = 0;
+    char *retval = NULL;
+
+    for (i = 0; i < SDLKBD_UI_HOTKEYS_MAX; ++i) {
+        if (sdlkbd_ui_hotkeys[i]) {
+            hotkey_path = sdl_ui_hotkey_path(sdlkbd_ui_hotkeys[i]);
+            if (!strcmp(hotkey_path, path)) {
+                mod_key = (i / SDL_NUM_SCANCODES);
+                key = i - (mod_key * SDL_NUM_SCANCODES);
+                found++;
+            }
+        }
+    }
+
+    if (found) {
+        retval = kbd_get_full_keyname(mod_key, key);
+    }
+    return retval;
 }
