@@ -55,7 +55,7 @@ extern void kbd_initialize_numpad_joykeys(int *joykeys);
 #define KMOD_META KMOD_GUI
 #define SDLK_LMETA SDLK_LGUI
 #define SDLK_RMETA SDLK_RGUI
-#define SDLK_COMPOSE	SDLK_APPLICATION
+#define SDLK_COMPOSE SDLK_APPLICATION
 #define SDLK_PRINT SDLK_PRINTSCREEN
 #define SDLK_NUMLOCK SDLK_NUMLOCKCLEAR
 #define SDLK_SCROLLOCK SDLK_SCROLLLOCK
@@ -95,6 +95,8 @@ extern void kbd_focus_change(void);
 
 extern int sdl_ui_menukeys[];
 
-extern const char *kbd_get_menu_keyname(void);
+extern char *kbd_get_menu_keyname(void);
+
+extern char *kbd_get_path_keyname(char *path);
 
 #endif
