@@ -42,7 +42,6 @@
 UI_MENU_DEFINE_TOGGLE(Sound)
 UI_MENU_DEFINE_RADIO(SoundSampleRate)
 UI_MENU_DEFINE_RADIO(SoundFragmentSize)
-UI_MENU_DEFINE_RADIO(SoundSpeedAdjustment)
 UI_MENU_DEFINE_RADIO(SoundDeviceName)
 UI_MENU_DEFINE_RADIO(SoundOutput)
 
@@ -77,7 +76,7 @@ static UI_MENU_CALLBACK(custom_buffer_size_callback)
         sprintf(buf, "%i", previous);
         value = sdl_ui_text_input_dialog("Enter buffer size in msec", buf);
         if (value) {
-            new_value = strtol(value, NULL, 0);
+            new_value = (int)strtol(value, NULL, 0);
             if (new_value != previous) {
                 resources_set_int("SoundBufferSize", new_value);
             }
@@ -102,7 +101,7 @@ static UI_MENU_CALLBACK(custom_frequency_callback)
         sprintf(buf, "%i", previous);
         value = sdl_ui_text_input_dialog("Enter frequency in Hz", buf);
         if (value) {
-            new_value = strtol(value, NULL, 0);
+            new_value = (int)strtol(value, NULL, 0);
             if (new_value != previous) {
                 resources_set_int("SoundSampleRate", new_value);
             }
@@ -154,12 +153,6 @@ static ui_menu_entry_t sound_output_driver_menu[] = {
       radio_SoundDeviceName_callback,
       (ui_callback_data_t)"ahi" },
 #endif
-#if defined(__MSDOS__) && !defined(USE_MIDAS_SOUND)
-    { "Allegro",
-      MENU_ENTRY_RESOURCE_RADIO,
-      radio_SoundDeviceName_callback,
-      (ui_callback_data_t)"allegro" },
-#endif
 #ifdef USE_ALSA
     { "ALSA",
       MENU_ENTRY_RESOURCE_RADIO,
@@ -181,12 +174,6 @@ static ui_menu_entry_t sound_output_driver_menu[] = {
       MENU_ENTRY_RESOURCE_RADIO,
       radio_SoundDeviceName_callback,
       (ui_callback_data_t)"coreaudio" },
-#endif
-#ifdef __OS2__
-    { "OS/2",
-      MENU_ENTRY_RESOURCE_RADIO,
-      radio_SoundDeviceName_callback,
-      (ui_callback_data_t)"dart" },
 #endif
     { "Dummy",
       MENU_ENTRY_RESOURCE_RADIO,
@@ -198,12 +185,6 @@ static ui_menu_entry_t sound_output_driver_menu[] = {
       radio_SoundDeviceName_callback,
       (ui_callback_data_t)"dx" },
 #endif
-#if defined(__MSDOS__) && defined(USE_MIDAS_SOUND)
-    { "Midas",
-      MENU_ENTRY_RESOURCE_RADIO,
-      radio_SoundDeviceName_callback,
-      (ui_callback_data_t)"midas" },
-#endif
 #ifdef USE_PULSE
     { "PulseAudio",
       MENU_ENTRY_RESOURCE_RADIO,
@@ -386,20 +367,6 @@ const ui_menu_entry_t sound_output_menu[] = {
       MENU_ENTRY_DIALOG,
       custom_frequency_callback,
       NULL },
-    SDL_MENU_ITEM_SEPARATOR,
-    SDL_MENU_ITEM_TITLE("Synchronization method"),
-    { "Flexible",
-      MENU_ENTRY_RESOURCE_RADIO,
-      radio_SoundSpeedAdjustment_callback,
-      (ui_callback_data_t)SOUND_ADJUST_FLEXIBLE },
-    { "Adjusting",
-      MENU_ENTRY_RESOURCE_RADIO,
-      radio_SoundSpeedAdjustment_callback,
-      (ui_callback_data_t)SOUND_ADJUST_ADJUSTING },
-    { "Exact",
-      MENU_ENTRY_RESOURCE_RADIO,
-      radio_SoundSpeedAdjustment_callback,
-      (ui_callback_data_t)SOUND_ADJUST_EXACT },
 
     SDL_MENU_LIST_END
 };
