@@ -44,13 +44,10 @@ void mousedrv_mouse_changed(void)
     ui_check_mouse_cursor();
 }
 
-int mousedrv_resources_init(mouse_func_t *funcs)
+int mousedrv_resources_init(const mouse_func_t *funcs)
 {
-    mouse_funcs.mbl = funcs->mbl;
-    mouse_funcs.mbr = funcs->mbr;
-    mouse_funcs.mbm = funcs->mbm;
-    mouse_funcs.mbu = funcs->mbu;
-    mouse_funcs.mbd = funcs->mbd;
+    /* Copy entire 'mouse_func_t' structure. */
+    mouse_funcs = *funcs;
     return 0;
 }
 
