@@ -48,7 +48,7 @@
 static int msg_mode_buttons[] = {
     1,  /* MESSAGE_OK */
     2,  /* MESSAGE_YESNO */
-    3,  /* MESSAGE_CPUJAM */
+    5,  /* MESSAGE_CPUJAM */
     5,  /* MESSAGE_UNIT_SELECT */
 };
 
@@ -96,7 +96,7 @@ static int handle_message_box(const char *title, const char *message, int messag
                         UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_TOPRIGHT_STRING, 2);
 
     /* make sure that the title length is not more than 28 chars. */
-    len = strlen(title);
+    len = (unsigned int)strlen(title);
     assert(len <= MAX_MSGBOX_LEN);
 
     /* calculate the position in the template to copy the title to. */
@@ -195,9 +195,13 @@ static int handle_message_box(const char *title, const char *message, int messag
             break;
         case MESSAGE_CPUJAM:
         default:
-            sdl_ui_print_center(UIFONT_VERTICAL_STRING " " UIFONT_TOPLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_TOPRIGHT_STRING "  " UIFONT_TOPLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_TOPRIGHT_STRING "  " UIFONT_TOPLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_TOPRIGHT_STRING " " UIFONT_VERTICAL_STRING, j + 1);
-            x = sdl_ui_print_center(UIFONT_VERTICAL_STRING " " UIFONT_VERTICAL_STRING "RESET" UIFONT_VERTICAL_STRING "  " UIFONT_VERTICAL_STRING "MONITOR" UIFONT_VERTICAL_STRING "  " UIFONT_VERTICAL_STRING "CONT" UIFONT_VERTICAL_STRING " " UIFONT_VERTICAL_STRING, j + 2);
-            sdl_ui_print_center(UIFONT_VERTICAL_STRING " " UIFONT_BOTTOMLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_BOTTOMRIGHT_STRING "  " UIFONT_BOTTOMLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_BOTTOMRIGHT_STRING "  " UIFONT_BOTTOMLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_BOTTOMRIGHT_STRING " " UIFONT_VERTICAL_STRING, j + 3);
+            sdl_ui_print_center(UIFONT_VERTICAL_STRING "   " UIFONT_TOPLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_TOPRIGHT_STRING "   " UIFONT_TOPLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_TOPRIGHT_STRING "   " UIFONT_VERTICAL_STRING, j + 1);
+            x = sdl_ui_print_center(UIFONT_VERTICAL_STRING "   " UIFONT_VERTICAL_STRING "RESET" UIFONT_VERTICAL_STRING "   " UIFONT_VERTICAL_STRING "HARD RESET" UIFONT_VERTICAL_STRING "   " UIFONT_VERTICAL_STRING, j + 2);
+            sdl_ui_print_center(UIFONT_VERTICAL_STRING "   " UIFONT_BOTTOMLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_BOTTOMRIGHT_STRING "   " UIFONT_BOTTOMLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_BOTTOMRIGHT_STRING "   " UIFONT_VERTICAL_STRING, j + 3);
+            sdl_ui_print_center(UIFONT_VERTICAL_STRING UIFONT_TOPLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_TOPRIGHT_STRING " " UIFONT_TOPLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_TOPRIGHT_STRING "  " UIFONT_TOPLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_TOPRIGHT_STRING UIFONT_VERTICAL_STRING, j + 4);
+            x = sdl_ui_print_center(UIFONT_VERTICAL_STRING UIFONT_VERTICAL_STRING "CONTINUE" UIFONT_VERTICAL_STRING " " UIFONT_VERTICAL_STRING "MONITOR" UIFONT_VERTICAL_STRING "  " UIFONT_VERTICAL_STRING "QUIT" UIFONT_VERTICAL_STRING UIFONT_VERTICAL_STRING, j + 5);
+            sdl_ui_print_center(UIFONT_VERTICAL_STRING UIFONT_BOTTOMLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_BOTTOMRIGHT_STRING " " UIFONT_BOTTOMLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_BOTTOMRIGHT_STRING "  " UIFONT_BOTTOMLEFT_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_HORIZONTAL_STRING UIFONT_BOTTOMRIGHT_STRING UIFONT_VERTICAL_STRING, j + 6);
+            j += 3;
             break;
     }
 
@@ -256,21 +260,21 @@ static int handle_message_box(const char *title, const char *message, int messag
                 break;
             case MESSAGE_CPUJAM:
             default:
-                if (cur_pos == 0) {
-                    sdl_ui_reverse_colors();
-                }
-                sdl_ui_print("RESET", x - 27, j + 2);
-                if (cur_pos < 2) {
-                    sdl_ui_reverse_colors();
-                }
-                sdl_ui_print("MONITOR", x - 18, j + 2);
-                if (cur_pos > 0) {
-                    sdl_ui_reverse_colors();
-                }
-                sdl_ui_print("CONT", x - 7, j + 2);
-                if (cur_pos == 2) {
-                    sdl_ui_reverse_colors();
-                }
+                if (cur_pos == 0) { sdl_ui_reverse_colors(); }
+                sdl_ui_print("RESET", x - 25, j - 1);
+                if (cur_pos == 0) { sdl_ui_reverse_colors(); }
+                if (cur_pos == 1) { sdl_ui_reverse_colors(); }
+                sdl_ui_print("HARD RESET", x - 15, j - 1);
+                if (cur_pos == 1) { sdl_ui_reverse_colors(); }
+                if (cur_pos == 2) { sdl_ui_reverse_colors(); }
+                sdl_ui_print("CONTINUE", x - 28, j + 2);
+                if (cur_pos == 2) { sdl_ui_reverse_colors(); }
+                if (cur_pos == 3) { sdl_ui_reverse_colors(); }
+                sdl_ui_print("MONITOR", x - 17, j + 2);
+                if (cur_pos == 3) { sdl_ui_reverse_colors(); }
+                if (cur_pos == 4) { sdl_ui_reverse_colors(); }
+                sdl_ui_print("QUIT", x - 6, j + 2);
+                if (cur_pos == 4) { sdl_ui_reverse_colors(); }
                 break;
         }
 
