@@ -106,6 +106,7 @@ static const float sdl_gl_vertex_coord[4 * 4] = {
 
 static int sdl_gl_aspect_mode;
 static char *aspect_ratio_s = NULL;
+static char *aspect_ratio_factory_value_s = NULL;
 static double aspect_ratio;
 
 static int sdl_gl_flipx;
@@ -145,11 +146,7 @@ static int set_sdl_bitdepth(int d, void *param)
         return 0;
     }
     sdl_bitdepth = d;
-#if defined(HAVE_HWSCALE)
-    if (!((d == 0) || (d == 24) || (d == 32))) {
-        resources_set_int("HwScalePossible", 0);
-    }
-#endif
+
     /* update */
     return 0;
 }
@@ -241,7 +238,7 @@ static int set_sdl_gl_aspect_mode(int v, void *param)
     sdl_gl_aspect_mode = v;
 
     if (old_v != v) {
-        if (sdl_active_canvas && sdl_active_canvas->videoconfig->hwscale) {
+        if (sdl_active_canvas) {
             video_viewport_resize(sdl_active_canvas, 1);
         }
     }
@@ -275,7 +272,7 @@ static int set_aspect_ratio(const char *val, void *param)
     util_string_set(&aspect_ratio_s, buf);
 
     if (old_aspect != aspect_ratio) {
-        if (sdl_active_canvas && sdl_active_canvas->videoconfig->hwscale) {
+        if (sdl_active_canvas) {
             video_viewport_resize(sdl_active_canvas, 1);
         }
     }
@@ -324,9 +321,10 @@ static int set_sdl_gl_filter(int v, void *param)
 }
 #endif /* HAVE_HWSCALE */
 
-static const resource_string_t resources_string[] = {
+static resource_string_t resources_string[] = {
 #if defined(HAVE_HWSCALE)
-    { "AspectRatio", "1.0", RES_EVENT_NO, NULL,
+    /* CAUTION: position hardcoded below */
+    { "AspectRatio", NULL, RES_EVENT_NO, NULL,
       &aspect_ratio_s, set_aspect_ratio, NULL },
 #endif
     RESOURCE_STRING_LIST_END
@@ -344,6 +342,8 @@ static const resource_string_t resources_string[] = {
 #define SDLCUSTOMHEIGHT_DEFAULT  600
 #endif
 
+/* FIXME: more resources should have the same name as their GTK counterparts,
+          and the SDL prefix removed */
 static const resource_int_t resources_int[] = {
     { "SDLBitdepth", VICE_DEFAULT_BITDEPTH, RES_EVENT_NO, NULL,
       &sdl_bitdepth, set_sdl_bitdepth, NULL },
@@ -353,9 +353,9 @@ static const resource_int_t resources_int[] = {
       &sdl_custom_width, set_sdl_custom_width, NULL },
     { "SDLCustomHeight", SDLCUSTOMHEIGHT_DEFAULT, RES_EVENT_NO, NULL,
       &sdl_custom_height, set_sdl_custom_height, NULL },
-    { "SDLWindowWidth", 0, RES_EVENT_NO, NULL,
+    { "Window0Width", 0, RES_EVENT_NO, NULL,
       &sdl_window_width, set_sdl_window_width, NULL },
-    { "SDLWindowHeight", 0, RES_EVENT_NO, NULL,
+    { "Window0Height", 0, RES_EVENT_NO, NULL,
       &sdl_window_height, set_sdl_window_height, NULL },
 #if defined(HAVE_HWSCALE)
     { "SDLGLAspectMode", SDL_ASPECT_MODE_TRUE, RES_EVENT_NO, NULL,
@@ -372,14 +372,25 @@ static const resource_int_t resources_int[] = {
 
 int video_arch_resources_init(void)
 {
+#if defined(HAVE_HWSCALE)
+    char buf[0x10];
+#endif
     DBG(("%s", __func__));
 
     if (machine_class == VICE_MACHINE_VSID) {
-        if (joy_arch_resources_init() < 0) {
+        if (joy_sdl_resources_init() < 0) {
             return -1;
         }
     }
 
+#if defined(HAVE_HWSCALE)
+    /* KLUDGES: setup the factory default with a string, needs to be done at
+       runtime since float format depends on locale */
+    sprintf(buf, "%f", 1.0f);
+    util_string_set(&aspect_ratio_factory_value_s, buf);
+    resources_string[0].factory_value = aspect_ratio_factory_value_s;
+#endif
+
     if (resources_register_string(resources_string) < 0) {
         return -1;
     }
@@ -403,6 +414,8 @@ void video_arch_resources_shutdown(void)
 /* ------------------------------------------------------------------------- */
 /* Video-related command-line options.  */
 
+/* FIXME: more options should have the same name as their GTK counterparts,
+          and the SDL prefix removed */
 static const cmdline_option_t cmdline_options[] =
 {
     { "-sdlbitdepth", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
@@ -418,10 +431,10 @@ static const cmdline_option_t cmdline_options[] =
       NULL, NULL, "SDLCustomHeight", NULL,
       "<height>", "Set custom resolution height" },
     { "-sdlinitialw", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
-      NULL, NULL, "SDLWindowWidth", NULL,
+      NULL, NULL, "Window0Width", NULL,
       "<width>", "Set intiial window width" },
     { "-sdlinitialh", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
-      NULL, NULL, "SDLWindowHeight", NULL,
+      NULL, NULL, "Window0Height", NULL,
       "<height>", "Set intiial window height" },
 #if defined(HAVE_HWSCALE)
     { "-sdlaspectmode", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
@@ -561,7 +574,7 @@ static video_canvas_t *sdl_canvas_create(video_canvas_t *canvas, unsigned int *w
     int hwscale = 0;
     int lightpen_updated = 0;
 #ifdef HAVE_HWSCALE
-    int rbits = 0, gbits = 0, bbits = 0;
+    int rbits = 0, gbits = 0, bbits = 0, abits = 0;
     const Uint32
 
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
@@ -581,12 +594,14 @@ static video_canvas_t *sdl_canvas_create(video_canvas_t *canvas, unsigned int *w
     new_width *= canvas->videoconfig->scalex;
     new_height *= canvas->videoconfig->scaley;
 
+    sdl_ui_set_window_icon(NULL);
+
     if ((canvas == sdl_active_canvas) && (canvas->fullscreenconfig->enable)) {
         fullscreen = 1;
     }
 
 #ifdef HAVE_HWSCALE
-    if ((canvas == sdl_active_canvas) && (canvas->videoconfig->hwscale)) {
+    if (canvas == sdl_active_canvas) {
         hwscale = 1;
     }
 #endif
@@ -648,18 +663,17 @@ static video_canvas_t *sdl_canvas_create(video_canvas_t *canvas, unsigned int *w
                 sdl_bitdepth = 32;
             /* fall through */
             case 32:
-                rbits = gbits = bbits = 8;
+                rbits = gbits = bbits = abits = 8;
                 sdl_gl_mode = GL_RGBA;
                 break;
             case 24:
                 rbits = gbits = bbits = 8;
+                abits = 0;
                 sdl_gl_mode = GL_RGB;
                 break;
             default:
                 log_error(sdlvideo_log, "%i bpp not supported in OpenGL.", sdl_bitdepth);
-                resources_set_int("HwScalePossible", 0);
                 hwscale = 0;
-                canvas->videoconfig->hwscale = 0;
                 flags = SDL_SWSURFACE;
                 break;
         }
@@ -667,6 +681,7 @@ static video_canvas_t *sdl_canvas_create(video_canvas_t *canvas, unsigned int *w
         SDL_GL_SetAttribute(SDL_GL_RED_SIZE, rbits);
         SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, gbits);
         SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, bbits);
+        SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, abits);
     }
 #endif
 
@@ -716,8 +731,6 @@ static video_canvas_t *sdl_canvas_create(video_canvas_t *canvas, unsigned int *w
                 new_screen = SDL_SetVideoMode(actual_width, actual_height, sdl_bitdepth, flags);
             }
             if (!new_screen) { /* Did not work out quite well. Let's try without hwscale */
-                resources_set_int("HwScalePossible", 0);
-                canvas->videoconfig->hwscale = 0;
                 return sdl_canvas_create(canvas, width, height);
             }
             actual_width = new_screen->w;
@@ -784,8 +797,8 @@ static video_canvas_t *sdl_canvas_create(video_canvas_t *canvas, unsigned int *w
 
     if (canvas == sdl_active_canvas) {
         if (!fullscreen) {
-            resources_set_int("SDLWindowWidth", actual_width);
-            resources_set_int("SDLWindowHeight", actual_height);
+            resources_set_int("Window0Width", actual_width);
+            resources_set_int("Window0Height", actual_height);
         }
     }
 
@@ -881,10 +894,10 @@ void video_canvas_refresh(struct video_canvas_s *canvas, unsigned int xs, unsign
 
         backup = canvas->draw_buffer->draw_buffer;
         canvas->draw_buffer->draw_buffer = canvas->draw_buffer_vsid->draw_buffer;
-        video_canvas_render(canvas, (uint8_t *)canvas->screen->pixels, w, h, xs, ys, xi, yi, canvas->screen->pitch, canvas->screen->format->BitsPerPixel);
+        video_canvas_render(canvas, (uint8_t *)canvas->screen->pixels, w, h, xs, ys, xi, yi, canvas->screen->pitch);
         canvas->draw_buffer->draw_buffer = backup;
     } else {
-        video_canvas_render(canvas, (uint8_t *)canvas->screen->pixels, w, h, xs, ys, xi, yi, canvas->screen->pitch, canvas->screen->format->BitsPerPixel);
+        video_canvas_render(canvas, (uint8_t *)canvas->screen->pixels, w, h, xs, ys, xi, yi, canvas->screen->pitch);
     }
 
     if (SDL_MUSTLOCK(canvas->screen)) {
@@ -892,7 +905,7 @@ void video_canvas_refresh(struct video_canvas_s *canvas, unsigned int xs, unsign
     }
 
 #if defined(HAVE_HWSCALE)
-    if (canvas->videoconfig->hwscale) {
+    {
         const float *v = &(sdl_gl_vertex_coord[sdl_gl_vertex_base]);
 
         if (canvas != sdl_active_canvas) {
@@ -949,16 +962,17 @@ void video_canvas_refresh(struct video_canvas_s *canvas, unsigned int xs, unsign
         glEnd();
 
         SDL_GL_SwapBuffers();
-    } else
-#endif
-
+    }
+#else
     SDL_UpdateRect(canvas->screen, xi, yi, w, h);
+#endif
     ui_autohide_mouse_cursor();
 }
 
 int video_canvas_set_palette(struct video_canvas_s *canvas, struct palette_s *palette)
 {
     unsigned int i, col = 0;
+    video_render_color_tables_t *color_tables = &canvas->videoconfig->color_tables;
     SDL_PixelFormat *fmt;
     SDL_Color colors[256];
 
@@ -994,7 +1008,7 @@ int video_canvas_set_palette(struct video_canvas_s *canvas, struct palette_s *pa
         SDL_SetColors(canvas->screen, colors, 0, palette->num_entries);
     } else {
         for (i = 0; i < 256; i++) {
-            video_render_setrawrgb(i, SDL_MapRGB(fmt, (Uint8)i, 0, 0), SDL_MapRGB(fmt, 0, (Uint8)i, 0), SDL_MapRGB(fmt, 0, 0, (Uint8)i));
+            video_render_setrawrgb(color_tables, i, SDL_MapRGB(fmt, (Uint8)i, 0, 0), SDL_MapRGB(fmt, 0, (Uint8)i, 0), SDL_MapRGB(fmt, 0, 0, (Uint8)i));
         }
         video_render_initraw(canvas->videoconfig);
     }
@@ -1030,10 +1044,10 @@ void video_canvas_resize(struct video_canvas_s *canvas, char resize_canvas)
             canvas->real_width = canvas->actual_width;
             canvas->real_height = canvas->actual_height;
         }
-	/* Recreating the video like this sometimes makes us lose the
-	   fact that keys were released or pressed. Reset the keyboard
-	   state. */
-	keyboard_key_clear();
+        /* Recreating the video like this sometimes makes us lose the
+        fact that keys were released or pressed. Reset the keyboard
+        state. */
+        keyboard_key_clear();
     }
 }
 
@@ -1050,7 +1064,7 @@ static void sdl_video_resize(unsigned int w, unsigned int h)
     vsync_suspend_speed_eval();
 
 #if defined(HAVE_HWSCALE)
-    if (sdl_active_canvas->videoconfig->hwscale && sdl_active_canvas->hwscale_screen) {
+    if (sdl_active_canvas->hwscale_screen) {
         int flags;
 
         if (sdl_active_canvas->fullscreenconfig->enable) {
@@ -1108,8 +1122,8 @@ void sdl_video_resize_event(unsigned int w, unsigned int h)
     }
     sdl_video_resize(w, h);
     if (!sdl_active_canvas->fullscreenconfig->enable) {
-        resources_set_int("SDLWindowWidth", sdl_active_canvas->actual_width);
-        resources_set_int("SDLWindowHeight", sdl_active_canvas->actual_height);
+        resources_set_int("Window0Width", sdl_active_canvas->actual_width);
+        resources_set_int("Window0Height", sdl_active_canvas->actual_height);
     }
 
 #endif /*  HAVE_HWSCALE */
@@ -1142,6 +1156,15 @@ void sdl_video_canvas_switch(int index)
     video_viewport_resize(canvas, 1);
 }
 
+int video_arch_get_active_chip(void)
+{
+    if (sdl_active_canvas_num == VIDEO_CANVAS_IDX_VDC) {
+        return VIDEO_CHIP_VDC;
+    } else {
+        return VIDEO_CHIP_VICII;
+    }
+}
+
 void video_arch_canvas_init(struct video_canvas_s *canvas)
 {
     DBG(("%s: (%p, %i)", __func__, canvas, sdl_num_screens));
@@ -1151,8 +1174,6 @@ void video_arch_canvas_init(struct video_canvas_s *canvas)
         archdep_vice_exit(-1);
     }
 
-    canvas->video_draw_buffer_callback = NULL;
-
     canvas->fullscreenconfig = lib_calloc(1, sizeof(fullscreenconfig_t));
 
     if (sdl_active_canvas_num == sdl_num_screens) {
