@@ -47,6 +47,7 @@
 #include "menu_help.h"
 #include "menu_jam.h"
 #include "menu_joyport.h"
+#include "menu_joystick.h"
 #include "menu_media.h"
 #include "menu_midi.h"
 #include "menu_monitor.h"
@@ -61,6 +62,7 @@
 #include "menu_sound.h"
 #include "menu_speed.h"
 #include "menu_tape.h"
+#include "menu_userport.h"
 #include "menu_video.h"
 #include "resources.h"
 #include "ui.h"
@@ -251,7 +253,9 @@ int c128ui_init(void)
 
     sdl_ui_set_menu_params = c128ui_set_menu_params;
 
-    uijoyport_menu_create(1, 1, 1, 1, 0);
+    uijoyport_menu_create(1, 1, 1, 1, 1);
+    uijoystick_menu_create(1, 1, 1, 1, 1);
+    uiuserport_menu_create(1);
     uisampler_menu_create();
     uicart_menu_create();
     uidrive_menu_create();
@@ -262,7 +266,7 @@ int c128ui_init(void)
     uiclockport_ide64_menu_create();
     uimedia_menu_create();
     sdl_ui_set_main_menu(x128_main_menu);
-    sdl_ui_vicii_font_init();
+    sdl_ui_font_init("chargen", 0, 0x800, 0);
     sdl_vkbd_set_vkbd(&vkbd_c128);
 
 #ifdef HAVE_FFMPEG
@@ -279,6 +283,9 @@ void c128ui_shutdown(void)
     uicart_menu_shutdown();
     uipalette_menu_shutdown();
     uijoyport_menu_shutdown();
+    uijoystick_menu_shutdown();
+    uiuserport_menu_shutdown();
+    uitapeport_menu_shutdown();
     uimedia_menu_shutdown();
 #ifdef HAVE_MIDI
     sdl_menu_midi_in_free();
@@ -292,5 +299,5 @@ void c128ui_shutdown(void)
 #ifdef HAVE_FFMPEG
     sdl_menu_ffmpeg_shutdown();
 #endif
-    sdl_ui_vicii_font_shutdown();
+    sdl_ui_font_shutdown();
 }
