@@ -32,6 +32,7 @@
 
 #include "debug.h"
 #include "lib.h"
+#include "machine.h"
 #include "menu_common.h"
 #include "menu_debug.h"
 #include "menu_drive.h"
@@ -40,6 +41,7 @@
 #include "menu_help.h"
 #include "menu_jam.h"
 #include "menu_joyport.h"
+#include "menu_joystick.h"
 #include "menu_media.h"
 #include "menu_monitor.h"
 #include "menu_network.h"
@@ -55,8 +57,10 @@
 #include "menu_sound.h"
 #include "menu_speed.h"
 #include "menu_tape.h"
+#include "menu_userport.h"
 #include "menu_video.h"
 #include "petmem.h"
+#include "pets.h"
 #include "petui.h"
 #include "pet-resources.h"
 #include "resources.h"
@@ -65,7 +69,9 @@
 #include "uimenu.h"
 #include "vkbd.h"
 
-static const ui_menu_entry_t xpet_main_menu[] = {
+static UI_MENU_CALLBACK(pause_callback_wrapper);
+
+static ui_menu_entry_t xpet_main_menu[] = {
     { "Autostart image",
       MENU_ENTRY_DIALOG,
       autostart_callback,
@@ -77,7 +83,7 @@ static const ui_menu_entry_t xpet_main_menu[] = {
     { "Tape",
       MENU_ENTRY_SUBMENU,
       submenu_callback,
-      (ui_callback_data_t)tape_menu },
+      (ui_callback_data_t)tape_pet_menu },
     { "Cartridge",
       MENU_ENTRY_SUBMENU,
       submenu_callback,
@@ -130,8 +136,9 @@ static const ui_menu_entry_t xpet_main_menu[] = {
 #endif
     { "Pause",
       MENU_ENTRY_OTHER_TOGGLE,
-      pause_callback,
+      pause_callback_wrapper,
       NULL },
+    /* Caution: index is hardcoded below */
     { "Advance Frame",
       MENU_ENTRY_OTHER,
       advance_frame_callback,
@@ -140,6 +147,7 @@ static const ui_menu_entry_t xpet_main_menu[] = {
       MENU_ENTRY_SUBMENU,
       submenu_callback,
       (ui_callback_data_t)monitor_menu },
+    /* Caution: index is hardcoded below */
     { "Virtual keyboard",
       MENU_ENTRY_OTHER,
       vkbd_callback,
@@ -175,6 +183,22 @@ static const ui_menu_entry_t xpet_main_menu[] = {
     SDL_MENU_LIST_END
 };
 
+#ifdef HAVE_NETWORK
+# define MENU_ADVANCE_FRAME_IDX      16
+# define MENU_VIRTUAL_KEYBOARD_IDX   18
+#else
+# define MENU_ADVANCE_FRAME_IDX      15
+# define MENU_VIRTUAL_KEYBOARD_IDX   17
+#endif
+static UI_MENU_CALLBACK(pause_callback_wrapper)
+{
+    xpet_main_menu[MENU_ADVANCE_FRAME_IDX].status = 
+        sdl_pause_state || !sdl_menu_state ? MENU_STATUS_ACTIVE : MENU_STATUS_INACTIVE;
+    xpet_main_menu[MENU_VIRTUAL_KEYBOARD_IDX].status =
+        sdl_pause_state ? MENU_STATUS_INACTIVE : MENU_STATUS_ACTIVE;
+    return pause_callback(activated, param);
+}
+
 /* FIXME: support all PET keyboards (see pet-resources.h) */
 
 static void petui_set_menu_params(int index, menu_draw_t *menu_draw)
@@ -220,6 +244,15 @@ int petui_init_early(void)
     return 0;
 }
 
+/* KLUDGES: this needs to be updated when ROM files are being renamed */
+static const char *get_chargen_name(void)
+{
+    if (machine_class == VICE_MACHINE_PET) {
+        return PET_CHARGEN_NAME;
+    }
+    return "chargen";
+}
+
 /** \brief  Initialize the UI
  *
  * \return  0 on success, -1 on failure
@@ -232,7 +265,9 @@ int petui_init(void)
 #endif
 
     sdl_ui_set_menu_params = petui_set_menu_params;
-    uijoyport_menu_create(0, 0, 1, 1, 0);
+    uijoyport_menu_create(0, 0, 1, 0, 0);
+    uijoystick_menu_create(0, 0, 1, 0, 0);
+    uiuserport_menu_create(1);
     uisampler_menu_create();
     uidrive_menu_create();
     uikeyboard_menu_create();
@@ -241,7 +276,7 @@ int petui_init(void)
     uimedia_menu_create();
 
     sdl_ui_set_main_menu(xpet_main_menu);
-    sdl_ui_crtc_font_init();
+    sdl_ui_font_init(get_chargen_name(), 0, 0x400, 0);
 
 #ifdef HAVE_FFMPEG
     sdl_menu_ffmpeg_init();
@@ -256,6 +291,9 @@ void petui_shutdown(void)
     uisid_menu_shutdown();
     uipalette_menu_shutdown();
     uijoyport_menu_shutdown();
+    uijoystick_menu_shutdown();
+    uiuserport_menu_shutdown();
+    uitapeport_menu_shutdown();
     uimedia_menu_shutdown();
 #ifdef SDL_DEBUG
     fprintf(stderr, "%s\n", __func__);
@@ -265,5 +303,5 @@ void petui_shutdown(void)
     sdl_menu_ffmpeg_shutdown();
 #endif
 
-    sdl_ui_crtc_font_shutdown();
+    sdl_ui_font_shutdown();
 }
