@@ -40,6 +40,7 @@
 #include "menu_help.h"
 #include "menu_jam.h"
 #include "menu_joyport.h"
+#include "menu_joystick.h"
 #include "menu_media.h"
 #include "menu_monitor.h"
 #include "menu_network.h"
@@ -55,6 +56,7 @@
 #include "menu_sound.h"
 #include "menu_speed.h"
 #include "menu_tape.h"
+#include "menu_userport.h"
 #include "menu_video.h"
 #include "plus4memrom.h"
 #include "plus4ui.h"
@@ -231,7 +233,9 @@ int plus4ui_init(void)
 
     sdl_ui_set_menu_params = plus4ui_set_menu_params;
     uisampler_menu_create();
-    uijoyport_menu_create(1, 1, 1, 1, 1);
+    uijoyport_menu_create(1, 1, 1, 1, 0);
+    uijoystick_menu_create(1, 1, 1, 1, 0);
+    uiuserport_menu_create(0);
     uidrive_menu_create();
     uikeyboard_menu_create();
     uipalette_menu_create("TED", NULL);
@@ -239,7 +243,7 @@ int plus4ui_init(void)
     uimedia_menu_create();
 
     sdl_ui_set_main_menu(xplus4_main_menu);
-    sdl_ui_ted_font_init();
+    sdl_ui_font_init("kernal", 0x1000, 0x1400, 0);
     sdl_vkbd_set_vkbd(&vkbd_plus4);
 
 #ifdef HAVE_FFMPEG
@@ -255,6 +259,9 @@ void plus4ui_shutdown(void)
     uisid_menu_shutdown();
     uipalette_menu_shutdown();
     uijoyport_menu_shutdown();
+    uijoystick_menu_shutdown();
+    uiuserport_menu_shutdown();
+    uitapeport_menu_shutdown();
     uimedia_menu_shutdown();
 #ifdef SDL_DEBUG
     fprintf(stderr, "%s\n", __func__);
@@ -264,5 +271,5 @@ void plus4ui_shutdown(void)
     sdl_menu_ffmpeg_shutdown();
 #endif
 
-    sdl_ui_ted_font_shutdown();
+    sdl_ui_font_shutdown();
 }
