@@ -37,24 +37,21 @@
 #include "log.h"
 #include "util.h"
 #include "archdep_defs.h"
-
-#include "archdep_exit.h"
 #include "archdep_home_path.h"
 
-/* TODO:    Include required headers for AmigaOS */
-
 #include "archdep_expand_path.h"
 
 
 /** \brief  Generate heap-allocated full pathname of \a orig_name
  *
  * Returns the absolute path of \a orig_name.
+ *
  * Expands '~' to the user's home path on Unix.
  * If the prefix in \a orig_name is not '\' and not '/' and not '~/' (Unix)
  * the file is assumed to reside in the current working directory, whatever
  * that may be.
  *
- * \param[out]  return_path pointer to expand path destination
+ * \param[out]  return_path pointer to expanded path destination
  * \param[in]   orig_name   original path
  *
  * \return  0
@@ -74,11 +71,17 @@ int archdep_expand_path(char **return_path, const char *orig_name)
         lib_free(cwd);
     }
     return 0;
+
 #elif defined(ARCHDEP_OS_WINDOWS)
     /* taken from the old WinVICE port (src/arch/win32/archdep.c): */
     *return_path = lib_strdup(orig_name);
 #elif defined(ARCHDEP_OS_BEOS)
     /* taken from src/arch/sdl/archdep_beos.c: */
+
+    /* XXX: Haiku uses a Unix-like approach, so we could use the Unix codepath
+     *      for it. But since non of the VICE devs use BeOS/Haiku, there's
+     *      little point in doing so.
+     */
     *return_path = lib_strdup(orig_name);
 #else
     /* fallback */
