@@ -44,6 +44,15 @@
 
 #include "archdep_kbd_get_host_mapping.h"
 
+
+/** \fn     archdep_kbd_get_host_mapping
+ * \brief   Get host keyboard mapping
+ *
+ * \return  Host keyboard mapping
+ *
+ * \see     keyboard.h
+ */
+
 #ifdef ARCHDEP_OS_WINDOWS
 
 /* returns host keyboard mapping. used to initialize the keyboard map when
@@ -53,19 +62,24 @@
    FIXME: add more languages, constants are defined in winnt.h
 
    https://msdn.microsoft.com/en-us/library/windows/desktop/dd318693%28v=vs.85%29.aspx
+
+   https://github.com/Alexpux/mingw-w64/blob/master/mingw-w64-headers/include/winnt.h
 */
 int archdep_kbd_get_host_mapping(void)
 {
     uintptr_t lang;
     int n;
+    /* we search for a full match including sublanguage first, then try again
+       with only the primary language. that means the preferred layout/language
+       must come first in this list */
     static const int maps[KBD_MAPPING_NUM] = {
         KBD_MAPPING_US, KBD_MAPPING_UK, KBD_MAPPING_DE, KBD_MAPPING_DA,
         KBD_MAPPING_NO, KBD_MAPPING_FI, KBD_MAPPING_IT, KBD_MAPPING_NL,
-        KBD_MAPPING_SE, KBD_MAPPING_CH
+        KBD_MAPPING_SE, KBD_MAPPING_CH, KBD_MAPPING_BE
     };
     static const int langids[KBD_MAPPING_NUM] = {
-        MAKELANGID(LANG_ENGLISH,    SUBLANG_ENGLISH_US),
-        MAKELANGID(LANG_ENGLISH,    SUBLANG_ENGLISH_UK),
+        MAKELANGID(LANG_ENGLISH,    SUBLANG_ENGLISH_US),        /* must be always first */
+        MAKELANGID(LANG_ENGLISH,    SUBLANG_ENGLISH_UK),        /* must come after US */
         MAKELANGID(LANG_GERMAN,     SUBLANG_GERMAN),
         MAKELANGID(LANG_DANISH,     SUBLANG_DANISH_DENMARK),
         MAKELANGID(LANG_NORWEGIAN,  SUBLANG_NORWEGIAN_BOKMAL),
@@ -74,6 +88,7 @@ int archdep_kbd_get_host_mapping(void)
         MAKELANGID(LANG_DUTCH,      SUBLANG_DUTCH),
         MAKELANGID(LANG_SWEDISH,    SUBLANG_SWEDISH),
         MAKELANGID(LANG_GERMAN,     SUBLANG_GERMAN_SWISS),
+        MAKELANGID(LANG_DUTCH,      SUBLANG_DUTCH_BELGIAN),    /* must come after regular dutch */
     };
 
     /* GetKeyboardLayout returns a pointer, but the first 16 bits of it return
@@ -118,7 +133,12 @@ int archdep_kbd_get_host_mapping(void)
    starting with a blank (default) config, so an educated guess works good
    enough most of the time :)
 
+   https://docs.oracle.com/cd/E23824_01/html/E26033/glset.html
+   https://docs.moodle.org/dev/Table_of_locales
+
    FIXME: add more languages
+
+   CAUTION: keep in sync with keyboard.c/.h
 */
 
 int archdep_kbd_get_host_mapping(void)
@@ -126,18 +146,20 @@ int archdep_kbd_get_host_mapping(void)
     int n;
     char *l;
     static const int maps[KBD_MAPPING_NUM] = {
-        KBD_MAPPING_US, KBD_MAPPING_UK, KBD_MAPPING_DE, KBD_MAPPING_DA,
-        KBD_MAPPING_NO, KBD_MAPPING_FI, KBD_MAPPING_IT, KBD_MAPPING_NL,
-        KBD_MAPPING_SE, KBD_MAPPING_CH
+        KBD_MAPPING_US, KBD_MAPPING_BE, KBD_MAPPING_UK, KBD_MAPPING_DE, KBD_MAPPING_DA,
+        KBD_MAPPING_NO, KBD_MAPPING_FI, KBD_MAPPING_FR, KBD_MAPPING_IT,
+        KBD_MAPPING_NL, KBD_MAPPING_ES, KBD_MAPPING_SE, KBD_MAPPING_CH,
     };
-    static const char * const s[KBD_MAPPING_NUM] = {
-        "en_US", "en_UK", "de", "da", "no", "fi", "it", "nl", "se", "ch" };
+    static const char * const langids[KBD_MAPPING_NUM] = {
+        "en_US", "nl_BE", "en_UK", "de", "da", "no", "fi", "fr", "it", "nl", "es", "se", "ch" };
     /* setup the locale */
     setlocale(LC_ALL, "");
     l = setlocale(LC_ALL, NULL);
     if (l && (strlen(l) > 1)) {
+        /* compare beginning of the locale string with the chars in our array, that means
+           more "specific" matches must come first in the array */
         for (n = 1; n < KBD_MAPPING_NUM; n++) {
-            if (strncmp(l, s[n], strlen(s[n])) == 0) {
+            if (langids[n] && strncmp(l, langids[n], strlen(langids[n])) == 0) {
                 return maps[n];
             }
         }
