@@ -1,3 +1,8 @@
+/** \file   archdep_list_drives.c
+ * \brief   Get a list of available Windows drives
+ * \author  (Unknown)
+ */
+
 /*
  * This file is part of VICE, the Versatile Commodore Emulator.
  * See README for copyright notice.
@@ -21,11 +26,18 @@
 
 #include "vice.h"
 #include "archdep_defs.h"
-
 #include "archdep.h"
 #include "lib.h"
 
-/* FIXME: includes for amiga */
+#include "archdep_list_drives.h"
+
+/** \fn     archdep_list_drives
+ * \brief   Get a list of available Windows drives
+ *
+ * \return  list of strings, NULL-terminated
+ *
+ * \note    the list and its elements need to be freed with lib_free().
+ */
 
 #if defined(ARCHDEP_OS_WINDOWS)
 
