@@ -30,6 +30,7 @@
 
 #include <string.h>
 #include <inttypes.h>
+#include <errno.h>
 
 #include "archdep.h"
 #include "attach.h"
@@ -40,7 +41,7 @@
 #include "mem.h"
 #include "resources.h"
 #include "util.h"
-
+#include "uiapi.h"
 #include "diskimage.h"
 #include "vdrive.h"
 #include "vdrive-iec.h"
@@ -70,9 +71,6 @@ static autostart_prg_t * load_prg(const char *file_name, fileio_info_t *finfo, l
     autostart_prg_t *prg;
 
     prg = lib_malloc(sizeof(autostart_prg_t));
-    if (prg == NULL) {
-        return NULL;
-    }
 
     /* get data size of file */
     prg->size = fileio_get_bytes_left(finfo);
@@ -81,30 +79,25 @@ static autostart_prg_t * load_prg(const char *file_name, fileio_info_t *finfo, l
     /* read start address */
     if ((fileio_read(finfo, &lo, 1) != 1) || (fileio_read(finfo, &hi, 1) != 1)) {
         log_error(log, "Cannot read start address from '%s'", file_name);
+        lib_free(prg);
         return NULL;
     }
 
     /* get load addr */
-    if (autostart_basic_load) {
-        mem_get_basic_text(&prg->start_addr, NULL);
-    } else {
-        prg->start_addr = (uint16_t)hi << 8 | (uint16_t)lo;
-    }
+    prg->start_addr = (uint16_t)hi << 8 | (uint16_t)lo;
+
     prg->size -= 2; /* skip load addr */
 
     /* check range */
     end = prg->start_addr + prg->size - 1;
     if (end > 0xffff) {
         log_error(log, "Invalid size of '%s': %" PRIu32, file_name, prg->size);
+        lib_free(prg);
         return NULL;
     }
 
     /* load to memory */
     prg->data = lib_malloc(prg->size);
-    if (prg->data == NULL) {
-        log_error(log, "No memory for '%s'", file_name);
-        return NULL;
-    }
 
     /* copy data to memory */
     ptr = prg->start_addr;
@@ -113,6 +106,7 @@ static autostart_prg_t * load_prg(const char *file_name, fileio_info_t *finfo, l
         if (fileio_read(finfo, &(prg->data[i]), 1) != 1) {
             log_error(log, "Error loading data from '%s'", file_name);
             lib_free(prg->data);
+            lib_free(prg);
             return NULL;
         }
         ptr++;
@@ -276,6 +270,11 @@ int autostart_prg_with_disk_image(int unit, int drive, const char *file_name,
         /* create empty image */
         if (vdrive_internal_create_format_disk_image(image_name, (char *)"AUTOSTART", disk_image_type) < 0) {
             log_error(log, "Error creating autostart disk image: %s", image_name);
+            ui_error("Error creating autostart disk image '%s'.\n"
+                     "(%d: %s)\n"
+                     "\n"
+                     "Make sure the directory exists and is writable.",
+                     image_name, errno, strerror(errno));
             break;
         }
 
@@ -285,8 +284,8 @@ int autostart_prg_with_disk_image(int unit, int drive, const char *file_name,
             break;
         }
 
-        /* get vdrive */
-        vdrive = file_system_get_vdrive((unsigned int)unit, drive);
+        /* get vdrive with current drive from attachement */
+        vdrive = file_system_get_vdrive((unsigned int)unit);
         if (vdrive == NULL) {
             break;
         }
@@ -365,6 +364,13 @@ int autostart_prg_perform_injection(log_t log)
         return -1;
     }
 
+    mem_get_basic_text(&start, &end);
+
+    /* load to basic start if requested */
+    if (autostart_basic_load) {
+        prg->start_addr = start;
+    }
+
     log_message(autostart_log, "Injecting program data at $%04x (size $%04x)",
                 prg->start_addr, (unsigned int)prg->size);
 
@@ -374,7 +380,6 @@ int autostart_prg_perform_injection(log_t log)
     }
 
     /* now simulate a basic load */
-    mem_get_basic_text(&start, &end);
     end = (uint16_t)(prg->start_addr + prg->size);
     mem_set_basic_text(start, end);
 
