@@ -35,6 +35,7 @@
 #include "cia.h"
 #include "drive-snapshot.h"
 #include "drive.h"
+#include "serial.h"
 #include "ioutil.h"
 #include "joyport.h"
 #include "joystick.h"
@@ -52,7 +53,7 @@
 #include "vicii.h"
 
 #define SNAP_MACHINE_NAME "C128"
-#define SNAP_MAJOR        0
+#define SNAP_MAJOR        1
 #define SNAP_MINOR        0
 
 int c128_snapshot_write(const char *name, int save_roms, int save_disks, int event_mode)
@@ -72,6 +73,7 @@ int c128_snapshot_write(const char *name, int save_roms, int save_disks, int eve
         || ciacore_snapshot_write_module(machine_context.cia2, s) < 0
         || sid_snapshot_write_module(s) < 0
         || drive_snapshot_write_module(s, save_disks, save_roms) < 0
+        || fsdrive_snapshot_write_module(s) < 0
         || vicii_snapshot_write_module(s) < 0
         || event_snapshot_write_module(s, event_mode) < 0
         || tapeport_snapshot_write_module(s, save_disks) < 0
@@ -114,6 +116,7 @@ int c128_snapshot_read(const char *name, int event_mode)
         || ciacore_snapshot_read_module(machine_context.cia2, s) < 0
         || sid_snapshot_read_module(s) < 0
         || drive_snapshot_read_module(s) < 0
+        || fsdrive_snapshot_read_module(s) < 0
         || vicii_snapshot_read_module(s) < 0
         || event_snapshot_read_module(s, event_mode) < 0
         || tapeport_snapshot_read_module(s) < 0
