@@ -50,6 +50,7 @@
 #include "types.h"
 #include "vdc.h"
 #include "vicii.h"
+#include "viciitypes.h"
 #include "z80.h"
 #include "z80mem.h"
 
@@ -149,6 +150,57 @@ static int mmu_is_in_shared_ram(uint16_t address)
     return 0;
 }
 
+static uint8_t mmu_is_valid_ram(uint8_t page, uint8_t bank, uint8_t current_bank)
+{
+    uint8_t hi_ram = (mmu[0] & 0x30) >> 4;
+    uint8_t mid_ram = (mmu[0] & 0x0c) >> 2;
+    uint8_t lo_ram = (mmu[0] & 0x02) >> 1;
+    uint8_t io_ram = (mmu[0] & 0x01);
+
+    /* check if the needed bank is the current ram bank */
+    if (bank != current_bank) {
+        /* needed bank is not current ram bank, so return not-valid */
+        return 0;
+    }
+
+    /* check if page is $00-$3f */
+    if (page <= 0x3f) {
+        /* always ram, return valid */
+        return 1;
+    }
+
+    /* check if page is $40-$7f */
+    if (page >= 0x40 && page <= 0x7f) {
+        if (lo_ram == 1) {
+            return 1;
+        }
+    }
+
+    /* check if page is $80-$bf */
+    if (page >= 0x80 && page <= 0xbf) {
+        if (mid_ram == 3) {
+            return 1;
+        }
+    }
+
+    /* check if page is $c0-$cf or $e0-$fe */
+    if ((page >= 0xc0 && page <= 0xcf) || (page >= 0xe0 && page <= 0xfe)) {
+        if (hi_ram == 3) {
+            return 1;
+        }
+    }
+
+    /* check if page is $d0-$df */
+    if (page >= 0xd0 && page <= 0xdf) {
+        if (io_ram == 1 && hi_ram == 3) {
+            return 1;
+        }
+    }
+
+    /* rom area, so return not-valid */
+    return 0;
+}
+
 static void mmu_toggle_column4080_key(void)
 {
     mmu_column4080_key = !mmu_column4080_key;
@@ -191,25 +243,34 @@ static void mmu_update_page01_pointers(void)
 {
     /* update pointers for page 0/1 in case they or the shared RAM settings changed */
     /* (shared window has priority over P0H/P1H) */
-    unsigned int page_zero_bank, page_one_bank;
+    uint8_t page_zero_bank, page_one_bank, current_bank;
 
     if (c128_full_banks) {
-        page_zero_bank = (mmu[0x8] & 0x3) * 0x10000U;
-        page_one_bank  = (mmu[0xa] & 0x3) * 0x10000U;
+        page_zero_bank = (mmu[0x8] & 0x3);
+        page_one_bank  = (mmu[0xa] & 0x3);
+        current_bank = ((mmu[0] >> 6) & 0x3);
     } else {
-        page_zero_bank = (mmu[0x8] & 0x1) * 0x10000U;
-        page_one_bank  = (mmu[0xa] & 0x1) * 0x10000U;
+        page_zero_bank = (mmu[0x8] & 0x1);
+        page_one_bank  = (mmu[0xa] & 0x1);
+        current_bank = ((mmu[0] >> 6) & 0x1);
     }
 
+    c128_cpu_set_mmu_page_0_target_ram(mmu_is_valid_ram(mmu[0x7], page_zero_bank, current_bank));
+    c128_cpu_set_mmu_page_1_target_ram(mmu_is_valid_ram(mmu[0x9], page_one_bank, current_bank));
+
+    c128_cpu_set_mmu_page_0_bank(page_zero_bank);
+    c128_cpu_set_mmu_page_1_bank(page_one_bank);
+
+    c128_cpu_set_mmu_zp_sp_shared(mmu_is_in_shared_ram(0));
+
     if (mmu_is_in_shared_ram(mmu[0x7] << 8)) {
         page_zero_bank = 0;
     }
     if (mmu_is_in_shared_ram(mmu[0x9] << 8)) {
         page_one_bank = 0;
     }
-
-    mem_page_zero = mem_ram + page_zero_bank + (mmu[0x7] << 8);
-    mem_page_one  = mem_ram + page_one_bank  + (mmu[0x9] << 8);
+    mem_page_zero = mem_ram + (page_zero_bank << 16) + (mmu[0x7] << 8);
+    mem_page_one  = mem_ram + (page_one_bank << 16) + (mmu[0x9] << 8);
 }
 
 /* returns 1 if MMU is in C64 mode */
@@ -234,8 +295,10 @@ static void mmu_switch_to_c64mode(void)
         mmu[5] = 0xf7;
         /* force standard addresses for stack and zeropage */
         mmu[7] = 0;
+        c128_cpu_set_mmu_page_0(0);
         mmu[8] = 0;
         mmu[9] = 1;
+        c128_cpu_set_mmu_page_1(1);
         mmu[10] = 0;
         mmu_update_page01_pointers();
     }
@@ -385,12 +448,17 @@ void mmu_store(uint16_t address, uint8_t value)
                 p1h_latch = value;
                 break;
             case 7:
+                mmu[8] = p0h_latch;
+                c128_cpu_set_mmu_page_0(mmu[7]);
+#ifdef MMU_DEBUG
+                log_message(mmu_log, "PAGE ZERO %05x PAGE ONE %05x",
+                            (mmu[0x8] & 0x1 ? 0x10000 : 0x00000) + (mmu[0x7] << 8),
+                            (mmu[0xa] & 0x1 ? 0x10000 : 0x00000) + (mmu[0x9] << 8));
+#endif
+                break;
             case 9:
-                if (address == 7) {
-                    mmu[8] = p0h_latch;
-                } else {
-                    mmu[10] = p1h_latch;
-                }
+                mmu[10] = p1h_latch;
+                c128_cpu_set_mmu_page_1(mmu[9]);
 #ifdef MMU_DEBUG
                 log_message(mmu_log, "PAGE ZERO %05x PAGE ONE %05x",
                             (mmu[0x8] & 0x1 ? 0x10000 : 0x00000) + (mmu[0x7] << 8),
@@ -410,20 +478,17 @@ void mmu_store(uint16_t address, uint8_t value)
 uint8_t mmu_ffxx_read(uint16_t addr)
 {
     if (addr >= 0xff00 && addr <= 0xff04) {
-        return mmu[addr & 0xf];
+        vicii.last_cpu_val = mmu[addr & 0xf];
+    } else if ((mmu[0] & 0x30) == 0x00) {
+        vicii.last_cpu_val = c128memrom_kernal_read(addr);
+    } else if ((mmu[0] & 0x30) == 0x10) {
+        vicii.last_cpu_val = internal_function_rom_read(addr);
+    } else if ((mmu[0] & 0x30) == 0x20) {
+        vicii.last_cpu_val = external_function_rom_read(addr);
+    } else {
+        vicii.last_cpu_val = top_shared_read(addr);
     }
-
-    if ((mmu[0] & 0x30) == 0x00) {
-        return c128memrom_kernal_read(addr);
-    }
-    if ((mmu[0] & 0x30) == 0x10) {
-        return internal_function_rom_read(addr);
-    }
-    if ((mmu[0] & 0x30) == 0x20) {
-        return external_function_rom_read(addr);
-    }
-
-    return top_shared_read(addr);
+    return vicii.last_cpu_val;
 }
 
 uint8_t mmu_ffxx_read_z80(uint16_t addr)
@@ -437,6 +502,7 @@ uint8_t mmu_ffxx_read_z80(uint16_t addr)
 
 void mmu_ffxx_store(uint16_t addr, uint8_t value)
 {
+    vicii.last_cpu_val = value;
     if (addr == 0xff00) {
         mmu_store(0, value);
         /* FIXME? [SRT] does reu_dma(-1) work here, or should
