@@ -35,6 +35,7 @@
 #include "c128rom.h"
 #include "c64memrom.h"
 #include "c64rom.h"
+#include "machine.h"
 #include "mem.h"
 #include "log.h"
 #include "resources.h"
@@ -48,23 +49,6 @@ static log_t c128rom_log = LOG_ERR;
 /* Flag: nonzero if the Kernal and BASIC ROMs have been loaded.  */
 static int rom_loaded = 0;
 
-#ifdef USE_EMBEDDED
-#include "c128kernal.h"
-#include "c128kernalde.h"
-#include "c128kernalfi.h"
-#include "c128kernalfr.h"
-#include "c128kernalit.h"
-#include "c128kernalno.h"
-#include "c128kernalse.h"
-#include "c128kernalch.h"
-
-#include "c128chargde.h"
-#include "c128chargen.h"
-#include "c128chargfr.h"
-#include "c128chargse.h"
-#include "c128chargch.h"
-#include "c128chargno.h"
-#else
 /* National Kernal ROM images. */
 static uint8_t kernal_int[C128_KERNAL_ROM_IMAGE_SIZE];
 static uint8_t kernal_de[C128_KERNAL_ROM_IMAGE_SIZE];
@@ -82,7 +66,6 @@ static uint8_t chargen_fr[C128_CHARGEN_ROM_SIZE];
 static uint8_t chargen_se[C128_CHARGEN_ROM_SIZE];
 static uint8_t chargen_ch[C128_CHARGEN_ROM_SIZE];
 static uint8_t chargen_no[C128_CHARGEN_ROM_SIZE];
-#endif
 
 int c128rom_kernal_checksum(void)
 {
@@ -111,7 +94,7 @@ int c128rom_load_kernal_int(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load international Kernal ROM.  */
-        if (sysfile_load(rom_name, kernal_int, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, kernal_int, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load kernal ROM `%s'.", rom_name);
             return -1;
         }
@@ -127,7 +110,7 @@ int c128rom_load_kernal_de(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load German Kernal ROM.  */
-        if (sysfile_load(rom_name, kernal_de, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, kernal_de, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load kernal ROM `%s'.", rom_name);
             return -1;
         }
@@ -143,7 +126,7 @@ int c128rom_load_kernal_fi(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load Finnish Kernal ROM.  */
-        if (sysfile_load(rom_name, kernal_fi, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, kernal_fi, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load kernal ROM `%s'.", rom_name);
             return -1;
         }
@@ -159,7 +142,7 @@ int c128rom_load_kernal_fr(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load French Kernal ROM.  */
-        if (sysfile_load(rom_name, kernal_fr, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, kernal_fr, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load kernal ROM `%s'.", rom_name);
             return -1;
         }
@@ -175,7 +158,7 @@ int c128rom_load_kernal_it(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load Italian Kernal ROM.  */
-        if (sysfile_load(rom_name, kernal_it, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, kernal_it, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load kernal ROM `%s'.", rom_name);
             return -1;
         }
@@ -191,7 +174,7 @@ int c128rom_load_kernal_no(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load Norwegian Kernal ROM.  */
-        if (sysfile_load(rom_name, kernal_no, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, kernal_no, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load kernal ROM `%s'.", rom_name);
             return -1;
         }
@@ -207,7 +190,7 @@ int c128rom_load_kernal_se(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load Swedish Kernal ROM.  */
-        if (sysfile_load(rom_name, kernal_se, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, kernal_se, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load kernal ROM `%s'.", rom_name);
             return -1;
         }
@@ -223,7 +206,7 @@ int c128rom_load_kernal_ch(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load Swiss Kernal ROM.  */
-        if (sysfile_load(rom_name, kernal_ch, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, kernal_ch, C128_KERNAL_ROM_IMAGE_SIZE, C128_KERNAL_ROM_IMAGE_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load kernal ROM `%s'.", rom_name);
             return -1;
         }
@@ -231,9 +214,37 @@ int c128rom_load_kernal_ch(const char *rom_name)
     return 0;
 }
 
+#define NUM_TRAP_DEVICES 9  /* FIXME: is there a better constant ? */
+static int trapfl[NUM_TRAP_DEVICES];
+static int trapdevices[NUM_TRAP_DEVICES + 1] = { 1, 4, 5, 6, 7, 8, 9, 10, 11, -1 };
+
+static void get_trapflags(void)
+{
+    int i;
+    for(i = 0; trapdevices[i] != -1; i++) {
+        resources_get_int_sprintf("VirtualDevice%d", &trapfl[i], trapdevices[i]);
+    }
+}
+
+static void clear_trapflags(void)
+{
+    int i;
+    for(i = 0; trapdevices[i] != -1; i++) {
+        resources_set_int_sprintf("VirtualDevice%d", 0, trapdevices[i]);
+    }
+}
+
+static void restore_trapflags(void)
+{
+    int i;
+    for(i = 0; trapdevices[i] != -1; i++) {
+        resources_set_int_sprintf("VirtualDevice%d", trapfl[i], trapdevices[i]);
+    }
+}
+
 int c128rom_kernal_setup(void)
 {
-    int trapfl, machine_type;
+    int machine_type;
     uint8_t *kernal = NULL;
 
     if (!rom_loaded) {
@@ -273,8 +284,8 @@ int c128rom_kernal_setup(void)
     }
 
     /* disable traps before loading the ROM */
-    resources_get_int("VirtualDevices", &trapfl);
-    resources_set_int("VirtualDevices", 0);
+    get_trapflags();
+    clear_trapflags();
 
     memcpy(&c128memrom_basic_rom[C128_BASIC_ROM_SIZE], kernal, C128_EDITOR_ROM_SIZE);
     memcpy(z80bios_rom, &kernal[C128_EDITOR_ROM_SIZE], C128_Z80BIOS_ROM_SIZE);
@@ -283,7 +294,7 @@ int c128rom_kernal_setup(void)
 
     c128rom_kernal_checksum();
 
-    resources_set_int("VirtualDevices", trapfl);
+    restore_trapflags();
 
     return 0;
 }
@@ -323,7 +334,7 @@ int c128rom_load_basiclo(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load Basic ROM.  */
-        if (sysfile_load(rom_name, c128memrom_basic_rom, C128_BASIC_ROM_IMAGELO_SIZE, C128_BASIC_ROM_IMAGELO_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, c128memrom_basic_rom, C128_BASIC_ROM_IMAGELO_SIZE, C128_BASIC_ROM_IMAGELO_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load basic ROM `%s'.", rom_name);
             return -1;
         }
@@ -339,7 +350,7 @@ int c128rom_load_basichi(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load Basic ROM.  */
-        if (sysfile_load(rom_name, &c128memrom_basic_rom[C128_BASIC_ROM_IMAGELO_SIZE], C128_BASIC_ROM_IMAGEHI_SIZE, C128_BASIC_ROM_IMAGEHI_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, &c128memrom_basic_rom[C128_BASIC_ROM_IMAGELO_SIZE], C128_BASIC_ROM_IMAGEHI_SIZE, C128_BASIC_ROM_IMAGEHI_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load basic ROM `%s'.", rom_name);
             return -1;
         }
@@ -397,7 +408,7 @@ int c128rom_load_chargen_int(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load chargen ROM.  */
-        if (sysfile_load(rom_name, chargen_int, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, chargen_int, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load character ROM `%s'.", rom_name);
             return -1;
         }
@@ -413,7 +424,7 @@ int c128rom_load_chargen_de(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load chargen ROM.  */
-        if (sysfile_load(rom_name, chargen_de, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, chargen_de, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load character ROM `%s'.", rom_name);
             return -1;
         }
@@ -429,7 +440,7 @@ int c128rom_load_chargen_fr(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load chargen ROM.  */
-        if (sysfile_load(rom_name, chargen_fr, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, chargen_fr, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load character ROM `%s'.", rom_name);
             return -1;
         }
@@ -445,7 +456,7 @@ int c128rom_load_chargen_se(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load chargen ROM.  */
-        if (sysfile_load(rom_name, chargen_se, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, chargen_se, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load character ROM `%s'.", rom_name);
             return -1;
         }
@@ -461,7 +472,7 @@ int c128rom_load_chargen_ch(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load chargen ROM.  */
-        if (sysfile_load(rom_name, chargen_ch, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, chargen_ch, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load character ROM `%s'.", rom_name);
             return -1;
         }
@@ -477,7 +488,7 @@ int c128rom_load_chargen_no(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load chargen ROM.  */
-        if (sysfile_load(rom_name, chargen_no, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, chargen_no, C128_CHARGEN_ROM_SIZE, C128_CHARGEN_ROM_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load character ROM `%s'.", rom_name);
             return -1;
         }
@@ -500,7 +511,7 @@ int c128rom_load_kernal64(const char *rom_name, uint8_t *cartkernal)
 
         if (!util_check_null_string(rom_name)) {
             /* Load C64 kernal ROM.  */
-            if (sysfile_load(rom_name, c64memrom_kernal64_rom, C128_KERNAL64_ROM_SIZE, C128_KERNAL64_ROM_SIZE) < 0) {
+            if (sysfile_load(rom_name, machine_name, c64memrom_kernal64_rom, C128_KERNAL64_ROM_SIZE, C128_KERNAL64_ROM_SIZE) < 0) {
                 log_error(c128rom_log, "Couldn't load C64 kernal ROM `%s'.", rom_name);
                 return -1;
             }
@@ -521,7 +532,7 @@ int c128rom_load_basic64(const char *rom_name)
 
     if (!util_check_null_string(rom_name)) {
         /* Load basic ROM.  */
-        if (sysfile_load(rom_name, c64memrom_basic64_rom, C128_BASIC64_ROM_SIZE, C128_BASIC64_ROM_SIZE) < 0) {
+        if (sysfile_load(rom_name, machine_name, c64memrom_basic64_rom, C128_BASIC64_ROM_SIZE, C128_BASIC64_ROM_SIZE) < 0) {
             log_error(c128rom_log, "Couldn't load C64 basic ROM `%s'.", rom_name);
             return -1;
         }
