@@ -27,12 +27,12 @@
 #ifndef VICE_C64ROM_H
 #define VICE_C64ROM_H
 
-extern int c64rom_load_kernal(const char *rom_name, BYTE *new_kernal);
+extern int c64rom_load_kernal(const char *rom_name, uint8_t *new_kernal);
 extern int c64rom_load_basic(const char *rom_name);
 extern int c64rom_load_chargen(const char *rom_name);
 
 extern int c64rom_get_kernal_checksum(void);
-extern int c64rom_get_kernal_chksum_id(WORD *sumout, int *idout);
+extern int c64rom_get_kernal_chksum_id(uint16_t *sumout, int *idout);
 extern int c64rom_get_basic_checksum(void);
 
 extern int c64rom_isloaded(void);
