@@ -28,7 +28,7 @@
 #include "daa.h"
 #include "types.h"
 
-const BYTE daa_reg_a[2048] = {
+const uint8_t daa_reg_a[2048] = {
     0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
     0x08, 0x09, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15,
     0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,
@@ -287,7 +287,7 @@ const BYTE daa_reg_a[2048] = {
     0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99
 };
 
-const BYTE daa_reg_f[2048] = {
+const uint8_t daa_reg_f[2048] = {
     0x44, 0x00, 0x00, 0x04, 0x00, 0x04, 0x04, 0x00,
     0x08, 0x0C, 0x10, 0x14, 0x14, 0x10, 0x14, 0x10,
     0x00, 0x04, 0x04, 0x00, 0x04, 0x00, 0x00, 0x04,
