@@ -53,15 +53,14 @@
 
 /*
     FIXME: this implementation is based on vague guesses on how the hardware
-           really works. remaining problems are:
-           - fastloader fails
+           really works.
 
-    Evesham Micros "Freeze Frame MK4"
+    Evesham Micros "Freeze Frame MK3 v2"
 
     - 2 Buttons (Freeze, Reset)
     - 16k ROM
 
-    Evesham Micros "Freeze Machine"
+    Evesham Micros "Freeze Frame MK4 LAZER", "Freeze Machine"
 
     - 2 Buttons (Freeze, Reset)
     - 7474, 74163, 2 * 7400, 7402
@@ -114,7 +113,7 @@ static uint8_t freezemachine_io1_peek(uint16_t addr)
 
 static void freezemachine_io1_store(uint16_t addr, uint8_t value)
 {
-    DBG(("io1 %04x %02x\n", addr, value));
+    DBG(("io1 w %04x %02x\n", addr, value));
 }
 
 static uint8_t freezemachine_io2_read(uint16_t addr)
@@ -135,7 +134,7 @@ static uint8_t freezemachine_io2_peek(uint16_t addr)
 
 static void freezemachine_io2_store(uint16_t addr, uint8_t value)
 {
-    DBG(("io2 %04x %02x\n", addr, value));
+    DBG(("io2 w %04x %02x\n", addr, value));
 }
 
 static io_source_t freezemachine_io1_device = {
