@@ -80,11 +80,6 @@
 #define debug(x)
 #endif
 
-#ifndef HAVE_FSEEKO
-#define fseeko(a, b, c) fseek(a, b, c)
-#define ftello(a) ftell(a)
-#endif
-
 #define LATENCY_TIMER 4000
 
 static int ide64_enabled = 0;
@@ -1483,7 +1478,7 @@ void ide64_config_init(void)
     struct drive_s *drive;
 
     debug("IDE64 init");
-    cart_config_changed_slotmain(0, 0, CMODE_READ | CMODE_PHI2_RAM);
+    cart_config_changed_slotmain(CMODE_8KGAME, CMODE_8KGAME, CMODE_READ | CMODE_PHI2_RAM);
     current_bank = 0;
     current_cfg = 0;
     kill_port = 0;
