@@ -45,6 +45,7 @@
 #include "machine.h"
 #include "mem.h"
 #include "monitor.h"
+#include "ram.h"
 #include "resources.h"
 #include "snapshot.h"
 #include "types.h"
@@ -178,6 +179,33 @@ static io_source_list_t *isepic_io2_list_item = NULL;
 
 /* ------------------------------------------------------------------------- */
 
+/* FIXME: this still needs to be tweaked to match the hardware */
+static RAMINITPARAM ramparam = {
+    .start_value = 255,
+    .value_invert = 2,
+    .value_offset = 1,
+
+    .pattern_invert = 0x100,
+    .pattern_invert_value = 255,
+
+    .random_start = 0,
+    .random_repeat = 0,
+    .random_chance = 0,
+};
+
+void isepic_powerup(void)
+{
+    DBG(("isepic_powerup"));
+    if ((isepic_filename != NULL) && (*isepic_filename != 0)) {
+        /* do not init ram if a file is used for ram content (like battery backup) */
+        return;
+    }
+    if (isepic_ram) {
+        DBG(("isepic_powerup ram clear"));
+        ram_init_with_pattern(isepic_ram, ISEPIC_RAM_SIZE, &ramparam);
+    }
+}
+
 int isepic_cart_enabled(void)
 {
     if (isepic_enabled) {
@@ -209,6 +237,7 @@ static int isepic_activate(void)
     if (isepic_ram == NULL) {
         isepic_ram = lib_malloc(ISEPIC_RAM_SIZE);
     }
+    ram_init_with_pattern(isepic_ram, ISEPIC_RAM_SIZE, &ramparam);
 
     if (!util_check_null_string(isepic_filename)) {
         log_message(LOG_DEFAULT, "Reading ISEPIC image %s.", isepic_filename);
@@ -523,8 +552,8 @@ void isepic_romh_store(uint16_t addr, uint8_t byte)
             case 0xfffb:
                 isepic_ram[(isepic_page * 256) + (addr & 0xff)] = byte;
                 break;
-		}
-	}
+        }
+    }
     mem_store_without_ultimax(addr, byte);
 }
 
@@ -627,7 +656,7 @@ void isepic_config_init(void)
 void isepic_reset(void)
 {
     if (isepic_state == ISEPIC_STATE_NMI_EXECUTING) {
-        cart_config_changed_slot1(2, 2, CMODE_READ | CMODE_RELEASE_FREEZE);
+        cart_config_changed_slot1(CMODE_RAM, CMODE_RAM, CMODE_READ | CMODE_RELEASE_FREEZE);
         isepic_state = ISEPIC_STATE_PROGRAMMING;
     }
 }
