@@ -219,7 +219,7 @@ void mach5_detach(void)
    ARRAY | ROML   |   0.0+  | 8192 BYTES of ROML data
  */
 
-static char snap_module_name[] = "CARTMACH5";
+static const char snap_module_name[] = "CARTMACH5";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   1
 
