@@ -101,7 +101,7 @@ static const export_resource_t export_res = {
 
 /* Some prototypes are needed */
 static int sfx_soundsampler_sound_machine_init(sound_t *psid, int speed, int cycles_per_sec);
-static int sfx_soundsampler_sound_machine_calculate_samples(sound_t **psid, int16_t *pbuf, int nr, int sound_output_channels, int sound_chip_channels, int *delta_t);
+static int sfx_soundsampler_sound_machine_calculate_samples(sound_t **psid, int16_t *pbuf, int nr, int sound_output_channels, int sound_chip_channels, CLOCK *delta_t);
 static void sfx_soundsampler_sound_machine_store(sound_t *psid, uint16_t addr, uint8_t val);
 static uint8_t sfx_soundsampler_sound_machine_read(sound_t *psid, uint16_t addr);
 static void sfx_soundsampler_sound_reset(sound_t *psid, CLOCK cpu_clk);
@@ -314,7 +314,7 @@ struct sfx_soundsampler_sound_s {
 
 static struct sfx_soundsampler_sound_s snd;
 
-static int sfx_soundsampler_sound_machine_calculate_samples(sound_t **psid, int16_t *pbuf, int nr, int soc, int scc, int *delta_t)
+static int sfx_soundsampler_sound_machine_calculate_samples(sound_t **psid, int16_t *pbuf, int nr, int soc, int scc, CLOCK *delta_t)
 {
     return sound_dac_calculate_samples(&sfx_soundsampler_dac, pbuf, (int)snd.voice0 * 128, nr, soc, (soc > 1) ? 3 : 1);
 }
