@@ -38,6 +38,7 @@
 #include "cartridge.h"
 #include "export.h"
 #include "log.h"
+#include "ram.h"
 #include "snapshot.h"
 #include "supersnapshot4.h"
 #include "types.h"
@@ -94,6 +95,8 @@
 #define DBG(x)
 #endif
 
+#define CART_RAM_SIZE (8 * 1024)
+
 /* Super Snapshot configuration flags.  */
 static uint8_t ramconfig = 0xff, romconfig = 9;
 
@@ -264,14 +267,33 @@ void supersnapshot_v4_roml_store(uint16_t addr, uint8_t value)
 
 /* ---------------------------------------------------------------------*/
 
+/* FIXME: this still needs to be tweaked to match the hardware */
+static RAMINITPARAM ramparam = {
+    .start_value = 255,
+    .value_invert = 2,
+    .value_offset = 1,
+
+    .pattern_invert = 0x100,
+    .pattern_invert_value = 255,
+
+    .random_start = 0,
+    .random_repeat = 0,
+    .random_chance = 0,
+};
+
+void supersnapshot_v4_powerup(void)
+{
+    ram_init_with_pattern(export_ram0, CART_RAM_SIZE, &ramparam);
+}
+
 void supersnapshot_v4_freeze(void)
 {
-    cart_config_changed_slotmain(3, 3, CMODE_READ | CMODE_EXPORT_RAM);
+    cart_config_changed_slotmain(CMODE_ULTIMAX, CMODE_ULTIMAX, CMODE_READ | CMODE_EXPORT_RAM);
 }
 
 void supersnapshot_v4_config_init(void)
 {
-    cart_config_changed_slotmain(1 | (1 << CMODE_BANK_SHIFT), 1 | (1 << CMODE_BANK_SHIFT), CMODE_READ);
+    cart_config_changed_slotmain(CMODE_16KGAME | (1 << CMODE_BANK_SHIFT), CMODE_16KGAME | (1 << CMODE_BANK_SHIFT), CMODE_READ);
 }
 
 void supersnapshot_v4_config_setup(uint8_t *rawcart)
