@@ -348,7 +348,7 @@ int psid_load_file(const char* filename)
     }
 
     if (!feof(f)) {
-        log_error(vlog, "More than 64K PSID data.");
+        log_error(vlog, "More than 64KiB PSID data.");
         goto fail;
     }
 
@@ -529,22 +529,24 @@ void psid_init_tune(int install_driver_hook)
             driver_info_text = lib_msprintf("Driver=$%04X, Image=$%04X-$%04X, Init=$%04X, Play=$%04X", reloc_addr, psid->load_addr,
                                             psid->load_addr + psid->data_size - 1, psid->init_addr, psid->play_addr);
             vsid_ui_setdrv(driver_info_text);
+            lib_free(driver_info_text);
+
             vsid_ui_set_driver_addr(reloc_addr);
             vsid_ui_set_load_addr(psid->load_addr);
             vsid_ui_set_init_addr(psid->init_addr);
             vsid_ui_set_play_addr(psid->play_addr);
             vsid_ui_set_data_size(psid->data_size);
-            lib_free(driver_info_text);
+            vsid_ui_set_default_tune(start_song);
+            vsid_ui_display_name((char *)(psid->name));
+            vsid_ui_display_author((char *)(psid->author));
+            vsid_ui_display_copyright((char *)(psid->copyright));
+            vsid_ui_display_sync(sync);
+            vsid_ui_display_sid_model(sid_model);
+            vsid_ui_display_irqtype(irq_str);
+            vsid_ui_display_nr_of_tunes(psid->songs);
+            vsid_ui_display_tune_nr(start_song);
+            vsid_ui_display_time(0);
         }
-        vsid_ui_display_name((char *)(psid->name));
-        vsid_ui_display_author((char *)(psid->author));
-        vsid_ui_display_copyright((char *)(psid->copyright));
-
-        vsid_ui_display_sync(sync);
-        vsid_ui_display_sid_model(sid_model);
-        vsid_ui_display_irqtype(irq_str);
-        vsid_ui_display_tune_nr(start_song);
-        vsid_ui_display_time(0);
     }
 
     /* Store parameters for PSID player. */
@@ -654,7 +656,7 @@ void psid_init_driver(void)
         if (((sid2loc >= 0xd420 && sid2loc < 0xd800) || sid2loc >= 0xde00)
             && (sid2loc & 0x10) == 0) {
             resources_set_int("SidStereo", 1);
-            resources_set_int("SidStereoAddressStart", sid2loc);
+            resources_set_int("Sid2AddressStart", sid2loc);
         }
         sid3loc = 0xd000 | ((psid->reserved << 4) & 0x0ff0);
         if (sid3loc != 0xd000) {
@@ -662,7 +664,7 @@ void psid_init_driver(void)
             if (((sid3loc >= 0xd420 && sid3loc < 0xd800) || sid3loc >= 0xde00)
                 && (sid3loc & 0x10) == 0) {
                 resources_set_int("SidStereo", 2);
-                resources_set_int("SidTripleAddressStart", sid3loc);
+                resources_set_int("Sid3AddressStart", sid3loc);
             }
         }
     }
@@ -835,7 +837,7 @@ static const unsigned char *copystring(unsigned char *d, const unsigned char *s)
         ++s;
     }
     *d = 0;
-    charset_petconvstring(d, 1);
+    charset_petconvstring(d, CONVERT_TO_ASCII);
     return s + 1;
 }
 
@@ -869,8 +871,9 @@ static int mus_load_file(const char* filename, int ispsid)
 {
     char *strname;
     FILE *f;
-    int n, stereo = 0;
-    int mus_datalen;
+    int stereo = 0;
+    size_t n;
+    size_t mus_datalen;
 
     if (!(f = zfile_fopen(filename, MODE_READ))) {
         return -1;
@@ -910,7 +913,7 @@ static int mus_load_file(const char* filename, int ispsid)
         }
         lib_free(strname);
         /* only extract credits if this is NOT a psid file */
-        mus_extract_credits(psid->data, mus_datalen);
+        mus_extract_credits(psid->data, (int)mus_datalen);
     }
 
     mus_install();
