@@ -59,17 +59,37 @@ void sid_sound_chip_init(void)
 
 /* ---------------------------------------------------------------------*/
 
-int machine_sid2_check_range(unsigned int sid2_adr)
+int machine_sid2_check_range(unsigned int sid_adr)
 {
     return 0;
 }
 
-int machine_sid3_check_range(unsigned int sid3_adr)
+int machine_sid3_check_range(unsigned int sid_adr)
 {
     return 0;
 }
 
-int machine_sid4_check_range(unsigned int sid3_adr)
+int machine_sid4_check_range(unsigned int sid_adr)
+{
+    return 0;
+}
+
+int machine_sid5_check_range(unsigned int sid_adr)
+{
+    return 0;
+}
+
+int machine_sid6_check_range(unsigned int sid_adr)
+{
+    return 0;
+}
+
+int machine_sid7_check_range(unsigned int sid_adr)
+{
+    return 0;
+}
+
+int machine_sid8_check_range(unsigned int sid_adr)
 {
     return 0;
 }
@@ -78,11 +98,6 @@ void machine_sid2_enable(int val)
 {
 }
 
-void sound_machine_prevent_clk_overflow(sound_t *psid, CLOCK sub)
-{
-    sid_sound_machine_prevent_clk_overflow(psid, sub);
-}
-
 char *sound_machine_dump_state(sound_t *psid)
 {
     return sid_sound_machine_dump_state(psid);
