@@ -48,7 +48,6 @@
 #include "cbm2tpi.h"
 #include "cbm2ui.h"
 #include "cia.h"
-#include "clkguard.h"
 #include "datasette.h"
 #include "debug.h"
 #include "debugcart.h"
@@ -61,8 +60,10 @@
 #include "fsdevice.h"
 #include "gfxoutput.h"
 #include "iecdrive.h"
+#include "inception.h"
 #include "init.h"
 #include "joyport.h"
+#include "joyport_io_sim.h"
 #include "joystick.h"
 #include "kbdbuf.h"
 #include "keyboard.h"
@@ -74,10 +75,12 @@
 #include "maincpu.h"
 #include "mem.h"
 #include "monitor.h"
+#include "multijoy.h"
 #include "network.h"
-#include "paperclip64.h"
+#include "ninja_snespad.h"
 #include "parallel.h"
 #include "printer.h"
+#include "protopad.h"
 #include "resources.h"
 #include "rs232drv.h"
 #include "sampler.h"
@@ -94,7 +97,9 @@
 #include "tapeport.h"
 #include "tpi.h"
 #include "traps.h"
+#include "trapthem_snespad.h"
 #include "types.h"
+#include "userport.h"
 #include "vice-event.h"
 #include "vicii.h"
 #include "vicii-resources.h"
@@ -106,7 +111,6 @@
 #include "mouse.h"
 #endif
 
-
 /** \brief  Delay in seconds before pasting -keybuf argument into the buffer
  */
 #define KBDBUF_ALARM_DELAY   8
@@ -162,6 +166,8 @@ static joyport_port_props_t control_port_1 =
     1,                      /* has a potentiometer connected to this port */
     0,                      /* officially has lightpen support on this port,
                                but no lightpen support is in the cbm5x0 code */
+    1,                      /* has joystick adapter on this port */
+    1,                      /* has output support on this port */
     1                       /* port is always active */
 };
 
@@ -170,15 +176,121 @@ static joyport_port_props_t control_port_2 =
     "Control port 2",
     1,                      /* has a potentiometer connected to this port */
     0,                      /* has NO lightpen support on this port */
+    1,                      /* has joystick adapter on this port */
+    1,                      /* has output support on this port */
     1                       /* port is always active */
 };
 
+static joyport_port_props_t joy_adapter_control_port_1 =
+{
+    "Joystick adapter port 1",
+    0,                      /* has NO potentiometer connected to this port */
+    0,                      /* has NO lightpen support on this port */
+    0,                      /* has NO joystick adapter on this port */
+    1,                      /* has output support on this port */
+    0                       /* port can be switched on/off */
+};
+
+static joyport_port_props_t joy_adapter_control_port_2 =
+{
+    "Joystick adapter port 2",
+    0,                      /* has NO potentiometer connected to this port */
+    0,                      /* has NO lightpen support on this port */
+    0,                      /* has NO joystick adapter on this port */
+    1,                      /* has output support on this port */
+    0                       /* port can be switched on/off */
+};
+
+static joyport_port_props_t joy_adapter_control_port_3 =
+{
+    "Joystick adapter port 3",
+    0,                      /* has NO potentiometer connected to this port */
+    0,                      /* has NO lightpen support on this port */
+    0,                      /* has NO joystick adapter on this port */
+    1,                      /* has output support on this port */
+    0                       /* port can be switched on/off */
+};
+
+static joyport_port_props_t joy_adapter_control_port_4 =
+{
+    "Joystick adapter port 4",
+    0,                      /* has NO potentiometer connected to this port */
+    0,                      /* has NO lightpen support on this port */
+    0,                      /* has NO joystick adapter on this port */
+    1,                      /* has output support on this port */
+    0                       /* port can be switched on/off */
+};
+
+static joyport_port_props_t joy_adapter_control_port_5 =
+{
+    "Joystick adapter port 5",
+    0,                      /* has NO potentiometer connected to this port */
+    0,                      /* has NO lightpen support on this port */
+    0,                      /* has NO joystick adapter on this port */
+    1,                      /* has output support on this port */
+    0                       /* port can be switched on/off */
+};
+
+static joyport_port_props_t joy_adapter_control_port_6 =
+{
+    "Joystick adapter port 6",
+    0,                      /* has NO potentiometer connected to this port */
+    0,                      /* has NO lightpen support on this port */
+    0,                      /* has NO joystick adapter on this port */
+    1,                      /* has output support on this port */
+    0                       /* port can be switched on/off */
+};
+
+static joyport_port_props_t joy_adapter_control_port_7 =
+{
+    "Joystick adapter port 7",
+    0,                      /* has NO potentiometer connected to this port */
+    0,                      /* has NO lightpen support on this port */
+    0,                      /* has NO joystick adapter on this port */
+    1,                      /* has output support on this port */
+    0                       /* port can be switched on/off */
+};
+
+static joyport_port_props_t joy_adapter_control_port_8 =
+{
+    "Joystick adapter port 8",
+    0,                      /* has NO potentiometer connected to this port */
+    0,                      /* has NO lightpen support on this port */
+    0,                      /* has NO joystick adapter on this port */
+    1,                      /* has output support on this port */
+    0                       /* port can be switched on/off */
+};
+
 static int init_joyport_ports(void)
 {
     if (joyport_port_register(JOYPORT_1, &control_port_1) < 0) {
         return -1;
     }
-    return joyport_port_register(JOYPORT_2, &control_port_2);
+    if (joyport_port_register(JOYPORT_2, &control_port_2) < 0) {
+        return -1;
+    }
+    if (joyport_port_register(JOYPORT_3, &joy_adapter_control_port_1) < 0) {
+        return -1;
+    }
+    if (joyport_port_register(JOYPORT_4, &joy_adapter_control_port_2) < 0) {
+        return -1;
+    }
+    if (joyport_port_register(JOYPORT_5, &joy_adapter_control_port_3) < 0) {
+        return -1;
+    }
+    if (joyport_port_register(JOYPORT_6, &joy_adapter_control_port_4) < 0) {
+        return -1;
+    }
+    if (joyport_port_register(JOYPORT_7, &joy_adapter_control_port_5) < 0) {
+        return -1;
+    }
+    if (joyport_port_register(JOYPORT_8, &joy_adapter_control_port_6) < 0) {
+        return -1;
+    }
+    if (joyport_port_register(JOYPORT_9, &joy_adapter_control_port_7) < 0) {
+        return -1;
+    }
+    return joyport_port_register(JOYPORT_10, &joy_adapter_control_port_8);
 }
 
 /* CBM-II-specific resource initialization.  This is called before initializing
@@ -209,20 +321,16 @@ int machine_resources_init(void)
         init_resource_fail("sid");
         return -1;
     }
+    /* Must be called after initializing cartridge resources. Some carts provide
+     * additional busses.  The drive resources check the validity of the drive
+     * type against the available busses on the system.  So if you had e.g. an
+     * IEEE cart enabled and an IEEE defined, on startup the drive code would
+     * reset the drive type to the default for the IEC bus. */
     if (drive_resources_init() < 0) {
         init_resource_fail("drive");
         return -1;
     }
-    /*
-     * This needs to be called before tapeport_resources_init(), otherwise
-     * the tapecart will fail to initialize due to the Datasette resource
-     * appearing after the Tapecart resources
-     */
-    if (datasette_resources_init() < 0) {
-        init_resource_fail("datasette");
-        return -1;
-    }
-    if (tapeport_resources_init() < 0) {
+    if (tapeport_resources_init(1) < 0) {
         init_resource_fail("tapeport");
         return -1;
     }
@@ -262,8 +370,28 @@ int machine_resources_init(void)
         init_resource_fail("joyport bbrtc");
         return -1;
     }
-    if (joyport_paperclip64_resources_init() < 0) {
-        init_resource_fail("joyport paperclip64 dongle");
+    if (joyport_trapthem_snespad_resources_init() < 0) {
+        init_resource_fail("joyport trapthem snespad");
+        return -1;
+    }
+    if (joyport_ninja_snespad_resources_init() < 0) {
+        init_resource_fail("joyport ninja snespad");
+        return -1;
+    }
+    if (joyport_protopad_resources_init() < 0) {
+        init_resource_fail("joyport protopad");
+        return -1;
+    }
+    if (joyport_inception_resources_init() < 0) {
+        init_resource_fail("joyport inception");
+        return -1;
+    }
+    if (joyport_multijoy_resources_init() < 0) {
+        init_resource_fail("joyport multijoy");
+        return -1;
+    }
+    if (joyport_io_sim_resources_init() < 0) {
+        init_resource_fail("joyport I/O simulation");
         return -1;
     }
     if (joystick_resources_init() < 0) {
@@ -390,10 +518,6 @@ int machine_cmdline_options_init(void)
         init_cmdline_options_fail("tapeport");
         return -1;
     }
-    if (datasette_cmdline_options_init() < 0) {
-        init_cmdline_options_fail("datasette");
-        return -1;
-    }
     if (acia1_cmdline_options_init() < 0) {
         init_cmdline_options_fail("acia1");
         return -1;
@@ -507,12 +631,6 @@ static void c500_powerline_clk_alarm_handler(CLOCK offset, void *data)
     SIGNAL_VERT_BLANK_ON
 }
 
-static void c500_powerline_clk_overflow_callback(CLOCK sub, void *data)
-{
-    c500_powerline_clk -= sub;
-}
-
-
 /*
  * C500 extra data (state of 50Hz clk)
  */
@@ -536,7 +654,7 @@ int cbm2_c500_snapshot_write_module(snapshot_t *p)
         return -1;
     }
 
-    SMW_DW(m, c500_powerline_clk - maincpu_clk);
+    SMW_CLOCK(m, c500_powerline_clk - maincpu_clk);
 
     snapshot_module_close(m);
 
@@ -547,7 +665,7 @@ int cbm2_c500_snapshot_read_module(snapshot_t *p)
 {
     uint8_t vmajor, vminor;
     snapshot_module_t *m;
-    uint32_t dword;
+    CLOCK qword;
 
     m = snapshot_module_open(p, module_name, &vmajor, &vminor);
     if (m == NULL) {
@@ -559,8 +677,8 @@ int cbm2_c500_snapshot_read_module(snapshot_t *p)
         return -1;
     }
 
-    SMR_DW(m, &dword);
-    c500_powerline_clk = maincpu_clk + dword;
+    SMR_CLOCK(m, &qword);
+    c500_powerline_clk = maincpu_clk + qword;
     alarm_set(c500_powerline_clk_alarm, c500_powerline_clk);
 
     snapshot_module_close(m);
@@ -652,8 +770,6 @@ int machine_specific_init(void)
                                          "C500PowerlineClk",
                                          c500_powerline_clk_alarm_handler,
                                          NULL);
-    clk_guard_add_callback(maincpu_clk_guard,
-                           c500_powerline_clk_overflow_callback, NULL);
     machine_timing.cycles_per_sec = C500_PAL_CYCLES_PER_SEC;
     machine_timing.rfsh_per_sec = C500_PAL_RFSH_PER_SEC;
     machine_timing.cycles_per_rfsh = C500_PAL_CYCLES_PER_RFSH;
@@ -767,12 +883,14 @@ void machine_specific_reset(void)
 
 void machine_specific_powerup(void)
 {
+    tapeport_powerup();
+    joyport_powerup();
 }
 
 void machine_specific_shutdown(void)
 {
     /* and the tape */
-    tape_image_detach_internal(1);
+    tape_image_detach_internal(TAPEPORT_PORT_1 + 1);
 
     ciacore_shutdown(machine_context.cia1);
     tpicore_shutdown(machine_context.tpi1);
@@ -790,7 +908,7 @@ void machine_specific_shutdown(void)
     }
 }
 
-void machine_handle_pending_alarms(int num_write_cycles)
+void machine_handle_pending_alarms(CLOCK num_write_cycles)
 {
 }
 
@@ -799,19 +917,9 @@ void machine_handle_pending_alarms(int num_write_cycles)
 /* This hook is called at the end of every frame.  */
 static void machine_vsync_hook(void)
 {
-    CLOCK sub;
-
     drive_vsync_hook();
 
-    autostart_advance();
-
     screenshot_record();
-
-    sub = clk_guard_prevent_overflow(maincpu_clk_guard);
-
-    /* The drive has to deal both with our overflowing and its own one, so
-       it is called even when there is no overflowing in the main CPU.  */
-    drive_cpu_prevent_clk_overflow_all(sub);
 }
 
 /* Dummy - no restore key.  */
@@ -877,9 +985,7 @@ void machine_change_timing(int timeval, int border_mode)
 #ifdef HAVE_MOUSE
     neos_mouse_set_machine_parameter(machine_timing.cycles_per_sec);
 #endif
-    clk_guard_set_clk_base(maincpu_clk_guard,
-                           (CLOCK)machine_timing.cycles_per_rfsh);
-
+    
     vicii_change_timing(&machine_timing, border_mode);
     cia1_set_timing(machine_context.cia1,
                     (int)machine_timing.cycles_per_sec,
@@ -931,6 +1037,16 @@ int machine_autodetect_psid(const char *name)
     return -1;
 }
 
+userport_desc_t *userport_get_valid_devices(int sort)
+{
+    return NULL;
+}
+
+const char *userport_get_device_type_desc(int type)
+{
+    return NULL;
+}
+
 int machine_screenshot(screenshot_t *screenshot, struct video_canvas_s *canvas)
 {
     if (canvas == vicii_get_canvas()) {
@@ -970,7 +1086,21 @@ uint8_t machine_tape_behaviour(void)
 int machine_addr_in_ram(unsigned int addr)
 {
     /* FIXME: handle the banking */
-    return (addr < 0xe000 && !(addr >= 0x8000 && addr < 0xc000)) ? 1 : 0;
+    
+    if (addr >= 0x25a && addr <= 0x25d) {
+        /* 'Pickup subroutine' */
+        return 0;
+    }
+    
+    if (addr >= 0x8000 && addr < 0xc000) {
+        return 0;
+    }
+    
+    if (addr > 0xe000) {
+        return 0;
+    }
+    
+    return 1;
 }
 
 const char *machine_get_name(void)
