@@ -79,7 +79,7 @@ int cbm2rom_load_chargen(const char *rom_name)
     if (!util_check_null_string(rom_name)) {
         memset(mem_chargen_rom, 0, CBM2_CHARGEN_ROM_SIZE);
 
-        if (sysfile_load(rom_name, mem_chargen_rom, 4096, 4096) < 0) {
+        if (sysfile_load(rom_name, machine_name, mem_chargen_rom, 4096, 4096) < 0) {
             log_error(cbm2rom_log, "Couldn't load character ROM '%s'.",
                       rom_name);
             return -1;
@@ -121,7 +121,7 @@ int cbm2rom_load_kernal(const char *rom_name)
 
     /* Load Kernal ROM.  */
     if (!util_check_null_string(rom_name)) {
-        if (sysfile_load(rom_name, mem_rom + 0xe000, 0x2000, 0x2000) < 0) {
+        if (sysfile_load(rom_name, machine_name, mem_rom + 0xe000, 0x2000, 0x2000) < 0) {
             log_error(cbm2rom_log, "Couldn't load ROM `%s'.", rom_name);
             return -1;
         }
@@ -137,7 +137,7 @@ int cbm2rom_load_basic(const char *rom_name)
     }
     /* Load BASIC ROM.  */
     if (!util_check_null_string(rom_name)) {
-        if ((sysfile_load(rom_name, mem_rom + 0x8000, 0x4000, 0x4000) < 0)) {
+        if ((sysfile_load(rom_name, machine_name, mem_rom + 0x8000, 0x4000, 0x4000) < 0)) {
             log_error(cbm2rom_log, "Couldn't load BASIC ROM `%s'.",
                       rom_name);
             return -1;
@@ -155,7 +155,7 @@ int cbm2rom_load_cart_1(const char *rom_name)
         return 0;  /* init not far enough */
     }
     if (!util_check_null_string(rom_name)) {
-        if ((sysfile_load(rom_name, mem_rom + 0x1000, 0x1000, 0x1000) < 0)) {
+        if ((sysfile_load(rom_name, machine_name, mem_rom + 0x1000, 0x1000, 0x1000) < 0)) {
             log_error(cbm2rom_log, "Couldn't load ROM `%s'.",
                       rom_name);
         }
@@ -171,7 +171,7 @@ int cbm2rom_load_cart_2(const char *rom_name)
         return 0;  /* init not far enough */
     }
     if (!util_check_null_string(rom_name)) {
-        if ((sysfile_load(rom_name, mem_rom + 0x2000, 0x2000, 0x2000) < 0)) {
+        if ((sysfile_load(rom_name, machine_name, mem_rom + 0x2000, 0x2000, 0x2000) < 0)) {
             log_error(cbm2rom_log, "Couldn't load ROM `%s'.",
                       rom_name);
         }
@@ -187,7 +187,7 @@ int cbm2rom_load_cart_4(const char *rom_name)
         return 0;  /* init not far enough */
     }
     if (!util_check_null_string(rom_name)) {
-        if ((sysfile_load(rom_name, mem_rom + 0x4000, 0x2000, 0x2000) < 0)) {
+        if ((sysfile_load(rom_name, machine_name, mem_rom + 0x4000, 0x2000, 0x2000) < 0)) {
             log_error(cbm2rom_log, "Couldn't load ROM `%s'.",
                       rom_name);
         }
@@ -203,7 +203,7 @@ int cbm2rom_load_cart_6(const char *rom_name)
         return 0;  /* init not far enough */
     }
     if (!util_check_null_string(rom_name)) {
-        if ((sysfile_load(rom_name, mem_rom + 0x6000, 0x2000, 0x2000) < 0)) {
+        if ((sysfile_load(rom_name, machine_name, mem_rom + 0x6000, 0x2000, 0x2000) < 0)) {
             log_error(cbm2rom_log, "Couldn't load ROM `%s'.",
                       rom_name);
         }
