@@ -36,6 +36,7 @@
 #include "archdep.h"
 #include "cmdline.h"
 #include "lib.h"
+#include "log.h"
 #include "resources.h"
 #include "types.h"
 #include "uicmdline.h"
@@ -377,3 +378,77 @@ int cmdline_get_num_options(void)
 {
     return num_options;
 }
+
+void cmdline_log_active(void)
+{
+    unsigned int i;
+    char *cmdline, *cmd;
+
+    cmdline = lib_strdup("-default");
+
+    for (i = 0; i < num_options; i++) {
+        const char *param = cmdline_options_get_param(i);
+        const char *resname = options[i].resource_name;
+        const char *optname = options[i].name;
+        const char *resval_str = NULL;
+        char *resval_str_default = NULL;
+        int restype = -1;
+        int resval_int = -1;
+        int resval_int_default = -1;
+        if (resname) {
+            restype = resources_query_type(resname);
+            if (restype == RES_INTEGER) {
+                resources_get_int(resname, &resval_int);
+                resources_get_default_value(resname, &resval_int_default);
+            } else if (restype == RES_STRING) {
+                resources_get_string(resname, &resval_str);
+                resources_get_default_value(resname, &resval_str_default);
+            }
+        }
+        cmd = NULL;
+        if ((options[i].attributes & CMDLINE_ATTRIB_NEED_ARGS) && param != NULL) {
+            /* the cmdline option needs a parameter, we assume this is what the resource got assigned */
+            if (restype == RES_INTEGER) {
+                if (resval_int != resval_int_default) {
+                    char tmp[32];
+                    sprintf(tmp, "%d", resval_int);
+                    cmd = util_concat(optname, " \"", tmp, "\"", NULL);
+                }
+            } else if (restype == RES_STRING) {
+                if ((resval_str != NULL) && (resval_str_default != NULL)) {
+                    if (strcmp(resval_str, resval_str_default)) {
+                        cmd = util_concat(optname, " \"", resval_str, "\"", NULL);
+                    }
+                }
+            }
+        } else {
+            /* the options does not need a parameter, the resource value should match the value
+               defined in the option itself */
+            if (restype == RES_INTEGER) {
+                if (resval_int != resval_int_default) {
+                    if (resval_int == vice_ptr_to_int(options[i].resource_value)) {
+                        cmd = lib_strdup(optname);
+                    }
+                }
+            } else if (restype == RES_STRING) {
+                if ((resval_str != NULL) && (resval_str_default != NULL) && (options[i].resource_value != NULL)) {
+                    if (strcmp(resval_str, resval_str_default)) {
+                        if (!strcmp(resval_str, options[i].resource_value)) {
+                            cmd = lib_strdup(optname);
+                        }
+                    }
+                }
+            }
+        }
+        if (cmd) {
+            char *p;
+            p = cmdline; /* remember old pointer */
+            cmdline = util_concat(p, " ", cmd, NULL);
+            lib_free(p); /* free old pointer */
+            lib_free(cmd); /* free old pointer */
+        }
+    }
+    log_message(LOG_DEFAULT, "\nreconstructed commandline options (might be incomplete):");
+    log_message(LOG_DEFAULT, "%s\n", cmdline);
+    lib_free(cmdline);
+}
