@@ -47,6 +47,8 @@ typedef struct console_s {
     struct console_private_s *private;
 } console_t;
 
+/* first set of functions, which will work with whatever "console" the respective
+   (g)ui/port comes up with. may or may not be equal to the native console */
 extern console_t *console_open(const char *id);
 extern int console_close(console_t *log);
 
@@ -65,6 +67,26 @@ extern int console_out(console_t *log, const char *format, ...);
 extern int console_flush(console_t *log);
 extern char *console_in(console_t *log, const char *prompt);
 
+/* the same set of functions, which will work with a "native" console of the host */
+extern console_t *native_console_open(const char *id);
+extern int native_console_close(console_t *log);
+
+/* the following must be called before any other
+  native_console_...() function is used */
+extern int native_console_init(void);
+
+/* The following should be called when quitting VICE
+ after calling native_console_close_all(), the native_console_...()
+ functions cannot be accessed unless native_console_init()
+ is called again.
+*/
+extern int native_console_close_all(void);
+
+extern int native_console_out(console_t *log, const char *format, ...);
+extern int native_console_petscii_out(console_t *log, const char *format, ...);
+extern int native_console_flush(console_t *log);
+extern char *native_console_in(console_t *log, const char *prompt);
+
 # ifndef HAVE_READLINE
 char *readline(const char *prompt);
 # endif
