@@ -50,11 +50,6 @@
 #include "maincpu.h"
 #include "monitor.h"
 
-#ifndef HAVE_FSEEKO
-#define fseeko(a, b, c) fseek(a, b, c)
-#define ftello(a) ftell(a)
-#endif
-
 #define ATA_UNC  0x40
 #define ATA_IDNF 0x10
 #define ATA_ABRT 0x04
@@ -1375,9 +1370,9 @@ int ata_register_dump(ata_drive_t *drv)
 int ata_snapshot_write_module(ata_drive_t *drv, snapshot_t *s)
 {
     snapshot_module_t *m;
-    uint32_t spindle_clk = CLOCK_MAX;
-    uint32_t head_clk = CLOCK_MAX;
-    uint32_t standby_clk = CLOCK_MAX;
+    CLOCK spindle_clk = CLOCK_MAX;
+    CLOCK head_clk = CLOCK_MAX;
+    CLOCK standby_clk = CLOCK_MAX;
     off_t pos = 0;
 
     m = snapshot_module_create(s, drv->myname,
@@ -1429,9 +1424,9 @@ int ata_snapshot_write_module(ata_drive_t *drv, snapshot_t *s)
     SMW_B(m, (uint8_t)drv->wcache);
     SMW_B(m, (uint8_t)drv->lookahead);
     SMW_B(m, (uint8_t)drv->busy);
-    SMW_DW(m, spindle_clk);
-    SMW_DW(m, head_clk);
-    SMW_DW(m, standby_clk);
+    SMW_CLOCK(m, spindle_clk);
+    SMW_CLOCK(m, head_clk);
+    SMW_CLOCK(m, standby_clk);
     SMW_DW(m, drv->standby);
     SMW_DW(m, drv->standby_max);
 
@@ -1443,9 +1438,9 @@ int ata_snapshot_read_module(ata_drive_t *drv, snapshot_t *s)
     uint8_t vmajor, vminor;
     snapshot_module_t *m;
     char *filename = NULL;
-    uint32_t spindle_clk;
-    uint32_t head_clk;
-    uint32_t standby_clk;
+    CLOCK spindle_clk;
+    CLOCK head_clk;
+    CLOCK standby_clk;
     int pos, type;
 
     m = snapshot_module_open(s, drv->myname, &vmajor, &vminor);
@@ -1453,16 +1448,24 @@ int ata_snapshot_read_module(ata_drive_t *drv, snapshot_t *s)
         return -1;
     }
 
-    if ((vmajor != CART_DUMP_VER_MAJOR) || (vminor != CART_DUMP_VER_MINOR)) {
+    if (!snapshot_version_is_equal(vmajor, vminor, CART_DUMP_VER_MAJOR, CART_DUMP_VER_MINOR)) {
+        snapshot_set_error(SNAPSHOT_MODULE_INCOMPATIBLE);
         snapshot_module_close(m);
         return -1;
     }
 
     SMR_STR(m, &filename);
     if (!drv->filename || strcmp(filename, drv->filename)) {
+        log_warning(drv->log, "IDE image filename mismatch. expected: %s got: %s\n",
+                    filename, drv->filename);
+    /* FIXME: this is really retarded - we should instead identify the image by
+              making a checksum instead of using the filename */
+#if 1
+        snapshot_set_error(SNAPSHOT_ATA_IMAGE_FILENAME_MISMATCH);
         lib_free(filename);
         snapshot_module_close(m);
         return -1;
+#endif
     }
     lib_free(filename);
     SMR_DW_INT(m, &type);
@@ -1536,9 +1539,9 @@ int ata_snapshot_read_module(ata_drive_t *drv, snapshot_t *s)
         drv->lookahead = 1;
     }
     SMR_B_INT(m, &drv->busy);
-    SMR_DW(m, &spindle_clk);
-    SMR_DW(m, &head_clk);
-    SMR_DW(m, &standby_clk);
+    SMR_CLOCK(m, &spindle_clk);
+    SMR_CLOCK(m, &head_clk);
+    SMR_CLOCK(m, &standby_clk);
     SMR_DW_INT(m, &drv->standby);
     SMR_DW_INT(m, &drv->standby_max);
     drv->busy &= 0x03;
