@@ -38,6 +38,7 @@
 #include "types.h"
 #include "snapshot.h"
 #include "scsi.h"
+#include "util.h"
 
 /* #define SCSILOG1 */
 /* #define SCSILOG2 */
@@ -66,13 +67,43 @@
 
 #define CRIT(_x_) log_message _x_
 
+#define MAXIDS 7
+#define MAXLUNS 8
+
+static off_t scsi_getmaxsize(struct scsi_context_s *context)
+{
+    off_t work;
+
+    /* make sure the target and lun are valid */
+    if (context->target >= MAXIDS || context->lun >= MAXLUNS) {
+        return 0;
+    }
+
+    /* make sure there is a file associated with the target and lun */
+    if (context->file[(context->target << 3) | context->lun]) {
+        if (!context->max_imagesize) {
+            /* if the max length setting is zero, return the image length */
+            work = util_file_length(context->file[(context->target << 3) | context->lun]);
+            /* turn the file size into 512 byte sectors */
+            work = (work >> 9) + (work & 511 ? 1 : 0);
+            return work;
+        } else {
+            /* other wise, return the setting itself */
+            return context->max_imagesize;
+        }
+    }
+
+    /* not a valid file, return 0 */
+    return 0;
+}
+
 static int32_t scsi_imagecheck(struct scsi_context_s *context)
 {
-    if (context->target >= context->max_ids || context->lun > 0) {
+    if (context->target >= MAXIDS || context->lun >= MAXLUNS) {
         return 1;
     }
 
-    if (!context->file[context->target]) {
+    if (!context->file[(context->target << 3) | context->lun]) {
         if ( context->target == 0 && context->lun == 0 &&
             !(context->log & SCSI_LOG_NODISK0) ) {
             CRIT((ERR, "SCSI: no image attached to disk 0;"
@@ -85,15 +116,15 @@ static int32_t scsi_imagecheck(struct scsi_context_s *context)
     return 0;
 }
 
-int scsi_image_detach(struct scsi_context_s *context, int id)
+int scsi_image_detach(struct scsi_context_s *context, int disk)
 {
-    if (id < 0 || id > 6) {
+    if (disk < 0 || disk > 55) {
         return 2;
     }
 
-    if (context->file[id]) {
-        fclose(context->file[id]);
-        context->file[id] = NULL;
+    if (context->file[disk]) {
+        fclose(context->file[disk]);
+        context->file[disk] = NULL;
         return 0;
     }
 
@@ -104,24 +135,24 @@ void scsi_image_detach_all(struct scsi_context_s *context)
 {
     int32_t i;
 
-    for (i = 0; i < 7; i++) {
+    for (i = 0; i < 56; i++) {
         scsi_image_detach(context, i);
     }
 
     return;
 }
 
-int scsi_image_attach(struct scsi_context_s *context, int id, char *filename)
+int scsi_image_attach(struct scsi_context_s *context, int disk, char *filename)
 {
-    if (id < 0 || id > 6) {
+    if (disk < 0 || disk > 55) {
         return 2;
     }
 
-    scsi_image_detach(context, id);
-    context->file[id] = fopen(filename, "rb+");
+    scsi_image_detach(context, disk);
+    context->file[disk] = fopen(filename, "rb+");
 
-    if (context->file[id]) {
-        setbuf(context->file[id], NULL);
+    if (context->file[disk]) {
+        setbuf(context->file[disk], NULL);
         return 0;
     } else {
         return 1;
@@ -137,9 +168,9 @@ int32_t scsi_image_read(struct scsi_context_s *context)
         return -1;
     }
 
-    fhd = context->file[context->target];
+    fhd = context->file[(context->target << 3) | context->lun];
 
-    if (fseeko(fhd, context->address * 512, SEEK_SET) < 0) {
+    if (fseeko(fhd, (off_t)context->address * 512, SEEK_SET) < 0) {
         CRIT((ERR, "SCSI: error seeking disk %d at sector 0x%x",
             context->target, context->address));
         return -3;
@@ -185,9 +216,9 @@ int32_t scsi_image_write(struct scsi_context_s *context)
         context->user_write(context);
     }
 
-    fhd = context->file[context->target];
+    fhd = context->file[(context->target << 3) | context->lun];
 
-    if (fseeko(fhd, context->address * 512, SEEK_SET) < 0) {
+    if (fseeko(fhd, (off_t)context->address * 512, SEEK_SET) < 0) {
         CRIT((ERR, "SCSI: error seeking disk %d at sector 0x%x",
             context->target, context->address));
         return -3;
@@ -208,7 +239,7 @@ int32_t scsi_image_write(struct scsi_context_s *context)
 
 /* default format handler */
 /* We don't actually format the disk, we just zero out the first sector */
-void scsi_format_sector0(struct scsi_context_s *context)
+static void scsi_format_sector0(struct scsi_context_s *context)
 {
     int32_t i;
 
@@ -317,7 +348,7 @@ void scsi_process_ack(struct scsi_context_s *context)
 {
     uint8_t data = context->databus ^ 0xff;
     uint8_t byte;
-    uint32_t j;
+    off_t j;
     int32_t i;
     unsigned char VENDORID[9] = "VICEEMUL"; /* 8 chars */
     unsigned char PRODID[17] = "SCSI Image File "; /* 16 chars */
@@ -376,7 +407,8 @@ void scsi_process_ack(struct scsi_context_s *context)
             if (context->seq >= context->data_max) {
                 /* write if WRITE command */
                 if (context->command == SCSI_COMMAND_WRITE_6 ||
-                    context->command == SCSI_COMMAND_WRITE_10) {
+                    context->command == SCSI_COMMAND_WRITE_10 ||
+                    context->command == SCSI_COMMAND_WRITE_VERIFY) {
                     if (scsi_image_write(context)) {
                         context->status = SCSI_STATUS_CHECKCONDITION;
                         context->state = SCSI_STATE_STATUS;
@@ -392,9 +424,7 @@ void scsi_process_ack(struct scsi_context_s *context)
                         context->state = SCSI_STATE_STATUS;
                         break;
                     }
-                    j = context->max_imagesize;
-                    j = j >> 9;
-                    if (context->address >= j) {
+                    if (context->address >= scsi_getmaxsize(context)) {
                         context->sensekey = SCSI_SENSEKEY_ILLEGALREQUEST;
                         context->asc = SCSI_SASC_LOGICALBLOCKADDRESSOUTOFRANGE;
                         context->status = SCSI_STATUS_CHECKCONDITION;
@@ -453,22 +483,38 @@ void scsi_process_ack(struct scsi_context_s *context)
                 context->command = context->cmd_buf[0];
                 switch (context->command) {
                 case SCSI_COMMAND_TEST_UNIT_READY:
+                    /* fall through */
                 case SCSI_COMMAND_REZERO_UNIT:
+                    /* fall through */
                 case SCSI_COMMAND_REQUEST_SENSE:
+                    /* fall through */
                 case SCSI_COMMAND_FORMAT_UNIT:
+                    /* fall through */
                 case SCSI_COMMAND_REASSIGN_BLOCKS:
+                    /* fall through */
                 case SCSI_COMMAND_READ_6:
+                    /* fall through */
                 case SCSI_COMMAND_WRITE_6:
+                    /* fall through */
                 case SCSI_COMMAND_INQUIRY:
+                    /* fall through */
                 case SCSI_COMMAND_MODE_SENSE:
+                    /* fall through */
                 case SCSI_COMMAND_START_STOP:
+                    /* fall through */
                 case SCSI_COMMAND_SEND_DIAGNOSTIC:
                     context->cmd_size = 6;
                     break;
                 case SCSI_COMMAND_READ_CAPACITY:
+                    /* fall through */
                 case SCSI_COMMAND_READ_10:
+                    /* fall through */
                 case SCSI_COMMAND_WRITE_10:
+                    /* fall through */
                 case SCSI_COMMAND_MODE_SENSE_10:
+                    /* fall through */
+                case SCSI_COMMAND_WRITE_VERIFY:
+                    /* fall through */
                 case SCSI_COMMAND_VERIFY:
                     context->cmd_size = 10;
                     break;
@@ -486,7 +532,7 @@ void scsi_process_ack(struct scsi_context_s *context)
                 switch (context->command) {
                 case SCSI_COMMAND_TEST_UNIT_READY:
                     context->lun = (context->cmd_buf[1] >> 5) & 7;
-                    context->link = context->cmd_buf[5]&1;
+                    context->link = context->cmd_buf[5] & 1;
                     if (scsi_imagecheck(context)) {
                         context->sensekey = SCSI_SENSEKEY_ILLEGALREQUEST;
                         context->status = SCSI_STATUS_CHECKCONDITION;
@@ -498,8 +544,14 @@ void scsi_process_ack(struct scsi_context_s *context)
                     break;
                 case SCSI_COMMAND_REQUEST_SENSE:
                     context->lun = (context->cmd_buf[1] >> 5) & 7;
-                    context->link = context->cmd_buf[5]&1;
-                    context->data_max = 18;
+                    context->link = context->cmd_buf[5] & 1;
+                    context->data_max = context->cmd_buf[4];
+                    if (!context->data_max) {
+                        context->data_max = 4;
+                    }
+                    if (context->data_max > 18) {
+                        context->data_max = 18;
+                    }
                     for (i = 0; i < 512; i++) {
                         context->data_buf[i] = 0;
                     }
@@ -514,12 +566,14 @@ void scsi_process_ack(struct scsi_context_s *context)
                     context->state = SCSI_STATE_DATAOUT;
                     context->data_max = 4;
                     context->seq = 0;
+                    break;
                 case SCSI_COMMAND_READ_6:
+                    /* fall through */
                 case SCSI_COMMAND_READ_10:
                     context->lun = (context->cmd_buf[1] >> 5) & 7;
                     if (context->command == SCSI_COMMAND_READ_6) {
-                        context->link = context->cmd_buf[5]&1;
-                        context->address = ((context->cmd_buf[1]&0x1f) << 16)|
+                        context->link = context->cmd_buf[5] & 1;
+                        context->address = ((context->cmd_buf[1] & 0x1f) << 16)|
                             (context->cmd_buf[2] << 8)|(context->cmd_buf[3]);
                         context->blocks = context->cmd_buf[4];
                         /* if blocks=0, it really means 256 */
@@ -527,7 +581,7 @@ void scsi_process_ack(struct scsi_context_s *context)
                             context->blocks = 256;
                         }
                     } else {
-                        context->link = context->cmd_buf[9]&1;
+                        context->link = context->cmd_buf[9] & 1;
                         context->address = (context->cmd_buf[2] << 24)|
                             (context->cmd_buf[3] << 16)|
                             (context->cmd_buf[4] << 8)|(context->cmd_buf[5]);
@@ -541,7 +595,7 @@ void scsi_process_ack(struct scsi_context_s *context)
                         }
                     }
                     context->data_max = 512;
-                    if (context->address >= (context->max_imagesize >> 9)) {
+                    if (context->address >= scsi_getmaxsize(context)) {
                         context->sensekey = SCSI_SENSEKEY_ILLEGALREQUEST;
                         context->asc = SCSI_SASC_LOGICALBLOCKADDRESSOUTOFRANGE;
                         context->status = SCSI_STATUS_CHECKCONDITION;
@@ -557,11 +611,14 @@ void scsi_process_ack(struct scsi_context_s *context)
                     }
                     break;
                 case SCSI_COMMAND_WRITE_6:
+                    /* fall through */
                 case SCSI_COMMAND_WRITE_10:
+                    /* fall through */
+                case SCSI_COMMAND_WRITE_VERIFY:
                     context->lun = (context->cmd_buf[1] >> 5) & 7;
                     if (context->command == SCSI_COMMAND_WRITE_6) {
-                        context->link = context->cmd_buf[5]&1;
-                        context->address = ((context->cmd_buf[1]&0x1f) << 16)|
+                        context->link = context->cmd_buf[5] & 1;
+                        context->address = ((context->cmd_buf[1] & 0x1f) << 16)|
                             (context->cmd_buf[2] << 8)|(context->cmd_buf[3]);
                         context->blocks = context->cmd_buf[4];
                         /* if blocks=0, it really means 256 */
@@ -569,7 +626,7 @@ void scsi_process_ack(struct scsi_context_s *context)
                             context->blocks = 256;
                         }
                     } else {
-                        context->link = context->cmd_buf[9]&1;
+                        context->link = context->cmd_buf[9] & 1;
                         context->address = (context->cmd_buf[2] << 24)|
                             (context->cmd_buf[3] << 16)|
                             (context->cmd_buf[4] << 8)|(context->cmd_buf[5]);
@@ -583,7 +640,7 @@ void scsi_process_ack(struct scsi_context_s *context)
                         }
                     }
                     context->data_max = 512;
-                    if (context->address >= (context->max_imagesize >> 9)) {
+                    if (context->address >= scsi_getmaxsize(context)) {
                         context->sensekey = SCSI_SENSEKEY_ILLEGALREQUEST;
                         context->asc = SCSI_SASC_LOGICALBLOCKADDRESSOUTOFRANGE;
                         context->status = SCSI_STATUS_CHECKCONDITION;
@@ -600,7 +657,7 @@ void scsi_process_ack(struct scsi_context_s *context)
                     break;
                 case SCSI_COMMAND_INQUIRY:
                     context->lun = (context->cmd_buf[1] >> 5) & 7;
-                    context->link = context->cmd_buf[5]&1;
+                    context->link = context->cmd_buf[5] & 1;
                     context->data_max = context->cmd_buf[4]; /* usually 96 */
                     for (i = 0; i < 512; i++) {
                         context->data_buf[i] = 0;
@@ -634,41 +691,47 @@ void scsi_process_ack(struct scsi_context_s *context)
                     break;
                 case SCSI_COMMAND_READ_CAPACITY:
                     context->lun = (context->cmd_buf[1] >> 5) & 7;
-                    context->link = context->cmd_buf[9]&1;
+                    context->link = context->cmd_buf[9] & 1;
                     context->data_max = 8;
-                    if (context->lun == 0) {
-                        j = (context->max_imagesize >> 9) - 1;
-                        i = 512;
-                    } else {
-                        j = 0;
+                    j = scsi_getmaxsize(context);
+                    if (j == 0) {
                         i = 0;
+                    } else {
+/* Limit size returned as some tools can't handle large disks */
+                        if (j > context->limit_imagesize) {
+                            j = context->limit_imagesize;
+                        }
+                        i = 512;
+/* SCSI spec says return last accessable block number */
+                        j--;
                     }
-                    context->data_buf[0] = (j >> 24)&255;
-                    context->data_buf[1] = (j >> 16)&255;
-                    context->data_buf[2] = (j >> 8)&255;
+                    context->data_buf[0] = (j >> 24) & 255;
+                    context->data_buf[1] = (j >> 16) & 255;
+                    context->data_buf[2] = (j >> 8) & 255;
                     context->data_buf[3] = j & 255;
-                    context->data_buf[4] = (i >> 24)&255;
-                    context->data_buf[5] = (i >> 16)&255;
-                    context->data_buf[6] = (i >> 8)&255;
+                    context->data_buf[4] = (i >> 24) & 255;
+                    context->data_buf[5] = (i >> 16) & 255;
+                    context->data_buf[6] = (i >> 8) & 255;
                     context->data_buf[7] = i & 255;
                     context->state = SCSI_STATE_DATAIN;
                     context->seq = 0;
                     break;
                 case SCSI_COMMAND_MODE_SENSE:
+                    /* fall through */
                 case SCSI_COMMAND_MODE_SENSE_10:
                     context->state = SCSI_STATE_DATAIN;
                     context->lun = (context->cmd_buf[1] >> 5) & 7;
                     context->seq = 0;
                     if (context->command == SCSI_COMMAND_MODE_SENSE) {
                         context->data_max = context->cmd_buf[4];
-                        context->link = context->cmd_buf[5]&1;
+                        context->link = context->cmd_buf[5] & 1;
                     } else {
                         context->data_max = (context->cmd_buf[7] << 8)|
                             (context->cmd_buf[8]);
                         if (context->data_max > 255) {
                             context->data_max = 255;
                         }
-                        context->link = context->cmd_buf[9]&1;
+                        context->link = context->cmd_buf[9] & 1;
                     }
                     for (i = 0; i < 512; i++) {
                         context->data_buf[i] = 0;
@@ -695,14 +758,14 @@ void scsi_process_ack(struct scsi_context_s *context)
                     break;
                 case SCSI_COMMAND_FORMAT_UNIT:
                     context->lun = (context->cmd_buf[1] >> 5) & 7;
-                    context->link = context->cmd_buf[5]&1;
+                    context->link = context->cmd_buf[5] & 1;
                     /* check if fmtdata is set, move on to dataout */
-                    if ((context->cmd_buf[1]&0x17) == 0x10) {
+                    if ((context->cmd_buf[1] & 0x17) == 0x10) {
                         context->state = SCSI_STATE_DATAOUT;
                         context->seq = 0;
                         context->data_max = 4;
                     /* cmplist can be anything, but list format must be 0 */
-                    } else if ((context->cmd_buf[1]&0x17) == 0x00) {
+                    } else if ((context->cmd_buf[1] & 0x17) == 0x00) {
                         context->status = SCSI_STATUS_GOOD;
                         context->state = SCSI_STATE_STATUS;
                         context->user_format(context);
@@ -714,16 +777,18 @@ void scsi_process_ack(struct scsi_context_s *context)
                     }
                     break;
                 case SCSI_COMMAND_REZERO_UNIT:
+                    /* fall through */
                 case SCSI_COMMAND_START_STOP:
+                    /* fall through */
                 case SCSI_COMMAND_SEND_DIAGNOSTIC:
                     context->lun = (context->cmd_buf[1] >> 5) & 7;
-                    context->link = context->cmd_buf[5]&1;
+                    context->link = context->cmd_buf[5] & 1;
                     context->status = SCSI_STATUS_GOOD;
                     context->state = SCSI_STATE_STATUS;
                     break;
                 case SCSI_COMMAND_VERIFY:
                     context->lun = (context->cmd_buf[1] >> 5) & 7;
-                    context->link = context->cmd_buf[9]&1;
+                    context->link = context->cmd_buf[9] & 1;
                     context->status = SCSI_STATUS_GOOD;
                     context->state = SCSI_STATE_STATUS;
                     break;
@@ -741,32 +806,35 @@ void scsi_process_ack(struct scsi_context_s *context)
                     context->seq = 0;
                     context->blocks--;
                     context->address++;
-                    if (context->address >= (context->max_imagesize >> 9)) {
+                    /* count down the number of blocks sent, keep going if
+                        necessary */
+                    if (!context->blocks) {
+                        context->status = SCSI_STATUS_GOOD;
+                        context->state = SCSI_STATE_STATUS;
+                        break;
+                    }
+                    /* if we are out of range, report an error */
+                    if (context->address >= scsi_getmaxsize(context)) {
                         context->sensekey = SCSI_SENSEKEY_ILLEGALREQUEST;
                         context->asc = SCSI_SASC_LOGICALBLOCKADDRESSOUTOFRANGE;
                         context->status = SCSI_STATUS_CHECKCONDITION;
                         context->state = SCSI_STATE_STATUS;
                         break;
                     }
-                    /* count down the number of blocks sent, keep going if
-                        necessary */
-                    if (context->blocks) {
-                        if (scsi_image_read(context)) {
-                            context->status = SCSI_STATUS_CHECKCONDITION;
-                            context->state = SCSI_STATE_STATUS;
-                            break;
-                        }
-                    } else {
-                        context->status = SCSI_STATUS_GOOD;
+                    /* read it, report error if a problem */
+                    if (scsi_image_read(context)) {
+                        context->status = SCSI_STATUS_CHECKCONDITION;
                         context->state = SCSI_STATE_STATUS;
                         break;
                     }
+                    /* all is good, keep state the same */
                 } else {
                     /* otherwise, finish up */
                     context->status = SCSI_STATUS_GOOD;
                     context->state = SCSI_STATE_STATUS;
                     break;
                 }
+                /* never gets here */
             }
             byte = context->data_buf[context->seq];
             SDBG((LOG, "SCSI: DATAIN[%02x]=%02x", context->seq,
@@ -779,7 +847,7 @@ void scsi_process_ack(struct scsi_context_s *context)
     }
 
     if (context->state == SCSI_STATE_STATUS) {
-        byte = context->status;
+        byte = (context->status << 1) | 0;
         SDBG((LOG, "SCSI: STATUS=%02x", context->status));
         context->io = 1;
         context->databus = byte ^ 0xff;
@@ -796,12 +864,12 @@ out:
 
 void scsi_reset(struct scsi_context_s *context)
 {
-    context->max_imagesize = 64*1024*1024;
+    context->max_imagesize = 512 * 1024 * 1024 / 512;
+    context->limit_imagesize = 512 * 1024 * 1024 / 512;
     context->rst = 1;
     scsi_process_noack(context);
     context->rst = 0;
     context->msg_after_status = 0;
-    context->max_ids = 7;
     context->user_format = scsi_format_sector0;
     context->user_read = NULL;
     context->user_write = NULL;
@@ -835,7 +903,6 @@ void scsi_reset(struct scsi_context_s *context)
    BYTE   | command            | command
    BYTE   | sensekey           | sensekey
    BYTE   | asc                | asc
-   BYTE   | max_ids            | max_ids
    BYTE   | msg_after_status   | msg_after_status
    DWORD  | seq                | seq
    DWORD  | cmd_size           | cmd_size
@@ -881,7 +948,6 @@ int scsi_snapshot_write_module(struct scsi_context_s *context, snapshot_t *s)
         || (SMW_B(m, (uint8_t)context->command) < 0)
         || (SMW_B(m, (uint8_t)context->sensekey) < 0)
         || (SMW_B(m, (uint8_t)context->asc) < 0)
-        || (SMW_B(m, (uint8_t)context->max_ids) < 0)
         || (SMW_B(m, (uint8_t)context->msg_after_status) < 0)
         || (SMW_DW(m, (uint32_t)context->seq) < 0)
         || (SMW_DW(m, (uint32_t)context->cmd_size) < 0)
@@ -935,7 +1001,6 @@ int scsi_snapshot_read_module(struct scsi_context_s *context, snapshot_t *s)
         || (SMR_B(m, &context->command) < 0)
         || (SMR_B(m, &context->sensekey) < 0)
         || (SMR_B(m, &context->asc) < 0)
-        || (SMR_B(m, &context->max_ids) < 0)
         || (SMR_B(m, &context->msg_after_status) < 0)
         || (SMR_DW(m, &context->seq) < 0)
         || (SMR_DW(m, &context->cmd_size) < 0)
