@@ -38,6 +38,7 @@
 #include "crtctypes.h"
 #include "log.h"
 #include "maincpu.h"
+#include "raster-snapshot.h"
 #include "snapshot.h"
 #include "types.h"
 
@@ -46,12 +47,12 @@
 
 static char snap_module_name[] = "CRTC";
 #define SNAP_MAJOR 1
-#define SNAP_MINOR 1
+#define SNAP_MINOR 2
 
 int crtc_snapshot_write_module(snapshot_t * s)
 {
     int i, ef = 0;
-    int current_char;
+    CLOCK current_char;
     int screen_rel;
     snapshot_module_t *m;
 
@@ -163,6 +164,10 @@ int crtc_snapshot_write_module(snapshot_t * s)
             ef = -1;
         }
     }
+    
+    if (raster_snapshot_write(m, &crtc.raster)) {
+        ef = -1;
+    }
 
     if (ef) {
         snapshot_module_close(m);
@@ -356,6 +361,10 @@ int crtc_snapshot_read_module(snapshot_t * s)
 
     force_repaint();
 #endif
+    
+    if (raster_snapshot_read(m, &crtc.raster)) {
+        ef = -1;
+    }
 
     crtc_update_window();
 
