@@ -31,7 +31,7 @@
 
 /* Some prototypes are needed */
 static int digimax_sound_machine_init(sound_t *psid, int speed, int cycles_per_sec);
-static int digimax_sound_machine_calculate_samples(sound_t **psid, int16_t *pbuf, int nr, int sound_output_channels, int sound_chip_channels, int *delta_t);
+static int digimax_sound_machine_calculate_samples(sound_t **psid, int16_t *pbuf, int nr, int sound_output_channels, int sound_chip_channels, CLOCK *delta_t);
 static void digimax_sound_machine_store(sound_t *psid, uint16_t addr, uint8_t val);
 static uint8_t digimax_sound_machine_read(sound_t *psid, uint16_t addr);
 static void digimax_sound_reset(sound_t *psid, CLOCK cpu_clk);
@@ -77,7 +77,7 @@ struct digimax_sound_s {
 
 static struct digimax_sound_s snd;
 
-static int digimax_sound_machine_calculate_samples(sound_t **psid, int16_t *pbuf, int nr, int soc, int scc, int *delta_t)
+static int digimax_sound_machine_calculate_samples(sound_t **psid, int16_t *pbuf, int nr, int soc, int scc, CLOCK *delta_t)
 {
     sound_dac_calculate_samples(&digimax_dac[0], pbuf, (int)snd.voice0 * 64, nr, soc, 1);
     sound_dac_calculate_samples(&digimax_dac[1], pbuf, (int)snd.voice1 * 64, nr, soc, (soc > 1) ? 2 : 1);
