@@ -35,7 +35,6 @@
 
 #include "6510core.h"
 #include "alarm.h"
-#include "clkguard.h"
 #include "debug.h"
 #include "drive.h"
 #include "drivecpu.h"
@@ -54,6 +53,7 @@
 #include "rotation.h"
 #include "snapshot.h"
 #include "types.h"
+#include "uiapi.h"
 
 
 #define DRIVE_CPU
@@ -61,7 +61,7 @@
 /* Global clock counters.  */
 CLOCK diskunit_clk[NUM_DISK_UNITS];
 
-static void drive_jam(diskunit_context_t *drv);
+static void drivecpu_jam(diskunit_context_t *drv);
 
 static void drivecpu_set_bank_base(void *context);
 
@@ -122,8 +122,6 @@ void drivecpu_setup_context(struct diskunit_context_s *drv, int i)
     cpu->monspace = monitor_diskspace_mem(drv->mynumber);
 
     if (i) {
-        drv->cpu->clk_guard = clk_guard_new(drv->clk_ptr, CLOCK_MAX - CLKGUARD_SUB_MIN);
-
         drv->cpu->alarm_context = alarm_context_new(drv->cpu->identification_string);
     }
 }
@@ -171,6 +169,7 @@ static void cpu_reset(diskunit_context_t *drv)
     preserve_monitor = drv->cpu->int_status->global_pending_int & IK_MONITOR;
 
     log_message(drv->log, "RESET.");
+    ui_display_reset(drv->mynumber + DRIVE_UNIT_MIN, 0);
 
     interrupt_cpu_status_reset(drv->cpu->int_status);
 
@@ -230,9 +229,6 @@ void drivecpu_shutdown(diskunit_context_t *drv)
     if (cpu->alarm_context != NULL) {
         alarm_context_destroy(cpu->alarm_context);
     }
-    if (cpu->clk_guard != NULL) {
-        clk_guard_destroy(cpu->clk_guard);
-    }
 
     monitor_interface_destroy(cpu->monitor_interface);
     interrupt_cpu_status_destroy(cpu->int_status);
@@ -270,28 +266,6 @@ inline void drivecpu_sleep(diskunit_context_t *drv)
     /* Currently does nothing.  But we might need this hook some day.  */
 }
 
-/* Make sure the drive clock counters never overflow; return nonzero if
-   they have been decremented to prevent overflow.  */
-CLOCK drivecpu_prevent_clk_overflow(diskunit_context_t *drv, CLOCK sub)
-{
-    if (sub != 0) {
-        /* First, get in sync with what the main CPU has done.  Notice that
-           `clk' has already been decremented at this point.  */
-        if (drv->enable) {
-            if (drv->cpu->last_clk < sub) {
-                /* Hm, this is kludgy.  :-(  */
-                drive_cpu_execute_all(maincpu_clk + sub);
-            }
-            drv->cpu->last_clk -= sub;
-        } else {
-            drv->cpu->last_clk = maincpu_clk;
-        }
-    }
-
-    /* Then, check our own clock counters.  */
-    return clk_guard_prevent_overflow(drv->cpu->clk_guard);
-}
-
 /* Handle a ROM trap. */
 inline static uint32_t drive_trap_handler(diskunit_context_t *drv)
 {
@@ -380,7 +354,7 @@ inline static int interrupt_check_irq_delay(interrupt_cpu_status_t *cs,
 void drivecpu_execute(diskunit_context_t *drv, CLOCK clk_value)
 {
     CLOCK cycles;
-    int tcycles;
+    CLOCK tcycles;
     drivecpu_context_t *cpu;
 
 #define reg_a   (cpu->cpu_regs.a)
@@ -391,6 +365,7 @@ void drivecpu_execute(diskunit_context_t *drv, CLOCK clk_value)
 #define reg_p   (cpu->cpu_regs.p)
 #define flag_z  (cpu->cpu_regs.z)
 #define flag_n  (cpu->cpu_regs.n)
+#define origin  (drv->mynumber + 1)
 
     cpu = drv->cpu;
 
@@ -412,11 +387,8 @@ void drivecpu_execute(diskunit_context_t *drv, CLOCK clk_value)
         cpu->cycle_accum &= 0xffff;
     }
 
-    /* Run drive CPU emulation until the stop_clk clock has been reached.
-     * There appears to be a nasty 32-bit overflow problem here, so we
-     * paper over it by only considering subtractions of 2nd complement
-     * integers. */
-    while ((int) (*(drv->clk_ptr) - cpu->stop_clk) < 0) {
+    /* Run drive CPU emulation until the stop_clk clock has been reached. */
+    while (*drv->clk_ptr < cpu->stop_clk) {
 /* Include the 6502/6510 CPU emulation core.  */
 
 #define CLK (*(drv->clk_ptr))
@@ -430,7 +402,7 @@ void drivecpu_execute(diskunit_context_t *drv, CLOCK clk_value)
 
 #define ALARM_CONTEXT (cpu->alarm_context)
 
-#define JAM() drive_jam(drv)
+#define JAM() drivecpu_jam(drv)
 
 #define ROM_TRAP_ALLOWED() 1
 
@@ -481,7 +453,7 @@ static void drivecpu_set_bank_base(void *context)
 }
 
 /* Inlining this fuction makes no sense and would only bloat the code.  */
-static void drive_jam(diskunit_context_t *drv)
+static void drivecpu_jam(diskunit_context_t *drv)
 {
     unsigned int tmp;
     char *dname = "  Drive";
@@ -540,7 +512,7 @@ static void drive_jam(diskunit_context_t *drv)
             break;
     }
 
-    tmp = machine_jam("%s (%d) CPU: JAM at $%04X  ", dname, drv->mynumber + 8, (int)reg_pc);
+    tmp = drive_jam(drv->mynumber, "%s (%d) CPU: JAM at $%04X  ", dname, drv->mynumber + 8, (int)reg_pc);
     switch (tmp) {
         case JAM_RESET:
             reg_pc = 0xeaa0;
@@ -563,7 +535,7 @@ static void drive_jam(diskunit_context_t *drv)
 /* ------------------------------------------------------------------------- */
 
 #define SNAP_MAJOR 1
-#define SNAP_MINOR 1
+#define SNAP_MINOR 2
 
 int drivecpu_snapshot_write_module(diskunit_context_t *drv, snapshot_t *s)
 {
@@ -579,7 +551,7 @@ int drivecpu_snapshot_write_module(diskunit_context_t *drv, snapshot_t *s)
     }
 
     if (0
-        || SMW_DW(m, (uint32_t) *(drv->clk_ptr)) < 0
+        || SMW_CLOCK(m, *(drv->clk_ptr)) < 0
         || SMW_B(m, (uint8_t)MOS6510_REGS_GET_A(&(cpu->cpu_regs))) < 0
         || SMW_B(m, (uint8_t)MOS6510_REGS_GET_X(&(cpu->cpu_regs))) < 0
         || SMW_B(m, (uint8_t)MOS6510_REGS_GET_Y(&(cpu->cpu_regs))) < 0
@@ -587,10 +559,10 @@ int drivecpu_snapshot_write_module(diskunit_context_t *drv, snapshot_t *s)
         || SMW_W(m, (uint16_t)MOS6510_REGS_GET_PC(&(cpu->cpu_regs))) < 0
         || SMW_B(m, (uint16_t)MOS6510_REGS_GET_STATUS(&(cpu->cpu_regs))) < 0
         || SMW_DW(m, (uint32_t)(cpu->last_opcode_info)) < 0
-        || SMW_DW(m, (uint32_t)(cpu->last_clk)) < 0
-        || SMW_DW(m, (uint32_t)(cpu->cycle_accum)) < 0
-        || SMW_DW(m, (uint32_t)(cpu->last_exc_cycles)) < 0
-        || SMW_DW(m, (uint32_t)(cpu->stop_clk)) < 0
+        || SMW_CLOCK(m, cpu->last_clk) < 0
+        || SMW_CLOCK(m, cpu->cycle_accum) < 0
+        || SMW_CLOCK(m, cpu->last_exc_cycles) < 0
+        || SMW_CLOCK(m, cpu->stop_clk) < 0
         ) {
         goto fail;
     }
@@ -658,7 +630,7 @@ int drivecpu_snapshot_read_module(diskunit_context_t *drv, snapshot_t *s)
 
     /* XXX: Assumes `CLOCK' is the same size as a `DWORD'.  */
     if (0
-        || SMR_DW(m, drv->clk_ptr) < 0
+        || SMR_CLOCK(m, drv->clk_ptr) < 0
         || SMR_B(m, &a) < 0
         || SMR_B(m, &x) < 0
         || SMR_B(m, &y) < 0
@@ -666,10 +638,10 @@ int drivecpu_snapshot_read_module(diskunit_context_t *drv, snapshot_t *s)
         || SMR_W(m, &pc) < 0
         || SMR_B(m, &status) < 0
         || SMR_DW_UINT(m, &(cpu->last_opcode_info)) < 0
-        || SMR_DW(m, &(cpu->last_clk)) < 0
-        || SMR_DW(m, &(cpu->cycle_accum)) < 0
-        || SMR_DW(m, &(cpu->last_exc_cycles)) < 0
-        || SMR_DW(m, &(cpu->stop_clk)) < 0
+        || SMR_CLOCK(m, &(cpu->last_clk)) < 0
+        || SMR_CLOCK(m, &(cpu->cycle_accum)) < 0
+        || SMR_CLOCK(m, &(cpu->last_exc_cycles)) < 0
+        || SMR_CLOCK(m, &(cpu->stop_clk)) < 0
         ) {
         goto fail;
     }
