@@ -26,9 +26,9 @@
 
 #include "vice.h"
 
+#include <stdio.h>
 #include <string.h>
 
-#include "clkguard.h"
 #include "diskimage.h"
 #include "drive.h"
 #include "drivetypes.h"
@@ -230,19 +230,6 @@ static void seek_alarm_handler(CLOCK offset, void *data)
     }
 }
 
-/* Clock overflow handling.  */
-static void clk_overflow_callback(CLOCK sub, void *data)
-{
-    pc8477_t *drv = (pc8477_t *)data;
-
-    if (drv->clk > (CLOCK) 0) {
-        drv->clk -= sub;
-    }
-    if (drv->motor_clk > (CLOCK) 0) {
-        drv->motor_clk -= sub;
-    }
-}
-
 /* Functions using drive context.  */
 void pc8477d_init(diskunit_context_t *drv)
 {
@@ -252,8 +239,6 @@ void pc8477d_init(diskunit_context_t *drv)
         pc8477_log = log_open("PC8477");
     }
 
-    clk_guard_add_callback(drv->cpu->clk_guard, clk_overflow_callback, drv->pc8477);
-
     name = lib_msprintf("%sEXEC", drv->pc8477->myname);
     drv->pc8477->seek_alarm = alarm_new(drv->cpu->alarm_context, name, seek_alarm_handler, drv->pc8477);
     lib_free(name);
@@ -649,7 +634,7 @@ static pc8477_state_t pc8477_execute(pc8477_t *drv)
                         /* fall through */
                     case 5:
                         if (drv->fifo_fill) {
-                            drv->clk += fdd_rotate(drv->fdd, (*drv->mycontext->clk_ptr - drv->clk) / BYTE_RATE) * BYTE_RATE;
+                            drv->clk += fdd_rotate(drv->fdd, (int)((*drv->mycontext->clk_ptr - drv->clk) / BYTE_RATE)) * BYTE_RATE;
                             return PC8477_READ;
                         }
                         if (drv->cmd[6] != drv->sector) {
@@ -964,7 +949,7 @@ static pc8477_state_t pc8477_execute(pc8477_t *drv)
                         break;
                 }
                 if (fdd_index_count(drv->fdd) > 1) {
-                    drv->clk += fdd_rotate(drv->fdd, (*drv->mycontext->clk_ptr - drv->clk) / BYTE_RATE) * BYTE_RATE;
+                    drv->clk += fdd_rotate(drv->fdd, (int)((*drv->mycontext->clk_ptr - drv->clk) / BYTE_RATE)) * BYTE_RATE;
                     drv->cmd[3] = drv->sector;
                     drv->st[0] |= 0x40;
                     return PC8477_RESULT;
@@ -998,7 +983,7 @@ static void pc8477_store(pc8477_t *drv, uint16_t addr, uint8_t byte)
                 pc8477_software_reset(drv);
             }
             drv->dor = byte;
-            drv->clk += fdd_rotate(drv->fdd, (*drv->mycontext->clk_ptr - drv->clk) / BYTE_RATE) * BYTE_RATE;
+            drv->clk += fdd_rotate(drv->fdd, (int)((*drv->mycontext->clk_ptr - drv->clk) / BYTE_RATE)) * BYTE_RATE;
             for (i = 0; i < 4; i++) {
                 if ((byte & (0x10 << i)) != drv->fdds[i].motor_on_out && drv->fdds[i].motor_on) {
                     (drv->fdds[i].motor_on)(drv->fdds[i].motor_on_data, drv->fdds[i].motor_on_out ? 0 : 1);
@@ -1054,7 +1039,7 @@ static void pc8477_store(pc8477_t *drv, uint16_t addr, uint8_t byte)
                     drv->int_step = 0;
                     drv->fifo_fill = 0;
                     drv->fifop2 = drv->fifop;
-                    drv->clk += fdd_rotate(drv->fdd, (*drv->mycontext->clk_ptr - drv->clk) / BYTE_RATE) * BYTE_RATE;
+                    drv->clk += fdd_rotate(drv->fdd, (int)((*drv->mycontext->clk_ptr - drv->clk) / BYTE_RATE)) * BYTE_RATE;
                     fdd_index_count_reset(drv->fdd);
                     drv->state = pc8477_execute(drv);
                     break;
