@@ -29,10 +29,10 @@
 
 #include "types.h"
 
-struct drive_s;
+struct diskunit_context_s;
 
 extern void iec128dcrrom_init(void);
-extern void iec128dcrrom_setup_image(struct drive_s *drive);
+extern void iec128dcrrom_setup_image(struct diskunit_context_s *drive);
 extern int iec128dcrrom_check_loaded(unsigned int type);
 
 extern int iec128dcrrom_load_1571cr(void);
