@@ -103,7 +103,7 @@ static int p00_check_name(const char *name)
 }
 
 static int p00_read_header(struct rawfile_info_s *info, uint8_t *cbmname_return,
-                           unsigned int *recsize_return)
+                           int *recsize_return)
 {
     uint8_t hdr[P00_HDR_LEN];
 
@@ -119,7 +119,7 @@ static int p00_read_header(struct rawfile_info_s *info, uint8_t *cbmname_return,
     memcpy(cbmname_return, hdr + P00_HDR_CBMNAME_OFFSET, P00_HDR_CBMNAME_LEN);
 
     if (recsize_return != NULL) {
-        *recsize_return = (unsigned int)hdr[P00_HDR_RECORDSIZE_OFFSET];
+        *recsize_return = (int)hdr[P00_HDR_RECORDSIZE_OFFSET];
     }
 
     return 0;
@@ -137,7 +137,7 @@ static int p00_write_header(struct rawfile_info_s *info, const uint8_t *cbmname,
     memcpy(hdr + P00_HDR_CBMNAME_OFFSET, cbmname, P00_HDR_CBMNAME_LEN);
     hdr[P00_HDR_RECORDSIZE_OFFSET] = (uint8_t)recsize;
 
-    if (rawfile_seek_set(info, 0) != 0) {
+    if (rawfile_seek(info, 0, SEEK_SET) != 0) {
         return -1;
     }
 
@@ -365,13 +365,15 @@ static char *p00_file_create(const char *file_name, const char *path,
 }
 
 fileio_info_t *p00_open(const char *file_name, const char *path,
-                        unsigned int command, unsigned int open_type)
+                        unsigned int command, unsigned int open_type,
+                        int *reclenp)
 {
     char rname[20]; /* FIXME */
     fileio_info_t *info;
     struct rawfile_info_s *rawfile;
     char *fname = NULL;
     int type;
+    int reclen = 0;
 
     if (command & FILEIO_COMMAND_FSNAME) {
         fname = lib_strdup(file_name);
@@ -379,16 +381,24 @@ fileio_info_t *p00_open(const char *file_name, const char *path,
         switch (command & FILEIO_COMMAND_MASK) {
             case FILEIO_COMMAND_STAT:
             case FILEIO_COMMAND_READ:
+            case FILEIO_COMMAND_READ_WRITE:
             case FILEIO_COMMAND_APPEND:
             case FILEIO_COMMAND_APPEND_READ:
                 fname = p00_file_find(file_name, path);
                 break;
             case FILEIO_COMMAND_WRITE:
+            case FILEIO_COMMAND_OVERWRITE:
                 fname = p00_file_create(file_name, path, open_type);
                 break;
         }
     }
 
+    if (fname == NULL &&
+            (command & FILEIO_COMMAND_MASK) == FILEIO_COMMAND_READ_WRITE) {
+        fname = p00_file_create(file_name, path, open_type);
+        command = (command & ~FILEIO_COMMAND_MASK) | FILEIO_COMMAND_WRITE;
+    }
+
     if (fname == NULL) {
         return NULL;
     }
@@ -405,14 +415,15 @@ fileio_info_t *p00_open(const char *file_name, const char *path,
     switch (command & FILEIO_COMMAND_MASK) {
         case FILEIO_COMMAND_STAT:
         case FILEIO_COMMAND_READ:
-            if (type < 0 || p00_read_header(rawfile, (uint8_t *)rname, NULL) < 0) {
+        case FILEIO_COMMAND_READ_WRITE:
+            if (type < 0 || p00_read_header(rawfile, (uint8_t *)rname, &reclen) < 0) {
                 rawfile_destroy(rawfile);
                 return NULL;
             }
             break;
         case FILEIO_COMMAND_APPEND:
         case FILEIO_COMMAND_APPEND_READ:
-            if (type < 0 || p00_read_header(rawfile, (uint8_t *)rname, NULL) < 0) {
+            if (type < 0 || p00_read_header(rawfile, (uint8_t *)rname, &reclen) < 0) {
                 rawfile_destroy(rawfile);
                 return NULL;
             }
@@ -423,15 +434,28 @@ fileio_info_t *p00_open(const char *file_name, const char *path,
 */
             break;
         case FILEIO_COMMAND_WRITE:
+        case FILEIO_COMMAND_OVERWRITE:
+            reclen = reclenp ? *reclenp : 0;
             memset(rname, 0, sizeof(rname));
             strncpy(rname, file_name, 16);
-            if (p00_write_header(rawfile, (uint8_t *)rname, 0) < 0) {
+            if (p00_write_header(rawfile, (uint8_t *)rname, reclen) < 0) {
                 rawfile_destroy(rawfile);
                 return NULL;
             }
             break;
     }
 
+    if (open_type == FILEIO_TYPE_REL && reclenp) {
+        if (*reclenp == 0) {
+            *reclenp = reclen;
+        } else if (*reclenp != reclen) {
+            /* The given record length does not match the stored record
+             * length */
+            log_verbose("p00_open: record size: found %d != expected %d => record size mismatch\n", reclen, *reclenp);
+            return NULL;
+        }
+    }
+
     info = lib_malloc(sizeof(fileio_info_t));
     info->name = (uint8_t *)lib_strdup(rname);
     info->length = (unsigned int)strlen((char *)(info->name));
@@ -544,3 +568,24 @@ unsigned int p00_get_bytes_left(struct fileio_info_s *info)
 {
     return rawfile_get_bytes_left(info->rawfile);
 }
+
+unsigned int p00_seek(struct fileio_info_s *info, off_t offset, int whence)
+{
+    if (whence == SEEK_SET) {
+        offset += P00_HDR_LEN;
+    }
+
+    /* Note: this has no protection against seeking into the P00 header */
+
+    return rawfile_seek(info->rawfile, offset, whence);
+}
+
+unsigned int p00_tell(struct fileio_info_s *info)
+{
+    long pos = rawfile_tell(info->rawfile);
+
+    if (pos == -1)
+        return -1;
+
+    return (unsigned int)pos - P00_HDR_LEN;
+}
