@@ -33,6 +33,8 @@
  *
  */
 
+/* #define DEBUG_FSDEVICE */
+
 #include "vice.h"
 
 #include <stdio.h>
@@ -59,6 +61,11 @@
 #include "vdrive-command.h"
 #include "vdrive.h"
 
+#ifdef DEBUG_FSDEVICE
+#define DBG(x)  log_debug x
+#else
+#define DBG(x)
+#endif
 
 fsdevice_dev_t fsdevice_dev[FSDEVICE_DEVICE_MAX];
 
@@ -183,7 +190,9 @@ int fsdevice_attach(unsigned int device, unsigned int drive, const char *name)
 {
     vdrive_t *vdrive;
 
-    vdrive = file_system_get_vdrive(device, drive);
+    DBG(("fsdevice_attach device: %u drive: %u name: %s", device, drive, name));
+
+    vdrive = file_system_get_vdrive(device);
 
     if (machine_bus_device_attach(device, name, fsdevice_read, fsdevice_write,
                                   fsdevice_open, fsdevice_close,
