@@ -1,5 +1,3 @@
-/* vim: set et ts=4 sw=4 sts=4 fdm=marker syntax=c.doxygen: */
-
 /** \file   src/lib/stil.c
  * \brief   Sid Tune Information List handling
  *
@@ -8,7 +6,7 @@
 
 /*
  *  HVSClib - a library to work with High Voltage SID Collection files
- *  Copyright (C) 2018-2020  Bas Wassink <b.wassink@ziggo.nl>
+ *  Copyright (C) 2018-2021  Bas Wassink <b.wassink@ziggo.nl>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,6 +28,7 @@
 #include <inttypes.h>
 #include <string.h>
 #include <ctype.h>
+#include "log.h"
 
 #include "hvsc.h"
 
@@ -213,7 +212,7 @@ static hvsc_stil_field_t *stil_field_new(int type,
         field->text = hvsc_strndup(text, tlen);
         if (field->text == NULL) {
             stil_field_free(field);
-            field = NULL;
+            return NULL;
         }
         if (album != NULL && *album != '\0') {
             field->album = hvsc_strndup(album, alen);
@@ -509,11 +508,14 @@ int hvsc_stil_open(const char *psid, hvsc_stil_t *handle)
     handle->entry_bufmax = HVSC_STIL_BUFFER_INIT;
     handle->entry_bufused = 0;
 
+    log_message(LOG_DEFAULT, "Vsid: Opening '%s'.", hvsc_stil_path);
     if (!hvsc_text_file_open(hvsc_stil_path, &(handle->stil))) {
+        log_warning(LOG_DEFAULT, "Vsid: Failed to open STIL.");
         hvsc_stil_close(handle);
         return 0;
     }
 
+
     /* make copy of psid, ripping off the HVSC root directory */
     handle->psid_path = hvsc_path_strip_root(psid);
 #if defined(_WIN32) || defined(_WIN64)
@@ -534,6 +536,7 @@ int hvsc_stil_open(const char *psid, hvsc_stil_t *handle)
             if (feof(handle->stil.fp)) {
                 /* EOF, so simply not found */
                 hvsc_errno = HVSC_ERR_NOT_FOUND;
+                log_message(LOG_DEFAULT, "Vsid: No STIL entry found.");
             }
             hvsc_stil_close(handle);
             /* I/O error is already set */
@@ -541,7 +544,8 @@ int hvsc_stil_open(const char *psid, hvsc_stil_t *handle)
         }
 
         if (strcmp(line, handle->psid_path) == 0) {
-            hvsc_dbg("Found '%s' at line %ld\n", line, handle->stil.lineno);
+            log_message(LOG_DEFAULT,
+                    "Vsid: Found '%s' at line %ld.", line, handle->stil.lineno);
             return 1;
         }
     }
@@ -976,6 +980,7 @@ int hvsc_stil_parse_entry(hvsc_stil_t *handle)
                 }
                 if (!stil_block_add_field(state.block, state.field)) {
                     hvsc_dbg("failed to add field to block\n");
+                    free(state.field);
                     return 0;
                 }
 
