@@ -44,10 +44,9 @@ extern int machine_bus_lib_write_sector(unsigned int unit, unsigned int track,
 
 extern unsigned int machine_bus_device_type_get(unsigned int unit);
 
-extern void machine_bus_status_truedrive_set(unsigned int enable);
-extern void machine_bus_status_drivetype_set(unsigned int unit,
-                                             unsigned int enable);
-extern void machine_bus_status_virtualdevices_set(unsigned int enable);
+extern void machine_bus_status_truedrive_set(unsigned int unit, unsigned int enable);
+extern void machine_bus_status_drivetype_set(unsigned int unit, unsigned int enable);
+extern void machine_bus_status_virtualdevices_set(unsigned int unit, unsigned int enable);
 
 extern void machine_bus_eof_callback_set(void (*func)(void));
 extern void machine_bus_attention_callback_set(void (*func)(void));
