@@ -62,6 +62,7 @@
 #include "tick.h"
 #include "types.h"
 #include "uiapi.h"
+#include "util.h"
 #include "version.h"
 #include "video.h"
 #include "vsyncapi.h"
@@ -79,11 +80,20 @@
 int console_mode = 0;
 int video_disabled_mode = 0;
 
+/** \brief  Help was requested on the command line
+ *
+ * The command line contained -?/-h/-help/--help.
+ *
+ * Include "machine.h" to use this variable.
+ */
+int help_requested = 0;
+
 void main_loop_forever(void);
 
 #ifdef USE_VICE_THREAD
 void *vice_thread_main(void *);
 static pthread_t vice_thread;
+static pthread_mutex_t init_lock = PTHREAD_MUTEX_INITIALIZER;
 #endif
 
 
@@ -102,32 +112,80 @@ int main_program(int argc, char **argv)
 {
     int i, n;
     const char *program_name;
-    int ishelp = 0;
     int loadconfig = 1;
     char term_tmp[TERM_TMP_SIZE];
     size_t name_len;
+    int reserr;
+    char *cmdline;
+
+#ifdef USE_VICE_THREAD
+    /*
+     * The init lock guarantees that all main thread init outcomes are visible
+     * to the VICE thread.
+     */
+    
+    pthread_mutex_lock(&init_lock);
+
+    archdep_thread_init();
+
+    mainlock_init();
+#endif
+    
+    /*
+     * OpenMP defaults to spinning threads for a couple hundred ms
+     * after they are used, which means they max out forever in our
+     * use case. Setting OMP_WAIT_POLICY to PASSIVE fixes that.
+     */
+    
+#if defined(WIN32_COMPILE)
+    _putenv("OMP_WAIT_POLICY=PASSIVE");
+#else
+    setenv("OMP_WAIT_POLICY", "PASSIVE", 1);
+#endif
 
     lib_init();
 
+    /* create string from the commandline that we can log later */
+    cmdline = lib_strdup(argv[0]);
+    for (i = 1; i < argc; i++) {
+        char *p;
+        p = cmdline; /* remember old pointer */
+        cmdline = util_concat(p, " ", argv[i], NULL);
+        lib_free(p); /* free old pointer */
+    }
+
     /* Check for some options at the beginning of the commandline before 
        initializing the user interface or loading the config file.
        -default => use default config, do not load any config
        -config  => use specified configuration file
        -console => no user interface
+       -verbose => more verbose logging
+       -silent => no logging
+       -seed => set the random seed
     */
     DBG(("main:early cmdline(argc:%d)\n", argc));
     for (i = 1; i < argc; i++) {
         if ((!strcmp(argv[i], "-console")) || (!strcmp(argv[i], "--console"))) {
             console_mode = 1;
-            video_disabled_mode = 1;
+            /* video_disabled_mode = 1;  Breaks exitscreenshot */
         } else
         if ((!strcmp(argv[i], "-config")) || (!strcmp(argv[i], "--config"))) {
             if ((i + 1) < argc) {
                 vice_config_file = lib_strdup(argv[++i]);
                 loadconfig = 1;
             }
-        } else if (!strcmp(argv[i], "-default")) {
+        } else if ((!strcmp(argv[i], "-default")) || (!strcmp(argv[i], "--default"))) {
             loadconfig = 0;
+        } else if ((!strcmp(argv[i], "-verbose")) || (!strcmp(argv[i], "--verbose"))) {
+            log_set_silent(0);
+            log_set_verbose(1);
+        } else if ((!strcmp(argv[i], "-silent")) || (!strcmp(argv[i], "--silent"))) {
+            log_set_silent(1);
+            log_set_verbose(0);
+        } else if ((!strcmp(argv[i], "-seed")) || (!strcmp(argv[i], "--seed"))) {
+            if ((i + 1) < argc) {
+                lib_rand_seed(strtoul(argv[++i], NULL, 0));
+            }
         } else {
             break;
         }
@@ -147,7 +205,7 @@ int main_program(int argc, char **argv)
                    (!strcmp(argv[i], "--help")) ||
                    (!strcmp(argv[i], "-h")) ||
                    (!strcmp(argv[i], "-?"))) {
-            ishelp = 1;
+            help_requested = 1;
         }
     }
 
@@ -166,7 +224,15 @@ int main_program(int argc, char **argv)
     /* Initialize system file locator.  */
     sysfile_init(machine_name);
 
-    gfxoutput_early_init(ishelp);
+
+    /* hotkeys init needs to be called after sysfile_init() but before the
+     * resources and cmdline options of the hotkeys are registered.
+     */
+    if (!help_requested) {
+        ui_hotkeys_init();
+    }
+
+    gfxoutput_early_init(help_requested);
     if ((init_resources() < 0) || (init_cmdline_options() < 0)) {
         return -1;
     }
@@ -177,18 +243,18 @@ int main_program(int argc, char **argv)
         return -1;
     }
 
-    /* Initialize the user interface.  `ui_init()' might need to handle the
+    /* Initialize the user interface.  `ui_init_with_args()' might need to handle the
        command line somehow, so we call it before parsing the options.
        (e.g. under X11, the `-display' option is handled independently).  */
     DBG(("main:ui_init(argc:%d)\n", argc));
-    if (!console_mode && ui_init(&argc, argv) < 0) {
-        archdep_startup_log_error("Cannot initialize the UI.\n");
-        return -1;
+    if (!console_mode) {
+        ui_init_with_args(&argc, argv);
     }
 
-    if ((!ishelp) && (loadconfig)) {
+    if ((!help_requested) && (loadconfig)) {
         /* Load the user's default configuration file.  */
-        if (resources_reset_and_load(NULL) < 0) {
+        reserr = resources_load(NULL);
+        if (reserr < 0 && reserr != RESERR_FILE_NOT_FOUND) {
             /* The resource file might contain errors, and thus certain
             resources might have been initialized anyway.  */
             if (resources_set_defaults() < 0) {
@@ -218,6 +284,13 @@ int main_program(int argc, char **argv)
         return -1;
     }
 
+    /* Initialize the user interface, 2nd part. */
+    DBG(("main:uidata_init(argc:%d)\n", argc));
+    if (!console_mode && ui_init() < 0) {
+        archdep_startup_log_error("Cannot initialize the UI.\n");
+        return -1;
+    }
+
     program_name = archdep_program_name();
 
     /* VICE boot sequence.  */
@@ -271,6 +344,9 @@ int main_program(int argc, char **argv)
     log_message(LOG_DEFAULT, " ");
 
     /* lib_free(program_name); */
+    lib_rand_printseed(); /* log the random seed */
+    log_message(LOG_DEFAULT, "command line was: %s", cmdline);
+    lib_free(cmdline);
 
     /* Complete the GUI initialization (after loading the resources and
        parsing the command-line) if necessary.  */
@@ -278,18 +354,18 @@ int main_program(int argc, char **argv)
         return -1;
     }
 
-    if (!console_mode && video_init() < 0) {
+    if (/*!console_mode && */video_init() < 0) {
         return -1;
     }
 
     if (initcmdline_check_psid() < 0) {
         return -1;
     }
-    
+
     if (init_main() < 0) {
         return -1;
     }
-    
+
     initcmdline_check_attach();
 
 #ifdef USE_VICE_THREAD
@@ -299,6 +375,8 @@ int main_program(int argc, char **argv)
         return 1;
     }
 
+    pthread_mutex_unlock(&init_lock);
+
 #else /* #ifdef USE_VICE_THREAD */
 
     main_loop_forever();
@@ -327,15 +405,18 @@ void vice_thread_shutdown(void)
         return;
     }
 
+    /* log resources with non default values */
+    resources_log_active();
+    /* log the active config as commandline options */
+    cmdline_log_active();
+
     if (pthread_equal(pthread_self(), vice_thread)) {
         printf("FIXME! VICE thread is trying to shut itself down directly, this needs to be called from the ui thread for a correct shutdown!\n");
         mainlock_initiate_shutdown();
         return;
     }
 
-    mainlock_obtain();
     mainlock_initiate_shutdown();
-    mainlock_release();
 
     pthread_join(vice_thread, NULL);
 
@@ -344,16 +425,14 @@ void vice_thread_shutdown(void)
 
 void *vice_thread_main(void *unused)
 {
-    archdep_thread_init();
-
-    mainlock_init();
-
-    main_loop_forever();
+    /* Let the mainlock system know which thread is the vice thread */
+    mainlock_set_vice_thread();
 
     /*
      * main_loop_forever() does not return, so we call archdep_thread_shutdown()
      * in the mainlock system which manages a direct pthread based thread exit.
      */
+    main_loop_forever();
 
     return NULL;
 }
