@@ -33,7 +33,7 @@
 #include "6510core.h"
 #include "alarm.h"
 #include "archdep.h"
-#include "clkguard.h"
+#include "autostart.h"
 #include "debug.h"
 #include "interrupt.h"
 #include "log.h"
@@ -231,7 +231,6 @@ struct interrupt_cpu_status_s *maincpu_int_status = NULL;
 #ifndef CYCLE_EXACT_ALARM
 alarm_context_t *maincpu_alarm_context = NULL;
 #endif
-clk_guard_t *maincpu_clk_guard = NULL;
 monitor_interface_t *maincpu_monitor_interface = NULL;
 
 /* Global clock counter.  */
@@ -461,6 +460,7 @@ void maincpu_resync_limits(void)
 
 void maincpu_mainloop(void)
 {
+#define origin (0)
 #ifndef C64DTV
     /* Notice that using a struct for these would make it a lot slower (at
        least, on gcc 2.7.2.x).  */
@@ -572,6 +572,8 @@ void maincpu_mainloop(void)
             log_error(LOG_DEFAULT, "cycle limit reached.");
             archdep_vice_exit(1);
         }
+
+        autostart_advance();
 #if 0
         if (CLK > 246171754) {
             debug.maincpu_traceflg = 1;
@@ -690,7 +692,7 @@ unsigned int maincpu_get_sp(void) {
 
 static char snap_module_name[] = "MAINCPU";
 #define SNAP_MAJOR 1
-#define SNAP_MINOR 1
+#define SNAP_MINOR 2
 
 int maincpu_snapshot_write_module(snapshot_t *s)
 {
@@ -703,7 +705,7 @@ int maincpu_snapshot_write_module(snapshot_t *s)
     }
 
 #ifdef C64DTV
-    if (SMW_DW(m, maincpu_clk) < 0
+    if (SMW_CLOCK(m, maincpu_clk) < 0
             || SMW_B(m, MOS6510DTV_REGS_GET_A(&maincpu_regs)) < 0
             || SMW_B(m, MOS6510DTV_REGS_GET_X(&maincpu_regs)) < 0
             || SMW_B(m, MOS6510DTV_REGS_GET_Y(&maincpu_regs)) < 0
@@ -732,7 +734,7 @@ int maincpu_snapshot_write_module(snapshot_t *s)
         goto fail;
     }
 #else
-    if (SMW_DW(m, maincpu_clk) < 0
+    if (SMW_CLOCK(m, maincpu_clk) < 0
             || SMW_B(m, MOS6510_REGS_GET_A(&maincpu_regs)) < 0
             || SMW_B(m, MOS6510_REGS_GET_X(&maincpu_regs)) < 0
             || SMW_B(m, MOS6510_REGS_GET_Y(&maincpu_regs)) < 0
@@ -780,8 +782,7 @@ int maincpu_snapshot_read_module(snapshot_t *s)
        wrong number of cycles.  */
     maincpu_rmw_flag = 0;
 
-    /* XXX: Assumes `CLOCK' is the same size as a `DWORD'.  */
-    if (SMR_DW(m, &maincpu_clk) < 0
+    if (SMR_CLOCK(m, &maincpu_clk) < 0
             || SMR_B(m, &a) < 0
             || SMR_B(m, &x) < 0
             || SMR_B(m, &y) < 0
