@@ -73,9 +73,9 @@ struct interrupt_cpu_status_s;
 struct monitor_cpu_type_s {
     CPU_TYPE_t cpu_type;
     unsigned int (*asm_addr_mode_get_size)(unsigned int mode, unsigned int p0,
-                                           unsigned int p1, unsigned int p2);
+                                           unsigned int p1, unsigned int p2, unsigned int p3);
     const struct asm_opcode_info_s *(*asm_opcode_info_get)(unsigned int p0, unsigned int p1,
-                                                           unsigned int p2);
+                                                           unsigned int p2, unsigned int p3);
     int (*mon_assemble_instr)(const char *opcode_name, asm_mode_addr_info_t operand);
     unsigned int (*mon_register_get_val)(int mem, int reg_id);
     void (*mon_register_set_val)(int mem, int reg_id, uint16_t val);
@@ -85,6 +85,16 @@ struct monitor_cpu_type_s {
 };
 typedef struct monitor_cpu_type_s monitor_cpu_type_t;
 
+
+struct supported_cpu_type_list_s {
+    monitor_cpu_type_t *monitor_cpu_type_p;
+    struct supported_cpu_type_list_s *next;
+};
+typedef struct supported_cpu_type_list_s supported_cpu_type_list_t;
+
+/* A linked list of supported monitor_cpu_types for each memspace */
+extern supported_cpu_type_list_t *monitor_cpu_type_supported[NUM_MEMSPACES];
+
 /* This is the standard interface through which the monitor accesses a
    certain CPU.  */
 struct monitor_interface_s {
@@ -232,9 +242,9 @@ typedef struct monitor_cartridge_commands_s monitor_cartridge_commands_t;
 extern monitor_cartridge_commands_t mon_cart_cmd;
 
 /* CPU history/memmap prototypes */
-extern void monitor_cpuhistory_store(uint32_t cycle, unsigned int addr, unsigned int op, unsigned int p1, unsigned int p2,
+extern void monitor_cpuhistory_store(CLOCK cycle, unsigned int addr, unsigned int op, unsigned int p1, unsigned int p2,
                                      uint8_t reg_a, uint8_t reg_x, uint8_t reg_y,
-                                     uint8_t reg_sp, unsigned int reg_st);
+                                     uint8_t reg_sp, unsigned int reg_st, uint8_t origin);
 extern void monitor_cpuhistory_fix_p2(unsigned int p2);
 extern void monitor_memmap_store(unsigned int addr, unsigned int type);
 
