@@ -58,7 +58,7 @@ static int mon_assemble_instr(const char *opcode_name, asm_mode_addr_info_t oper
     do {
         const asm_opcode_info_t *opinfo;
 
-        opinfo = (monitor_cpu_for_memspace[mem]->asm_opcode_info_get)(i, 0, 0);
+        opinfo = (monitor_cpu_for_memspace[mem]->asm_opcode_info_get)(i, 0, 0, 0);
         if (!strcasecmp(opinfo->mnemonic, opcode_name)) {
             if (opinfo->addr_mode == operand_mode) {
                 opcode = i;
@@ -126,6 +126,17 @@ static int mon_assemble_instr(const char *opcode_name, asm_mode_addr_info_t oper
                 found = TRUE;
                 break;
             }
+
+            /* Special case: Absolute Indirect with operand < 256
+             * (which can only be JMP ($xx)
+             */
+            if (operand_mode == ASM_ADDR_MODE_INDIRECT
+                && opinfo->addr_mode == ASM_ADDR_MODE_ABS_INDIRECT) {
+                opcode = i;
+                operand_mode = ASM_ADDR_MODE_ABS_INDIRECT;
+                found = TRUE;
+                break;
+            }
         }
         i++;
     } while (i != 0);
@@ -136,7 +147,7 @@ static int mon_assemble_instr(const char *opcode_name, asm_mode_addr_info_t oper
     }
 
     len = (monitor_cpu_for_memspace[mem]->asm_addr_mode_get_size)
-              ((unsigned int)(operand_mode), 0, 0, 0);
+              ((unsigned int)(operand_mode), 0, 0, 0, 0);
 
     mon_set_mem_val(mem, loc, opcode);
     if (len >= 2) {
