@@ -32,7 +32,8 @@
 #include <string.h>
 
 #include "attach.h"
-#include "diskcontents.h"
+#include "charset.h"
+#include "diskcontents-block.h"
 #include "diskimage.h"
 #include "imagecontents.h"
 #include "lib.h"
@@ -54,13 +55,14 @@
 void mon_drive_block_cmd(int op, int track, int sector, MON_ADDR addr)
 {
     vdrive_t *vdrive;
+    /* TODO: drive 1? */
+    unsigned int drive = 0;
 
     mon_evaluate_default_addr(&addr);
 
-    /* TODO: other units, drive 1? */
-    vdrive = file_system_get_vdrive(8, 0);
+    vdrive = file_system_get_vdrive(8);
 
-    if (!vdrive || vdrive->image == NULL) {
+    if (!vdrive) {
         mon_out("No disk attached\n");
         return;
     }
@@ -71,7 +73,7 @@ void mon_drive_block_cmd(int op, int track, int sector, MON_ADDR addr)
         MEMSPACE dest_mem;
 
         /* We ignore disk error codes here.  */
-        if (vdrive_read_sector(vdrive, readdata, track, sector)
+        if (vdrive_ext_read_sector(vdrive, drive, readdata, track, sector)
             < 0) {
             mon_out("Error reading track %d sector %d\n", track, sector);
             return;
@@ -111,7 +113,7 @@ void mon_drive_block_cmd(int op, int track, int sector, MON_ADDR addr)
             writedata[i] = mon_get_mem_val(src_mem, ADDR_LIMIT(src + i));
         }
 
-        if (vdrive_write_sector(vdrive, writedata, track, sector)) {
+        if (vdrive_ext_write_sector(vdrive, drive, writedata, track, sector)) {
             mon_out("Error writing track %d sector %d\n", track, sector);
             return;
         }
@@ -127,11 +129,11 @@ void mon_drive_execute_disk_cmd(char *cmd)
     unsigned int len;
     vdrive_t *vdrive;
 
-    /* FIXME */
-    vdrive = file_system_get_vdrive(8, 0);
+    /* Unit? */
+    vdrive = file_system_get_vdrive(8);
 
     len = (unsigned int)strlen(cmd);
-
+    charset_petconvstring((uint8_t*)cmd, CONVERT_TO_PETSCII);
     vdrive_command_execute(vdrive, (uint8_t *)cmd, len);
 }
 
@@ -141,8 +143,8 @@ int mon_drive_is_fsdevice(int drive_unit)
 {
     int virtualdev = 0, truedrive = 0, iecdevice = 0 /* , fsdevice = 0 */;
     /* FIXME: unsure if this check really works as advertised */
-    resources_get_int("VirtualDevices", &virtualdev);
-    resources_get_int("DriveTrueEmulation", &truedrive);
+    resources_get_int_sprintf("VirtualDevice%d", &virtualdev, drive_unit);
+    resources_get_int_sprintf("Drive%dTrueEmulation", &truedrive, drive_unit);
     resources_get_int_sprintf("IECDevice%i", &iecdevice, drive_unit);
     /* resources_get_int_sprintf("FileSystemDevice%i", &fsdevice, drive_unit); */
     if ((virtualdev && !truedrive) || (!virtualdev && iecdevice)) {
@@ -167,18 +169,16 @@ const char *mon_drive_get_fsdevice_path(int drive_unit)
 
 void mon_drive_list(int drive_unit)
 {
-    const char *name;
     image_contents_t *listing;
     vdrive_t *vdrive;
     /* TODO: drive 1? */
-    unsigned int drive = 0;
     const char *fspath = NULL;
 
     if ((drive_unit < 8) || (drive_unit > 11)) {
         drive_unit = 8;
     }
 
-    vdrive = file_system_get_vdrive(drive_unit, drive);
+    vdrive = file_system_get_vdrive(drive_unit);
 
     if (vdrive == NULL || vdrive->image == NULL) {
         if ((fspath = mon_drive_get_fsdevice_path(drive_unit))) {
@@ -189,12 +189,10 @@ void mon_drive_list(int drive_unit)
         return;
     }
 
-    name = disk_image_name_get(vdrive->image);
-
-    listing = diskcontents_read(name, drive_unit, drive);
+    listing = diskcontents_block_read(vdrive, 0);
 
     if (listing != NULL) {
-        char *string = image_contents_to_string(listing, 1);
+        char *string = image_contents_to_string(listing, IMAGE_CONTENTS_STRING_ASCII);
         image_contents_file_list_t *element = listing->file_list;
 
         mon_out("%s\n", string);
@@ -204,7 +202,7 @@ void mon_drive_list(int drive_unit)
             mon_out("Empty image\n");
         } else {
             do {
-                string = image_contents_file_to_string(element, 1);
+                string = image_contents_file_to_string(element, IMAGE_CONTENTS_STRING_ASCII);
                 mon_out("%s\n", string);
                 lib_free(string);
             }
