@@ -50,6 +50,7 @@
 #endif
 
 #include "archdep.h"
+#include "archdep_defs.h"
 #include "cartio.h"
 #include "charset.h"
 #include "cmdline.h"
@@ -84,11 +85,17 @@
 #include "monitor_network.h"
 #include "monitor_binary.h"
 #include "montypes.h"
+
+#include "userport_io_sim.h"
+#include "joyport_io_sim.h"
+#include "joyport.h"
+
 #include "resources.h"
 #include "screenshot.h"
 #include "sysfile.h"
 #include "traps.h"
 #include "types.h"
+#include "ui.h"
 #include "uiapi.h"
 #include "uimon.h"
 #include "util.h"
@@ -127,8 +134,6 @@ static int init_break_address = -1;
 
 console_t *console_log = NULL;
 
-static int monitor_trap_triggered = 0;
-
 monitor_cartridge_commands_t mon_cart_cmd;
 
 /* Types */
@@ -167,8 +172,10 @@ static int mon_console_close_on_leaving = 0;
 int sidefx = 0;
 int break_on_dummy_access = 0;
 RADIXTYPE default_radix;
-MEMSPACE default_memspace;
+MEMSPACE default_memspace = e_comp_space;
 static bool inside_monitor = false;
+static bool should_pause_on_exit_mon = false;
+static bool pause_on_exit_mon = false;
 static unsigned int instruction_count;
 static bool skip_jsrs;
 static int wait_for_return_level;
@@ -202,7 +209,7 @@ static bool recording;
 static FILE *recording_fp;
 static char *recording_name;
 
-#define PLAYBACK_MAX_RECUSRION 128
+#define PLAYBACK_MAX_FP_DEPTH 128
 
 static char **playback_filename_stack = NULL;
 static FILE **playback_fp_stack = NULL;
@@ -215,7 +222,7 @@ static bool playback_exit_after_all_playback = false;
 
 static void playback_end_file(void);
 
-static void monitor_close(int check);
+static void monitor_close(bool check_exit);
 static int monitor_set_moncommands_file(const char *param, void *extra_param);
 
 /* Disassemble the current opcode on entry.  Used for single step.  */
@@ -225,14 +232,8 @@ static int disassemble_on_entry = 0;
 /* This gets initialized in monitor_init(). */
 monitor_cpu_type_t *monitor_cpu_for_memspace[NUM_MEMSPACES];
 
-struct supported_cpu_type_list_s {
-    monitor_cpu_type_t *monitor_cpu_type_p;
-    struct supported_cpu_type_list_s *next;
-};
-typedef struct supported_cpu_type_list_s supported_cpu_type_list_t;
-
 /* A linked list of supported monitor_cpu_types for each memspace */
-static supported_cpu_type_list_t *monitor_cpu_type_supported[NUM_MEMSPACES];
+supported_cpu_type_list_t *monitor_cpu_type_supported[NUM_MEMSPACES];
 
 struct monitor_cpu_type_list_s {
     monitor_cpu_type_t monitor_cpu_type;
@@ -638,12 +639,12 @@ bool check_drive_emu_level_ok(int drive_num)
 
 void monitor_cpu_type_set(const char *cpu_type)
 {
-    int serchcpu;
+    int searchcpu;
     monitor_cpu_type_t *monitor_cpu_type_p = NULL;
 
-    serchcpu = find_cpu_type_from_string(cpu_type);
-    if (serchcpu > -1) {
-        monitor_cpu_type_p = monitor_find_cpu_for_memspace(default_memspace, serchcpu);
+    searchcpu = find_cpu_type_from_string(cpu_type);
+    if (searchcpu > -1) {
+        monitor_cpu_type_p = monitor_find_cpu_for_memspace(default_memspace, searchcpu);
     }
     if (monitor_cpu_type_p) {
         monitor_cpu_for_memspace[default_memspace] = monitor_cpu_type_p;
@@ -815,10 +816,12 @@ const char *mon_get_current_bank_name(MEMSPACE mem)
     return mon_get_bank_name_for_bank(mem, mon_interfaces[mem]->current_bank);
 }
 
-const int mon_get_current_bank_index(MEMSPACE mem)
+#if 0
+static const int mon_get_current_bank_index(MEMSPACE mem)
 {
     return mon_get_bank_index_for_bank(mem, mon_interfaces[mem]->current_bank);
 }
+#endif
 
 /*
     main entry point for the monitor to read a value from memory
@@ -956,6 +959,11 @@ void mon_jump(MON_ADDR addr)
 void mon_go(void)
 {
     exit_mon = 1;
+    
+    if (should_pause_on_exit_mon || ui_pause_active()) {
+        should_pause_on_exit_mon = false;
+        pause_on_exit_mon = true;
+    }
 }
 
 /* exit monitor, close monitor window  */
@@ -963,6 +971,11 @@ void mon_exit(void)
 {
     exit_mon = 1;
     mon_console_close_on_leaving = 1;
+    
+    if (should_pause_on_exit_mon || ui_pause_active()) {
+        should_pause_on_exit_mon = false;
+        pause_on_exit_mon = true;
+    }
 }
 
 /* If we want 'quit' for OS/2 I couldn't leave the emulator by calling exit(0)
@@ -1170,10 +1183,7 @@ void mon_show_dir(const char *path)
             if (isdir) {
                 mon_out("     <dir> %s\n", name);
             } else {
-                /* Should use '%zu' here for `len`, but that would break
-                 * Windows :(
-                 */
-                mon_out("%10u %s\n", (unsigned int)len, name);
+                mon_out("%"PRI_SIZE_T" %s\n", len, name);
             }
         } else {
             mon_out("%-20s?????\n", name);
@@ -1231,12 +1241,12 @@ void mon_reset_machine(int type)
     }
 }
 
-void mon_tape_ctrl(int command)
+void mon_tape_ctrl(int port, int command)
 {
     if ((command < 0) || (command > 6)) {
         mon_out("Unknown command.\n");
     } else {
-        datasette_control(command);
+        datasette_control(port, command);
     }
 }
 
@@ -1249,6 +1259,73 @@ void mon_cart_freeze(void)
     }
 }
 
+IO_SIM_RESULT mon_userport_set_output(int value)
+{ 
+    if (machine_class == VICE_MACHINE_CBM5x0) {
+        mon_out("Unsupported.\n");
+        return e_IO_SIM_RESULT_GENERAL_FAILURE;
+    } else if (value >= 0x00 && value <= 0xff) {
+        userport_io_sim_set_pbx_out_lines((uint8_t)value);
+        return e_IO_SIM_RESULT_OK;
+    } else {
+        mon_out("Illegal value.\n");
+        return e_IO_SIM_RESULT_ILLEGAL_VALUE;
+    }
+
+    return 0;
+}
+
+IO_SIM_RESULT mon_joyport_set_output(int port, int value)
+{
+    int command_ok = 0;
+    int port_ok = 1;
+
+    if (value < 0x00 || value > 0xff) {
+        mon_out("Illegal value.\n");
+        return e_IO_SIM_RESULT_ILLEGAL_VALUE;
+    }
+
+    switch (machine_class) {
+        case VICE_MACHINE_C64:
+        case VICE_MACHINE_C128:
+        case VICE_MACHINE_CBM5x0:
+        case VICE_MACHINE_C64DTV:
+        case VICE_MACHINE_C64SC:
+        case VICE_MACHINE_SCPU64:
+            if (port == JOYPORT_1 || port == JOYPORT_2) {
+                command_ok = 1;
+            } else {
+                port_ok = 0;
+            }
+            break;
+        case VICE_MACHINE_VIC20:
+            if (port == JOYPORT_1) {
+                command_ok = 1;
+            } else {
+                port_ok = 0;
+            }
+            break;
+        case VICE_MACHINE_PLUS4:
+            if (port == JOYPORT_1 || port == JOYPORT_2 || port == JOYPORT_6) {
+                command_ok = 1;
+            } else {
+                port_ok = 0;
+            }
+            break;
+    }
+
+    if (command_ok) {
+        joyport_io_sim_set_out_lines((uint8_t)value, port);
+        return e_IO_SIM_RESULT_OK;
+    } else if (!port_ok) {
+        mon_out("Illegal port.\n");
+        return e_IO_SIM_RESULT_ILLEGAL_PORT;
+    } else {
+        mon_out("Unsupported.\n");
+        return e_IO_SIM_RESULT_GENERAL_FAILURE;
+    }
+}
+
 void mon_export(void)
 {
     if (mon_cart_cmd.export_dump != NULL) {
@@ -1436,7 +1513,7 @@ void monitor_shutdown(void)
     
     if (inside_monitor) {
         /* Can happen if VICE thread exited while monitor open */
-        monitor_close(0);
+        monitor_close(false);
     }
     
     if (last_cmd) {
@@ -1600,7 +1677,7 @@ static const resource_int_t resources_int[] = {
     { "MonitorLogEnabled", 0, RES_EVENT_NO, NULL,
       &monitorlogenabled, set_monitor_log_enabled, NULL },
 #ifdef FEATURE_CPUMEMHISTORY
-    { "MonitorChisLines", 4096, RES_EVENT_NO, NULL,
+    { "MonitorChisLines", 8192, RES_EVENT_NO, NULL,
       &monitorchislines, set_monitor_chis_lines, NULL },
 #endif
     { "MonitorScrollbackLines", 4096, RES_EVENT_NO, NULL,
@@ -1642,7 +1719,7 @@ static const cmdline_option_t cmdline_options[] =
       NULL, "Disable logging monitor output to a file" },
     { "-initbreak", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
       monitor_set_initial_breakpoint, NULL, NULL, NULL,
-      "<value>", "Set an initial breakpoint for the monitor" },
+      "<value>", "Set an initial breakpoint for the monitor: <address>, ready, or reset" },
 #ifdef ARCHDEP_SEPERATE_MONITOR_WINDOW
     { "-keepmonopen", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
       NULL, NULL, "KeepMonitorOpen", (resource_value_t)1,
@@ -1749,7 +1826,7 @@ void mon_display_screen(long addr)
                Do we want monitor sidefx in a function that's *supposed*
                to just read from screen memory? */
             data = mon_get_mem_val_ex_nosfx(e_comp_space, bank, (uint16_t)ADDR_LIMIT(base++));
-            data = charset_p_toascii(charset_screencode_to_petcii(data), 1);
+            data = charset_p_toascii(charset_screencode_to_petcii(data), CONVERT_WITH_CTRLCODES);
 
             mon_out("%c", data);
         }
@@ -1972,7 +2049,7 @@ void mon_end_recording(void)
 
 static int monitor_set_moncommands_file(const char *filename, void *extra_param)
 {
-    if (mon_playback_commands(filename) != 0) {
+    if (mon_playback_commands(filename, false) != 0) {
         return -1;
     }
     
@@ -1996,29 +2073,30 @@ static int monitor_set_moncommands_file(const char *filename, void *extra_param)
 }
 
 
-int mon_playback_commands(const char *filename)
+int mon_playback_commands(const char *filename, bool interrupt_current_playback)
 {
     FILE *fp;
     
     log_message(LOG_DEFAULT, "Opening monitor command playback file: %s", filename);
-    
+
     if (playback_fp_stack_size == playback_fp_stack_size_max) {
-        
-        /* Need to grow the playback FP stack. But look out for insane playback include depth. */
-        if (playback_fp_stack_size_max >= PLAYBACK_MAX_RECUSRION) {
-            log_error(LOG_ERR, "Max level of playback file recursion depth %d reached, exiting", playback_fp_stack_size_max);
-            exit(1);
+        /*
+         * Need to grow the playback FP stack. But look out for insane playback include depth.
+         */
+        if (playback_fp_stack_size_max >= PLAYBACK_MAX_FP_DEPTH) {
+            log_error(LOG_ERR, "Max level of playback file depth %d reached, exiting", playback_fp_stack_size_max);
+            archdep_vice_exit(1);
         }
         
         playback_fp_stack_size_max += 1;
         playback_fp_stack       = lib_realloc(playback_fp_stack,       playback_fp_stack_size_max * sizeof(FILE *));
         playback_filename_stack = lib_realloc(playback_filename_stack, playback_fp_stack_size_max * sizeof(char *));
     }
-    
+
     fp = fopen(filename, MODE_READ_TEXT);
     
     if (fp == NULL) {
-        fp = sysfile_open(filename, NULL, MODE_READ_TEXT);
+        fp = sysfile_open(filename, NULL, NULL, MODE_READ_TEXT);
     }
     
     if (fp == NULL) {
@@ -2026,9 +2104,29 @@ int mon_playback_commands(const char *filename)
         return -1;
     }
     
-    playback_filename_stack[playback_fp_stack_size] = lib_strdup(filename);
-    playback_fp_stack[playback_fp_stack_size] = fp;
-    playback_fp = fp;
+    if (interrupt_current_playback || playback_fp_stack_size == 0) {
+        /*
+         * Place the new file at the top of the stack and set the current playback fp,
+         * which means that the commands within this file will be be next commands to execute.
+         */
+        
+        playback_fp = fp;
+
+        playback_fp_stack[playback_fp_stack_size] = fp;
+        playback_filename_stack[playback_fp_stack_size] = lib_strdup(filename);
+    } else {
+        /*
+         * Place the new file at the bottom of the stack, which means that the commands
+         * within will execute after all queued playback files have finished. If no
+         * playback is currently active, start with this file.
+         */
+        
+        memmove(playback_fp_stack + 1,       playback_fp_stack,       playback_fp_stack_size * sizeof(char *));
+        memmove(playback_filename_stack + 1, playback_filename_stack, playback_fp_stack_size * sizeof(FILE *));
+
+        playback_fp_stack[0] = fp;
+        playback_filename_stack[0] = lib_strdup(filename);
+    }
     
     playback_fp_stack_size++;
     
@@ -2059,7 +2157,6 @@ static void playback_end_file(void)
     /* Should we return to emulation? (Yes if -moncommands arg used) */
     if (playback_exit_after_all_playback) {
         playback_exit_after_all_playback = false;
-        
         mon_exit();
     }
 }
@@ -2179,6 +2276,7 @@ void mon_add_name_to_symbol_table(MON_ADDR addr, char *name)
     int old_addr;
     MEMSPACE mem = addr_memspace(addr);
     uint16_t loc = addr_location(addr);
+    int silent = (playback_fp != NULL); /* suppress warnings when playing back label file */
 
     if (mem == e_default_space) {
         mem = default_memspace;
@@ -2193,11 +2291,11 @@ void mon_add_name_to_symbol_table(MON_ADDR addr, char *name)
 
     old_name = mon_symbol_table_lookup_name(mem, loc);
     old_addr = mon_symbol_table_lookup_addr(mem, name);
-    if (old_name && (MON_ADDR)addr_location(old_addr) != addr) {
+    if ((old_name && (MON_ADDR)addr_location(old_addr) != addr) && (!silent)) {
         mon_out("Warning: label(s) for address $%04x already exist.\n", loc);
     }
     if (old_addr >= 0) {
-        if (old_addr != loc) {
+        if ((old_addr != loc) && (!silent)) {
             mon_out("Changing address of label %s from $%04x to $%04x\n",
                     name, (unsigned int)old_addr, loc);
         }
@@ -2770,6 +2868,10 @@ int monitor_diskspace_mem(int dnr)
 
 void monitor_change_device(MEMSPACE mem)
 {
+    /* if no argument given, switch back to computer */
+    if (mem == e_default_space) {
+        mem = e_comp_space;
+    }
     mon_out("Setting default device to `%s'\n", _mon_space_strings[(int) mem]);
     default_memspace = mem;
 }
@@ -2793,6 +2895,7 @@ void monitor_abort(void)
 static void monitor_open(void)
 {
     supported_cpu_type_list_t *slist, *slist_next;
+    monitor_cpu_type_t *monitor_cpu_type_p = NULL;
     unsigned int dnr;
     int i;
 
@@ -2810,7 +2913,8 @@ static void monitor_open(void)
         if (console_log) {
             console_log = uimon_window_resume();
         } else {
-            console_log = uimon_window_open();
+            /* We 'open' the monitor, but only display it if we're not in -moncommands playback mode */
+            console_log = uimon_window_open(playback_fp == NULL);
             uimon_set_interface(mon_interfaces, NUM_MEMSPACES);
         }
     }
@@ -2818,20 +2922,19 @@ static void monitor_open(void)
     if (console_log == NULL) {
         log_error(LOG_DEFAULT, "monitor_open: could not open monitor console.");
         exit_mon = 1;
-        monitor_trap_triggered = false;
         return;
     }
 
-#ifdef FEATURE_CPUHISTORY
+#ifdef FEATURE_CPUMEMHISTORY
     memmap_state |= MEMMAP_STATE_IN_MONITOR;
 #endif
     inside_monitor = true;
-    monitor_trap_triggered = false;
     vsync_suspend_speed_eval();
     sound_suspend();
 
     uimon_notify_change();
 
+    /* free the list of supported CPUs */
     for (i = 0; i < NUM_MEMSPACES; i++) {
         slist = monitor_cpu_type_supported[i];
         while (slist != NULL) {
@@ -2841,6 +2944,7 @@ static void monitor_open(void)
         }
         monitor_cpu_type_supported[i] = NULL;
     }
+
     /* We should really be told what CPUs are supported by each memspace, but that will
      * require a bunch of changes, so for now we detect it based on the available registers. */
     find_supported_monitor_cpu_types(&monitor_cpu_type_supported[e_comp_space], mon_interfaces[e_comp_space]);
@@ -2851,14 +2955,39 @@ static void monitor_open(void)
     }
 
     /* Build array of pointers to monitor_cpu_type structs */
+
+    /* NOTE: We can't just init the struct(s) with the default CPU for each memspace, if
+             we did that, the monitor will not be able to eg single step on any CPU that
+             is not the first in the list. This loop makes sure the last active CPU is
+             still active in the monitor. */
+    for (i = 0; i < NUM_MEMSPACES; i++) {
+        slist = monitor_cpu_type_supported[i];
+        monitor_cpu_type_p = NULL;
+        /* check if the CPU was already set to an available type */
+        while (slist != NULL) {
+            if (slist->monitor_cpu_type_p == monitor_cpu_for_memspace[i]) {
+                monitor_cpu_type_p = slist->monitor_cpu_type_p;
+            }
+            slist = slist->next;
+        }
+        /* if no matching CPU was set, use the first supported one, if there is any */
+        if (monitor_cpu_type_p == NULL && monitor_cpu_type_supported[i]) {
+            monitor_cpu_for_memspace[i] = monitor_cpu_type_supported[i]->monitor_cpu_type_p;
+        }
+    }
+
+#if 0
     monitor_cpu_for_memspace[e_comp_space] =
         monitor_cpu_type_supported[e_comp_space]->monitor_cpu_type_p;
+
     for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
         monitor_cpu_for_memspace[monitor_diskspace_mem(dnr)] =
             monitor_cpu_type_supported[monitor_diskspace_mem(dnr)]->monitor_cpu_type_p;
     }
+#endif
     /* Safety precaution */
-    monitor_cpu_for_memspace[default_memspace] = monitor_cpu_for_memspace[default_memspace];
+    /* FIXME: this makes no sense at all */
+    /* monitor_cpu_for_memspace[default_memspace] = monitor_cpu_for_memspace[default_memspace]; */
 
     dot_addr[e_comp_space] = new_addr(e_comp_space, ((uint16_t)((monitor_cpu_for_memspace[e_comp_space]->mon_register_get_val)(e_comp_space, e_PC))));
 
@@ -2915,7 +3044,7 @@ static int monitor_process(char *cmd)
                 lib_free(trimmed_command);
                 trimmed_command = NULL;
             }
-
+            
             parse_and_execute_line(cmd);
         }
     }
@@ -2934,9 +3063,9 @@ static int monitor_process(char *cmd)
     return exit_mon;
 }
 
-static void monitor_close(int check)
+static void monitor_close(bool check_exit)
 {
-#ifdef FEATURE_CPUHISTORY
+#ifdef FEATURE_CPUMEMHISTORY
     memmap_state &= ~(MEMMAP_STATE_IN_MONITOR);
 #endif
     inside_monitor = false;
@@ -2945,7 +3074,7 @@ static void monitor_close(int check)
         exit_mon--;
     }
 
-    if (check && exit_mon) {
+    if (check_exit && exit_mon) {
         if (!monitor_is_remote()) {
             uimon_window_close();
         }
@@ -2984,6 +3113,21 @@ void monitor_startup(MEMSPACE mem)
 {
     char prompt[40];
     char *p;
+    
+    if (inside_monitor) {
+        /*
+         * Drive cpu interrupt can enter the monitor .. and then the monitor
+         * can tell the drive to catch up, re-triggering the incomplete interrupt
+         * .. recursion, etc
+         */
+        return;
+    }
+    
+    if (ui_pause_active()) {
+        should_pause_on_exit_mon = true;
+        
+        ui_pause_disable();
+    }
 
     if (mem != e_default_space) {
         default_memspace = mem;
@@ -2995,6 +3139,10 @@ void monitor_startup(MEMSPACE mem)
         if (playback_fp) {
             playback_next_command();
         } else {
+            /*
+             * We are about to go interactive.
+             */
+
             if (refresh_on_break) {
                 /* Render all in-progress frames as we enter the prompt */
                 video_canvas_refresh_all_tracked();
@@ -3005,7 +3153,7 @@ void monitor_startup(MEMSPACE mem)
             if (p) {
                 exit_mon = monitor_process(p);
             } else if (exit_mon < 1) {
-                exit_mon = 1;
+                mon_exit();
             }
         }
 
@@ -3014,7 +3162,15 @@ void monitor_startup(MEMSPACE mem)
             break;
         }
     }
-    monitor_close(1);
+    monitor_close(true);
+    
+    if (pause_on_exit_mon) {
+        pause_on_exit_mon = false;
+        
+        ui_pause_enable();
+        
+        while (ui_pause_loop_iteration());
+    }
 }
 
 static void monitor_trap(uint16_t addr, void *unused_data)
@@ -3024,8 +3180,7 @@ static void monitor_trap(uint16_t addr, void *unused_data)
 
 void monitor_startup_trap(void)
 {
-    if (!monitor_trap_triggered && !inside_monitor) {
-        monitor_trap_triggered = true;
+    if (!inside_monitor) {
         interrupt_maincpu_trigger_trap(monitor_trap, 0);
     }
 }
