@@ -381,7 +381,7 @@ static int network_send_buffer(vice_network_socket_t * s, const uint8_t *buf, in
 #define NUM_OF_TESTPACKETS 50
 
 typedef struct {
-    unsigned long t;
+    tick_t t;
     unsigned char buf[0x60];
 } testpacket;
 
@@ -392,7 +392,7 @@ static void network_test_delay(void)
     unsigned char *buf;
     testpacket pkt;
 
-    long packet_delay[NUM_OF_TESTPACKETS];
+    tick_t packet_delay[NUM_OF_TESTPACKETS];
     char st[256];
 
     ui_display_statustext("Testing best frame delay...", 0);
@@ -406,13 +406,13 @@ static void network_test_delay(void)
                 || network_recv_buffer(network_socket, buf, sizeof(testpacket)) < 0) {
                 return;
             }
-            packet_delay[i] = tick_delta(pkt.t);
+            packet_delay[i] = tick_now_delta(pkt.t);
         }
         /* Sort the packets delays*/
         for (i = 0; i < NUM_OF_TESTPACKETS - 1; i++) {
             for (j = i + 1; j < NUM_OF_TESTPACKETS; j++) {
                 if (packet_delay[i] < packet_delay[j]) {
-                    long d = packet_delay[i];
+                    tick_t d = packet_delay[i];
                     packet_delay[i] = packet_delay[j];
                     packet_delay[j] = d;
                 }
@@ -571,7 +571,7 @@ static void network_client_connect_trap(uint16_t addr, void *data)
 void network_event_record(unsigned int type, void *data, unsigned int size)
 {
     unsigned int control = 0;
-    uint8_t joyport;
+    uint16_t joyport;
 
     switch (type) {
         case EVENT_KEYBOARD_MATRIX:
@@ -590,7 +590,7 @@ void network_event_record(unsigned int type, void *data, unsigned int size)
             control = NETWORK_CONTROL_RSRC;
             break;
         case EVENT_JOYSTICK_VALUE:
-            joyport = ((uint8_t *)data)[0];
+            joyport = ((uint16_t *)data)[0];
             if (joyport == 1) {
                 control = NETWORK_CONTROL_JOY1;
             }
@@ -804,7 +804,7 @@ static void network_hook_connected_send(void)
         network_disconnect();
     }
 #ifdef NETWORK_DEBUG
-    t2 = tick_after(t1);
+    t2 = tick_now_after(t1);
 #endif
 
     lib_free(local_event_buf);
@@ -854,7 +854,7 @@ static void network_hook_connected_receive(void)
         }
 
 #ifdef NETWORK_DEBUG
-        t3 = tick_after(t2);
+        t3 = tick_now_after(t2);
 #endif
 
         remote_event_list = network_create_event_list(remote_event_buf);
@@ -893,7 +893,7 @@ static void network_hook_connected_receive(void)
     }
     network_prepare_next_frame();
 #ifdef NETWORK_DEBUG
-    t4 = tick_after(t3);
+    t4 = tick_now_after(t3);
 #endif
 }
 
