@@ -105,6 +105,12 @@ static const cmdline_option_t cmdline_options[] =
     { "+eoiblank", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
       NULL, NULL, "EoiBlank", (void *)0,
       NULL, "EOI does not blank screen" },
+    { "-screen2001", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "Screen2001", (void *)1,
+      NULL, "4x1K screen mirrors on 2001" },
+    { "+screen2001", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "Screen2001", (void *)0,
+      NULL, "screen mirrors like later models" },
     { "-cpu6502", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
       NULL, NULL, "CPUswitch", (void *)SUPERPET_CPU_6502,
       NULL, "Set SuperPET CPU switch to '6502'" },
