@@ -33,6 +33,7 @@
 #include "6809.h"
 #include "crtc.h"
 #include "drive-snapshot.h"
+#include "serial.h"
 #include "ioutil.h"
 #include "joystick.h"
 #include "keyboard.h"
@@ -54,7 +55,7 @@
 #include "via.h"
 #include "vice-event.h"
 
-#define SNAP_MAJOR 0
+#define SNAP_MAJOR 1
 #define SNAP_MINOR 0
 
 int pet_snapshot_write(const char *name, int save_roms, int save_disks,
@@ -80,6 +81,7 @@ int pet_snapshot_write(const char *name, int save_roms, int save_disks,
         || petdww_snapshot_write_module(s) < 0
         || viacore_snapshot_write_module(machine_context.via, s) < 0
         || drive_snapshot_write_module(s, save_disks, save_roms) < 0
+        || fsdrive_snapshot_write_module(s) < 0
         || event_snapshot_write_module(s, event_mode) < 0
         || tapeport_snapshot_write_module(s, save_disks) < 0
         || keyboard_snapshot_write_module(s) < 0
@@ -128,6 +130,7 @@ int pet_snapshot_read(const char *name, int event_mode)
         || petdww_snapshot_read_module(s) < 0
         || viacore_snapshot_read_module(machine_context.via, s) < 0
         || drive_snapshot_read_module(s) < 0
+        || fsdrive_snapshot_read_module(s) < 0
         || event_snapshot_read_module(s, event_mode) < 0
         || tapeport_snapshot_read_module(s) < 0
         || keyboard_snapshot_read_module(s) < 0
