@@ -100,6 +100,8 @@ static const char module_ram_name[] = "PETMEM";
  *                              Added in format V1.2
  * BYTE         EOIBLANK        bit 0=0: EOI does not blank screen
  *                                   =1: EOI does blank screen
+ *                              bit 1=0: Screen memory like 3000 and later
+ *                                    1: Screen memory like 2001
  *
  *                              Added in format V1.3
  * WORD         CPU_SWITCH      6502 / 6809 / PROG
@@ -174,7 +176,8 @@ static int mem_write_ram_snapshot_module(snapshot_t *s)
     /* V1.1 */
     SMW_B(m, (uint8_t)(kbdindex & 1));
     /* V1.2 */
-    SMW_B(m, (uint8_t)(petres.eoiblank ? 1 : 0));
+    SMW_B(m, (uint8_t)((petres.eoiblank ? 1 : 0) |
+                       (petres.screenmirrors2001 ? 2 : 0)));
     /* V1.3 */
     SMW_W(m, (uint16_t)petres.superpet_cpu_switch);
     SMW_B(m, (uint8_t)dongle6702.val);
@@ -204,8 +207,11 @@ static int mem_read_ram_snapshot_module(snapshot_t *s)
     snapshot_module_t *m;
     uint8_t config, rconf, byte, memsize, conf8x96, superpet;
     petinfo_t peti = {
-        32, 0x0800, 1, 80, 0, 0, 0, 0, 0, 0, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL, NULL, { NULL }
+        /* Defaults like a 8032 */
+        .ramSize = RAM_32K,
+        .IOSize = IO_2048,
+        .crtc = HAS_CRTC,
+        .video = COLS_80,
     };
     int old6809mode;
     int spetbank = 0;
@@ -301,6 +307,7 @@ static int mem_read_ram_snapshot_module(snapshot_t *s)
     if (vminor > 1) {
         SMR_B(m, &byte);
         resources_set_int("EoiBlank", byte & 1);
+        resources_set_int("Screen2001", (byte & 2) >> 1);
     }
     if (vminor > 2) {
         int new6809mode, i;
@@ -351,6 +358,37 @@ static int mem_read_ram_snapshot_module(snapshot_t *s)
     return 0;
 }
 
+#define NUM_TRAP_DEVICES 9  /* FIXME: is there a better constant ? */
+static int trapfl[NUM_TRAP_DEVICES];
+static int trapdevices[NUM_TRAP_DEVICES + 1] = { 1, 4, 5, 6, 7, 8, 9, 10, 11, -1 };
+
+static void get_trapflags(void)
+{
+    int i;
+    for(i = 0; trapdevices[i] != -1; i++) {
+        resources_get_int_sprintf("VirtualDevice%d", &trapfl[i], trapdevices[i]);
+        printf("got %d = %d\n", trapdevices[i], trapfl[i]);
+    }
+}
+
+static void clear_trapflags(void)
+{
+    int i;
+    for(i = 0; trapdevices[i] != -1; i++) {
+        resources_set_int_sprintf("VirtualDevice%d", 0, trapdevices[i]);
+        printf("clear %d = %d\n", trapdevices[i], 0);
+    }
+}
+
+static void restore_trapflags(void)
+{
+    int i;
+    for(i = 0; trapdevices[i] != -1; i++) {
+        resources_set_int_sprintf("VirtualDevice%d", trapfl[i], trapdevices[i]);
+        printf("restore %d = %d\n", trapdevices[i], trapfl[i]);
+    }
+}
+
 static const char module_rom_name[] = "PETROM";
 #define PETROM_DUMP_VER_MAJOR   1
 #define PETROM_DUMP_VER_MINOR   1
@@ -382,7 +420,7 @@ static int mem_write_rom_snapshot_module(snapshot_t *s, int save_roms)
 {
     snapshot_module_t *m;
     uint8_t config;
-    int i, trapfl;
+    int i;
 
     if (!save_roms) {
         return 0;
@@ -395,8 +433,8 @@ static int mem_write_rom_snapshot_module(snapshot_t *s, int save_roms)
     }
 
     /* disable traps before saving the ROM */
-    resources_get_int("VirtualDevices", &trapfl);
-    resources_set_int("VirtualDevices", 0);
+    get_trapflags();
+    clear_trapflags();
     petrom_unpatch_2001();
 
     config = (petrom_9_loaded ? 1 : 0)
@@ -449,7 +487,7 @@ static int mem_write_rom_snapshot_module(snapshot_t *s, int save_roms)
     }
 
     /* enable traps again when necessary */
-    resources_set_int("VirtualDevices", trapfl);
+    restore_trapflags();
     petrom_patch_2001();
 
     snapshot_module_close(m);
@@ -462,7 +500,7 @@ static int mem_read_rom_snapshot_module(snapshot_t *s)
     uint8_t vmajor, vminor;
     snapshot_module_t *m;
     uint8_t config;
-    int trapfl, new_iosize;
+    int new_iosize;
 
     m = snapshot_module_open(s, module_rom_name, &vmajor, &vminor);
     if (m == NULL) {
@@ -477,8 +515,8 @@ static int mem_read_rom_snapshot_module(snapshot_t *s)
     }
 
     /* disable traps before loading the ROM */
-    resources_get_int("VirtualDevices", &trapfl);
-    resources_set_int("VirtualDevices", 0);
+    get_trapflags();
+    clear_trapflags();
     petrom_unpatch_2001();
 
     config = (petrom_9_loaded ? 1 : 0)
@@ -561,7 +599,7 @@ static int mem_read_rom_snapshot_module(snapshot_t *s)
     petrom_patch_2001();
 
     /* enable traps again when necessary */
-    resources_set_int("VirtualDevices", trapfl);
+    restore_trapflags();
 
     snapshot_module_close(m);
 
