@@ -32,7 +32,7 @@
 
 extern void petsound_store_onoff(int value);
 extern void petsound_store_rate(CLOCK t);
-extern void petsound_store_sample(uint8_t value);
+extern void petsound_store_waveform(uint8_t value);
 extern void petsound_store_manual(int value);
 
 extern void petsound_reset(sound_t *psid, CLOCK cpu_clk);
