@@ -30,6 +30,7 @@
 #include <string.h>
 #include <assert.h>
 
+#include "archdep.h"
 #include "cartio.h"
 #include "cartridge.h"
 #include "cmdline.h"
@@ -422,6 +423,13 @@ io_source_list_t *io_source_register(io_source_t *device)
         case 0xfe00:
             current = &plus4io_fe00_head;
             break;
+        default:
+            log_error(LOG_DEFAULT,
+                    "io_source_register internal error: I/O range 0x%04x "
+                    "does not exist",
+                    device->start_address & 0xff00U);
+            archdep_vice_exit(-1);
+            break;
     }
 
     while (current->next != NULL) {
