@@ -31,6 +31,7 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "cartridge.h"
 #include "cartio.h"
 #include "datasette.h"
 #include "digiblaster.h"
@@ -39,7 +40,9 @@
 #include "mem.h"
 #include "monitor.h"
 #include "plus4iec.h"
+#include "plus4cart.h"
 #include "plus4mem.h"
+#include "plus4memrom.h"
 #include "plus4memcsory256k.h"
 #include "plus4memhannes256k.h"
 #include "plus4memhacks.h"
@@ -65,19 +68,6 @@ static int hard_reset_flag = 1;
 /* The Plus4 memory.  */
 uint8_t mem_ram[PLUS4_RAM_SIZE];
 
-#ifdef USE_EMBEDDED
-#include "plus43plus1lo.h"
-#include "plus43plus1hi.h"
-#else
-uint8_t extromlo1[PLUS4_BASIC_ROM_SIZE];
-uint8_t extromhi1[PLUS4_KERNAL_ROM_SIZE];
-#endif
-
-uint8_t extromlo2[PLUS4_BASIC_ROM_SIZE];
-uint8_t extromlo3[PLUS4_BASIC_ROM_SIZE];
-uint8_t extromhi2[PLUS4_KERNAL_ROM_SIZE];
-uint8_t extromhi3[PLUS4_KERNAL_ROM_SIZE];
-
 /* Pointers to the currently used memory read and write tables.  */
 read_func_ptr_t *_mem_read_tab_ptr;
 store_func_ptr_t *_mem_write_tab_ptr;
@@ -141,6 +131,8 @@ static uint8_t *chargen_tab[8][16] = {
             RAM4, RAM4, RAM4, RAM4,
             RAM4, RAM4, RAM4, RAM4 },
     /* 8000-bfff, ROM selected  */
+    /* FIXME: we cant directly point to cartridge ROM here, we need a better
+              (indirect) way to do this */
     {  plus4memrom_basic_rom, extromlo1, extromlo2, extromlo3,
        plus4memrom_basic_rom, extromlo1, extromlo2, extromlo3,
        plus4memrom_basic_rom, extromlo1, extromlo2, extromlo3,
@@ -190,14 +182,14 @@ inline static void mem_proc_port_store(void)
 
     if (((~pport.dir | pport.data) & 0x02) != old_port_write_bit) {
         old_port_write_bit = (~pport.dir | pport.data) & 0x02;
-        tapeport_toggle_write_bit((~pport.dir | ~pport.data) & 0x02);
+        tapeport_toggle_write_bit(TAPEPORT_PORT_1, (~pport.dir | ~pport.data) & 0x02);
     }
 
     (*iecbus_callback_write)((uint8_t)~pport.data_out, last_write_cycle);
 
     if (((pport.dir & pport.data) & 0x08) != old_port_data_out) {
         old_port_data_out = (pport.dir & pport.data) & 0x08;
-        tapeport_set_motor(!old_port_data_out);
+        tapeport_set_motor(TAPEPORT_PORT_1, !old_port_data_out);
     }
 }
 
@@ -333,7 +325,7 @@ static void mem_config_set(unsigned int config)
     mem_config = config;
 
     mem_update_tab_ptrs(watchpoints_active);
-    
+
     _mem_read_base_tab_ptr = mem_read_base_tab[mem_config];
     mem_read_limit_tab_ptr = mem_read_limit_tab[mem_config];
 
@@ -730,32 +722,36 @@ void mem_initialize_memory(void)
         mem_read_base_tab[1][i] = plus4memrom_basic_rom + ((i & 0x3f) << 8);
         mem_read_tab[3][i] = plus4memrom_extromlo1_read;
         mem_read_base_tab[3][i] = extromlo1 + ((i & 0x3f) << 8);
-        mem_read_tab[5][i] = plus4memrom_extromlo2_read;
-        mem_read_base_tab[5][i] = extromlo2 + ((i & 0x3f) << 8);
+        mem_read_tab[5][i] = plus4cart_c1lo_read;
+        /*mem_read_base_tab[5][i] = extromlo2 + ((i & 0x3f) << 8);*/
+        mem_read_base_tab[5][i] = NULL;
         mem_read_tab[7][i] = plus4memrom_extromlo3_read;
         mem_read_base_tab[7][i] = extromlo3 + ((i & 0x3f) << 8);
         mem_read_tab[9][i] = plus4memrom_basic_read;
         mem_read_base_tab[9][i] = plus4memrom_basic_rom + ((i & 0x3f) << 8);
         mem_read_tab[11][i] = plus4memrom_extromlo1_read;
         mem_read_base_tab[11][i] = extromlo1 + ((i & 0x3f) << 8);
-        mem_read_tab[13][i] = plus4memrom_extromlo2_read;
-        mem_read_base_tab[13][i] = extromlo2 + ((i & 0x3f) << 8);
+        mem_read_tab[13][i] = plus4cart_c1lo_read;
+        /*mem_read_base_tab[13][i] = extromlo2 + ((i & 0x3f) << 8);*/
+        mem_read_base_tab[13][i] = NULL;
         mem_read_tab[15][i] = plus4memrom_extromlo3_read;
         mem_read_base_tab[15][i] = extromlo3 + ((i & 0x3f) << 8);
         mem_read_tab[17][i] = plus4memrom_basic_read;
         mem_read_base_tab[17][i] = plus4memrom_basic_rom + ((i & 0x3f) << 8);
         mem_read_tab[19][i] = plus4memrom_extromlo1_read;
         mem_read_base_tab[19][i] = extromlo1 + ((i & 0x3f) << 8);
-        mem_read_tab[21][i] = plus4memrom_extromlo2_read;
-        mem_read_base_tab[21][i] = extromlo2 + ((i & 0x3f) << 8);
+        mem_read_tab[21][i] = plus4cart_c1lo_read;
+        /*mem_read_base_tab[21][i] = extromlo2 + ((i & 0x3f) << 8);*/
+        mem_read_base_tab[21][i] = NULL;
         mem_read_tab[23][i] = plus4memrom_extromlo3_read;
         mem_read_base_tab[23][i] = extromlo3 + ((i & 0x3f) << 8);
         mem_read_tab[25][i] = plus4memrom_basic_read;
         mem_read_base_tab[25][i] = plus4memrom_basic_rom + ((i & 0x3f) << 8);
         mem_read_tab[27][i] = plus4memrom_extromlo1_read;
         mem_read_base_tab[27][i] = extromlo1 + ((i & 0x3f) << 8);
-        mem_read_tab[29][i] = plus4memrom_extromlo2_read;
-        mem_read_base_tab[29][i] = extromlo2 + ((i & 0x3f) << 8);
+        mem_read_tab[29][i] = plus4cart_c1lo_read;
+        /*mem_read_base_tab[29][i] = extromlo2 + ((i & 0x3f) << 8);*/
+        mem_read_base_tab[29][i] = NULL;
         mem_read_tab[31][i] = plus4memrom_extromlo3_read;
         mem_read_base_tab[31][i] = extromlo3 + ((i & 0x3f) << 8);
     }
@@ -782,14 +778,18 @@ void mem_initialize_memory(void)
         mem_read_base_tab[13][i] = extromhi1 + ((i & 0x3f) << 8);
         mem_read_tab[15][i] = plus4memrom_extromhi1_read;
         mem_read_base_tab[15][i] = extromhi1 + ((i & 0x3f) << 8);
-        mem_read_tab[17][i] = plus4memrom_extromhi2_read;
-        mem_read_base_tab[17][i] = extromhi2 + ((i & 0x3f) << 8);
-        mem_read_tab[19][i] = plus4memrom_extromhi2_read;
-        mem_read_base_tab[19][i] = extromhi2 + ((i & 0x3f) << 8);
-        mem_read_tab[21][i] = plus4memrom_extromhi2_read;
-        mem_read_base_tab[21][i] = extromhi2 + ((i & 0x3f) << 8);
-        mem_read_tab[23][i] = plus4memrom_extromhi2_read;
-        mem_read_base_tab[23][i] = extromhi2 + ((i & 0x3f) << 8);
+        mem_read_tab[17][i] = plus4cart_c1hi_read;
+        /*mem_read_base_tab[17][i] = extromhi2 + ((i & 0x3f) << 8);*/
+        mem_read_base_tab[17][i] = NULL;
+        mem_read_tab[19][i] = plus4cart_c1hi_read;
+        /*mem_read_base_tab[19][i] = extromhi2 + ((i & 0x3f) << 8);*/
+        mem_read_base_tab[19][i] = NULL;
+        mem_read_tab[21][i] = plus4cart_c1hi_read;
+        /*mem_read_base_tab[21][i] = extromhi2 + ((i & 0x3f) << 8);*/
+        mem_read_base_tab[21][i] = NULL;
+        mem_read_tab[23][i] = plus4cart_c1hi_read;
+        /*mem_read_base_tab[23][i] = extromhi2 + ((i & 0x3f) << 8);*/
+        mem_read_base_tab[23][i] = NULL;
         mem_read_tab[25][i] = plus4memrom_extromhi3_read;
         mem_read_base_tab[25][i] = extromhi3 + ((i & 0x3f) << 8);
         mem_read_tab[27][i] = plus4memrom_extromhi3_read;
@@ -885,9 +885,13 @@ void mem_mmu_translate(unsigned int addr, uint8_t **base, int *start, int *limit
 {
     uint8_t *p = _mem_read_base_tab_ptr[addr >> 8];
 
-    *base = (p == NULL) ? NULL : (p - (addr & 0xff00));
-    *start = addr; /* TODO */
-    *limit = mem_read_limit_tab_ptr[addr >> 8];
+    if (p != NULL && addr > 1) {
+        *base = (p - (addr & 0xff00));
+        *start = addr; /* TODO */
+        *limit = mem_read_limit_tab_ptr[addr >> 8];
+    } else {
+        cartridge_mmu_translate(addr, base, start, limit);
+    }
 }
 
 /* ------------------------------------------------------------------------- */
@@ -979,7 +983,7 @@ static const char *banknames[MAXBANKS + 1] = {
     NULL
 };
 
-static const int banknums[MAXBANKS + 1] = { 1, 0, 1, 2, 6, 3, 4, 5, -1 };
+static const int banknums[MAXBANKS + 1] = { 0, 1, 1, 2, 6, 3, 4, 5, -1 };
 static const int bankindex[MAXBANKS + 1] = { -1, -1, -1, -1, -1, -1, -1, -1, -1 };
 static const int bankflags[MAXBANKS + 1] = { 0, 0, 0, 0, 0, 0, 0, 0, -1 };
 
@@ -1098,39 +1102,39 @@ uint8_t mem_bank_peek(int bank, uint16_t addr, void *context)
                 $0000-$7fff   RAM
                 $8000-$9fff   RAM / BASIC / Function LO
                 $a000-$bfff   RAM / Kernal / Function HI
-                
+
                 $c000-$cfff   RAM / Basic Extension
-                
+
                 $d000-$d7ff   RAM / character ROM / Function HI
                 $d800-$fbff   RAM / operating system
-              
+
                 $FC00-        Kernal Routines for switching banks
-              
+
                 $FD00-$FF3F always I/O:
 
                     $FD00-FD0F: 6551  (only on the +4.  4 registers.)
                     $FD10-FD1F: 6529B (1 register)
                     $FD30-FD3F: 6529B (1 register)
-                
+
                 $FDD0-$FDDF ROM bank select
-                
+
                     a0 a1 bank
                     0  0  BASIC (low internal #1)
                     0  1  Function LO (low internal #2)
                     1  0  Cartridge LO (low external #1)
                     1  1  reserved
-                
+
                     a2 a3 bank
                     0  0  Kernal (hi internal #1)
                     0  1  Function HI (hi internal #2)
                     1  0  Cartridge HI (hi external #1)
                     1  1  reserved
-                
+
                 $FF00-  TED registers
-                
+
                 $FF3E   ROM select, Write switches on ROM bank
                 $FF3F   RAM select, Write switches on RAM bank                
-                
+
                 $FF40-$FFFF RAM / Kernal / Function HI
             */
             if ((addr >= 0xfd00) && (addr <= 0xfd3f)) {
@@ -1177,10 +1181,12 @@ uint8_t mem_bank_read(int bank, uint16_t addr, void *context)
             break;
         case 4:                   /* cart1rom */
             if (addr >= 0x8000 && addr <= 0xbfff) {
-                return extromlo2[addr & 0x3fff];
+                /* return extromlo2[addr & 0x3fff]; */
+                return plus4cart_c1lo_read(addr);
             }
             if (addr >= 0xc000) {
-                return extromhi2[addr & 0x3fff];
+                /* return extromhi2[addr & 0x3fff]; */
+                return plus4cart_c1hi_read(addr);
             }
             break;
         case 5:                   /* cart2rom */
@@ -1288,19 +1294,25 @@ void mem_get_cursor_parameter(uint16_t *screen_addr, uint8_t *cursor_column, uin
 {
     unsigned int cursorposition = (ted_peek(0xff0d) + ((ted_peek(0xff0c) & 3) * 256));
     unsigned int screenbase = ((ted_peek(0xff14) & 0xf8) << 8 | 0x400);
-#if 0
-    /* this does not work, apparently the kernal does update those too late */
-    *screen_addr = mem_ram[0xc8] + mem_ram[0xc9] * 256; /* Current Screen Line Address */
-    *cursor_column = mem_ram[0xca];    /* Cursor Column on Current Line */
-#endif
+
     *line_length = 40;                 /* Physical Screen Line Length */
-    /* we will derive screen line and column from the position of the hardware cursor */
-    *cursor_column = cursorposition % 40; /* Cursor Column on Current Line */
-    *screen_addr = screenbase + cursorposition - *cursor_column; /* Current Screen Line Address */
-    /* Cursor Blink enable: 1 = Flash Cursor, 0 = Cursor disabled, -1 = n/a
-       we have to check cursor position >= 1000, which means it is not visible */
-    *blinking = (cursorposition < 1000);
-/*    printf("mem_get_cursor_parameter screen_addr:%04x column: %d line len: %d blinking:%d\n",
+
+    if (cursorposition < 1000) {
+        /* cursor position < 1000 means the cursor is visible/enabled, in
+           this case we will derive screen line and column from the position
+           of the hardware cursor */
+        *cursor_column = cursorposition % 40; /* Cursor Column on Current Line */
+        *screen_addr = screenbase + cursorposition - *cursor_column; /* Current Screen Line Address */
+        /* Cursor Blink enable: 1 = Flash Cursor, 0 = Cursor disabled, -1 = n/a */
+        *blinking = 1;
+    } else {
+        /* when cursor is disabled, we use the kernal pointers and hope they
+           are not completely wrong */
+        *screen_addr = mem_ram[0xc8] + mem_ram[0xc9] * 256; /* Current Screen Line Address */
+        *cursor_column = mem_ram[0xca];    /* Cursor Column on Current Line */
+        *blinking = 0;
+    }
+    /* printf("mem_get_cursor_parameter screen_addr:%04x column: %d line len: %d blinking:%d\n",
            *screen_addr, *cursor_column, *line_length, *blinking); */
 }
 
@@ -1332,8 +1344,8 @@ static mem_config_t mem_config_table[] = {
 
 static int memconfig_dump(void)
 {
-    mon_out("$8000-$BFFF: %s", (mem_config & 1) ? mem_config_table[mem_config >> 1].mem_8000 : "RAM");
-    mon_out("$C000-$FFFF: %s", (mem_config & 1) ? mem_config_table[mem_config >> 1].mem_c000 : "RAM");
+    mon_out("$8000-$BFFF: %s\n", (mem_config & 1) ? mem_config_table[mem_config >> 1].mem_8000 : "RAM");
+    mon_out("$C000-$FFFF: %s\n", (mem_config & 1) ? mem_config_table[mem_config >> 1].mem_c000 : "RAM");
 
     return 0;
 }
