@@ -51,15 +51,7 @@ uint8_t pio1_read(uint16_t addr)
     /*  Correct clock */
     ted_handle_pending_alarms(0);
 
-    /* The functions below will gradually be removed as the functionality is added to the new userport system. */
-    if (diskunit_context[0]->parallel_cable
-        || diskunit_context[1]->parallel_cable) {
-        pio1_value = parallel_cable_cpu_read(DRIVE_PC_STANDARD, pio1_value);
-    } else {
-        pio1_value = pio1_data;
-    }
-
-    pio1_value = read_userport_pbx(0xff, pio1_value);
+    pio1_value = read_userport_pbx(pio1_data);
 
     if (tape_sense) {
         pio1_value &= ~4;
@@ -83,13 +75,7 @@ void pio1_store(uint16_t addr, uint8_t value)
         pio1_outline &= ~4;
     }
 
-    store_userport_pbx(pio1_outline);
-
-    /* The functions below will gradually be removed as the functionality is added to the new userport system. */
-    if (diskunit_context[0]->parallel_cable
-        || diskunit_context[1]->parallel_cable) {
-        parallel_cable_cpu_write(DRIVE_PC_STANDARD, pio1_outline);
-    }
+    store_userport_pbx(pio1_outline, USERPORT_NO_PULSE);
 }
 
 void pio1_set_tape_sense(int sense)
@@ -104,10 +90,7 @@ void pio1_set_tape_sense(int sense)
         pio1_outline &= ~4;
     }
 
-    if (diskunit_context[0]->parallel_cable
-        || diskunit_context[1]->parallel_cable) {
-        parallel_cable_cpu_write(DRIVE_PC_STANDARD, pio1_outline);
-    }
+    store_userport_pbx(pio1_outline, USERPORT_NO_PULSE);
 }
 
 /*
