@@ -26,6 +26,8 @@
 
 #include "vice.h"
 
+#include <limits.h>
+#include <math.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -36,55 +38,59 @@
 #include "resources.h"
 #include "types.h"
 
-static int start_value = 255;
-static int value_invert = 128;
-static int value_offset = 0;
+#define RANDOM_CHANCE_MAX   0x1000
 
-static int pattern_invert = 0;
-static int pattern_invert_value = 0;
+static RAMINITPARAM mainramparam = {
+    .start_value = 255,
+    .value_invert = 128,
+    .value_offset = 0,
 
-static int random_start = 0;
-static int random_repeat = 0;
-static int random_chance = 0;
+    .pattern_invert = 0,
+    .pattern_invert_value = 0,
+
+    .random_start = 0,
+    .random_repeat = 0,
+    .random_chance = 0,
+};
 
 static int set_start_value(int val, void *param)
 {
-    start_value = val;
-    if (start_value < 0) {
-        start_value = 0;
+    mainramparam.start_value = val;
+    if (mainramparam.start_value < 0) {
+        mainramparam.start_value = 0;
     }
-    if (start_value > 0xff) {
-        start_value = 0xff;
+    if (mainramparam.start_value > 0xff) {
+        mainramparam.start_value = 0xff;
     }
     return 0;
 }
 
 static int set_value_invert(int val, void *param)
 {
-    value_invert = val;
+    mainramparam.value_invert = val;
     return 0;
 }
 
 static int set_value_offset(int val, void *param)
 {
-    value_offset = val;
+    mainramparam.value_offset = val;
     return 0;
 }
 
 static int set_pattern_invert(int val, void *param)
 {
-    pattern_invert = val;
+    mainramparam.pattern_invert = val;
     return 0;
 }
 
 static int set_pattern_invert_value(int val, void *param)
 {
-    pattern_invert_value = val;
-    if (pattern_invert_value < 0) {
-        pattern_invert_value = 0;
+    mainramparam.pattern_invert_value = val;
+    if (mainramparam.pattern_invert_value < 0) {
+        mainramparam.pattern_invert_value = 0;
     }
-    if (pattern_invert_value > 0xff) {
-        pattern_invert_value = 0xff;
+    if (mainramparam.pattern_invert_value > 0xff) {
+        mainramparam.pattern_invert_value = 0xff;
     }
     return 0;
 }
@@ -92,53 +98,58 @@ static int set_pattern_invert_value(int val, void *param)
 
 static int set_random_start(int val, void *param)
 {
-    random_start = val;
-    if (random_start < 0) {
-        random_start = 0;
+    mainramparam.random_start = val;
+    if (mainramparam.random_start < 0) {
+        mainramparam.random_start = 0;
     }
-    if (random_start > 0xff) {
-        random_start = 0xff;
+    if (mainramparam.random_start > 0xff) {
+        mainramparam.random_start = 0xff;
     }
     return 0;
 }
 
 static int set_random_repeat(int val, void *param)
 {
-    random_repeat = val;
+    mainramparam.random_repeat = val;
     return 0;
 }
 
 static int set_random_chance(int val, void *param)
 {
-    random_chance = val;
+    if (val > RANDOM_CHANCE_MAX) {
+        val = RANDOM_CHANCE_MAX;
+    } else if (val < 0) {
+        val = 0;
+    }
+    mainramparam.random_chance = val;
     return 0;
 }
 
 /* FIXME: the defaults have been choosen so the result matches a real reported
           pattern in x64sc, AND from those one was picked so all raminitvalue 
           tests pass.
-          
+
           however, the respective defaults should probably be different per
           emulator/machine.
 */
 /* RAM-related resources. */
 static const resource_int_t resources_int[] = {
     { "RAMInitValueOffset", 2, RES_EVENT_SAME, NULL,
-      &value_offset, set_value_offset, NULL },
+      &mainramparam.value_offset, set_value_offset, NULL },
     { "RAMInitStartValue", 0, RES_EVENT_SAME, NULL,
-      &start_value, set_start_value, NULL },
+      &mainramparam.start_value, set_start_value, NULL },
     { "RAMInitValueInvert", 4, RES_EVENT_SAME, NULL,
-      &value_invert, set_value_invert, NULL },
+      &mainramparam.value_invert, set_value_invert, NULL },
     { "RAMInitPatternInvert", 16384, RES_EVENT_SAME, NULL,
-      &pattern_invert, set_pattern_invert, NULL },
+      &mainramparam.pattern_invert, set_pattern_invert, NULL },
     { "RAMInitPatternInvertValue", 255, RES_EVENT_SAME, NULL,
-      &pattern_invert_value, set_pattern_invert_value, NULL },
+      &mainramparam.pattern_invert_value, set_pattern_invert_value, NULL },
     { "RAMInitStartRandom", 0, RES_EVENT_SAME, NULL,
-      &random_start, set_random_start, NULL },
+      &mainramparam.random_start, set_random_start, NULL },
     { "RAMInitRepeatRandom", 0, RES_EVENT_SAME, NULL,
-      &random_repeat, set_random_repeat, NULL },
+      &mainramparam.random_repeat, set_random_repeat, NULL },
     { "RAMInitRandomChance", 1, RES_EVENT_SAME, NULL,
-      &random_chance, set_random_chance, NULL },
+      &mainramparam.random_chance, set_random_chance, NULL },
     RESOURCE_INT_LIST_END
 };
 
@@ -187,46 +198,116 @@ int ram_cmdline_options_init(void)
     return 0;
 }
 
+/* Methods for randomly selecting bits to flip. */
+typedef enum {
+  RANDOM_METHOD_NONE,   /* flip no (or all) bits */
+  RANDOM_METHOD_GEOM,   /* generate bit intervals between flips */
+  RANDOM_METHOD_UNIFORM /* generate discrete uniform per bit */
+} random_method_t;
 
-void ram_init(uint8_t *memram, unsigned int ramsize)
+/* Generate a random variate from the geometric distribution with success
+ * probability p, where the function parameter is the value log(1 - p).  If each
+ * bit has a probability p of being flipped, this distribution corresponds to
+ * the number of non-flipped bits between each subsequent flipped bit. */
+static unsigned int random_method_geom_next(double log_1mp)
+{
+    /* For a uniform random variable U \in [0, 1], then X = log(U) / log(1 - p))
+     * is exponentially distributed with rate -log(1 - p), and floor(X) is
+     * geometrically distributed with success probability p. */
+    double u = lib_double_rand_unit();
+    /* u may be 0 but not 1; use 1 - u to avoid taking the log of 0. */
+    double g = floor(log1p(-u) / log_1mp);
+    /* Avoid overflow when converting to an unsigned int. */
+    return (g > (double)UINT_MAX) ? UINT_MAX : g;
+}
+
+/* this can be used to init arbitrary memory */
+void ram_init_with_pattern(uint8_t *memram, unsigned int ramsize, RAMINITPARAM *ramparam)
 {
     unsigned int offset, j, k;
     uint8_t value;
-    
+
+    random_method_t random_method = RANDOM_METHOD_NONE;
+    unsigned int random_mask_initial = 0;
+    double log_1mp = -INFINITY;
+    unsigned int random_next = UINT_MAX;
+
+    if (ramparam->random_chance <= 0) {
+        /* flipping no bits */
+        random_method = RANDOM_METHOD_NONE;
+        random_mask_initial = 0x00;
+    } else if (ramparam->random_chance >= RANDOM_CHANCE_MAX) {
+        /* flipping all bits; same as no bits, but with the opposite mask */
+        random_method = RANDOM_METHOD_NONE;
+        random_mask_initial = 0xff;
+    } else if (ramparam->random_chance == (RANDOM_CHANCE_MAX / 2)) {
+        /* flipping bits or not with equal probability; worst-case for the
+         * geometric spacing method, so handle separately */
+        random_method = RANDOM_METHOD_UNIFORM;
+    } else if (ramparam->random_chance < (RANDOM_CHANCE_MAX / 2)) {
+        /* some other probability less than 0.5; generate the number of bits
+         * un-flipped between each flipped bit. */
+        random_method = RANDOM_METHOD_GEOM;
+        random_mask_initial = 0x00;
+        log_1mp = log1p((double)-ramparam->random_chance / RANDOM_CHANCE_MAX);
+        random_next = random_method_geom_next(log_1mp);
+    } else {
+        /* some other probability greater than 0.5; generate the number of bits
+         * flipped between each un-flipped bit. */
+        random_method = RANDOM_METHOD_GEOM;
+        random_mask_initial = 0xff;
+        log_1mp = log((double)ramparam->random_chance / RANDOM_CHANCE_MAX);
+        random_next = random_method_geom_next(log_1mp);
+    }
+
     for (offset = 0; offset < ramsize; offset++) {
-        
+
         j = k = 0;
-        if (value_invert) {
-            j = (((offset + value_offset) / value_invert) & 1) ? 0xff : 0x00;
+        if (ramparam->value_invert) {
+            j = (((offset + ramparam->value_offset) / ramparam->value_invert) & 1) ? 0xff : 0x00;
         }
 
-        if (pattern_invert) {
-            k = ((offset / pattern_invert) & 1) ? pattern_invert_value : 0x00;
+        if (ramparam->pattern_invert) {
+            k = ((offset / ramparam->pattern_invert) & 1) ? ramparam->pattern_invert_value : 0x00;
         }
 
-        value = start_value ^ j ^ k;
-        
-        j = k = 0;
-        if (random_start && random_repeat) {
-            k = ((offset % random_repeat) < random_start) ? lib_unsigned_rand(0, 0xff) : 0;
+        value = ramparam->start_value ^ j ^ k;
+
+        k = 0;
+        if (ramparam->random_start && ramparam->random_repeat) {
+            k = ((offset % ramparam->random_repeat) < ramparam->random_start) ? lib_unsigned_rand(0, 0xff) : 0;
         }
-        if (random_chance) {
-            j |= lib_unsigned_rand(0, 0x1000) < random_chance ? 0x80 : 0;
-            j |= lib_unsigned_rand(0, 0x1000) < random_chance ? 0x40 : 0;
-            j |= lib_unsigned_rand(0, 0x1000) < random_chance ? 0x20 : 0;
-            j |= lib_unsigned_rand(0, 0x1000) < random_chance ? 0x10 : 0;
-            j |= lib_unsigned_rand(0, 0x1000) < random_chance ? 0x08 : 0;
-            j |= lib_unsigned_rand(0, 0x1000) < random_chance ? 0x04 : 0;
-            j |= lib_unsigned_rand(0, 0x1000) < random_chance ? 0x02 : 0;
-            j |= lib_unsigned_rand(0, 0x1000) < random_chance ? 0x01 : 0;
+
+        j = 0;
+        switch (random_method) {
+            case RANDOM_METHOD_NONE:
+                j = random_mask_initial;
+                break;
+            case RANDOM_METHOD_UNIFORM:
+                j = lib_unsigned_rand(0x00, 0xff);
+                break;
+            case RANDOM_METHOD_GEOM:
+                j = random_mask_initial;
+                while (random_next < 8) {
+                    j ^= (1 << random_next);
+                    random_next += 1 + random_method_geom_next(log_1mp);
+                }
+                random_next -= 8;
+                break;
         }
-        
+
         value ^= k ^ j;
-        
+
         memram[offset] = value;
     }
 }
 
+/* used to initialize the main memory of the machine */
+void ram_init(uint8_t *memram, unsigned int ramsize)
+{
+    ram_init_with_pattern(memram, ramsize, &mainramparam);
+}
+
 /* create a preview of the RAM init pattern - this should be as fast as
    possible since it is used in the GUI */
 void ram_init_print_pattern(char *s, int len, char *eol)
