@@ -62,36 +62,56 @@ static int raster_calc_frame_buffer_width(raster_t *raster)
            + raster->geometry->extra_offscreen_border_right;
 }
 
+void raster_calculate_padding_size(unsigned int fb_width, unsigned int fb_height,
+                                   unsigned int *padded_size, unsigned int *unpadded_offset)
+{
+    *padded_size = fb_width * (fb_height + 4);
+    *unpadded_offset = fb_width * 2;
+}
+
 static int raster_draw_buffer_alloc(video_canvas_t *canvas,
                                     unsigned int fb_width,
                                     unsigned int fb_height,
                                     unsigned int *fb_pitch)
 {
-    if (canvas->video_draw_buffer_callback) {
-        return canvas->video_draw_buffer_callback->draw_buffer_alloc(canvas, &canvas->draw_buffer->draw_buffer, fb_width, fb_height, fb_pitch);
-    }
-
+    unsigned int padded_size;
+    unsigned int unpadded_offset;
+    
     /*
      * FIXME: We have to allocate memory either size of the draw buffer because both the CRT and Scale2x
      * filters will access memory both before and after the draw_buffer. This is a workaround that will 
      * no doubt survive until we shift filters to the GPU.
      */
     
-    canvas->draw_buffer->draw_buffer_padded_allocation = lib_calloc(1, fb_width * (fb_height + 4));
-    canvas->draw_buffer->draw_buffer = canvas->draw_buffer->draw_buffer_padded_allocation + (fb_height * 2);
+    raster_calculate_padding_size(fb_width, fb_height, &padded_size, &unpadded_offset);
+
+    canvas->draw_buffer->draw_buffer_padded_allocations[0] = lib_calloc(1, padded_size);
+    canvas->draw_buffer->draw_buffer_non_padded[0] = canvas->draw_buffer->draw_buffer_padded_allocations[0] + unpadded_offset;
+    canvas->draw_buffer->draw_buffer = canvas->draw_buffer->draw_buffer_non_padded[0];
+
+    if (canvas->videoconfig->cap->interlace_allowed) {
+        /*
+         * Interlaced rendering maintains two buffers, one for even frames and
+         * and one for odd frames. This allows mid-frame rendering to show the
+         * correct previous frame contents after the raster beam location.
+         * This was added for rendering within the monitor.
+         */
+
+        canvas->draw_buffer->draw_buffer_padded_allocations[1] = lib_calloc(1, padded_size);
+        canvas->draw_buffer->draw_buffer_non_padded[1] = canvas->draw_buffer->draw_buffer_padded_allocations[1] + unpadded_offset;
+    }
+
     *fb_pitch = fb_width;
     return 0;
 }
 
 static void raster_draw_buffer_free(video_canvas_t *canvas)
 {
-    if (canvas->video_draw_buffer_callback) {
-        canvas->video_draw_buffer_callback->draw_buffer_free(canvas, canvas->draw_buffer->draw_buffer);
-        return;
-    }
+    lib_free(canvas->draw_buffer->draw_buffer_padded_allocations[0]);
+    lib_free(canvas->draw_buffer->draw_buffer_padded_allocations[1]);
 
-    lib_free(canvas->draw_buffer->draw_buffer_padded_allocation);
-    canvas->draw_buffer->draw_buffer_padded_allocation = NULL;
+    canvas->draw_buffer->draw_buffer_padded_allocations[0] = NULL;
+    canvas->draw_buffer->draw_buffer_padded_allocations[1] = NULL;
     canvas->draw_buffer->draw_buffer = NULL;
 }
 
@@ -100,11 +120,6 @@ static void raster_draw_buffer_clear(video_canvas_t *canvas, uint8_t value,
                                      unsigned int fb_height,
                                      unsigned int fb_pitch)
 {
-    if (canvas->video_draw_buffer_callback) {
-        canvas->video_draw_buffer_callback->draw_buffer_clear(canvas, canvas->draw_buffer->draw_buffer, value, fb_width, fb_height, fb_pitch);
-        return;
-    }
-
     memset(canvas->draw_buffer->draw_buffer, value, fb_width * fb_height);
 }
 
@@ -291,7 +306,6 @@ void raster_reset(raster_t *raster)
     raster->xsmooth_shift_left = 0;
     raster->xsmooth_shift_right = 0;
     raster->sprite_xsmooth_shift_right = 0;
-    raster->skip_frame = 0;
 
     raster->blank_off = 0;
     raster->blank_enabled = 0;
@@ -305,6 +319,11 @@ void raster_reset(raster_t *raster)
     raster->ycounter = 0;
     raster->video_mode = 0;
     raster->last_video_mode = -1;
+
+    if (raster->canvas) {
+        raster->canvas->videoconfig->interlaced = 0;
+        raster->canvas->videoconfig->interlace_field = 0;
+    }
 }
 
 typedef struct raster_list_t {
@@ -502,11 +521,6 @@ void raster_set_title(raster_t *raster, const char *name)
     lib_free(title);
 }
 
-void raster_skip_frame(raster_t *raster, int skip)
-{
-    raster->skip_frame = skip;
-}
-
 void raster_enable_cache(raster_t *raster, int enable)
 {
 #if 0 /* disabled cache hack */
