@@ -37,14 +37,16 @@
 #include <stdio.h>
 
 #include "alarm.h"
-#include "clkguard.h"
 #include "cmdline.h"
+#include "joyport.h"
 #include "log.h"
 #include "maincpu.h"
 #include "resources.h"
+#include "rs232.h"
 #include "rs232drv.h"
 #include "rsuser.h"
 #include "types.h"
+#include "userport.h"
 
 static int fd = -1;
 
@@ -65,8 +67,6 @@ static CLOCK clk_end_tx = 0;
 static void (*start_bit_trigger)(void);
 static void (*byte_rx_func)(uint8_t);
 
-static void clk_overflow_callback(CLOCK sub, void *data);
-
 static void int_rsuser(CLOCK offset, void *data);
 
 #undef DEBUG
@@ -105,14 +105,60 @@ static void int_rsuser(CLOCK offset, void *data);
 static int dtr = 0;
 static int rts = 0;
 
+/***********************************************************************
+ * UP9600 detection variables
+ */
+static int old_dsr;
+static unsigned int dsr_cnt;
+
+/***********************************************************************
+ * userport interface handling
+ */
+
+/* some prototypes are needed */
+static void rsuser_tx_byte(uint8_t b);
+static void rsuser_write_ctrl(uint8_t b, int pulse);
+static uint8_t rsuser_read_ctrl(uint8_t orig);
+static void rsuser_set_tx_bit(uint8_t b);
+static int userport_rs232_enable(int value);
+static void rsuser_reset(void);
+
+static userport_device_t rs232_device = {
+    "Userport RS232/Modem",     /* device name */
+    JOYSTICK_ADAPTER_ID_NONE,   /* NOT a joystick adapter */
+    USERPORT_DEVICE_TYPE_MODEM, /* device is a modem */
+    userport_rs232_enable,      /* enable function */
+    rsuser_read_ctrl,           /* read pb0-pb7 function */
+    rsuser_write_ctrl,          /* store pb0-pb7 function */
+    NULL,                       /* NO read pa2 pin function */
+    rsuser_set_tx_bit,          /* store pa2 pin function */
+    NULL,                       /* NO read pa3 pin function */
+    NULL,                       /* NO store pa3 pin function */
+    0,                          /* pc pin is NOT needed */
+    rsuser_tx_byte,             /* store sp1 pin function */
+    NULL,                       /* NO read sp1 pin function */
+    NULL,                       /* NO store sp2 pin function */
+    NULL,                       /* NO read sp2 pin function */
+    rsuser_reset,               /* NO reset function */
+    NULL,                       /* NO power toggle function */
+    NULL,                       /* NO snapshot write function */
+    NULL                        /* NO snapshot read function */
+};
+
 /***********************************************************************
  * resource handling
  */
 
-int rsuser_enabled = 0;                 /* enable flag */
+static int rsuser_up9600 = 0;           /* UP9600 enable flag */
+static int rsuser_enabled = 0;          /* enable flag */
+static int rsuser_rtsinv = 0;           /* RTS invert flag - 0 = Normal Kernal level, 1 = Inverted level */
+static int rsuser_ctsinv = 0;           /* CTS invert flag - 0 = Normal Kernal level, 1 = Inverted level */
+static int rsuser_dsrinv = 0;           /* DSR invert flag - 0 = Normal Kernal level, 1 = Inverted level */
+static int rsuser_dcdinv = 0;           /* DCD invert flag - 0 = Normal Kernal level, 1 = Inverted level */
+static int rsuser_dtrinv = 0;           /* DTR invert flag - 0 = Normal Kernal level, 1 = Inverted level */
 static int rsuser_baudrate = 300;       /* saves the baud rate given */
 static int char_clk_ticks = 0;          /* clk ticks per character */
-static int bit_clk_ticks = 0;           /* clk ticks per character */
+static int bit_clk_ticks = 0;           /* clk ticks per bit */
 
 static int rsuser_device;
 
@@ -129,17 +175,17 @@ static void calculate_baudrate(void)
                char_clk_ticks, cycles_per_sec));
 }
 
-static int set_enable(int value, void *param)
+static int userport_rs232_enable(int value)
 {
     int newval = value ? 1 : 0;
 
     if (newval && !rsuser_enabled) {
-        dtr = DTR_OUT;  /* inactive */
-        rts = RTS_OUT;  /* inactive */
+        dtr = rsuser_dtrinv ? DTR_OUT : 0;  /* inactive */
+        rts = rsuser_rtsinv ? RTS_OUT : 0;  /* inactive */
         fd = -1;
     }
     if (rsuser_enabled && !newval) {
-        if (fd != -1) {
+        if (fd >= 0) {
             /* if (clk_start_tx) rs232drv_putc(fd, rxdata); */
             rs232drv_close(fd);
         }
@@ -160,6 +206,21 @@ static int set_enable(int value, void *param)
     return 0;
 }
 
+static int set_up9600(int val, void *param)
+{
+    if (val >= 0){
+        rsuser_up9600 = val;
+    } else {
+        rsuser_up9600 = 0;
+    }
+
+    if (rsuser_up9600 == true){
+        old_dsr = 0;
+        dsr_cnt = 0;
+    }
+
+    return 0;
+}
 static int set_baudrate(int val, void *param)
 {
     if (val < 1) {
@@ -181,42 +242,121 @@ static int set_up_device(int val, void *param)
 
     rsuser_device = val;
 
-    if (fd != -1) {
+    if (fd >= 0) {
         rs232drv_close(fd);
         fd = rs232drv_open(rsuser_device);
     }
     return 0;
 }
 
+static int set_rtsinv(int value, void *param)
+{
+    rsuser_rtsinv = value ? 1 : 0;
+    /* Prevent the state machine to be left in a locked state if the software leaves RTS */
+    /* in a fixed state, triggering the RTS disabled state on initialization.            */
+    if (rxstate == 2){
+        rxstate = 0;
+    }
+    return 0;
+}
+
+static int set_ctsinv(int value, void *param)
+{
+    rsuser_ctsinv = value ? 1 : 0;
+    return 0;
+}
+
+static int set_dsrinv(int value, void *param)
+{
+    rsuser_dsrinv = value ? 1 : 0;
+    return 0;
+}
+
+static int set_dcdinv(int value, void *param)
+{
+    rsuser_dcdinv = value ? 1 : 0;
+    return 0;
+}
+
+static int set_dtrinv(int value, void *param)
+{
+    rsuser_dtrinv = value ? 1 : 0;
+    return 0;
+}
+
 static const resource_int_t resources_int[] = {
-    { "RsUserEnable", 0, RES_EVENT_STRICT, (resource_value_t)0,
-      &rsuser_enabled, set_enable, NULL },
+    { "RsUserUP9600", 0, RES_EVENT_NO, NULL,
+      &rsuser_up9600, set_up9600, NULL},
     { "RsUserBaud", 2400, RES_EVENT_NO, NULL,
       &rsuser_baudrate, set_baudrate, NULL },
     { "RsUserDev", 0, RES_EVENT_NO, NULL,
       &rsuser_device, set_up_device, NULL },
+    { "RsUserRTSInv", 0, RES_EVENT_NO, (resource_value_t)0,
+      &rsuser_rtsinv, set_rtsinv, NULL },
+    { "RsUserCTSInv", 0, RES_EVENT_NO, (resource_value_t)0,
+      &rsuser_ctsinv, set_ctsinv, NULL },
+    { "RsUserDSRInv", 0, RES_EVENT_NO, (resource_value_t)0,
+      &rsuser_dsrinv, set_dsrinv, NULL },
+    { "RsUserDCDInv", 0, RES_EVENT_NO, (resource_value_t)0,
+      &rsuser_dcdinv, set_dcdinv, NULL },
+    { "RsUserDTRInv", 0, RES_EVENT_NO, (resource_value_t)0,
+      &rsuser_dtrinv, set_dtrinv, NULL },
     RESOURCE_INT_LIST_END
 };
 
 int rsuser_resources_init(void)
 {
+    if (userport_device_register(USERPORT_DEVICE_RS232_MODEM, &rs232_device) < 0) {
+        return -1;
+    }
+
     return resources_register_int(resources_int);
 }
 
 static const cmdline_option_t cmdline_options[] =
 {
-    { "-rsuser", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
-      NULL, NULL, "RsUserEnable", (void *)1,
-      NULL, "Enable RS232 userport emulation" },
-    { "+rsuser", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
-      NULL, NULL, "RsUserEnable", (void *)0,
-      NULL, "Disable RS232 userport emulation" },
+    { "-rsuserup9600", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserUP9600", (void *)1,
+      NULL, "Enable UP9600 interface emulation."},
+    { "+rsuserup9600", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserUP9600", (void *)0,
+      NULL, "Disable UP9600 interface emulation."},
     { "-rsuserbaud", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "RsUserBaud", NULL,
       "<baud>", "Set the baud rate of the RS232 userport emulation." },
     { "-rsuserdev", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "RsUserDev", NULL,
       "<0-3>", "Specify VICE RS232 device for userport" },
+    { "-rsuserrtsinv", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserRTSInv", (void *)1,
+      NULL, "Invert RS232 userport emulation RTS line" },
+    { "+rsuserrtsinv", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserRTSInv", (void *)0,
+      NULL, "Do not invert RS232 userport emulation RTS line" },
+    { "-rsuserctsinv", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserCTSInv", (void *)1,
+      NULL, "Invert RS232 userport emulation CTS line" },
+    { "+rsuserctsinv", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserCTSInv", (void *)0,
+      NULL, "Do not invert RS232 userport emulation CTS line" },
+    { "-rsuserdsrinv", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserDSRInv", (void *)1,
+      NULL, "Invert RS232 userport emulation DSR line" },
+    { "+rsuserdsrinv", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserDSRInv", (void *)0,
+      NULL, "Do not invert RS232 userport emulation DSR line" },
+    { "-rsuserdcdinv", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserDCDInv", (void *)1,
+      NULL, "Invert RS232 userport emulation DCD line" },
+    { "+rsuserdcdinv", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserDCDInv", (void *)0,
+      NULL, "Do not invert RS232 userport emulation DCD line" },
+    { "-rsuserdtrinv", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserDTRInv", (void *)1,
+      NULL, "Invert RS232 userport emulation DTR line" },
+    { "+rsuserdtrinv", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
+      NULL, NULL, "RsUserDTRInv", (void *)0,
+      NULL, "Do not invert RS232 userport emulation DTR line" },
     CMDLINE_LIST_END
 };
 
@@ -236,6 +376,12 @@ static const unsigned int masks[] =
     0x1000, 0x2000, 0x4000, 0x8000
 };
 
+void rsuser_change_timing(long cycles)
+{
+    cycles_per_sec = cycles;
+    calculate_baudrate();
+}
+
 void rsuser_init(long cycles, void (*startfunc)(void), void (*bytefunc)(uint8_t))
 {
     int i, j;
@@ -243,10 +389,7 @@ void rsuser_init(long cycles, void (*startfunc)(void), void (*bytefunc)(uint8_t)
 
     rsuser_alarm = alarm_new(maincpu_alarm_context, "RSUser", int_rsuser, NULL);
 
-    clk_guard_add_callback(maincpu_clk_guard, clk_overflow_callback, NULL);
-
-    cycles_per_sec = cycles;
-    calculate_baudrate();
+    rsuser_change_timing(cycles);
 
     start_bit_trigger = startfunc;
     byte_rx_func = bytefunc;
@@ -263,29 +406,33 @@ void rsuser_init(long cycles, void (*startfunc)(void), void (*bytefunc)(uint8_t)
         code[i] = d;
     }
 
-    dtr = DTR_OUT;      /* inactive */
-    rts = RTS_OUT;      /* inactive */
+    dtr = rsuser_dtrinv ? DTR_OUT : 0;      /* inactive */
+    rts = rsuser_dtrinv ? RTS_OUT : 0;      /* inactive */
     fd = -1;
 
+
+
     LOG_DEBUG(("rsuser_init: fd:%d dtr:%d rts:%d", fd, dtr, rts));
     
     buf = (unsigned int)(~0); /* all 1s */
     valid = 0;
 }
 
-void rsuser_reset(void)
+static void rsuser_reset(void)
 {
     rxstate = 0;
     clk_start_rx = 0;
     clk_start_tx = 0;
     clk_start_bit = 0;
     clk_end_tx = 0;
-    if (fd != -1) {
+    dsr_cnt = 0;
+
+    if (fd >= 0) {
         rs232drv_close(fd);
+        fd = -1;
     }
 
     alarm_unset(rsuser_alarm);
-    fd = -1;
 }
 
 static void rsuser_setup(void)
@@ -294,21 +441,23 @@ static void rsuser_setup(void)
     clk_start_rx = 0;
     clk_start_tx = 0;
     clk_start_bit = 0;
-    if (fd < 0) {
+    dsr_cnt = 0;
+    if (fd < 0 && rsuser_enabled) {
         fd = rs232drv_open(rsuser_device);
     }
-    alarm_set(rsuser_alarm, maincpu_clk + char_clk_ticks / 8);
+    alarm_set(rsuser_alarm, maincpu_clk + char_clk_ticks / 10);
 }
 
 /* called by VIA/CIA when cpu writes to user port */
-void rsuser_write_ctrl(uint8_t status)
+static void rsuser_write_ctrl(uint8_t status, int pulse)
 {
     enum rs232handshake_out modem_status = 0;
-    int new_dtr = status & DTR_OUT;  /* = 0 is active, != 0 is inactive */
-    int new_rts = status & RTS_OUT;  /* = 0 is active, != 0 is inactive */
+    int new_dtr = status & DTR_OUT;  /* != 0 is active, = 0 is inactive for Kernal */
+    int new_rts = status & RTS_OUT;  /* != 0 is active, = 0 is inactive for Kernal */
+    int new_dsr = status & DSR_IN;   /* Written to by UP9600 install routine */
 #ifdef LOG_MODEM_STATUS
     static int oldstatus = -1;
-#endif    
+#endif
 
 #ifdef LOG_MODEM_STATUS
     if (status != oldstatus) {
@@ -318,6 +467,17 @@ void rsuser_write_ctrl(uint8_t status)
 #endif
     
     if (rsuser_enabled) {
+        if (fd < 0) {
+            if (new_dtr == (rsuser_dtrinv ? 0: DTR_OUT)) {
+                rsuser_setup();
+            }
+        }
+        if (fd >= 0) {
+            modem_status |= (new_dtr == (rsuser_dtrinv ? 0 : DTR_OUT)) ? RS232_HSO_DTR : 0;
+            rs232drv_set_status(fd, modem_status);
+        }
+
+#if 0
         if (dtr && !new_dtr) {
             /* DTR low->high transition, set up userport, set DTR active */
             rsuser_setup();
@@ -327,6 +487,16 @@ void rsuser_write_ctrl(uint8_t status)
                 rs232drv_set_status(fd, modem_status);
             }
         }
+#endif
+        if (rts != new_rts && fd >= 0) {
+            /* RTS handling */
+            if ((rsuser_rtsinv ? 0 : RTS_OUT) != new_rts) {
+                rxstate = 2;
+            } else {
+                rxstate = 0;
+            }
+        }
+#if 0
         if (new_dtr && !dtr && fd != -1) {
 #if 0   /* This is a bug in the X-line handshake of the C64... */
             LOG_DEBUG(("switch rs232 off."));
@@ -338,10 +508,28 @@ void rsuser_write_ctrl(uint8_t status)
             modem_status |= dtr ? 0 : RS232_HSO_DTR;
             rs232drv_set_status(fd, modem_status);
         }
+#endif
+        if (rsuser_up9600){
+            if (!old_dsr && new_dsr) {
+                /* DSR low-high transition, UP9600 install routine pulsing PB7*/
+                dsr_cnt++;
+                if (dsr_cnt == 8){
+                    /* 8 PB7 pulses: Write a dummy byte to the shift register input  */
+                    /* this will set the SDR IRQ flag and UP9600 detection will pass */
+                    byte_rx_func(0x00);
+                    dsr_cnt = 0;
+                }
+            } /*else if (old_dsr == new_dsr) { */
+              /* Reset count if two consecutive writes have the same DSR state
+                 ---- This isn't working, as rsuser_write_ctrl is being called twice per write */
+              /* dsr_cnt = 0; */
+            /* } */
+        }
     }
 
     dtr = new_dtr;
     rts = new_rts;
+    old_dsr = new_dsr;
 
 #ifdef LOG_MODEM_STATUS
     if (status != oldstatus) {
@@ -370,7 +558,7 @@ static void check_tx_buffer(void)
             log_error(LOG_DEFAULT, "rsuser: framing mismatch - outgoing baudrates ok?");
         } else {
             c = (buf >> (valid - 9)) & 0xff;
-            if (fd != -1) {
+            if (fd >= 0) {
                 LOG_DEBUG(("\"%c\" (%02x).", code[c], code[c]));
                 rs232drv_putc(fd, ((uint8_t)(code[c])));
             }
@@ -411,7 +599,7 @@ static void keepup_tx_buffer(void)
     }
 }
 
-void rsuser_set_tx_bit(int b)
+static void rsuser_set_tx_bit(uint8_t b)
 {
     LOG_DEBUG(("rsuser_set_tx(clk=%d, clk_start_tx=%d, b=%d).",
                maincpu_clk, clk_start_tx, b));
@@ -419,14 +607,14 @@ void rsuser_set_tx_bit(int b)
     LOG_DEBUG_TIMING_TX(("rsuser_set_tx(clk=%d, clk_start_tx=%d, b=%d).",
                          maincpu_clk, clk_start_tx, b));
 
-    if (fd == -1 || rsuser_baudrate > 2400) {
+    if (fd < 0 || rsuser_up9600) {
         clk_start_tx = 0;
         return;
     }
 
     /* feeds the output buffer with enough bits till clk */
     keepup_tx_buffer();
-    txbit = b;
+    txbit = b << 2;
 
     if (!clk_start_tx && !b) {
         /* the clock where we start sampling - in the middle of the bit */
@@ -440,9 +628,10 @@ void rsuser_set_tx_bit(int b)
     }
 }
 
-uint8_t rsuser_get_rx_bit(void)
+static uint8_t rsuser_get_rx_bit(void)
 {
-    int bit = 0, byte = 1;
+    CLOCK bit = 0;
+    int byte = 1;
     LOG_DEBUG_TIMING_RX(("rsuser_get_rx_bit(clk=%d, clk_start_rx=%d).",
                          maincpu_clk, clk_start_rx));
 
@@ -467,8 +656,9 @@ uint8_t rsuser_get_rx_bit(void)
 }
 
 /* called by VIA/CIA when cpu reads from user port */
-uint8_t rsuser_read_ctrl(uint8_t b)
+static uint8_t rsuser_read_ctrl(uint8_t orig)
 {
+    uint8_t b = orig;
     enum rs232handshake_in modem_status = rs232drv_get_status(fd);
     uint8_t status = 0;
 #ifdef LOG_MODEM_STATUS
@@ -483,17 +673,17 @@ uint8_t rsuser_read_ctrl(uint8_t b)
                modem_status & RS232_HSI_RI ? 1 : 0
               );
     }
-#endif    
-    /* all handshake lines are inverted by the RS232 interface */
-    if (!(modem_status & RS232_HSI_CTS)) {
+#endif
+    if ((rsuser_ctsinv ? 0 : RS232_HSI_CTS) == (modem_status & RS232_HSI_CTS)) {
         status |= CTS_IN;
     }
-    if (!(modem_status & RS232_HSI_DCD)) {
-        if (!(rsuser_baudrate > 2400)) {
+    if ((rsuser_dcdinv ? 0 : RS232_HSI_DCD) == (modem_status & RS232_HSI_DCD)) {
+        if (!rsuser_up9600) {
             status |= DCD_IN;
         }
     }
-    if (!(modem_status & RS232_HSI_DSR)) {
+    /* DSR line is not connected in the UP9600 interface */
+    if (((rsuser_dsrinv ? 0 : RS232_HSI_DSR) == (modem_status & RS232_HSI_DSR)) && !rsuser_up9600) {
         status |= DSR_IN;
     }
 #ifdef LOG_MODEM_STATUS
@@ -506,12 +696,14 @@ uint8_t rsuser_read_ctrl(uint8_t b)
               );
         oldstatus = status;
     }
-#endif    
+#endif
+    /* Reset UP9600 detection counter */
+    dsr_cnt = 0;
+
     return b & (rsuser_get_rx_bit() | status);
-/*  return b & (rsuser_get_rx_bit() | CTS_IN | (rsuser_baudrate > 2400 ? 0 : DCD_IN)); */
 }
 
-void rsuser_tx_byte(uint8_t b)
+static void rsuser_tx_byte(uint8_t b)
 {
     buf = (buf << 8) | b;
     valid += 8;
@@ -527,7 +719,7 @@ static void int_rsuser(CLOCK offset, void *data)
 
     switch (rxstate) {
         case 0:
-            if (fd != -1 && rs232drv_getc(fd, &rxdata)) {
+            if ((rsuser_rtsinv ? 0 : RTS_OUT) == rts && fd >= 0 && rs232drv_getc(fd, &rxdata)) {
                 /* byte received, signal startbit on flag */
                 rxstate++;
                 if (start_bit_trigger) {
@@ -535,29 +727,20 @@ static void int_rsuser(CLOCK offset, void *data)
                 }
                 clk_start_rx = rclk;
             }
-            alarm_set(rsuser_alarm, maincpu_clk + char_clk_ticks);
+            alarm_set(rsuser_alarm, maincpu_clk + char_clk_ticks - bit_clk_ticks); /* Wait one char - 1 stop bit */
             break;
         case 1:
             /* now byte should be in shift register */
-            if (byte_rx_func) {
+            if (byte_rx_func && rsuser_up9600) {
                 byte_rx_func((uint8_t)(code[rxdata]));
             }
             rxstate = 0;
             clk_start_rx = 0;
-            alarm_set(rsuser_alarm, maincpu_clk + char_clk_ticks / 8);
+            alarm_set(rsuser_alarm, maincpu_clk + char_clk_ticks / 10);
+            break;
+        case 2:
+            /* RTS disabled */
+            alarm_set(rsuser_alarm, maincpu_clk + char_clk_ticks / 10);
             break;
     }
 }
-
-static void clk_overflow_callback(CLOCK sub, void *data)
-{
-    if (clk_start_tx) {
-        clk_start_tx -= sub;
-    }
-    if (clk_start_rx) {
-        clk_start_rx -= sub;
-    }
-    if (clk_start_bit) {
-        clk_start_bit -= sub;
-    }
-}
