@@ -71,10 +71,37 @@ static log_t c64_snapshot_log = LOG_ERR;
 
 static const char snap_rom_module_name[] = "C64ROM";
 
+#define NUM_TRAP_DEVICES 9  /* FIXME: is there a better constant ? */
+static int trapfl[NUM_TRAP_DEVICES];
+static int trapdevices[NUM_TRAP_DEVICES + 1] = { 1, 4, 5, 6, 7, 8, 9, 10, 11, -1 };
+
+static void get_trapflags(void)
+{
+    int i;
+    for(i = 0; trapdevices[i] != -1; i++) {
+        resources_get_int_sprintf("VirtualDevice%d", &trapfl[i], trapdevices[i]);
+    }
+}
+
+static void clear_trapflags(void)
+{
+    int i;
+    for(i = 0; trapdevices[i] != -1; i++) {
+        resources_set_int_sprintf("VirtualDevice%d", 0, trapdevices[i]);
+    }
+}
+
+static void restore_trapflags(void)
+{
+    int i;
+    for(i = 0; trapdevices[i] != -1; i++) {
+        resources_set_int_sprintf("VirtualDevice%d", trapfl[i], trapdevices[i]);
+    }
+}
+
 static int scpu64_snapshot_write_rom_module(snapshot_t *s)
 {
     snapshot_module_t *m;
-    int trapfl;
 
     /* Main memory module.  */
 
@@ -84,8 +111,8 @@ static int scpu64_snapshot_write_rom_module(snapshot_t *s)
     }
 
     /* disable traps before saving the ROM */
-    resources_get_int("VirtualDevices", &trapfl);
-    resources_set_int("VirtualDevices", 0);
+    get_trapflags();
+    clear_trapflags();
 
     if (SMW_BA(m, mem_chargen_rom, SCPU64_CHARGEN_ROM_SIZE) < 0
         || SMW_BA(m, scpu64rom_scpu64_rom, SCPU64_SCPU64_ROM_MAXSIZE) < 0) {
@@ -96,7 +123,7 @@ static int scpu64_snapshot_write_rom_module(snapshot_t *s)
         goto fail;
     }
 
-    resources_set_int("VirtualDevices", trapfl);
+    restore_trapflags();
 
     return 0;
 
@@ -105,7 +132,7 @@ fail:
         snapshot_module_close(m);
     }
 
-    resources_set_int("VirtualDevices", trapfl);
+    restore_trapflags();
 
     return -1;
 }
@@ -114,7 +141,6 @@ static int scpu64_snapshot_read_rom_module(snapshot_t *s)
 {
     uint8_t major_version, minor_version;
     snapshot_module_t *m;
-    int trapfl;
 
     /* Main memory module.  */
 
@@ -132,8 +158,8 @@ static int scpu64_snapshot_read_rom_module(snapshot_t *s)
     }
 
     /* disable traps before loading the ROM */
-    resources_get_int("VirtualDevices", &trapfl);
-    resources_set_int("VirtualDevices", 0);
+    get_trapflags();
+    clear_trapflags();
 
     if (SMR_BA(m, mem_chargen_rom, SCPU64_CHARGEN_ROM_SIZE) < 0
         || SMR_BA(m, scpu64rom_scpu64_rom, SCPU64_SCPU64_ROM_MAXSIZE) < 0) {
@@ -145,7 +171,7 @@ static int scpu64_snapshot_read_rom_module(snapshot_t *s)
     }
 
     /* enable traps again when necessary */
-    resources_set_int("VirtualDevices", trapfl);
+    restore_trapflags();
 
     return 0;
 
@@ -153,7 +179,7 @@ fail:
     if (m != NULL) {
         snapshot_module_close(m);
     }
-    resources_set_int("VirtualDevices", trapfl);
+    restore_trapflags();
     return -1;
 }
 
@@ -218,7 +244,6 @@ int scpu64_snapshot_read_module(snapshot_t *s)
     uint8_t major_version, minor_version;
     snapshot_module_t *m;
     unsigned int simm_mask;
-    int trapfl;
 
     /* Main memory module.  */
 
@@ -251,13 +276,13 @@ int scpu64_snapshot_read_module(snapshot_t *s)
     }
 
     /* disable traps before loading the ROM */
-    resources_get_int("VirtualDevices", &trapfl);
-    resources_set_int("VirtualDevices", 0);
+    get_trapflags();
+    clear_trapflags();
 
     memcpy(mem_trap_ram, mem_sram + 0x1e000, SCPU64_KERNAL_ROM_SIZE);
 
     /* enable traps again when necessary */
-    resources_set_int("VirtualDevices", trapfl);
+    restore_trapflags();
 
     switch (simm_mask) {
     case 0x0:
