@@ -25,6 +25,8 @@
  *
  */
 
+/* #define DEBUG_SNAPSHOT */
+
 #include "vice.h"
 
 #include <stdio.h>
@@ -45,11 +47,18 @@
 #include "vsync.h"
 #include "zfile.h"
 
+#ifdef DEBUG_SNAPSHOT
+#define DBG(x)  printf x
+#else
+#define DBG(x)
+#endif
+
 static int snapshot_error = SNAPSHOT_NO_ERROR;
 static char *current_module = NULL;
 static char read_name[SNAPSHOT_MACHINE_NAME_LEN];
 static char *current_machine_name = NULL;
 static char *current_filename = NULL;
+static size_t current_fpos = 0;
 
 static const char snapshot_magic_string[] = "VICE Snapshot File\032";
 static const char snapshot_version_magic_string[] = "VICE Version\032";
@@ -89,6 +98,7 @@ struct snapshot_s {
 
 static int snapshot_write_byte(FILE *f, uint8_t data)
 {
+    current_fpos = ftell(f);
     if (fputc(data, f) == EOF) {
         snapshot_error = SNAPSHOT_WRITE_EOF_ERROR;
         return -1;
@@ -99,6 +109,7 @@ static int snapshot_write_byte(FILE *f, uint8_t data)
 
 static int snapshot_write_word(FILE *f, uint16_t data)
 {
+    current_fpos = ftell(f);
     if (snapshot_write_byte(f, (uint8_t)(data & 0xff)) < 0
         || snapshot_write_byte(f, (uint8_t)(data >> 8)) < 0) {
         return -1;
@@ -109,6 +120,7 @@ static int snapshot_write_word(FILE *f, uint16_t data)
 
 static int snapshot_write_dword(FILE *f, uint32_t data)
 {
+    current_fpos = ftell(f);
     if (snapshot_write_word(f, (uint16_t)(data & 0xffff)) < 0
         || snapshot_write_word(f, (uint16_t)(data >> 16)) < 0) {
         return -1;
@@ -117,11 +129,23 @@ static int snapshot_write_dword(FILE *f, uint32_t data)
     return 0;
 }
 
+static int snapshot_write_qword(FILE *f, uint64_t data)
+{
+    current_fpos = ftell(f);
+    if (snapshot_write_dword(f, (uint32_t)(data & 0xffffffff)) < 0
+        || snapshot_write_dword(f, (uint32_t)(data >> 32)) < 0) {
+        return -1;
+    }
+
+    return 0;
+}
+
 static int snapshot_write_double(FILE *f, double data)
 {
     uint8_t *byte_data = (uint8_t *)&data;
     int i;
 
+    current_fpos = ftell(f);
     for (i = 0; i < sizeof(double); i++) {
         if (snapshot_write_byte(f, byte_data[i]) < 0) {
             return -1;
@@ -136,6 +160,7 @@ static int snapshot_write_padded_string(FILE *f, const char *s, uint8_t pad_char
     int i, found_zero;
     uint8_t c;
 
+    current_fpos = ftell(f);
     for (i = found_zero = 0; i < len; i++) {
         if (!found_zero && s[i] == 0) {
             found_zero = 1;
@@ -151,6 +176,7 @@ static int snapshot_write_padded_string(FILE *f, const char *s, uint8_t pad_char
 
 static int snapshot_write_byte_array(FILE *f, const uint8_t *data, unsigned int num)
 {
+    current_fpos = ftell(f);
     if (num > 0 && fwrite(data, (size_t)num, 1, f) < 1) {
         snapshot_error = SNAPSHOT_WRITE_BYTE_ARRAY_ERROR;
         return -1;
@@ -163,6 +189,7 @@ static int snapshot_write_word_array(FILE *f, const uint16_t *data, unsigned int
 {
     unsigned int i;
 
+    current_fpos = ftell(f);
     for (i = 0; i < num; i++) {
         if (snapshot_write_word(f, data[i]) < 0) {
             return -1;
@@ -176,6 +203,7 @@ static int snapshot_write_dword_array(FILE *f, const uint32_t *data, unsigned in
 {
     unsigned int i;
 
+    current_fpos = ftell(f);
     for (i = 0; i < num; i++) {
         if (snapshot_write_dword(f, data[i]) < 0) {
             return -1;
@@ -192,6 +220,7 @@ static int snapshot_write_string(FILE *f, const char *s)
 
     len = s ? (strlen(s) + 1) : 0;      /* length includes nullbyte */
 
+    current_fpos = ftell(f);
     if (snapshot_write_word(f, (uint16_t)len) < 0) {
         return -1;
     }
@@ -209,6 +238,7 @@ static int snapshot_read_byte(FILE *f, uint8_t *b_return)
 {
     int c;
 
+    current_fpos = ftell(f);
     c = fgetc(f);
     if (c == EOF) {
         snapshot_error = SNAPSHOT_READ_EOF_ERROR;
@@ -222,6 +252,7 @@ static int snapshot_read_word(FILE *f, uint16_t *w_return)
 {
     uint8_t lo, hi;
 
+    current_fpos = ftell(f);
     if (snapshot_read_byte(f, &lo) < 0 || snapshot_read_byte(f, &hi) < 0) {
         return -1;
     }
@@ -234,6 +265,7 @@ static int snapshot_read_dword(FILE *f, uint32_t *dw_return)
 {
     uint16_t lo, hi;
 
+    current_fpos = ftell(f);
     if (snapshot_read_word(f, &lo) < 0 || snapshot_read_word(f, &hi) < 0) {
         return -1;
     }
@@ -242,6 +274,19 @@ static int snapshot_read_dword(FILE *f, uint32_t *dw_return)
     return 0;
 }
 
+static int snapshot_read_qword(FILE *f, uint64_t *qw_return)
+{
+    uint32_t lo, hi;
+
+    current_fpos = ftell(f);
+    if (snapshot_read_dword(f, &lo) < 0 || snapshot_read_dword(f, &hi) < 0) {
+        return -1;
+    }
+
+    *qw_return = lo | ((uint64_t)hi << 32);
+    return 0;
+}
+
 static int snapshot_read_double(FILE *f, double *d_return)
 {
     int i;
@@ -249,6 +294,7 @@ static int snapshot_read_double(FILE *f, double *d_return)
     double val;
     uint8_t *byte_val = (uint8_t *)&val;
 
+    current_fpos = ftell(f);
     for (i = 0; i < sizeof(double); i++) {
         c = fgetc(f);
         if (c == EOF) {
@@ -263,6 +309,7 @@ static int snapshot_read_double(FILE *f, double *d_return)
 
 static int snapshot_read_byte_array(FILE *f, uint8_t *b_return, unsigned int num)
 {
+    current_fpos = ftell(f);
     if (num > 0 && fread(b_return, (size_t)num, 1, f) < 1) {
         snapshot_error = SNAPSHOT_READ_BYTE_ARRAY_ERROR;
         return -1;
@@ -275,6 +322,7 @@ static int snapshot_read_word_array(FILE *f, uint16_t *w_return, unsigned int nu
 {
     unsigned int i;
 
+    current_fpos = ftell(f);
     for (i = 0; i < num; i++) {
         if (snapshot_read_word(f, w_return + i) < 0) {
             return -1;
@@ -288,6 +336,7 @@ static int snapshot_read_dword_array(FILE *f, uint32_t *dw_return, unsigned int
 {
     unsigned int i;
 
+    current_fpos = ftell(f);
     for (i = 0; i < num; i++) {
         if (snapshot_read_dword(f, dw_return + i) < 0) {
             return -1;
@@ -307,6 +356,7 @@ static int snapshot_read_string(FILE *f, char **s)
     lib_free(*s);
     *s = NULL;      /* don't leave a bogus pointer */
 
+    current_fpos = ftell(f);
     if (snapshot_read_word(f, &w) < 0) {
         return -1;
     }
@@ -360,6 +410,16 @@ int snapshot_module_write_dword(snapshot_module_t *m, uint32_t dw)
     return 0;
 }
 
+int snapshot_module_write_qword(snapshot_module_t *m, uint64_t qw)
+{
+    if (snapshot_write_qword(m->file, qw) < 0) {
+        return -1;
+    }
+
+    m->size += 8;
+    return 0;
+}
+
 int snapshot_module_write_double(snapshot_module_t *m, double db)
 {
     if (snapshot_write_double(m->file, db) < 0) {
@@ -427,6 +487,7 @@ int snapshot_module_write_string(snapshot_module_t *m, const char *s)
 
 int snapshot_module_read_byte(snapshot_module_t *m, uint8_t *b_return)
 {
+    current_fpos = ftell(m->file);
     if (ftell(m->file) + sizeof(uint8_t) > m->offset + m->size) {
         snapshot_error = SNAPSHOT_READ_OUT_OF_BOUNDS_ERROR;
         return -1;
@@ -437,6 +498,7 @@ int snapshot_module_read_byte(snapshot_module_t *m, uint8_t *b_return)
 
 int snapshot_module_read_word(snapshot_module_t *m, uint16_t *w_return)
 {
+    current_fpos = ftell(m->file);
     if (ftell(m->file) + sizeof(uint16_t) > m->offset + m->size) {
         snapshot_error = SNAPSHOT_READ_OUT_OF_BOUNDS_ERROR;
         return -1;
@@ -447,6 +509,7 @@ int snapshot_module_read_word(snapshot_module_t *m, uint16_t *w_return)
 
 int snapshot_module_read_dword(snapshot_module_t *m, uint32_t *dw_return)
 {
+    current_fpos = ftell(m->file);
     if (ftell(m->file) + sizeof(uint32_t) > m->offset + m->size) {
         snapshot_error = SNAPSHOT_READ_OUT_OF_BOUNDS_ERROR;
         return -1;
@@ -455,8 +518,20 @@ int snapshot_module_read_dword(snapshot_module_t *m, uint32_t *dw_return)
     return snapshot_read_dword(m->file, dw_return);
 }
 
+int snapshot_module_read_qword(snapshot_module_t *m, uint64_t *qw_return)
+{
+    current_fpos = ftell(m->file);
+    if (ftell(m->file) + sizeof(uint64_t) > m->offset + m->size) {
+        snapshot_error = SNAPSHOT_READ_OUT_OF_BOUNDS_ERROR;
+        return -1;
+    }
+
+    return snapshot_read_qword(m->file, qw_return);
+}
+
 int snapshot_module_read_double(snapshot_module_t *m, double *db_return)
 {
+    current_fpos = ftell(m->file);
     if (ftell(m->file) + sizeof(double) > m->offset + m->size) {
         snapshot_error = SNAPSHOT_READ_OUT_OF_BOUNDS_ERROR;
         return -1;
@@ -467,6 +542,7 @@ int snapshot_module_read_double(snapshot_module_t *m, double *db_return)
 
 int snapshot_module_read_byte_array(snapshot_module_t *m, uint8_t *b_return, unsigned int num)
 {
+    current_fpos = ftell(m->file);
     if ((long)(ftell(m->file) + num) > (long)(m->offset + m->size)) {
         snapshot_error = SNAPSHOT_READ_OUT_OF_BOUNDS_ERROR;
         return -1;
@@ -487,6 +563,7 @@ int snapshot_module_read_word_array(snapshot_module_t *m, uint16_t *w_return, un
 
 int snapshot_module_read_dword_array(snapshot_module_t *m, uint32_t *dw_return, unsigned int num)
 {
+    current_fpos = ftell(m->file);
     if ((long)(ftell(m->file) + num * sizeof(uint32_t)) > (long)(m->offset + m->size)) {
         snapshot_error = SNAPSHOT_READ_OUT_OF_BOUNDS_ERROR;
         return -1;
@@ -497,6 +574,7 @@ int snapshot_module_read_dword_array(snapshot_module_t *m, uint32_t *dw_return,
 
 int snapshot_module_read_string(snapshot_module_t *m, char **charp_return)
 {
+    current_fpos = ftell(m->file);
     if (ftell(m->file) + sizeof(uint16_t) > m->offset + m->size) {
         snapshot_error = SNAPSHOT_READ_OUT_OF_BOUNDS_ERROR;
         return -1;
@@ -516,6 +594,17 @@ int snapshot_module_read_byte_into_int(snapshot_module_t *m, int *value_return)
     return 0;
 }
 
+int snapshot_module_read_byte_into_uint(snapshot_module_t *m, unsigned int *value_return)
+{
+    uint8_t b;
+
+    if (snapshot_module_read_byte(m, &b) < 0) {
+        return -1;
+    }
+    *value_return = (unsigned int)b;
+    return 0;
+}
+
 int snapshot_module_read_word_into_int(snapshot_module_t *m, int *value_return)
 {
     uint16_t b;
@@ -527,6 +616,17 @@ int snapshot_module_read_word_into_int(snapshot_module_t *m, int *value_return)
     return 0;
 }
 
+int snapshot_module_read_word_into_uint(snapshot_module_t *m, unsigned int *value_return)
+{
+    uint16_t b;
+
+    if (snapshot_module_read_word(m, &b) < 0) {
+        return -1;
+    }
+    *value_return = (unsigned int)b;
+    return 0;
+}
+
 int snapshot_module_read_dword_into_ulong(snapshot_module_t *m, unsigned long *value_return)
 {
     uint32_t b;
@@ -560,6 +660,17 @@ int snapshot_module_read_dword_into_uint(snapshot_module_t *m, unsigned int *val
     return 0;
 }
 
+int snapshot_module_read_qword_into_int64(snapshot_module_t *m, int64_t *value_return)
+{
+    uint64_t qw;
+
+    if (snapshot_module_read_qword(m, &qw) < 0) {
+        return -1;
+    }
+    *value_return = (int64_t)qw;
+    return 0;
+}
+
 /* ------------------------------------------------------------------------- */
 
 snapshot_module_t *snapshot_module_create(snapshot_t *s, const char *name, uint8_t major_version, uint8_t minor_version)
@@ -601,6 +712,7 @@ snapshot_module_t *snapshot_module_open(snapshot_t *s, const char *name, uint8_t
 
     if (fseek(s->file, s->first_module_offset, SEEK_SET) < 0) {
         snapshot_error = SNAPSHOT_FIRST_MODULE_NOT_FOUND_ERROR;
+        DBG(("snapshot_module_open error: name: '%s' NOT found\n", name));
         return NULL;
     }
 
@@ -610,6 +722,8 @@ snapshot_module_t *snapshot_module_open(snapshot_t *s, const char *name, uint8_t
 
     m->offset = s->first_module_offset;
 
+    DBG(("snapshot_module_open name: '%s'\n", name));
+
     /* Search for the module name.  This is quite inefficient, but I don't
        think we care.  */
     while (1) {
@@ -636,33 +750,48 @@ snapshot_module_t *snapshot_module_open(snapshot_t *s, const char *name, uint8_t
     }
 
     m->size_offset = ftell(s->file) - sizeof(uint32_t);
-
+#if 0
+    /* HACK: if any of the errors *this* function can produce is still pending
+             in snapshot_error, clear it out - else we might fail for no reason
+             eg when trying to open the C64ROM module, which isnt strictly
+             required to exist in the snapshot */
+    if ((snapshot_error == SNAPSHOT_FIRST_MODULE_NOT_FOUND_ERROR) ||
+        (snapshot_error == SNAPSHOT_MODULE_HEADER_READ_ERROR) ||
+        (snapshot_error == SNAPSHOT_MODULE_NOT_FOUND_ERROR)) {
+        snapshot_error = SNAPSHOT_NO_ERROR;
+    }
+#endif
+    DBG(("snapshot_module_open name: '%s', version %u.%u found\n", name, *major_version_return, *minor_version_return));
     return m;
 
 fail:
     fseek(s->file, s->first_module_offset, SEEK_SET);
     lib_free(m);
+    DBG(("snapshot_module_open error: name: '%s' NOT found\n", name));
     return NULL;
 }
 
 int snapshot_module_close(snapshot_module_t *m)
 {
-
+    DBG(("snapshot_module_close name: '%s'\n", current_module));
     /* Backpatch module size if writing.  */
     if (m->write_mode
         && (fseek(m->file, m->size_offset, SEEK_SET) < 0
             || snapshot_write_dword(m->file, m->size) < 0)) {
         snapshot_error = SNAPSHOT_MODULE_CLOSE_ERROR;
+        DBG(("snapshot_module_close error\n"));
         return -1;
     }
 
     /* Skip module.  */
     if (fseek(m->file, m->offset + m->size, SEEK_SET) < 0) {
         snapshot_error = SNAPSHOT_MODULE_SKIP_ERROR;
+        DBG(("snapshot_module_close error\n"));
         return -1;
     }
 
     lib_free(m);
+    DBG(("snapshot_module_close ok\n"));
     return 0;
 }
 
@@ -911,6 +1040,20 @@ void snapshot_display_error(void)
                 ui_error("Out of bounds reading error in snapshot %s", current_filename);
             }
             break;
+        case SNAPSHOT_ATA_IMAGE_FILENAME_MISMATCH:
+            if (current_module) {
+                ui_error("Filename of ATA Image file does not match in module %s in snapshot %s", current_module, current_filename);
+            } else {
+                ui_error("Filename of ATA Image file does not match in snapshot %s", current_filename);
+            }
+            break;
+        case SNAPSHOT_VICII_MODEL_MISMATCH:
+            if (current_module) {
+                ui_error("VICII model mismatch in module %s in snapshot %s", current_module, current_filename);
+            } else {
+                ui_error("VICII model mismatch in snapshot %s", current_filename);
+            }
+            break;
         case SNAPSHOT_ILLEGAL_OFFSET_ERROR:
             ui_error("Illegal offset while attempting to create module %s in snapshot %s", current_module, current_filename);
             break;
@@ -918,7 +1061,8 @@ void snapshot_display_error(void)
             ui_error("Cannot find first module in snapshot %s", current_filename);
             break;
         case SNAPSHOT_MODULE_HEADER_READ_ERROR:
-            ui_error("Error while reading module header in snapshot %s", current_filename);
+            ui_error("Error while reading module header (after module '%s' at pos 0x%lx) in snapshot %s",
+                     current_module, current_fpos, current_filename);
             break;
         case SNAPSHOT_MODULE_NOT_FOUND_ERROR:
             ui_error("Cannot find module %s in snapshot %s", current_module, current_filename);
@@ -976,6 +1120,10 @@ void snapshot_display_error(void)
             ui_error("Cannot read snapshot %s", current_filename);
             break;
     }
+    if (snapshot_error != SNAPSHOT_NO_ERROR) {
+        log_error(LOG_DEFAULT, "snapshot error at position 0x%lx module '%s' in file '%s'",
+                  current_fpos, current_module, current_filename);
+    }
 }
 
 void snapshot_set_error(int error)
