@@ -66,7 +66,7 @@ static int tape_snapshot_read_t64image_module(snapshot_t *s)
 #define TAPIMAGE_SNAP_MAJOR 1
 #define TAPIMAGE_SNAP_MINOR 0
 
-static int tape_snapshot_write_tapimage_module(snapshot_t *s)
+static int tape_snapshot_write_tapimage_module(int port, snapshot_t *s)
 {
     snapshot_module_t *m;
     FILE *ftap;
@@ -81,7 +81,7 @@ static int tape_snapshot_write_tapimage_module(snapshot_t *s)
     }
 
     /* get the file descriptor */
-    ftap = ((tap_t*)tape_image_dev1->data)->fd;
+    ftap = ((tap_t*)tape_image_dev[port]->data)->fd;
     if (!ftap) {
         log_error(tape_snapshot_log, "Cannot open tapfile for reading");
         return -1;
@@ -130,7 +130,7 @@ static int tape_snapshot_write_tapimage_module(snapshot_t *s)
 }
 
 
-static int tape_snapshot_read_tapimage_module(snapshot_t *s)
+static int tape_snapshot_read_tapimage_module(int port, snapshot_t *s)
 {
     uint8_t major_version, minor_version;
     snapshot_module_t *m;
@@ -178,7 +178,7 @@ static int tape_snapshot_read_tapimage_module(snapshot_t *s)
 
     lib_free(buffer);
     fclose(ftap);
-    tape_image_attach(1, filename);
+    tape_image_attach(port + 1, filename);
     lib_free(filename);
     snapshot_module_close(m);
     return 0;
@@ -194,24 +194,24 @@ fail:
 
 static const char snap_module_name[] = "TAPE";
 
-int tape_snapshot_write_module(snapshot_t *s, int save_image)
+int tape_snapshot_write_module(int port, snapshot_t *s, int save_image)
 {
     snapshot_module_t *m;
     tap_t *tap;
 
-    if (tape_image_dev1 == NULL || tape_image_dev1->name == NULL) {
+    if (tape_image_dev[port] == NULL || tape_image_dev[port]->name == NULL) {
         return 0;
     }
 
     if (save_image) {
-        switch (tape_image_dev1->type) {
+        switch (tape_image_dev[port]->type) {
             case TAPE_TYPE_T64:
                 if (tape_snapshot_write_t64image_module(s) < 0) {
                     return -1;
                 }
                 break;
             case TAPE_TYPE_TAP:
-                if (tape_snapshot_write_tapimage_module(s) < 0) {
+                if (tape_snapshot_write_tapimage_module(port, s) < 0) {
                     return -1;
                 }
                 break;
@@ -226,17 +226,17 @@ int tape_snapshot_write_module(snapshot_t *s, int save_image)
     }
 
     if (0
-        || SMW_B(m, (uint8_t)tape_image_dev1->read_only) < 0
-        || SMW_B(m, (uint8_t)tape_image_dev1->type) < 0) {
+        || SMW_B(m, (uint8_t)tape_image_dev[port]->read_only) < 0
+        || SMW_B(m, (uint8_t)tape_image_dev[port]->type) < 0) {
         snapshot_module_close(m);
         return -1;
     }
 
-    switch (tape_image_dev1->type) {
+    switch (tape_image_dev[port]->type) {
         case TAPE_TYPE_T64:
             break;
         case TAPE_TYPE_TAP:
-            tap = (tap_t*)tape_image_dev1->data;
+            tap = (tap_t*)tape_image_dev[port]->data;
             if (tap == NULL
                 || SMW_DW(m, tap->size) < 0
                 || SMW_B(m, tap->version) < 0
@@ -262,14 +262,14 @@ int tape_snapshot_write_module(snapshot_t *s, int save_image)
 }
 
 
-int tape_snapshot_read_module(snapshot_t *s)
+int tape_snapshot_read_module(int port, snapshot_t *s)
 {
     uint8_t major_version, minor_version;
     snapshot_module_t *m;
     unsigned int snap_type;
     tap_t *tap;
 
-    if (tape_snapshot_read_tapimage_module(s) < 0
+    if (tape_snapshot_read_tapimage_module(port, s) < 0
         || tape_snapshot_read_t64image_module(s) < 0) {
         return -1;
     }
@@ -280,18 +280,18 @@ int tape_snapshot_read_module(snapshot_t *s)
 
     if (m == NULL) {
         /* no tape attached */
-        tape_image_detach_internal(1);
+        tape_image_detach_internal(port + 1);
         return 0;
     }
 
     if (0
-        || SMR_B_INT(m, (int *)&tape_image_dev1->read_only) < 0
+        || SMR_B_INT(m, (int *)&tape_image_dev[port]->read_only) < 0
         || SMR_B_INT(m, (int *)&snap_type) < 0) {
         snapshot_module_close(m);
         return -1;
     }
 
-    if (snap_type != tape_image_dev1->type) {
+    if (snap_type != tape_image_dev[port]->type) {
         /* attached image type is not correct */
         log_error(tape_snapshot_log,
                   "No tape image attached or type not correct.");
@@ -299,11 +299,11 @@ int tape_snapshot_read_module(snapshot_t *s)
         return -1;
     }
 
-    switch (tape_image_dev1->type) {
+    switch (tape_image_dev[port]->type) {
         case TAPE_TYPE_T64:
             break;
         case TAPE_TYPE_TAP:
-            tap = (tap_t*)tape_image_dev1->data;
+            tap = (tap_t*)tape_image_dev[port]->data;
             if (tap == NULL
                 || SMR_DW(m, (uint32_t *)&tap->size) < 0
                 || SMR_B(m, &tap->version) < 0
