@@ -46,10 +46,12 @@
 #include "tape-internal.h"
 #include "tape.h"
 #include "tapeimage.h"
+#include "tapeport.h"
 #include "traps.h"
 #include "types.h"
 #include "uiapi.h"
 #include "vice-event.h"
+#include "arch/shared/archdep_real_path.h"
 
 /* #define DEBUG_TAPE */
 
@@ -80,7 +82,7 @@ static const trap_t *tape_traps;
 static log_t tape_log = LOG_ERR;
 
 /* The tape image for device 1. */
-tape_image_t *tape_image_dev1 = NULL;
+tape_image_t *tape_image_dev[TAPEPORT_MAX_PORTS] = { NULL };
 
 /* ------------------------------------------------------------------------- */
 
@@ -134,6 +136,8 @@ static void tape_init_vars(const tape_init_t *init)
 /* FIXME: This should be passed through a struct.  */
 int tape_init(const tape_init_t *init)
 {
+    int i;
+
     if (tape_log == LOG_ERR) {
         tape_log = log_open("Tape");
     }
@@ -141,8 +145,10 @@ int tape_init(const tape_init_t *init)
     tape_internal_init();
     tape_image_init();
 
-    lib_free(tape_image_dev1);
-    tape_image_dev1 = lib_calloc(1, sizeof(tape_image_t));
+    for (i = 0; i < TAPEPORT_MAX_PORTS; i++) {
+        lib_free(tape_image_dev[i]);
+        tape_image_dev[i] = lib_calloc(1, sizeof(tape_image_t));
+    }
 
     tap_init(init);
 
@@ -171,18 +177,25 @@ int tape_reinit(const tape_init_t *init)
 
 void tape_shutdown(void)
 {
-    lib_free(tape_image_dev1);
+    int i;
+
+    for (i = 0; i < TAPEPORT_MAX_PORTS; i++) {
+        lib_free(tape_image_dev[i]);
+    }
 }
 
 int tape_deinstall(void)
 {
+    int i;
+
     if (!tape_is_initialized) {
         return -1;
     }
 
-    if (tape_image_dev1->name != NULL &&
-        tape_image_dev1->type == TAPE_TYPE_T64) {
-        tape_image_detach_internal(1);
+    for (i = 0; i < TAPEPORT_MAX_PORTS; i++) {
+        if (tape_image_dev[i]->name != NULL && tape_image_dev[i]->type == TAPE_TYPE_T64) {
+            tape_image_detach_internal(i + 1);
+        }
     }
 
     tape_traps_deinstall();
@@ -209,14 +222,14 @@ int tape_find_header_trap(void)
 
     cassette_buffer = mem_ram + (mem_read(buffer_pointer_addr) | (mem_read((uint16_t)(buffer_pointer_addr + 1)) << 8));
 
-    if (tape_image_dev1->name == NULL
-        || tape_image_dev1->type != TAPE_TYPE_T64) {
+    if (tape_image_dev[TAPEPORT_PORT_1]->name == NULL
+        || tape_image_dev[TAPEPORT_PORT_1]->type != TAPE_TYPE_T64) {
         err = 1;
     } else {
         t64_t *t64;
         t64_file_record_t *rec;
 
-        t64 = (t64_t *)tape_image_dev1->data;
+        t64 = (t64_t *)tape_image_dev[TAPEPORT_PORT_1]->data;
         rec = NULL;
 
         err = 0;
@@ -276,14 +289,14 @@ int tape_find_header_trap_plus4(void)
 
     cassette_buffer = mem_ram + buffer_pointer_addr;
 
-    if (tape_image_dev1->name == NULL
-        || tape_image_dev1->type != TAPE_TYPE_T64) {
+    if (tape_image_dev[TAPEPORT_PORT_1]->name == NULL
+        || tape_image_dev[TAPEPORT_PORT_1]->type != TAPE_TYPE_T64) {
         err = 1;
     } else {
         t64_t *t64;
         t64_file_record_t *rec;
 
-        t64 = (t64_t *)tape_image_dev1->data;
+        t64 = (t64_t *)tape_image_dev[TAPEPORT_PORT_1]->data;
         rec = NULL;
 
         err = 0;
@@ -359,7 +372,7 @@ int tape_receive_trap(void)
                 int amount;
 
                 len = (int)(end - start);
-                amount = t64_read((t64_t *)tape_image_dev1->data, mem_ram + (int)start, len);
+                amount = t64_read((t64_t *)tape_image_dev[TAPEPORT_PORT_1]->data, mem_ram + (int)start, len);
                 if (amount == len) {
                     st = 0x40;  /* EOF */
                 } else {
@@ -402,7 +415,7 @@ int tape_receive_trap_plus4(void)
     /* Read block.  */
     len = end - start;
 
-    if (t64_read((t64_t *)tape_image_dev1->data,
+    if (t64_read((t64_t *)tape_image_dev[TAPEPORT_PORT_1]->data,
                  mem_ram + (int) start, (int)len) == (int) len) {
         st = 0x40;      /* EOF */
     } else {
@@ -419,19 +432,19 @@ int tape_receive_trap_plus4(void)
     return 1;
 }
 
-const char *tape_get_file_name(void)
+const char *tape_get_file_name(int port)
 {
-    if (tape_image_dev1 == NULL) {
+    if (tape_image_dev[port] == NULL) {
         return "";
     }
 
-    return tape_image_dev1->name;
+    return tape_image_dev[port]->name;
 }
 
-int tape_tap_attached(void)
+int tape_tap_attached(int port)
 {
-    if (tape_image_dev1->name != NULL
-        && tape_image_dev1->type == TAPE_TYPE_TAP) {
+    if (tape_image_dev[port]->name != NULL
+        && tape_image_dev[port]->type == TAPE_TYPE_TAP) {
         return 1;
     }
 
@@ -446,36 +459,36 @@ int tape_image_detach_internal(unsigned int unit)
     int retval = 0;
     char event_data[2];
 
-    if (unit != 1) {
+    if (unit != 1 && unit != 2) {
         return -1;
     }
 
-    if (tape_image_dev1 == NULL || tape_image_dev1->name == NULL) {
+    if (tape_image_dev[unit - 1] == NULL || tape_image_dev[unit - 1]->name == NULL) {
         return 0;
     }
 
-    switch (tape_image_dev1->type) {
+    switch (tape_image_dev[unit - 1]->type) {
         case TAPE_TYPE_T64:
             log_message(tape_log,
-                        "Detaching T64 image `%s'.", tape_image_dev1->name);
+                        "Detaching T64 image `%s'.", tape_image_dev[unit - 1]->name);
             /* Tape detached: release play button.  */
-            datasette_set_tape_sense(0);
+            datasette_set_tape_sense(unit - 1, 0);
             break;
         case TAPE_TYPE_TAP:
             log_message(tape_log,
-                        "Detaching TAP image `%s'.", tape_image_dev1->name);
-            datasette_set_tape_image(NULL);
+                        "Detaching TAP image `%s'.", tape_image_dev[unit - 1]->name);
+            datasette_set_tape_image(unit - 1, NULL);
 
             tape_traps_install();
             break;
         default:
             log_error(tape_log, "Unknown tape type %u.",
-                      tape_image_dev1->type);
+                      tape_image_dev[unit - 1]->type);
     }
 
-    retval = tape_image_close(tape_image_dev1);
+    retval = tape_image_close(tape_image_dev[unit - 1]);
 
-    ui_display_tape_current_image("");
+    ui_display_tape_current_image(unit - 1, "");
 
     event_data[0] = (char)unit;
     event_data[1] = 0;
@@ -489,7 +502,7 @@ int tape_image_detach(unsigned int unit)
 {
     char event_data[2];
 
-    if (unit != 1) {
+    if (unit != 1 && unit != 2) {
         return -1;
     }
 
@@ -513,12 +526,21 @@ static int tape_image_attach_internal(unsigned int unit, const char *name)
 {
     tape_image_t tape_image;
 
-    if (unit != 1) {
+    if (unit != 1 && unit != 2) {
         return -1;
     }
 
     if (!name || !*name) {
         return -1;
+    } else {
+        /* Make sure this tape isn't already in the other datasette */
+        unsigned int other_index = 2 - unit;
+        if (tape_image_dev[other_index] &&
+            tape_image_dev[other_index]->name &&
+            archdep_real_path_equal(tape_image_dev[other_index]->name, name)) {
+            log_error(tape_log, "File `%s' already mounted on other tape unit", name);
+            return -1;
+        }
     }
 
     tape_image.name = lib_strdup(name);
@@ -532,27 +554,27 @@ static int tape_image_attach_internal(unsigned int unit, const char *name)
 
     tape_image_detach_internal(unit);
 
-    memcpy(tape_image_dev1, &tape_image, sizeof(tape_image_t));
+    memcpy(tape_image_dev[unit - 1], &tape_image, sizeof(tape_image_t));
 
-    ui_display_tape_current_image(tape_image_dev1->name);
+    ui_display_tape_current_image(unit - 1, tape_image_dev[unit - 1]->name);
 
-    switch (tape_image_dev1->type) {
+    switch (tape_image_dev[unit - 1]->type) {
         case TAPE_TYPE_T64:
             log_message(tape_log, "T64 image '%s' attached.", name);
             /* Tape attached: press play button.  */
-            datasette_set_tape_sense(1);
+            datasette_set_tape_sense(unit - 1, 1);
             break;
         case TAPE_TYPE_TAP:
-            datasette_set_tape_image((tap_t *)tape_image_dev1->data);
+            datasette_set_tape_image(unit - 1, (tap_t *)tape_image_dev[unit - 1]->data);
             log_message(tape_log, "TAP image '%s' attached.", name);
             log_message(tape_log, "TAP image version: %i, system: %i.",
-                        ((tap_t *)tape_image_dev1->data)->version,
-                        ((tap_t *)tape_image_dev1->data)->system);
+                        ((tap_t *)tape_image_dev[unit - 1]->data)->version,
+                        ((tap_t *)tape_image_dev[unit - 1]->data)->system);
             tape_traps_deinstall();
             break;
         default:
             log_error(tape_log, "Unknown tape type %u.",
-                      tape_image_dev1->type);
+                      tape_image_dev[unit - 1]->type);
             return -1;
     }
 
