@@ -30,55 +30,82 @@
 #include "snapshot.h"
 #include "types.h"
 
-#define USERPORT_COLLISION_METHOD_DETACH_ALL    0
-#define USERPORT_COLLISION_METHOD_DETACH_LAST   1
-#define USERPORT_COLLISION_METHOD_AND_WIRES     2
-
 /* #define USERPORT_EXPERIMENTAL_DEVICES */
 
+#define USERPORT_NO_PULSE   0
+#define USERPORT_PULSE      1
+
+#define USERPORT_DEVICE_NONE                0
+#define USERPORT_DEVICE_PRINTER             1
+#define USERPORT_DEVICE_RS232_MODEM         2
+#define USERPORT_DEVICE_JOYSTICK_CGA        3
+#define USERPORT_DEVICE_JOYSTICK_PET        4
+#define USERPORT_DEVICE_JOYSTICK_HUMMER     5
+#define USERPORT_DEVICE_JOYSTICK_OEM        6
+#define USERPORT_DEVICE_JOYSTICK_HIT        7
+#define USERPORT_DEVICE_JOYSTICK_KINGSOFT   8
+#define USERPORT_DEVICE_JOYSTICK_STARBYTE   9
+#define USERPORT_DEVICE_JOYSTICK_SYNERGY    10
+#define USERPORT_DEVICE_DAC                 11
+#define USERPORT_DEVICE_DIGIMAX             12
+#define USERPORT_DEVICE_4BIT_SAMPLER        13
+#define USERPORT_DEVICE_8BSS                14
+#define USERPORT_DEVICE_RTC_58321A          15
+#define USERPORT_DEVICE_RTC_DS1307          16
+#define USERPORT_DEVICE_PETSCII_SNESPAD     17
+#define USERPORT_DEVICE_SUPERPAD64          18
+#define USERPORT_DEVICE_DIAG_586220_HARNESS 19
+#define USERPORT_DEVICE_DRIVE_PAR_CABLE     20
+#define USERPORT_DEVICE_IO_SIMULATION       21
+#define USERPORT_DEVICE_WIC64               22
+#define USERPORT_DEVICE_SPACEBALLS          23
+
+#define USERPORT_MAX_DEVICES                24
+
 enum {
-    USERPORT_DEVICE_PRINTER = 0,
-    USERPORT_DEVICE_JOYSTICK_CGA,
-    USERPORT_DEVICE_JOYSTICK_PET,
-    USERPORT_DEVICE_JOYSTICK_HUMMER,
-    USERPORT_DEVICE_JOYSTICK_OEM,
-    USERPORT_DEVICE_JOYSTICK_HIT,
-    USERPORT_DEVICE_JOYSTICK_KINGSOFT,
-    USERPORT_DEVICE_JOYSTICK_STARBYTE,
-    USERPORT_DEVICE_DAC,
-    USERPORT_DEVICE_DIGIMAX,
-    USERPORT_DEVICE_4BIT_SAMPLER,
-    USERPORT_DEVICE_8BSS,
-    USERPORT_DEVICE_RTC_58321A,
-    USERPORT_DEVICE_RTC_DS1307,
+    USERPORT_DEVICE_TYPE_NONE = 0,
+    USERPORT_DEVICE_TYPE_PRINTER,
+    USERPORT_DEVICE_TYPE_MODEM,
+    USERPORT_DEVICE_TYPE_DRIVE_PAR_CABLE,
+    USERPORT_DEVICE_TYPE_JOYSTICK_ADAPTER,
+    USERPORT_DEVICE_TYPE_AUDIO_OUTPUT,
+    USERPORT_DEVICE_TYPE_SAMPLER,
+    USERPORT_DEVICE_TYPE_RTC,
 #ifdef USERPORT_EXPERIMENTAL_DEVICES
-    USERPORT_DEVICE_DIAG_586220_HARNESS,
+    USERPORT_DEVICE_TYPE_HARNESS,
+    USERPORT_DEVICE_TYPE_WIFI,
 #endif
-    USERPORT_MAX_DEVICES
+    USERPORT_DEVICE_TYPE_IO_SIMULATION,
 };
 
 /* this structure is used by userport devices */
 typedef struct userport_device_s {
-    /* ID of the device */
-    int id;
-
     /* Name of the device */
     char *name;
 
+    /* flag to indicate that the device is a joystick/pad adapter */
+    int joystick_adapter_id;
+
+    /* flag to indicate the device type */
+    int device_type;
+
+    /* Device enable/disable */
+    int (*enable)(int val);
+
     /* Read pb0-7 pins */
-    void (*read_pbx)(void);
+    uint8_t (*read_pbx)(uint8_t orig);
 
     /* Store pb0-7 pins */
-    void (*store_pbx)(uint8_t val);
+    void (*store_pbx)(uint8_t val, int pulse);
 
     /* Read pa2 pin */
-    void (*read_pa2)(void);
+    uint8_t (*read_pa2)(uint8_t orig);
 
     /* Store pa2 pin */
     void (*store_pa2)(uint8_t val);
 
     /* Read pa3 pin */
-    void (*read_pa3)(void);
+    uint8_t (*read_pa3)(uint8_t orig);
 
     /* Store pa3 pin */
     void (*store_pa3)(uint8_t val);
@@ -90,28 +117,25 @@ typedef struct userport_device_s {
     void (*store_sp1)(uint8_t val);
 
     /* Read sp1 pin */
-    void (*read_sp1)(void);
+    uint8_t (*read_sp1)(uint8_t orig);
 
     /* Store sp2 pin */
     void (*store_sp2)(uint8_t val);
 
     /* Read sp2 pin */
-    void (*read_sp2)(void);
+    uint8_t (*read_sp2)(uint8_t orig);
 
-    /* detach device */
-    char *resource;
+    /* device reset function, reset line on the userport */
+    void (*reset)(void);
 
-    /* return value of a read */
-    uint8_t retval;
+    /* device powerup function, gets called when a hard reset is done */
+    void (*powerup)(void);
 
-    /* validity mask of a read */
-    uint8_t mask;
+    /* Snapshot write */
+    int (*write_snapshot)(struct snapshot_s *s);
 
-    /* involved in collision */
-    int collision;
-
-    /* a tag to indicate the order of insertion */
-    unsigned int order;
+    /* Snapshot read */
+    int (*read_snapshot)(struct snapshot_s *s);  /* pointer to the device snapshot read function */
 } userport_device_t;
 
 /* this structure is used by userport ports */
@@ -121,57 +145,43 @@ typedef struct userport_port_props_s {
     void (*set_flag)(uint8_t val); /* pointer to set flag function */
     int has_pc;                    /* port has the pc line */
     int has_sp12;                  /* port has the sp1 and sp2 lines */
+    int has_reset;                 /* port had the reset line */
 } userport_port_props_t;
 
-typedef struct userport_device_list_s {
-    struct userport_device_list_s *previous;
-    userport_device_t *device;
-    struct userport_device_list_s *next;
-} userport_device_list_t;
-
-extern userport_device_list_t *userport_device_register(userport_device_t *device);
-extern void userport_device_unregister(userport_device_list_t *device);
+typedef struct userport_desc_s {
+    char *name;
+    int id;
+    int device_type;
+} userport_desc_t;
 
 extern void userport_port_register(userport_port_props_t *props);
+extern int userport_device_register(int id, userport_device_t *device);
 
-extern uint8_t read_userport_pbx(uint8_t mask, uint8_t orig);
-extern void store_userport_pbx(uint8_t val);
+extern uint8_t read_userport_pbx(uint8_t orig);
+extern void store_userport_pbx(uint8_t val, int pulse);
 extern uint8_t read_userport_pa2(uint8_t orig);
 extern void store_userport_pa2(uint8_t val);
 extern uint8_t read_userport_pa3(uint8_t orig);
 extern void store_userport_pa3(uint8_t val);
-extern void set_userport_flag(uint8_t val);
 extern uint8_t read_userport_pc(uint8_t orig);
 extern uint8_t read_userport_sp1(uint8_t orig);
 extern void store_userport_sp1(uint8_t val);
 extern uint8_t read_userport_sp2(uint8_t orig);
 extern void store_userport_sp2(uint8_t val);
+extern void userport_reset(void);
+extern void userport_powerup(void);
+
+/* use this function from userport device code to set the userport flag */
+extern void set_userport_flag(uint8_t val);
 
 extern int userport_resources_init(void);
-extern void userport_resources_shutdown(void);
 extern int userport_cmdline_options_init(void);
 
+extern userport_desc_t *userport_get_valid_devices(int sort);
+extern const char *userport_get_device_type_desc(int type);
+
 extern void userport_enable(int val);
 
-typedef struct userport_snapshot_s {
-    /* ID of the device */
-    int id;
-
-    /* write snapshot */
-    int (*write_snapshot)(snapshot_t *s);
-
-    /* read snapshot */
-    int (*read_snapshot)(snapshot_t *s);
-} userport_snapshot_t;
-
-typedef struct userport_snapshot_list_s {
-    struct userport_snapshot_list_s *previous;
-    userport_snapshot_t *snapshot;
-    struct userport_snapshot_list_s *next;
-} userport_snapshot_list_t;
-
-extern void userport_snapshot_register(userport_snapshot_t *s);
-
 extern int userport_snapshot_write_module(snapshot_t *s);
 extern int userport_snapshot_read_module(snapshot_t *s);
 
