@@ -38,6 +38,7 @@
 #include "snapshot.h"
 #include "sound.h"
 #include "uiapi.h"
+#include "joyport.h"
 #include "userport.h"
 #include "userport_digimax.h"
 #include "util.h"
@@ -66,41 +67,35 @@ C64/C128 | CBM2 | TLC7226 DAC | NOTES
 */
 
 /* Some prototypes are needed */
-static void userport_digimax_store_pbx(uint8_t value);
+static void userport_digimax_store_pbx(uint8_t value, int pulse);
 static void userport_digimax_store_pa2(uint8_t value);
 static void userport_digimax_store_pa3(uint8_t value);
 static int userport_digimax_write_snapshot_module(snapshot_t *s);
 static int userport_digimax_read_snapshot_module(snapshot_t *s);
+static int digimax_enable(int value);
 
 static userport_device_t digimax_device = {
-    USERPORT_DEVICE_DIGIMAX,    /* device id */
-    "Userport DigiMAX",         /* device name */
-    NULL,                       /* NO read pb0-pb7 function */
-    userport_digimax_store_pbx, /* store pb0-pb7 function */
-    NULL,                       /* NO read pa2 pin function */
-    userport_digimax_store_pa2, /* store pa2 pin function */
-    NULL,                       /* NO read pa3 pin function */
-    userport_digimax_store_pa3, /* store pa3 pin function */
-    1,                          /* pc pin is needed */
-    NULL,                       /* NO store sp1 pin function */
-    NULL,                       /* NO read sp1 pin function */
-    NULL,                       /* NO store sp2 pin function */
-    NULL,                       /* NO read sp2 pin function */
-    "UserportDigiMax",          /* resource used by the device */
-    0xff,                       /* return value from a read, not used since the device is write only */
-    0x0f,                       /* validity mask of the device, doesn't change */
-    0,                          /* device involved in a read collision, to be filled in by the collision detection system */
-    0                           /* a tag to indicate the order of insertion */
+    "Userport DigiMAX",                     /* device name */
+    JOYSTICK_ADAPTER_ID_NONE,               /* NOT a joystick adapter */
+    USERPORT_DEVICE_TYPE_AUDIO_OUTPUT,      /* device is an audio output */
+    digimax_enable,                         /* enable function */
+    NULL,                                   /* NO read pb0-pb7 function */
+    userport_digimax_store_pbx,             /* store pb0-pb7 function */
+    NULL,                                   /* NO read pa2 pin function */
+    userport_digimax_store_pa2,             /* store pa2 pin function */
+    NULL,                                   /* NO read pa3 pin function */
+    userport_digimax_store_pa3,             /* store pa3 pin function */
+    1,                                      /* pc pin is needed */
+    NULL,                                   /* NO store sp1 pin function */
+    NULL,                                   /* NO read sp1 pin function */
+    NULL,                                   /* NO store sp2 pin function */
+    NULL,                                   /* NO read sp2 pin function */
+    NULL,                                   /* NO reset function */
+    NULL,                                   /* NO powerup function */
+    userport_digimax_write_snapshot_module, /* snapshot write function */
+    userport_digimax_read_snapshot_module   /* snapshot read function */
 };
 
-static userport_snapshot_t digimax_snapshot = {
-    USERPORT_DEVICE_DIGIMAX,
-    userport_digimax_write_snapshot_module,
-    userport_digimax_read_snapshot_module
-};
-
-static userport_device_list_t *userport_digimax_list_item = NULL;
-
 /* ------------------------------------------------------------------------- */
 
 static uint8_t userport_digimax_address = 3;
@@ -122,7 +117,7 @@ static void userport_digimax_store_pa3(uint8_t value)
     userport_digimax_address |= ((value & 1) << 1);
 }
 
-static void userport_digimax_store_pbx(uint8_t value)
+static void userport_digimax_store_pbx(uint8_t value, int pulse)
 {
     uint8_t addr = 0;
 
@@ -147,18 +142,13 @@ static void userport_digimax_store_pbx(uint8_t value)
 
 /* ---------------------------------------------------------------------*/
 
-static int set_digimax_enabled(int value, void *param)
+static int digimax_enable(int value)
 {
     int val = value ? 1 : 0;
 
     if (!digimax_sound_chip.chip_enabled && val) {
-        userport_digimax_list_item = userport_device_register(&digimax_device);
         digimax_sound_chip.chip_enabled = 1;
     } else if (digimax_sound_chip.chip_enabled && !val) {
-        if (userport_digimax_list_item != NULL) {
-            userport_device_unregister(userport_digimax_list_item);
-            userport_digimax_list_item = NULL;
-        }
         digimax_sound_chip.chip_enabled = 0;
     }
     return 0;
@@ -166,35 +156,9 @@ static int set_digimax_enabled(int value, void *param)
 
 /* ---------------------------------------------------------------------*/
 
-static const resource_int_t resources_int[] = {
-    { "UserportDIGIMAX", 0, RES_EVENT_STRICT, (resource_value_t)0,
-      &digimax_sound_chip.chip_enabled, set_digimax_enabled, NULL },
-    RESOURCE_INT_LIST_END
-};
-
 int userport_digimax_resources_init(void)
 {
-    userport_snapshot_register(&digimax_snapshot);
-
-    return resources_register_int(resources_int);
-}
-
-/* ---------------------------------------------------------------------*/
-
-static const cmdline_option_t cmdline_options[] =
-{
-    { "-userportdigimax", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
-      NULL, NULL, "UserportDIGIMAX", (resource_value_t)1,
-      NULL, "Enable the userport DigiMAX device" },
-    { "+userportdigimax", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
-      NULL, NULL, "UserportDIGIMAX", (resource_value_t)0,
-      NULL, "Disable the userport DigiMAX device" },
-    CMDLINE_LIST_END
-};
-
-int userport_digimax_cmdline_options_init(void)
-{
-    return cmdline_register_options(cmdline_options);
+    return userport_device_register(USERPORT_DEVICE_DIGIMAX, &digimax_device);
 }
 
 /* ---------------------------------------------------------------------*/
@@ -211,9 +175,9 @@ int userport_digimax_cmdline_options_init(void)
    BYTE  | voice 3 | voice 3 data
  */
 
-static char snap_module_name[] = "USERPORT_DIGIMAX";
+static const char snap_module_name[] = "UPDIGIMAX";
 #define SNAP_MAJOR   0
-#define SNAP_MINOR   0
+#define SNAP_MINOR   1
 
 static int userport_digimax_write_snapshot_module(snapshot_t *s)
 {
@@ -243,9 +207,6 @@ static int userport_digimax_read_snapshot_module(snapshot_t *s)
     uint8_t major_version, minor_version;
     snapshot_module_t *m;
 
-    /* enable device */
-    set_digimax_enabled(1, NULL);
-
     m = snapshot_module_open(s, snap_module_name, &major_version, &minor_version);
 
     if (m == NULL) {
