@@ -45,6 +45,7 @@ C64/C128 | CBM2 | PET | VIC20 | NAME
 #include "resources.h"
 #include "snapshot.h"
 #include "uiapi.h"
+#include "joyport.h"
 #include "userport.h"
 #include "userport_rtc_ds1307.h"
 
@@ -63,43 +64,37 @@ static uint8_t ds1307_pb1_scl = 1;
 /* ------------------------------------------------------------------------- */
 
 /* Some prototypes are needed */
-static void userport_rtc_read_pbx(void);
-static void userport_rtc_store_pbx(uint8_t value);
+static uint8_t userport_rtc_read_pbx(uint8_t orig);
+static void userport_rtc_store_pbx(uint8_t value, int pulse);
 static int userport_rtc_write_snapshot_module(snapshot_t *s);
 static int userport_rtc_read_snapshot_module(snapshot_t *s);
+static int userport_rtc_enable(int value);
 
 static userport_device_t rtc_device = {
-    USERPORT_DEVICE_RTC_DS1307, /* device id */
-    "Userport RTC (DS1307)",    /* device name */
-    userport_rtc_read_pbx,      /* read pb0-pb7 function */
-    userport_rtc_store_pbx,     /* store pb0-pb7 function */
-    NULL,                       /* NO read pa2 pin function */
-    NULL,                       /* NO store pa2 pin function */
-    NULL,                       /* NO read pa3 pin function */
-    NULL,                       /* NO store pa3 pin function */
-    0,                          /* pc pin is NOT needed */
-    NULL,                       /* NO store sp1 pin function */
-    NULL,                       /* NO read sp1 pin function */
-    NULL,                       /* NO store sp2 pin function */
-    NULL,                       /* NO read sp2 pin function */
-    "UserportRTC",              /* resource used by the device */
-    0xff,                       /* return value from a read, to be filled in by the device */
-    0x03,                       /* validity mask of the device, doesn't change */
-    0,                          /* device involved in a read collision, to be filled in by the collision detection system */
-    0                           /* a tag to indicate the order of insertion */
+    "Userport RTC (DS1307)",            /* device name */
+    JOYSTICK_ADAPTER_ID_NONE,           /* NOT a joystick adapter */
+    USERPORT_DEVICE_TYPE_RTC,           /* device is an RTC */
+    userport_rtc_enable,                /* enable function */
+    userport_rtc_read_pbx,              /* read pb0-pb7 function */
+    userport_rtc_store_pbx,             /* store pb0-pb7 function */
+    NULL,                               /* NO read pa2 pin function */
+    NULL,                               /* NO store pa2 pin function */
+    NULL,                               /* NO read pa3 pin function */
+    NULL,                               /* NO store pa3 pin function */
+    0,                                  /* pc pin is NOT needed */
+    NULL,                               /* NO store sp1 pin function */
+    NULL,                               /* NO read sp1 pin function */
+    NULL,                               /* NO store sp2 pin function */
+    NULL,                               /* NO read sp2 pin function */
+    NULL,                               /* NO reset function */
+    NULL,                               /* NO powerup function */
+    userport_rtc_write_snapshot_module, /* snapshot write function */
+    userport_rtc_read_snapshot_module   /* snapshot read function */
 };
 
-static userport_snapshot_t rtc_snapshot = {
-    USERPORT_DEVICE_RTC_DS1307,
-    userport_rtc_write_snapshot_module,
-    userport_rtc_read_snapshot_module
-};
-
-static userport_device_list_t *userport_rtc_list_item = NULL;
-
 /* ------------------------------------------------------------------------- */
 
-static int set_userport_rtc_enabled(int value, void *param)
+static int userport_rtc_enable(int value)
 {
     int val = value ? 1 : 0;
 
@@ -109,10 +104,6 @@ static int set_userport_rtc_enabled(int value, void *param)
 
     if (val) {
         ds1307_context = ds1307_init("USERDS1307");
-        userport_rtc_list_item = userport_device_register(&rtc_device);
-        if (userport_rtc_list_item == NULL) {
-            return -1;
-        }
         ds1307_set_data_line(ds1307_context, 1);
         ds1307_set_clk_line(ds1307_context, 1);
     } else {
@@ -120,8 +111,6 @@ static int set_userport_rtc_enabled(int value, void *param)
             ds1307_destroy(ds1307_context, ds1307_rtc_save);
             ds1307_context = NULL;
         }
-        userport_device_unregister(userport_rtc_list_item);
-        userport_rtc_list_item = NULL;
     }
 
     userport_rtc_ds1307_enabled = val;
@@ -137,8 +126,6 @@ static int set_userport_rtc_save(int val, void *param)
 
 
 static const resource_int_t resources_int[] = {
-    { "UserportRTCDS1307", 0, RES_EVENT_STRICT, (resource_value_t)0,
-      &userport_rtc_ds1307_enabled, set_userport_rtc_enabled, NULL },
     { "UserportRTCDS1307Save", 0, RES_EVENT_STRICT, (resource_value_t)0,
       &ds1307_rtc_save, set_userport_rtc_save, NULL },
     RESOURCE_INT_LIST_END
@@ -146,19 +133,15 @@ static const resource_int_t resources_int[] = {
 
 int userport_rtc_ds1307_resources_init(void)
 {
-    userport_snapshot_register(&rtc_snapshot);
+    if (userport_device_register(USERPORT_DEVICE_RTC_DS1307, &rtc_device) < 0) {
+        return -1;
+    }
 
     return resources_register_int(resources_int);
 }
 
 static const cmdline_option_t cmdline_options[] =
 {
-    { "-userportrtcds1307", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
-      NULL, NULL, "UserportRTCDS1307", (resource_value_t)1,
-      NULL, "Enable Userport RTC (DS1307)" },
-    { "+userportrtcds1307", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
-      NULL, NULL, "UserportRTCDS1307", (resource_value_t)0,
-      NULL, "Disable Userport RTC (DS1307)" },
     { "-userportrtcds1307save", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
       NULL, NULL, "UserportRTCDS1307Save", (resource_value_t)1,
       NULL, "Enable saving of the Userport RTC (DS1307) data when changed." },
@@ -183,7 +166,7 @@ void userport_rtc_ds1307_resources_shutdown(void)
 
 /* ---------------------------------------------------------------------*/
 
-static void userport_rtc_store_pbx(uint8_t value)
+static void userport_rtc_store_pbx(uint8_t value, int pulse)
 {
     uint8_t rtcdata = (value & 1) ? 1 : 0;
     uint8_t rtcclk = (value & 2) ? 1 : 0;
@@ -198,28 +181,29 @@ static void userport_rtc_store_pbx(uint8_t value)
     }
 }
 
-static void userport_rtc_read_pbx(void)
+static uint8_t userport_rtc_read_pbx(uint8_t orig)
 {
     uint8_t retval = ds1307_pb1_scl << 1;
 
     retval |= (ds1307_read_data_line(ds1307_context) & 1);
 
-    rtc_device.retval = retval;
+    return retval;
 }
 
 /* ---------------------------------------------------------------------*/
 
 /* UP_RTC_DS1307 snapshot module format:
 
-   type  | name | description
-   --------------------------
-   BYTE  | SDA  | SDA line state
-   BYTE  | SCL  | SCL line state
+   type  |   name   | description
+   ------------------------------
+   BYTE  |   SDA    | SDA line state
+   BYTE  |   SCL    | SCL line state
+   BYTE  | rtc save | save rtc offset upon detacht
  */
 
-static char snap_module_name[] = "UP_RTC_DS1307";
+static const char snap_module_name[] = "UPRTCDS1307";
 #define SNAP_MAJOR   0
-#define SNAP_MINOR   0
+#define SNAP_MINOR   1
 
 static int userport_rtc_write_snapshot_module(snapshot_t *s)
 {
@@ -233,7 +217,8 @@ static int userport_rtc_write_snapshot_module(snapshot_t *s)
 
     if (0
         || SMW_B(m, ds1307_pb0_sda) < 0
-        || SMW_B(m, ds1307_pb1_scl) < 0) {
+        || SMW_B(m, ds1307_pb1_scl) < 0
+        || SMW_B(m, (uint8_t)ds1307_rtc_save) < 0) {
         snapshot_module_close(m);
         return -1;
     }
@@ -247,9 +232,6 @@ static int userport_rtc_read_snapshot_module(snapshot_t *s)
     uint8_t major_version, minor_version;
     snapshot_module_t *m;
 
-    /* enable device */
-    set_userport_rtc_enabled(1, NULL);
-
     m = snapshot_module_open(s, snap_module_name, &major_version, &minor_version);
 
     if (m == NULL) {
@@ -264,8 +246,9 @@ static int userport_rtc_read_snapshot_module(snapshot_t *s)
 
     if (0
         || SMR_B(m, &ds1307_pb0_sda) < 0
-        || SMR_B(m, &ds1307_pb1_scl) < 0) {
-        goto fail;
+        || SMR_B(m, &ds1307_pb1_scl) < 0
+        || SMR_B_INT(m, &ds1307_rtc_save) < 0) {
+       goto fail;
     }
     snapshot_module_close(m);
 
