@@ -79,17 +79,17 @@ static void vdc_update_renderer(void)
     if (vdc_resources.stretchy) {
         video_chip_cap.single_mode.sizex = 1;
         video_chip_cap.single_mode.sizey = 2;
-        video_chip_cap.single_mode.rmode = VIDEO_RENDER_CRT_1X2;
+        video_chip_cap.single_mode.rmode = VIDEO_RENDER_RGBI_1X2;
         video_chip_cap.double_mode.sizex = 2;
         video_chip_cap.double_mode.sizey = 4;
-        video_chip_cap.double_mode.rmode = VIDEO_RENDER_CRT_2X4;
+        video_chip_cap.double_mode.rmode = VIDEO_RENDER_RGBI_2X4;
     } else {
         video_chip_cap.single_mode.sizex = 1;
         video_chip_cap.single_mode.sizey = 1;
-        video_chip_cap.single_mode.rmode = VIDEO_RENDER_CRT_1X1;
+        video_chip_cap.single_mode.rmode = VIDEO_RENDER_RGBI_1X1;
         video_chip_cap.double_mode.sizex = 2;
         video_chip_cap.double_mode.sizey = 2;
-        video_chip_cap.double_mode.rmode = VIDEO_RENDER_CRT_2X2;
+        video_chip_cap.double_mode.rmode = VIDEO_RENDER_RGBI_2X2;
     }
 }
 
@@ -121,8 +121,8 @@ int vdc_resources_init(void)
     video_chip_cap.dsize_limit_width = 0;
     video_chip_cap.dsize_limit_height = 0;
     video_chip_cap.dscan_allowed = ARCHDEP_VDC_DSCAN;
-    video_chip_cap.hwscale_allowed = ARCHDEP_VDC_HWSCALE;
     video_chip_cap.scale2x_allowed = 0;
+    video_chip_cap.interlace_allowed = 1;
     video_chip_cap.external_palette_name = "vdc_deft";
     video_chip_cap.double_buffering_allowed = ARCHDEP_VDC_DBUF;
 
