@@ -76,7 +76,6 @@
 
 
 struct alarm_context_s;
-struct clk_guard_s;
 struct interrupt_cpu_status_s;
 struct snapshot_s;
 struct via_context_s;
@@ -161,8 +160,7 @@ typedef struct via_context_s {
 extern void viacore_setup_context(struct via_context_s *via_context);
 extern void viacore_init(struct via_context_s *via_context,
                          struct alarm_context_s *alarm_context,
-                         struct interrupt_cpu_status_s *int_status,
-                         struct clk_guard_s *clk_guard);
+                         struct interrupt_cpu_status_s *int_status);
 extern void viacore_shutdown(struct via_context_s *via_context);
 extern void viacore_reset(struct via_context_s *via_context);
 extern void viacore_disable(struct via_context_s *via_context);
