@@ -34,6 +34,7 @@
 #include "behrbonz.h"
 #include "cartio.h"
 #include "cartridge.h"
+#include "crt.h"
 #include "export.h"
 #include "lib.h"
 #include "log.h"
@@ -172,6 +173,45 @@ static int zfile_load(const char *filename, uint8_t *dest, size_t size)
     return 0;
 }
 
+int behrbonz_crt_attach(FILE *fd, uint8_t *rawcart)
+{
+    crt_chip_header_t chip;
+    int idx = 0;
+
+    if (!cart_rom) {
+        cart_rom = lib_malloc(CART_ROM_SIZE);
+    }
+
+    for (idx = 0; idx < 256; idx++) {
+        if (crt_read_chip_header(&chip, fd)) {
+            goto exiterror;
+        }
+
+        DBG(("chip %d at %02x len %02x\n", idx, chip.start, chip.size));
+        if (chip.size != 0x2000) {
+            goto exiterror;
+        }
+
+        if (crt_read_chip(&cart_rom[0x2000 * idx], 0, &chip, fd)) {
+            goto exiterror;
+        }
+    }
+
+    if (export_add(&export_res) < 0) {
+        goto exiterror;
+    }
+    mem_cart_blocks = VIC_CART_BLK1 | VIC_CART_BLK2 | VIC_CART_BLK3 | VIC_CART_BLK5 | VIC_CART_IO3;
+    mem_initialize_memory();
+
+    behrbonz_io3_list_item = io_source_register(&behrbonz_io3_device);
+
+    return CARTRIDGE_VIC20_BEHRBONZ;
+
+exiterror:
+    behrbonz_detach();
+    return -1;
+}
+
 int behrbonz_bin_attach(const char *filename)
 {
     if (!cart_rom) {
