@@ -24,6 +24,8 @@
  *
  */
 
+/* #define DEBUGCART */
+
 #include "vice.h"
 
 #include <stdio.h>
@@ -34,6 +36,7 @@
 #include "cartio.h"
 #include "cartridge.h"
 #include "cmdline.h"
+#include "crt.h"
 #include "export.h"
 #include "lib.h"
 #include "log.h"
@@ -41,6 +44,7 @@
 #include "megacart.h"
 #include "mem.h"
 #include "monitor.h"
+#include "ram.h"
 #include "resources.h"
 #include "snapshot.h"
 #include "types.h"
@@ -50,6 +54,12 @@
 #include "vic20mem.h"
 #include "zfile.h"
 
+#ifdef DEBUGCART
+#define DBG(x) log_debug x
+#else
+#define DBG(x)
+#endif
+
 /* ------------------------------------------------------------------------- */
 
 /*
@@ -318,15 +328,18 @@ static uint8_t megacart_io3_read(uint16_t addr)
 
 static uint8_t megacart_io3_peek(uint16_t addr)
 {
-    if ((addr & 0x180) == 0x080) { /* $9c80 */
+    if ((addr & 0x3ff) == 0x080) { /* $9c80 */
         return bank_high_reg;
     }
 
-    if ((addr & 0x180) == 0x100) { /* $9d00 */
+    if ((addr & 0x3ff) == 0x100) { /* $9d00 */
         return bank_low_reg;
     }
 
-    return cart_nvram[0x1c00 + (addr & 0x3ff)];
+    if (nvram_en_flop) {
+        return cart_nvram[0x1c00 + (addr & 0x3ff)];
+    }
+    return vic20_cpu_last_data;
 }
 
 /* store 0x9c00-0x9fff (nvram 0x1c00 - 0x1fff) */
@@ -359,31 +372,6 @@ static void megacart_io3_store(uint16_t addr, uint8_t value)
 
 /* ------------------------------------------------------------------------- */
 
-void megacart_init(void)
-{
-    if (megacart_log == LOG_ERR) {
-        megacart_log = log_open(CARTRIDGE_VIC20_NAME_MEGACART);
-    }
-
-    reset_mode = BUTTON_RESET;
-    oe_flop = 0;
-    nvram_en_flop = 0;
-}
-
-void megacart_reset(void)
-{
-    if (reset_mode == SOFTWARE_RESET) {
-        oe_flop = !oe_flop;
-    } else {
-        oe_flop = 0;
-    }
-    reset_mode = BUTTON_RESET;
-}
-
-void megacart_config_setup(uint8_t *rawcart)
-{
-}
-
 static int zfile_load(const char *filename, uint8_t *dest, size_t size)
 {
     FILE *fd;
@@ -406,15 +394,16 @@ static int zfile_load(const char *filename, uint8_t *dest, size_t size)
 
 static int try_nvram_load(const char *filename)
 {
+    DBG(("cart_nvram: %p filename: %s", cart_nvram, filename ? filename : "NULL"));
     if (cart_nvram && filename && *filename != '\0') {
         if (zfile_load(filename, cart_nvram, (size_t)CART_NVRAM_SIZE) < 0) {
             log_message(megacart_log, "Failed to read NvRAM image `%s'!", filename);
             return -1;
         } else {
             log_message(megacart_log, "Read NvRAM image `%s'.", filename);
+            return 1;
         }
     }
-
     return 0;
 }
 
@@ -442,7 +431,37 @@ static int try_nvram_save(const char *filename)
     return ret;
 }
 
-int megacart_bin_attach(const char *filename)
+/* ------------------------------------------------------------------------- */
+
+/* FIXME: this still needs to be tweaked to match the hardware */
+static RAMINITPARAM ramparam = {
+    .start_value = 255,
+    .value_invert = 2,
+    .value_offset = 1,
+
+    .pattern_invert = 0x100,
+    .pattern_invert_value = 255,
+
+    .random_start = 0,
+    .random_repeat = 0,
+    .random_chance = 0,
+};
+
+/* FIXME: this still needs to be tweaked to match the hardware */
+static RAMINITPARAM nvramparam = {
+    .start_value = 255,
+    .value_invert = 2,
+    .value_offset = 1,
+
+    .pattern_invert = 0x100,
+    .pattern_invert_value = 255,
+
+    .random_start = 0,
+    .random_repeat = 0,
+    .random_chance = 0,
+};
+
+static void allocate_rom_ram(void)
 {
     if (!cart_ram) {
         cart_ram = lib_malloc(CART_RAM_SIZE);
@@ -453,6 +472,99 @@ int megacart_bin_attach(const char *filename)
     if (!cart_rom) {
         cart_rom = lib_malloc(CART_ROM_SIZE);
     }
+}
+
+static void clear_ram(void)
+{
+    if (cart_ram) {
+        DBG(("clear_ram: cart_ram"));
+        ram_init_with_pattern(cart_ram, CART_RAM_SIZE, &ramparam);
+    }
+    if (cart_nvram) {
+        DBG(("clear_ram: cart_nvram"));
+        ram_init_with_pattern(cart_nvram, CART_NVRAM_SIZE, &nvramparam);
+    }
+}
+
+void megacart_init(void)
+{
+    if (megacart_log == LOG_ERR) {
+        megacart_log = log_open(CARTRIDGE_VIC20_NAME_MEGACART);
+    }
+}
+
+void megacart_reset(void)
+{
+    if (reset_mode == SOFTWARE_RESET) {
+        oe_flop = !oe_flop;
+    } else {
+        oe_flop = 0;
+    }
+    reset_mode = BUTTON_RESET;
+}
+
+void megacart_powerup(void)
+{
+    reset_mode = BUTTON_RESET;
+    oe_flop = 0;
+    nvram_en_flop = 1;
+}
+
+void megacart_config_setup(uint8_t *rawcart)
+{
+}
+
+int megacart_crt_attach(FILE *fd, uint8_t *rawcart)
+{
+    crt_chip_header_t chip;
+    int idx = 0;
+
+    allocate_rom_ram();
+    clear_ram();
+
+    for (idx = 0; idx < 256; idx++) {
+        if (crt_read_chip_header(&chip, fd)) {
+            goto exiterror;
+        }
+
+        DBG(("chip %d at %02x len %02x", idx, chip.start, chip.size));
+        if (chip.size != 0x2000) {
+            goto exiterror;
+        }
+
+        if (crt_read_chip(&cart_rom[0x2000 * idx], 0, &chip, fd)) {
+            goto exiterror;
+        }
+    }
+
+    if (export_add(&export_res) < 0) {
+        goto exiterror;
+    }
+
+    try_nvram_load(nvram_filename);
+
+    cart_rom_low = cart_rom;
+    cart_rom_high = cart_rom + 0x100000;
+
+    mem_cart_blocks = VIC_CART_RAM123 |
+                      VIC_CART_BLK1 | VIC_CART_BLK2 | VIC_CART_BLK3 | VIC_CART_BLK5 |
+                      VIC_CART_IO2 | VIC_CART_IO3;
+    mem_initialize_memory();
+
+    megacart_io2_list_item = io_source_register(&megacart_io2_device);
+    megacart_io3_list_item = io_source_register(&megacart_io3_device);
+
+    return CARTRIDGE_VIC20_MEGACART;
+
+exiterror:
+    megacart_detach();
+    return -1;
+}
+
+int megacart_bin_attach(const char *filename)
+{
+    allocate_rom_ram();
+    clear_ram();
 
     if (zfile_load(filename, cart_rom, (size_t)CART_ROM_SIZE) < 0) {
         megacart_detach();
