@@ -52,7 +52,7 @@ extern void generic_reset(void);
 extern void generic_config_setup(uint8_t *rawcart);
 extern int generic_bin_attach(int type, const char *filename);
 /* extern int generic_bin_attach(const char *filename, uint8_t *rawcart); */
-/* extern int generic_crt_attach(FILE *fd, uint8_t *rawcart); */
+extern int generic_crt_attach(FILE *fd, uint8_t *rawcart);
 extern void generic_detach(void);
 
 extern int generic_resources_init(void);
