@@ -91,16 +91,16 @@ int vic_resources_init(void)
     video_chip_cap.dsize_limit_width = 0;
     video_chip_cap.dsize_limit_height = 0;
     video_chip_cap.dscan_allowed = ARCHDEP_VIC_DSCAN;
-    video_chip_cap.hwscale_allowed = ARCHDEP_VIC_HWSCALE;
     video_chip_cap.scale2x_allowed = ARCHDEP_VIC_DSIZE;
+    video_chip_cap.interlace_allowed = 1;
     video_chip_cap.external_palette_name = "mike-pal";
     video_chip_cap.double_buffering_allowed = ARCHDEP_VIC_DBUF;
     video_chip_cap.single_mode.sizex = 1;
     video_chip_cap.single_mode.sizey = 1;
-    video_chip_cap.single_mode.rmode = VIDEO_RENDER_PAL_1X1;
+    video_chip_cap.single_mode.rmode = VIDEO_RENDER_PAL_NTSC_1X1;
     video_chip_cap.double_mode.sizex = 2;
     video_chip_cap.double_mode.sizey = 2;
-    video_chip_cap.double_mode.rmode = VIDEO_RENDER_PAL_2X2;
+    video_chip_cap.double_mode.rmode = VIDEO_RENDER_PAL_NTSC_2X2;
 
     fullscreen_capability(&(video_chip_cap.fullscreen));
 
