@@ -30,13 +30,8 @@
 #include "vic20memrom.h"
 #include "types.h"
 
-#ifdef USE_EMBEDDED
-#include "vic20basic.h"
-#include "vic20kernal.h"
-#else
 uint8_t vic20memrom_basic_rom[VIC20_BASIC_ROM_SIZE];
 uint8_t vic20memrom_kernal_rom[VIC20_KERNAL_ROM_SIZE];
-#endif
 
 uint8_t vic20memrom_kernal_trap_rom[VIC20_KERNAL_ROM_SIZE];
 
