@@ -48,17 +48,15 @@ static int border_set_func(const char *value, void *extra_param)
     resources_get_int("MachineVideoStandard", &video);
 
     if (strcmp(value, "1") == 0 || strcmp(value, "full") == 0) {
-        vicii_resources.border_mode = VICII_FULL_BORDERS;
+        resources_set_int("VICIIBorderMode", 1);
     } else if (strcmp(value, "2") == 0 || strcmp(value, "debug") == 0) {
-        vicii_resources.border_mode = VICII_DEBUG_BORDERS;
+        resources_set_int("VICIIBorderMode", 2);
     } else if (strcmp(value, "3") == 0 || strcmp(value, "none") == 0) {
-        vicii_resources.border_mode = VICII_NO_BORDERS;
+        resources_set_int("VICIIBorderMode", 3);
     } else {
-        vicii_resources.border_mode = VICII_NORMAL_BORDERS;
+        resources_set_int("VICIIBorderMode", 0);
     }
 
-    machine_change_timing(video, vicii_resources.border_mode);
-
     return 0;
 }
 
