@@ -49,17 +49,15 @@ static int border_set_func(const char *value, void *extra_param)
     resources_get_int("MachineVideoStandard", &video);
 
     if (strcmp(value, "1") == 0 || strcmp(value, "full") == 0) {
-        vicii_resources.border_mode = VICII_FULL_BORDERS;
+        resources_set_int("VICIIBorderMode", 1);
     } else if (strcmp(value, "2") == 0 || strcmp(value, "debug") == 0) {
-        vicii_resources.border_mode = VICII_DEBUG_BORDERS;
+        resources_set_int("VICIIBorderMode", 2);
     } else if (strcmp(value, "3") == 0 || strcmp(value, "none") == 0) {
-        vicii_resources.border_mode = VICII_NO_BORDERS;
+        resources_set_int("VICIIBorderMode", 3);
     } else {
-        vicii_resources.border_mode = VICII_NORMAL_BORDERS;
+        resources_set_int("VICIIBorderMode", 0);
     }
 
-    machine_change_timing(video, vicii_resources.border_mode);
-
     return 0;
 }
 
@@ -68,7 +66,7 @@ struct model_s {
     int model;
 };
 
-static struct model_s model_match[] = {
+static const struct model_s model_match[] = {
     /* PAL, 63 cycle, 9 luma, "old" */
     { "6569", VICII_MODEL_6569 },
     { "6569r3", VICII_MODEL_6569 },
