@@ -62,6 +62,16 @@ static void on_vsync_set_border_mode(void *unused)
 
 static int set_border_mode(int val, void *param)
 {
+    switch (val) {
+        case VICII_NORMAL_BORDERS:
+        case VICII_FULL_BORDERS:
+        case VICII_DEBUG_BORDERS:
+        case VICII_NO_BORDERS:
+            break;
+        default:
+            return -1;
+    }
+    
     next_border_mode = val;
     vsync_on_vsync_do(on_vsync_set_border_mode, NULL);
 
@@ -92,7 +102,7 @@ struct vicii_model_info_s {
     int luma;
 };
 
-static struct vicii_model_info_s vicii_info[] = {
+static const struct vicii_model_info_s vicii_info[] = {
     /* VICII_MODEL_6569: PAL, 63 cycle, 9 luma, "old" */
     { MACHINE_SYNC_PAL, 1 },
 
@@ -175,16 +185,15 @@ int vicii_resources_init(void)
     video_chip_cap.dsize_limit_width = 0;
     video_chip_cap.dsize_limit_height = 0;
     video_chip_cap.dscan_allowed = ARCHDEP_VICII_DSCAN;
-    video_chip_cap.hwscale_allowed = ARCHDEP_VICII_HWSCALE;
     video_chip_cap.scale2x_allowed = ARCHDEP_VICII_DSIZE;
     video_chip_cap.external_palette_name = "pepto-pal";
     video_chip_cap.double_buffering_allowed = ARCHDEP_VICII_DBUF;
     video_chip_cap.single_mode.sizex = 1;
     video_chip_cap.single_mode.sizey = 1;
-    video_chip_cap.single_mode.rmode = VIDEO_RENDER_PAL_1X1;
+    video_chip_cap.single_mode.rmode = VIDEO_RENDER_PAL_NTSC_1X1;
     video_chip_cap.double_mode.sizex = 2;
     video_chip_cap.double_mode.sizey = 2;
-    video_chip_cap.double_mode.rmode = VIDEO_RENDER_PAL_2X2;
+    video_chip_cap.double_mode.rmode = VIDEO_RENDER_PAL_NTSC_2X2;
 
     fullscreen_capability(&(video_chip_cap.fullscreen));
 
