@@ -29,6 +29,10 @@
 
 #include "types.h"
 
+/* video chip type */
+#define VIDEO_CHIP_VICII 0
+#define VIDEO_CHIP_VDC   1
+
 /* video filter type, resource "CHIPFilter" */
 #define VIDEO_FILTER_NONE         0
 #define VIDEO_FILTER_CRT          1
@@ -37,20 +41,20 @@
 /* These constants are used to configure the video output.  */
 
 /* no video output (dummy) */
-#define VIDEO_RENDER_NULL       0
+#define VIDEO_RENDER_NULL             0
 /* PAL or NTSC TV/Monitor (like VIC, VIC-II or TED) */
-#define VIDEO_RENDER_PAL_1X1    1
-#define VIDEO_RENDER_PAL_2X2    2
-/* no filtering */
-#define VIDEO_RENDER_RGB_1X1    3
-#define VIDEO_RENDER_RGB_1X2    4 /* needed for y-stretch */
-#define VIDEO_RENDER_RGB_2X2    5
-#define VIDEO_RENDER_RGB_2X4    6 /* needed for y-stretch */
-/* RGB(I) or monochrome Monitor (CRTC and VDC) */
-#define VIDEO_RENDER_CRT_1X1    7 /* FIXME: to be written */
-#define VIDEO_RENDER_CRT_1X2    8 /* needed for y-stretch */
-#define VIDEO_RENDER_CRT_2X2    9
-#define VIDEO_RENDER_CRT_2X4   10 /* needed for y-stretch */
+#define VIDEO_RENDER_PAL_NTSC_1X1     1
+#define VIDEO_RENDER_PAL_NTSC_2X2     2
+/* monochrome Monitor (CRTC) */
+#define VIDEO_RENDER_CRT_MONO_1X1     3
+#define VIDEO_RENDER_CRT_MONO_1X2     4 /* needed for y-stretch */
+#define VIDEO_RENDER_CRT_MONO_2X2     5
+#define VIDEO_RENDER_CRT_MONO_2X4     6 /* needed for y-stretch */
+/* RGB(I) Monitor (VDC) */
+#define VIDEO_RENDER_RGBI_1X1         7
+#define VIDEO_RENDER_RGBI_1X2         8 /* needed for y-stretch */
+#define VIDEO_RENDER_RGBI_2X2         9
+#define VIDEO_RENDER_RGBI_2X4        10 /* needed for y-stretch */
 
 struct video_canvas_s;
 struct video_cbm_palette_s;
@@ -70,8 +74,10 @@ struct canvas_refresh_s {
 typedef struct canvas_refresh_s canvas_refresh_t;
 
 struct draw_buffer_s {
-    /* The real drawing buffer, with padding bytes on either side to workaround CRT and Scale2x bugs */
-    uint8_t *draw_buffer_padded_allocation;
+    /* The real drawing buffers, with padding bytes on either side to workaround CRT and Scale2x bugs */
+    uint8_t *draw_buffer_padded_allocations[2];
+    /* Pointers into the non padded buffers within the padded buffers */
+    uint8_t *draw_buffer_non_padded[2];
     /* The memory buffer where the screen of the emulated machine is drawn. Palettized, 1 byte per pixel */
     uint8_t *draw_buffer;
     /* Width of draw_buffer in pixels */
@@ -128,9 +134,9 @@ struct video_chip_cap_s {
     unsigned int dsize_limit_width;
     unsigned int dsize_limit_height;
     unsigned int dscan_allowed;
-    unsigned int hwscale_allowed;
     unsigned int scale2x_allowed;
     unsigned int double_buffering_allowed;
+    unsigned int interlace_allowed;
     const char *external_palette_name;
     cap_render_t single_mode;
     cap_render_t double_mode;
@@ -160,6 +166,25 @@ struct video_render_color_tables_s {
     int32_t line_yuv_0[VIDEO_MAX_OUTPUT_WIDTH * 3];
     int16_t prevrgbline[VIDEO_MAX_OUTPUT_WIDTH * 3];
     uint8_t rgbscratchbuffer[VIDEO_MAX_OUTPUT_WIDTH * 4];
+    
+    /*
+     * All values below here formerly were globals in video-color.h.
+     * This resulted in palette leaks between multiple rendering windows (VDC / VICII)
+     */
+    uint32_t gamma_red[256 * 3];
+    uint32_t gamma_grn[256 * 3];
+    uint32_t gamma_blu[256 * 3];
+
+    uint32_t gamma_red_fac[256 * 3 * 2];
+    uint32_t gamma_grn_fac[256 * 3 * 2];
+    uint32_t gamma_blu_fac[256 * 3 * 2];
+
+    /* optional alpha value for 32bit rendering */
+    uint32_t alpha;
+    
+    uint32_t color_red[256];
+    uint32_t color_grn[256];
+    uint32_t color_blu[256];
 };
 typedef struct video_render_color_tables_s video_render_color_tables_t;
 
@@ -190,13 +215,14 @@ struct video_render_config_s {
     int scalex;                    /* Horizontal scaling */
     int scaley;                    /* Vertical scaling */
     int doublescan;                /* Doublescan enabled?  */
-    int hwscale;                   /* Hardware scaling enabled? */
     int scale2x;                   /* Scale2x enabled?  */
     int filter;                    /* VIDEO_FILTER_NONE, VIDEO_FILTER_CRT, VIDEO_FILTER_SCALE2X */
     int external_palette;          /* Use an external palette?  */
     char *external_palette_name;   /* Name of the external palette.  */
     int double_buffer;             /* Double buffering enabled? */
     int readable;                  /* reading of frame buffer is safe and fast */
+    int interlaced;                /* Is the output currently interlaced? */
+    int interlace_field;           /* Which of the two interlaced frames is current? */
     struct video_cbm_palette_s *cbm_palette; /* Internal palette.  */
     struct video_render_color_tables_s color_tables;
     int fullscreen_enabled;
@@ -212,9 +238,9 @@ typedef struct video_render_config_s video_render_config_t;
 extern void video_render_initconfig(video_render_config_t *config);
 extern void video_render_setphysicalcolor(video_render_config_t *config,
                                           int index, uint32_t color, int depth);
-extern void video_render_setrawrgb(unsigned int index, uint32_t r, uint32_t g,
-                                   uint32_t b);
-extern void video_render_setrawalpha(uint32_t a);
+extern void video_render_setrawrgb(video_render_color_tables_t *color_tab, unsigned int index,
+                                   uint32_t r, uint32_t g, uint32_t b);
+extern void video_render_setrawalpha(video_render_color_tables_t *color_tab, uint32_t a);
 extern void video_render_initraw(struct video_render_config_s *videoconfig);
 
 /**************************************************************/
@@ -228,6 +254,7 @@ extern struct video_canvas_s *video_canvas_create(struct video_canvas_s *canvas,
                                                   unsigned int *width, unsigned int *height,
                                                   int mapped);
 extern void video_arch_canvas_init(struct video_canvas_s *canvas);
+extern int video_arch_get_active_chip(void);
 extern void video_canvas_shutdown(struct video_canvas_s *canvas);
 extern struct video_canvas_s *video_canvas_init(void);
 extern void video_canvas_refresh_all_tracked(void);
@@ -247,7 +274,7 @@ extern void video_canvas_unmap(struct video_canvas_s *canvas);
 extern void video_canvas_resize(struct video_canvas_s *canvas, char resize_canvas);
 extern void video_canvas_render(struct video_canvas_s *canvas, uint8_t *trg,
                                 int width, int height, int xs, int ys,
-                                int xt, int yt, int pitcht, int depth);
+                                int xt, int yt, int pitcht);
 extern void video_canvas_refresh_all(struct video_canvas_s *canvas);
 extern char video_canvas_can_resize(struct video_canvas_s *canvas);
 extern void video_viewport_get(struct video_canvas_s *canvas,
@@ -258,16 +285,6 @@ extern void video_viewport_title_set(struct video_canvas_s *canvas,
                                      const char *title);
 extern void video_viewport_title_free(struct viewport_s *viewport);
 
-typedef struct video_draw_buffer_callback_s {
-    int (*draw_buffer_alloc)(struct video_canvas_s *canvas, uint8_t **draw_buffer,
-                             unsigned int fb_width, unsigned int fb_height,
-                             unsigned int *fb_pitch);
-    void (*draw_buffer_free)(struct video_canvas_s *canvas, uint8_t *draw_buffer);
-    void (*draw_buffer_clear)(struct video_canvas_s *canvas, uint8_t *draw_buffer,
-                              uint8_t value, unsigned int fb_width,
-                              unsigned int fb_height, unsigned int fb_pitch);
-} video_draw_buffer_callback_t;
-
 struct raster_s;
 
 extern int video_resources_init(void);
@@ -310,10 +327,4 @@ extern void video_color_palette_internal(struct video_canvas_s *canvas,
 extern int video_color_update_palette(struct video_canvas_s *canvas);
 extern void video_color_palette_free(struct palette_s *palette);
 
-/* FIXME: implement a central function that inits the renderer(s) */
-extern void video_render_1x2_init(void);
-extern void video_render_2x2_init(void);
-extern void video_render_pal_init(void);
-extern void video_render_crt_init(void);
-
 #endif
