@@ -50,10 +50,9 @@
 #endif
 
 #include "archdep_exit.h"
-#include "clkguard.h"
 #include "cmdline.h"
 #include "debug.h"
-#include "joy.h"
+#include "joystick.h"
 #include "kbdbuf.h"
 #include "lib.h"
 #include "log.h"
@@ -68,6 +67,7 @@
 #include "sound.h"
 #include "types.h"
 #include "tick.h"
+#include "videoarch.h"
 #include "vsync.h"
 #include "vsyncapi.h"
 
@@ -94,37 +94,51 @@ static int    vsync_metric_warp_enabled;
 
 /* ------------------------------------------------------------------------- */
 
-static vsync_callback_t *vsync_callback_queue;
-static int vsync_callback_queue_size_max;
-static int vsync_callback_queue_size;
+typedef struct callback_queue_s {
+    vsync_callback_t *queue;
+    int size_max;
+    int size;
+} callback_queue_t;
+
+static callback_queue_t callback_queues[2];
+static callback_queue_t *callback_queue = callback_queues;
+static int callback_queue_index;
 
 /** \brief Call callback_func(callback_param) once at vsync time (or machine reset) */
 void vsync_on_vsync_do(vsync_callback_func_t callback_func, void *callback_param)
 {
-    mainlock_assert_lock_obtained();
-    
     /* Grow the queue as needed */
-    if (vsync_callback_queue_size == vsync_callback_queue_size_max) {
-        vsync_callback_queue_size_max += 1;
-        vsync_callback_queue = lib_realloc(vsync_callback_queue, vsync_callback_queue_size_max * sizeof(vsync_callback_t));
+    if (callback_queue->size == callback_queue->size_max) {
+        callback_queue->size_max++;
+        callback_queue->queue = lib_realloc(callback_queue->queue, callback_queue->size_max * sizeof(vsync_callback_t));
     }
 
-    vsync_callback_queue[vsync_callback_queue_size].callback = callback_func;
-    vsync_callback_queue[vsync_callback_queue_size].param = callback_param;
+    callback_queue->queue[callback_queue->size].callback = callback_func;
+    callback_queue->queue[callback_queue->size].param = callback_param;
 
-    vsync_callback_queue_size++;
+    callback_queue->size++;
 }
 
+/** \brief Keep executing on_vsync_do callbacks until none are scheduled. */
 static void execute_vsync_callbacks(void)
 {
     int i;
+    callback_queue_t *executing_queue;
     
-    /* Execute each callback in turn. */
-    if (vsync_callback_queue_size) {
-        for (i = 0; i < vsync_callback_queue_size; i++) {
-            vsync_callback_queue[i].callback(vsync_callback_queue[i].param);
+    while (callback_queue->size) {
+        
+        /* We'll iterate over this queue */
+        executing_queue = callback_queue;
+        
+        /* Flip to the other queue to collect any new callbacks queued within these callbacks */
+        callback_queue_index = 1 - callback_queue_index;
+        callback_queue = &callback_queues[callback_queue_index];
+        
+        for (i = 0; i < executing_queue->size; i++) {
+            executing_queue->queue[i].callback(executing_queue->queue[i].param);
         }
-        vsync_callback_queue_size = 0;
+        
+        executing_queue->size = 0;
     }
 }
 
@@ -137,8 +151,15 @@ static int relative_speed;
 
 /* "Warp mode".  If nonzero, attempt to run as fast as possible. */
 static int warp_enabled;
-static unsigned long warp_render_tick_interval;
-static unsigned long warp_next_render_tick;
+
+/* "InitialWarpMode" resource controlling whether warp should be enabled from launch. */
+static int initial_warp_mode_resource;
+
+/* warp mode cmdline arg controlling whether warp should be enabled from launch. Overrides InitialWarpMode resource. */
+static int initial_warp_mode_cmdline = -1;
+
+/* When the next frame should be rendered, not skipped, during warp. */
+static tick_t warp_render_tick_interval;
 
 /* Triggers the vice thread to update its priorty */
 static volatile int update_thread_priority = 1;
@@ -149,7 +170,7 @@ static int set_relative_speed(int val, void *param)
         val = 100;
         log_warning(LOG_DEFAULT, "Setting speed to 0 is no longer supported - use warp instead.");
     }
-    
+
     relative_speed = val;
     sound_set_relative_speed(relative_speed);
     set_timer_speed(relative_speed);
@@ -157,34 +178,38 @@ static int set_relative_speed(int val, void *param)
     return 0;
 }
 
-static int set_warp_mode(int val, void *param)
+void vsync_set_warp_mode(int val)
 {
     warp_enabled = val ? 1 : 0;
 
     sound_set_warp_mode(warp_enabled);
     vsync_suspend_speed_eval();
-    
-    if (warp_enabled) {
-        warp_next_render_tick = tick_now() + warp_render_tick_interval;
-    }
 
     update_thread_priority = 1;
+}
+
+int vsync_get_warp_mode(void)
+{
+    return warp_enabled;
+}
+
+static int set_initial_warp_mode_resource(int val, void *param)
+{
+    initial_warp_mode_resource = val ? 1 : 0;
     
     return 0;
 }
 
-
 /* Vsync-related resources. */
 static const resource_int_t resources_int[] = {
     { "Speed", 100, RES_EVENT_SAME, NULL,
       &relative_speed, set_relative_speed, NULL },
-    { "WarpMode", 0, RES_EVENT_STRICT, (resource_value_t)0,
+    { "InitialWarpMode", 0, RES_EVENT_STRICT, (resource_value_t)0,
       /* FIXME: maybe RES_EVENT_NO */
-      &warp_enabled, set_warp_mode, NULL },
+      &initial_warp_mode_resource, set_initial_warp_mode_resource, NULL },
     RESOURCE_INT_LIST_END
 };
 
-
 int vsync_resources_init(void)
 {
     return resources_register_int(resources_int);
@@ -192,18 +217,30 @@ int vsync_resources_init(void)
 
 /* ------------------------------------------------------------------------- */
 
+static int set_initial_warp_mode_cmdline(const char *param, void *extra_param)
+{
+    /*
+     * We don't want -warp / +warp to end up in the config file,
+     * so we don't use a resource.
+     */
+    
+    initial_warp_mode_cmdline = vice_ptr_to_int(extra_param) ? 1 : 0;
+
+    return 0;
+}
+
 /* Vsync-related command-line options. */
 static const cmdline_option_t cmdline_options[] =
 {
     { "-speed", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "Speed", NULL,
       "<percent or negative fps>", "Limit emulation speed to specified value" },
-    { "-warp", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
-      NULL, NULL, "WarpMode", (resource_value_t)1,
-      NULL, "Enable warp mode" },
-    { "+warp", SET_RESOURCE, CMDLINE_ATTRIB_NONE,
-      NULL, NULL, "WarpMode", (resource_value_t)0,
-      NULL, "Disable warp mode" },
+    { "-warp", CALL_FUNCTION, CMDLINE_ATTRIB_NONE,
+      set_initial_warp_mode_cmdline, int_to_void_ptr(1), NULL, NULL,
+      NULL, "Initially enable warp mode" },
+    { "+warp", CALL_FUNCTION, CMDLINE_ATTRIB_NONE,
+      set_initial_warp_mode_cmdline, int_to_void_ptr(0), NULL, NULL,
+      NULL, "Do not initially enable warp mode (default)" },
     CMDLINE_LIST_END
 };
 
@@ -214,7 +251,7 @@ int vsync_cmdline_options_init(void)
 
 /* ------------------------------------------------------------------------- */
 
-/* Maximum consecutive length of time we can skip rendering frames when 
+/* Maximum consecutive length of time we can skip rendering frames when
    adjusting the refresh rate dynamically for slow host CPU situations. */
 #define MAX_RENDER_SKIP_MS (1000 / 10)
 
@@ -231,17 +268,18 @@ static void (*vsync_hook)(void);
 
 /* static guarantees zero values. */
 static double ticks_per_frame;
+static double cycles_per_frame;
 static double emulated_clk_per_second;
 
-static unsigned long last_sync_emulated_tick;
-static unsigned long last_sync_tick;
+static tick_t last_sync_emulated_tick;
+static tick_t last_sync_tick;
 static CLOCK last_sync_clk;
 
-static unsigned long sync_target_tick;
+static tick_t sync_target_tick;
 
 static int timer_speed = 0;
-static int speed_eval_suspended = 1;
-static int sync_reset = 1;
+static bool sync_reset = true;
+static bool metrics_reset = false;
 
 /* Initialize vsync timers and set relative speed of emulation in percent. */
 static int set_timer_speed(int speed)
@@ -251,7 +289,7 @@ static int set_timer_speed(int speed)
     timer_speed = speed;
 
     vsync_suspend_speed_eval();
-    
+
     if (refresh_frequency <= 0) {
         /* Happens during init */
         return -1;
@@ -259,15 +297,14 @@ static int set_timer_speed(int speed)
 
     if (speed < 0) {
         /* negative speeds are fps targets */
-
         cpu_percent = 100.0 * ((0 - speed) / refresh_frequency);
     } else {
         /* positive speeds are cpu percent targets */
         cpu_percent = speed;
     }
 
-    ticks_per_frame = tick_per_second() * 100 / cpu_percent / refresh_frequency;
-    emulated_clk_per_second = cycles_per_sec * cpu_percent / 100;
+    ticks_per_frame = tick_per_second() * 100.0 / cpu_percent / refresh_frequency;
+    emulated_clk_per_second = cycles_per_sec * cpu_percent / 100.0;
 
     return 0;
 }
@@ -278,6 +315,7 @@ void vsync_set_machine_parameter(double refresh, long cycles)
 {
     refresh_frequency = refresh;
     cycles_per_sec = cycles;
+    cycles_per_frame = (double)cycles / refresh;
     set_timer_speed(relative_speed);
 }
 
@@ -288,19 +326,31 @@ double vsync_get_refresh_frequency(void)
 
 void vsync_init(void (*hook)(void))
 {
+    /* Set the initial warp state. */
+    if (initial_warp_mode_cmdline != -1) {
+        /* Command line overrides config resource setting. */
+        vsync_set_warp_mode(initial_warp_mode_cmdline);
+    } else {
+        vsync_set_warp_mode(initial_warp_mode_resource);
+    }
+    
     /* Limit warp rendering to 10fps */
     warp_render_tick_interval = tick_per_second() / 10.0;
-    
+
     vsync_hook = hook;
     vsync_suspend_speed_eval();
 }
 
-/* FIXME: This function is not needed here anymore, however it is
-   called from sound.c and can only be removed if all other ports are
-   changed to use similar vsync code. */
-int vsync_disable_timer(void)
+void vsync_shutdown(void)
 {
-    return 0;
+    int i;
+    
+    for (i = 0; i < 2; i++) {
+        if (callback_queues[0].queue) {
+            lib_free(callback_queues[0].queue);
+            callback_queues[0].queue = NULL;
+        }
+    }
 }
 
 /* This should be called whenever something that has nothing to do with the
@@ -310,12 +360,12 @@ void vsync_suspend_speed_eval(void)
     /* TODO - Is this needed any more now that late vsync is detected
        in vsync_do_vsync() */
     network_suspend();
-    speed_eval_suspended = 1;
+    sync_reset = true;
 }
 
 void vsync_reset_hook(void)
 {
-    execute_vsync_callbacks();    
+    execute_vsync_callbacks();
 
     vsync_suspend_speed_eval();
 }
@@ -323,205 +373,230 @@ void vsync_reset_hook(void)
 void vsyncarch_get_metrics(double *cpu_percent, double *emulated_fps, int *is_warp_enabled)
 {
     METRIC_LOCK();
-    
+
     *cpu_percent = vsync_metric_cpu_percent;
     *emulated_fps = vsync_metric_emulated_fps;
     *is_warp_enabled = vsync_metric_warp_enabled;
-    
+
     METRIC_UNLOCK();
 }
 
+/*
+ * TODO: Grow measurements array as needed so 5 seconds can be stored.
+ * This will allow warp measurements to be stablise!
+ */
 #define MEASUREMENT_SMOOTH_FACTOR 0.99
 #define MEASUREMENT_FRAME_WINDOW  250
 
-static void update_performance_metrics(unsigned long frame_time)
-{
-    static int oldest_measurement_index;
-    static int next_measurement_index;
-    static int frames_counted;
+static int measurement_count;
+static int next_measurement_index;
 
-    /* For knowing the relevant timespan */
-    static unsigned long frame_times[MEASUREMENT_FRAME_WINDOW];
-    
-    /* For measuring emulator cpu cycles per second */
-    static CLOCK clocks[MEASUREMENT_FRAME_WINDOW];
+/* For knowing the relevant timespan */
+static tick_t last_tick;
+static tick_t tick_deltas[MEASUREMENT_FRAME_WINDOW];
+static uint64_t cumulative_tick_delta;
+
+/* For measuring emulator cpu cycles per second */
+static CLOCK last_clock;
+static CLOCK clock_deltas[MEASUREMENT_FRAME_WINDOW];
+static uint64_t cumulative_clock_delta;
+
+static void reset_performance_metrics(tick_t frame_tick)
+{
+    /*
+     * The emulator is just starting, or resuming from pause, or entering
+     * warp, or exiting warp. So we reset the fps calculations.
+     */
+
+    last_tick = frame_tick;
+    last_clock = maincpu_clk;
+
+    measurement_count = 0;
+    next_measurement_index = 0;
     
+    cumulative_tick_delta = 0;
+    cumulative_clock_delta = 0;
+
+    METRIC_LOCK();
+
+    /* The final smoothing function requires that we initialise the public metrics. */
+    if (timer_speed > 0) {
+        vsync_metric_emulated_fps = (double)timer_speed * refresh_frequency / 100.0;
+        vsync_metric_cpu_percent  = timer_speed;
+    } else {
+        vsync_metric_emulated_fps = (0.0 - timer_speed);
+        vsync_metric_cpu_percent  = (0.0 - timer_speed) / refresh_frequency * 100;
+    }
+
+    METRIC_UNLOCK();
+}
+
+static void update_performance_metrics(tick_t frame_tick)
+{
     /* how many seconds of wallclock time the measurement window covers */
     double frame_timespan_seconds;
-    
+
     /* how many emulated seconds of cpu time have been emulated */
     double clock_delta_seconds;
-    
-    METRIC_LOCK();
-    
-    if (sync_reset) {
-        /*
-         * The emulator is just starting, or resuming from pause, or entering
-         * warp, or exiting warp. So we reset the fps calculations. We also
-         * throw away this initial measurement, because the emulator is just
-         * about to skip the timing sleep and immediately produce the next frame.
-         *
-         * TODO: Don't reset numbers on unpause, and account for the gap.
-         */
-        sync_reset = 0;
-        
-        frame_times[0] = frame_time;
-        clocks[0] = maincpu_clk;
-        
-        next_measurement_index = 1;
-        
-        if (warp_enabled) {
-            /* Don't bother with seed measurements when entering warp mode, just reset */
-            oldest_measurement_index = 0;
-            frames_counted = 1;
-        } else {
-            int i;
-            /*
-             * For normal speed changes, exiting warp etc, initialise with a full set
-             * of fake perfect measurements
-             */
-            for (i = 1; i < MEASUREMENT_FRAME_WINDOW; i++) {
-                frame_times[i] = frame_time - ((MEASUREMENT_FRAME_WINDOW - i) * ticks_per_frame);
-                clocks[i] = maincpu_clk - (CLOCK)((MEASUREMENT_FRAME_WINDOW - i) * cycles_per_sec / refresh_frequency);
-            }
-            
-            oldest_measurement_index = 1;
-            frames_counted = MEASUREMENT_FRAME_WINDOW;
-        }
-    
-        /* The final smoothing function requires that we initialise the public metrics */
-        if (timer_speed > 0) {
-            vsync_metric_emulated_fps = (double)timer_speed * refresh_frequency / 100.0;
-            vsync_metric_cpu_percent  = timer_speed;
-        } else {
-            vsync_metric_emulated_fps = (0.0 - timer_speed);
-            vsync_metric_cpu_percent  = (0.0 - timer_speed) / refresh_frequency * 100;
-        }
-        
-        /* printf("INIT frames_counted %d %.3f seconds - %0.3f%% cpu, %.3f fps\n", frames_counted, frame_timespan_seconds, vsync_metric_cpu_percent, vsync_metric_emulated_fps); fflush(stdout); */
 
-        METRIC_UNLOCK();
-        
+    CLOCK main_cpu_clock = maincpu_clk;
+
+    if (metrics_reset) {
+        metrics_reset = false;
+        reset_performance_metrics(frame_tick);
         return;
     }
-    
-    /* Capure this frame's measurements */
-    frame_times[next_measurement_index] = frame_time;
-    clocks[next_measurement_index] = maincpu_clk;
 
-    if(frames_counted == MEASUREMENT_FRAME_WINDOW) {
-        if (++oldest_measurement_index == MEASUREMENT_FRAME_WINDOW) {
-            oldest_measurement_index = 0;
-        }
+    if (measurement_count == MEASUREMENT_FRAME_WINDOW) {
+        /* Remove the oldest measurement */
+        cumulative_tick_delta -= tick_deltas[next_measurement_index];
+        cumulative_clock_delta -= clock_deltas[next_measurement_index];
     } else {
-        frames_counted++;
+        measurement_count++;
     }
-    
+
+    /* Add this frame's measurement */
+    tick_deltas[next_measurement_index] = frame_tick - last_tick;
+    clock_deltas[next_measurement_index] = main_cpu_clock - last_clock;
+
+    cumulative_tick_delta += tick_deltas[next_measurement_index];
+    cumulative_clock_delta += clock_deltas[next_measurement_index];
+
+    last_tick = frame_tick;
+    last_clock = main_cpu_clock;
+
     /* Calculate our final metrics */
-    frame_timespan_seconds = (double)(frame_time - frame_times[oldest_measurement_index]) / tick_per_second();
-    clock_delta_seconds = (double)(maincpu_clk - clocks[oldest_measurement_index]) / cycles_per_sec;
+    frame_timespan_seconds = (double)cumulative_tick_delta / tick_per_second();
+    clock_delta_seconds = (double)cumulative_clock_delta / cycles_per_sec;
+
+    METRIC_LOCK();
 
     /* smooth and make public */
     vsync_metric_cpu_percent  = (MEASUREMENT_SMOOTH_FACTOR * vsync_metric_cpu_percent)  + (1.0 - MEASUREMENT_SMOOTH_FACTOR) * (clock_delta_seconds / frame_timespan_seconds * 100.0);
-    vsync_metric_emulated_fps = (MEASUREMENT_SMOOTH_FACTOR * vsync_metric_emulated_fps) + (1.0 - MEASUREMENT_SMOOTH_FACTOR) * ((double)(frames_counted - 1) / frame_timespan_seconds);
+    vsync_metric_emulated_fps = (MEASUREMENT_SMOOTH_FACTOR * vsync_metric_emulated_fps) + (1.0 - MEASUREMENT_SMOOTH_FACTOR) * ((double)measurement_count / frame_timespan_seconds);
     vsync_metric_warp_enabled = warp_enabled;
-    
-    /* printf("frames_counted %d %.3f seconds - %0.3f%% cpu, %.3f fps\n", frames_counted, frame_timespan_seconds, vsync_metric_cpu_percent, vsync_metric_emulated_fps); fflush(stdout); */
-    
+
+    /* printf("%.3f seconds - %0.3f%% cpu, %.3f fps (CLOCK delta: %u)\n", frame_timespan_seconds, vsync_metric_cpu_percent, vsync_metric_emulated_fps, clock_deltas[next_measurement_index]); fflush(stdout); */
+
+    METRIC_UNLOCK();
+
     /* Get ready for next invoke */
     if (++next_measurement_index == MEASUREMENT_FRAME_WINDOW) {
         next_measurement_index = 0;
     }
-    
-    METRIC_UNLOCK();
 }
 
 void vsync_do_end_of_line(void)
 {
     const int microseconds_between_sync = 2 * 1000;
-    
-    unsigned long tick_between_sync = tick_per_second() * microseconds_between_sync / 1000000;
-    unsigned long tick_now;
-    unsigned long tick_delta;
-    
+
+    tick_t tick_between_sync = tick_per_second() / (1000000 / microseconds_between_sync);
+    tick_t tick_now;
+    tick_t tick_delta;
+    tick_t ticks_until_target;
+
     bool tick_based_sync_timing;
 
+    CLOCK main_cpu_clock = maincpu_clk;
     CLOCK sync_clk_delta;
-    unsigned long sync_emulated_ticks;
-    
+    double sync_emulated_ticks;
+
+    /* used to preserve the fractional ticks betwen calls */
+    static double sync_emulated_ticks_offset;
+
     /*
      * Ideally the vic chip draw alarm wouldn't be triggered
      * during shutdown but here we are - apply workaround.
      */
-    
+
     if (archdep_is_exiting()) {
-        mainlock_yield_once();
+        mainlock_yield();
         return;
     }
 
     /* deal with any accumulated sound immediately */
     tick_based_sync_timing = sound_flush();
-    
-    tick_now = tick_after(last_sync_tick);
 
-    /*
-     * If it's been a long time between scanlines, (such as when paused in
-     * debuggeretc) then reset speed eval and sync. Otherwise, the emulator
-     * will warp until it catches up, which is rarely good when this far out
-     * of sync. This means the emulator will run slower overall than hoped.
-     */
+    tick_now = tick_now_after(last_sync_tick);
 
-    if (!speed_eval_suspended && (tick_now - last_sync_tick) > ticks_per_frame * 5) {
-        if (last_sync_tick != 0) {
-            log_warning(LOG_DEFAULT, "sync is far too late, resetting sync");
-        }
-        vsync_suspend_speed_eval();
-    }
+    if (sync_reset) {
+        log_message(LOG_DEFAULT, "Sync reset");
+        sync_reset = false;
+        metrics_reset = true;
 
-    if (speed_eval_suspended) {
         last_sync_emulated_tick = tick_now;
         last_sync_tick = tick_now;
-        last_sync_clk = maincpu_clk;
+        last_sync_clk = main_cpu_clock;
         sync_target_tick = tick_now;
 
-        speed_eval_suspended = 0;
-        sync_reset = 1;
         return;
     }
 
     /* how many host ticks since last sync. */
     tick_delta = tick_now - last_sync_tick;
-    
+
     /* is it time to consider keyboard, joystick ? */
     if (tick_delta >= tick_between_sync) {
         
-        /* deal with user input */
-        joystick();
-                
-        /* Do we need to slow down the emulation here or can we rely on the audio device? */
-        if (tick_based_sync_timing && !warp_enabled) {
-            
-            /* add the emulated clock cycles since last sync. */
-            sync_clk_delta = maincpu_clk - last_sync_clk;
-            
+        if (warp_enabled) {
+            /* During warp we need to periodically allow the UI a chance with the mainlock */
+            tick_sleep(1);
+        } else {
+            /*
+             * Compare the emulated time vs host time.
+             *
+             * First, add the emulated clock cycles since last sync.
+             */
+
+            sync_clk_delta = main_cpu_clock - last_sync_clk;
+
             /* amount of host ticks that represents the emulated duration */
             sync_emulated_ticks = (double)tick_per_second() * sync_clk_delta / emulated_clk_per_second;
 
-            /* 
-             * We sleep so that our host is blocked for long enough to catch up
-             * with how long the emulated machine would have taken.
-             */
-
+            /* combine with leftover offset from last sync */
+            sync_emulated_ticks += sync_emulated_ticks_offset;
+            
+            /* calculate the ideal host tick representing the emulated tick */
             sync_target_tick += sync_emulated_ticks;
 
-            /* Some tricky wrap around cases to deal with */
-            if (sync_target_tick - tick_now > 0 && sync_target_tick - tick_now < tick_per_second()) {
-                tick_sleep(sync_target_tick - tick_now);
+            /* Preserve the fractional component for next time */
+            sync_emulated_ticks_offset = sync_emulated_ticks - (tick_t)sync_emulated_ticks;
+
+            /*
+             * How many host ticks to catch up with emulator?
+             *
+             * If we are behind the emulator, this will be a giant number
+             */
+
+            ticks_until_target = sync_target_tick - tick_now;
+
+            if (ticks_until_target < tick_per_second()) {
+
+                /*
+                 * Emulation timing / sync is OK.
+                 */
+
+                /* If we can't rely on the audio device for timing, slow down here. */
+                if (tick_based_sync_timing) {
+                    tick_sleep(ticks_until_target);
+                }
+
+            } else if ((tick_t)0 - ticks_until_target > tick_per_second()) {
+
+                /*
+                 * We are more than a second behind, reset sync and accept that we're not running at full speed.
+                 */
+
+                log_warning(LOG_DEFAULT, "Sync is %.3f ms behind", (double)TICK_TO_MICRO((tick_t)0 - ticks_until_target) / 1000);
+                sync_reset = true;
             }
         }
         
+        /* deal with pending user input */
+        joystick();
+
         last_sync_tick = tick_now;
-        last_sync_clk = maincpu_clk;        
+        last_sync_clk = main_cpu_clock;
     }
 
     /* Do we need to update the thread priority? */
@@ -538,27 +613,57 @@ void vsync_do_end_of_line(void)
     }
 }
 
-/* This is called at the end of each screen frame. */
-int vsync_do_vsync(struct video_canvas_s *c, int been_skipped)
+bool vsync_should_skip_frame(struct video_canvas_s *canvas)
 {
-    // static unsigned long next_frame_start = 0;
-    static int skipped_redraw_count = 0;
-    static unsigned long last_vsync;
+    tick_t now = tick_now();
 
-    unsigned long now;
-    unsigned long network_hook_time = 0;
-    // long delay;
-    int skip_next_frame = 0;
-    
     /*
-     * Ideally the vic chip draw alarm wouldn't be triggered
+     * Ideally the draw alarm wouldn't be triggered
      * during shutdown but here we are - apply workaround.
      */
-    
+
     if (archdep_is_exiting()) {
-        return 1;
+        /* skip this frame */
+        return true;
     }
 
+    /*
+     * Limit rendering fps if we're in warp mode.
+     * It's ugly enough for dqh to weep but makes warp faster.
+     */
+
+    if (warp_enabled) {
+        if (now < canvas->warp_next_render_tick) {
+            if (now < canvas->warp_next_render_tick - warp_render_tick_interval) {
+                /* next render tick is further ahead than it should be */
+                canvas->warp_next_render_tick = now + warp_render_tick_interval;    
+            }
+            /* skip this frame */
+            return true;
+        } else {
+            canvas->warp_next_render_tick += warp_render_tick_interval;
+
+            if (canvas->warp_next_render_tick < now) {
+                /* Warp rendering is behind, catch up */
+                canvas->warp_next_render_tick = now + warp_render_tick_interval;
+            }
+            /* render this frame */
+            return false;
+        }
+    }
+    
+    /* render this frame */
+    return false;
+}
+
+/* This is called at the end of each screen frame. */
+void vsync_do_vsync(struct video_canvas_s *c)
+{
+    static tick_t last_vsync;
+
+    tick_t now;
+    tick_t network_hook_time = 0;
+
     monitor_vsync_hook();
 
     /*
@@ -576,10 +681,10 @@ int vsync_do_vsync(struct video_canvas_s *c, int been_skipped)
 
     if (network_connected()) {
         /* TODO - re-eval if any of this network stuff makes sense */
-        network_hook_time = tick_delta(network_hook_time);
+        network_hook_time = tick_now_delta(network_hook_time);
 
-        if (network_hook_time > (unsigned long)ticks_per_frame) {
-            // next_frame_start += network_hook_time;
+        if (network_hook_time > (tick_t)ticks_per_frame) {
+            /* next_frame_start += network_hook_time; */
             last_vsync += network_hook_time;
         }
     }
@@ -589,36 +694,19 @@ int vsync_do_vsync(struct video_canvas_s *c, int been_skipped)
     debug_check_autoplay_mode();
 #endif
 
-    now = tick_after(last_vsync);
+    now = tick_now_after(last_vsync);
     update_performance_metrics(now);
 
-    /*
-     * Limit rendering fps if we're in warp mode.
-     * It's ugly enough for dqh to weep but makes warp faster.
-     */
-    
-    if (warp_enabled) {
-        if (now < warp_next_render_tick) {
-            skip_next_frame = 1;
-            skipped_redraw_count++;
-        } else {
-            warp_next_render_tick += warp_render_tick_interval;
-            skipped_redraw_count = 0;
-        }
-    }
-
     vsyncarch_postsync();
 
 #ifdef VSYNC_DEBUG
-    log_debug("vsync: start:%lu  delay:%ld  sound-delay:%lf  end:%lu  next-frame:%lu  frame-ticks:%lu", 
+    log_debug("vsync: start:%lu  delay:%ld  sound-delay:%lf  end:%lu  next-frame:%lu  frame-ticks:%lu",
                 now, delay, sound_delay * 1000000, tick_now(), next_frame_start, ticks_per_frame);
 #endif
-    
-    execute_vsync_callbacks();
-    
-    kbdbuf_flush();
-    
-    last_vsync = now;
 
-    return skip_next_frame;
+    execute_vsync_callbacks();
+
+    kbdbuf_flush();
+
+    last_vsync = now;
 }
